/********************************************************************************/
/* File:  iany.hpp                                                              */
/* Class: IAny                                                                  */
/*                                                                              */
/* DAX's  implementation of IAny                                                */
/*                                                                              */
/* The implementation is coded after OMG IDL to C++ mapping, with minor         */
/* differences.                                                                 */
/* There is a set of type-safe operations defined on IAny, using <<= and >>=    */
/* operators. There is also type-unsafe interface to set and get typecode and   */
/* value inside IAny.                                                           */
/* IMPORTANT: If type-unsafe operations are used to create/change IAny object,  */
/* type-safe operations CAN't BE USED to access IAny correctly.                 */
/********************************************************************************/

/******************************************************************************/
/*                                                                            */
/* IBM(R) VisualAge(TM) C++ for OS/2(R), Version 3                            */
/* (C) Copyright IBM Corp. 1991, 1995.                                        */
/*  - Licensed Material - Program-Property of IBM - All Rights Reserved.      */
/* US Government Users Restricted Rights - Use, duplication or disclosure     */
/* restricted by GSA ADP Schedule Contract with IBM Corp.                     */
/*                                                                            */
/******************************************************************************/

#ifndef _IANY_HPP_
#define _IANY_HPP_

#include <ibase.hpp>
#include "itypecod.hpp"
#include <istring.hpp>
#include "dax.hpp"

 class EXPORT IAny : public IBase
 {

 public:
   IAny();                                      // type-safe; create tk_null value
   IAny (const ITypeCode& tc, void* _vP, IBoolean release=False);   // type-unsafe

   virtual ~IAny();

// This operations are used to create IAny given another IAny.
// Note: In all cases, the new IAny will NOT be responsible for managing space
//       allocated for the value.

   IAny(const IAny&);
   IAny& operator= (const IAny&);


// ************************** TYPE-SAFE methods *****************************************

   void    operator<<=(short);
   void    operator<<=(unsigned short);
   void    operator<<=(long);
   void    operator<<=(unsigned long);
   void    operator<<=(float);
   void    operator<<=(double);
   void    operator<<=(char);
   void    operator<<=(const IString&);
   void    operator<<=(const ITypeCode&);

   void    operator>>=(short&) const;
   void    operator>>=(unsigned short&) const;
   void    operator>>=(long&) const;
   void    operator>>=(unsigned long&) const;
   void    operator>>=(float&) const;
   void    operator>>=(double&) const;
   void    operator>>=(char&) const;
   void    operator>>=(IString&) const;



// ************************** TYPE-UNSAFE methods **********************************

   void
     replace (const ITypeCode&, void* value, Boolean release=False);
   ITypeCode*
     getType() const; // allocates new ITypeCode. Receiver responsible for allocated mem.
   void*
     getValue() const;


// ****************************** PRIVATE SECTION **********************************
 private:

   union {
     short          _value_short;
     unsigned short _value_ushort;
     long           _value_long;
     unsigned long  _value_ulong;
     float          _value_float;
     double         _value_double;
     char           _value_char;
     IString*       _value_string;
     ITypeCode*     _value_typecode;
     void*          _value_void;
   } ;

   ITypeCode*        _tpCode;     // typecode
   ITypeCode::TCKind _value_type; // type of one of the union members
   IBoolean          _release;    // if TRUE, assume ownership of the storage

   void _cleanUp();
 };

#ifndef _IANY_INL_
#include "iany.inl"
#endif

#endif
