/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <iptr.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)

template < class Element, class Key >
IAPriorityQueue < Element, Key > ::~IAPriorityQueue ()
{
}

template < class Element, class Key >
void IAPriorityQueue < Element, Key >::
addAllFrom (IAPriorityQueue < Element, Key > const& collection)
{ ICHECK (! isIdentical (collection),
          IIdenticalCollectionException, IIdenticalCollectionText)
  IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
  forCursor (*cursor) {
    add (collection.elementAt (*cursor));
  }
};

template < class Element, class Key >
void IAPriorityQueue < Element, Key >::
copy (IAPriorityQueue < Element, Key > const& collection)
{ if (! isIdentical (collection)) {
    removeAll ();
    addAllFrom (collection);
  }
};

template < class Element, class Key >
void* IAPriorityQueue < Element, Key >::
identity () const
{ return (void*) this;
}

template <class Element, class Key>
IBoolean IAPriorityQueue < Element, Key >::
containsAllKeysFrom (IAPriorityQueue < Element,
                                       Key > const& collection) const
{ IBoolean result = True;
  IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
  forCursor (*cursor) {
    if (! containsElementWithKey
            (key (collection.elementAt (*cursor)))) {
      result = False;
      break;
    }
  }
  return result;
};

template < class Element, class Key >
long IAPriorityQueue < Element, Key >::
compare (IAPriorityQueue < Element, Key > const& collection,
         long (*comparisonFunction)
           (Element const&, Element const&)) const
{ long result = 0;
  IAutoPointer < ICursor > thisCursor (newCursor (), IINIT);
  IAutoPointer < ICursor >
    collectionCursor (collection.newCursor (), IINIT);

  for (thisCursor->setToFirst (), collectionCursor->setToFirst ();
       result == 0 &&
         thisCursor->isValid () && collectionCursor->isValid ();
       thisCursor->setToNext (), collectionCursor->setToNext ()) {
    result = (*comparisonFunction)
                 (this->elementAt (*thisCursor),
                  collection.elementAt (*collectionCursor));
  }

  if (result == 0) {
    if (thisCursor->isValid () && ! collectionCursor->isValid ())
      result = 1;
    else if (! thisCursor->isValid () && collectionCursor->isValid ())
      result = -1;
    else {
    }
  }
  return result;
}

#pragma info (restore)
