/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IAQUEUE_H
#define _IAQUEUE_H

#include <iglobals.h>
#include <icursor.h>
#include <ibexcept.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

template < class Element >
class IAQueue {
public:

  virtual
          ~IAQueue          ();

  virtual
  IBoolean add              (Element const&) = 0;

  virtual
  IBoolean add              (Element const&, ICursor&) = 0;

  virtual
  void     addAllFrom       (IAQueue <Element> const&);

  virtual
  void     addAsLast        (Element const&) = 0;

  virtual
  void     addAsLast        (Element const&, ICursor&) = 0;

  virtual
  IBoolean allElementsDo    (IBoolean (*function)
                              (Element const&, void*),
                             void* additionalArgument = 0) const = 0;

  virtual
  IBoolean allElementsDo    (IConstantIterator <Element>&) const = 0;

  virtual
  Element const&
           anyElement       () const = 0;

  virtual
  long     compare          (IAQueue < Element > const&,
                             long (*comparisonFunction)
                              (Element const&, Element const&)) const;

  virtual
  void     copy             (IAQueue <Element> const&);

  virtual
  void     dequeue          () = 0;

  virtual
  void     dequeue          (Element&) = 0;

  virtual
  Element const&
           elementAt        (ICursor const&) const = 0;

  virtual
  Element const&
           elementAtPosition (IPosition) const = 0;

  virtual
  void     enqueue          (Element const&) = 0;

  virtual
  void     enqueue          (Element const&, ICursor&) = 0;

  virtual
  Element const&
           firstElement     () const = 0;

  virtual
  IBoolean isBounded        () const = 0;

  virtual
  IBoolean isEmpty          () const = 0;

  virtual
  IBoolean isFirst          (ICursor const&) const = 0;

  virtual
  IBoolean isFull           () const = 0;

  virtual
  IBoolean isLast           (ICursor const&) const = 0;

  virtual
  Element const&
           lastElement      () const = 0;

  virtual
  INumber  maxNumberOfElements () const = 0;

  virtual
  ICursor* newCursor        () const = 0;

  virtual
  INumber  numberOfElements () const = 0;

  virtual
  void     removeAll        () = 0;

  virtual
  void     removeFirst      () = 0;

  virtual
  IBoolean setToFirst       (ICursor&) const = 0;

  virtual
  IBoolean setToLast        (ICursor&) const = 0;

  virtual
  IBoolean setToNext        (ICursor&) const = 0;

  virtual
  void     setToPosition    (IPosition, ICursor&) const = 0;

#ifndef __BIN_COMPAT_BASE_43__
  virtual
  IPosition     position    (ICursor const&) const = 0;
#endif

  virtual
  IBoolean setToPrevious    (ICursor&) const = 0;

  virtual
  IBoolean isConsistent     () const = 0;

protected:

  virtual
  void*    identity () const;

  IBoolean isIdentical (IAQueue < Element > const& collection) const
           { return identity () == collection.identity ();
           }

};

#ifndef __TEMPINC__
#include <iaqueue.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
