/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <new.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)


#ifdef ICL_NO_INLINE
#define inline
#include <iavlkss.inl>
#undef inline
#endif

// ********************** IGAvlKeySortedSet::Cursor ********************

template < class Element, class Key, class ElementOps >
IBoolean IGAvlKeySortedSetCursor < Element, Key, ElementOps >::
setToFirst ()
{ return ivCollection->setToFirst (*this);
}

template < class Element, class Key, class ElementOps >
IBoolean IGAvlKeySortedSetCursor < Element, Key, ElementOps >::
setToNext ()
{ return ivCollection->setToNext (*this);
}

template < class Element, class Key, class ElementOps >
IBoolean IGAvlKeySortedSetCursor < Element, Key, ElementOps >::
isValid () const
{ return ivNode != 0;
}

template < class Element, class Key, class ElementOps >
void IGAvlKeySortedSetCursor < Element, Key, ElementOps >::
invalidate ()
{ ivNode = 0;
}

template < class Element, class Key, class ElementOps >
void IGAvlKeySortedSetCursor < Element, Key, ElementOps >::
copy (ICursor const& cursor)
{ operator= ((IGAvlKeySortedSetCursor
               < Element, Key, ElementOps > const&) cursor);
}

template < class Element, class Key, class ElementOps >
IBoolean IGAvlKeySortedSetCursor < Element, Key, ElementOps >::
operator== (ICursor const& cursor) const
{ return operator== ((IGAvlKeySortedSetCursor
                       < Element, Key, ElementOps > const&) cursor);
}

template < class Element, class Key, class ElementOps >
Element const& IGAvlKeySortedSetCursor < Element, Key, ElementOps >::
element () const
{ return ivCollection->elementAt (*this);
}

template < class Element, class Key, class ElementOps >
IBoolean IGAvlKeySortedSetCursor < Element, Key, ElementOps >::
setToLast ()
{ return ivCollection->setToLast (*this);
}

template < class Element, class Key, class ElementOps >
IBoolean IGAvlKeySortedSetCursor < Element, Key, ElementOps >::
setToPrevious ()
{ return ivCollection->setToPrevious (*this);
}

// ************************** IGAvlKeySortedSet ************************


template < class Element, class Key, class ElementOps >
IAvlKeySortedSetImplNode*
IGAvlKeySortedSet < Element, Key, ElementOps >::
newNodeFromElem (Element const& element) const
{ void* nodeSpace = ivElementOps.allocate
                      (sizeof (Node)
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
  ICHECK (nodeSpace != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return new (nodeSpace) Node (element);
}

template < class Element, class Key, class ElementOps >
IAvlKeySortedSetImplNode*
IGAvlKeySortedSet < Element, Key, ElementOps >::
newNode (void const* element) const
{ return newNodeFromElem (*(Element const*) element);
}

template < class Element, class Key, class ElementOps >
IAvlKeySortedSetImplNode*
IGAvlKeySortedSet < Element, Key, ElementOps >::
copyNode (void const* node) const
{ return newNodeFromElem (((Node const*) node)->ivElement);
}

template < class Element, class Key, class ElementOps >
void IGAvlKeySortedSet < Element, Key, ElementOps >::
deleteNode (void* node) const
{ ((Node*) node)->~Node ();
  ivElementOps.deallocate
                      (node
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
}

template < class Element, class Key, class ElementOps >
void* IGAvlKeySortedSet < Element, Key, ElementOps >::
elementAtNode (void* node) const
{ return &((Node*) node)->ivElement;
}

template < class Element, class Key, class ElementOps >
void const* IGAvlKeySortedSet < Element, Key, ElementOps >::
constElementAtNode (void const* node) const
{ return &((Node*) node)->ivElement;
}

template < class Element, class Key, class ElementOps >
void const* IGAvlKeySortedSet < Element, Key, ElementOps >::
keyAt (void const* node) const
{ return &ivElementOps.key (((Node*) node)->ivElement);
}

template < class Element, class Key, class ElementOps >
long IGAvlKeySortedSet < Element, Key, ElementOps >::
compareKeyToElement (void const* node, void const* element) const
{ return ivElementOps.keyOps.compare
           (ivElementOps.key (((Node const*)node)->ivElement),
            ivElementOps.key (*(Element const*)element));
}

template < class Element, class Key, class ElementOps >
long IGAvlKeySortedSet < Element, Key, ElementOps >::
compareKeyToKey (void const* node, void const* key) const
{ return ivElementOps.keyOps.compare
           (ivElementOps.key (((Node const*)node)->ivElement),
            (*(Key const*)key));
}

template < class Element, class Key, class ElementOps >
void IGAvlKeySortedSet < Element, Key, ElementOps >::
copyFrom (void* node, void const* element) const
{ ivElementOps.assign
    (((Node*)node)->ivElement, *(Element const*)element);
}

template < class Element, class Key, class ElementOps >
IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
constantFunctionIteration (void *iterationFunction,
                           void* env,
                           void const* node) const
{ return (*(IBoolean (*) (Element const&, void*)) iterationFunction)
           (((Node const*)node)->ivElement, env);
}

template < class Element, class Key, class ElementOps >
IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
functionIteration (void *iterationFunction, void* env, void* node) const
{ return (*(IBoolean (*) (Element&, void*)) iterationFunction)
           (((Node*)node)->ivElement, env);
}

template < class Element, class Key, class ElementOps >
IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
constantIteratorIteration (void* iterator, void const* node) const
{ return ((IConstantIterator < Element >*)iterator)->
           applyTo (((Node const*)node)->ivElement);
}

template < class Element, class Key, class ElementOps >
IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
iteratorIteration (void* iterator, void* node) const
{ return ((IIterator < Element >*)iterator)->
           applyTo (((Node*)node)->ivElement);
}

template < class Element, class Key, class ElementOps >
long IGAvlKeySortedSet < Element, Key, ElementOps >::
functionComparison (void *comparisonFunction,
                    void const* node1,
                    void const* node2) const
{ return (*(long (*) (Element const&, Element const&) const)
            comparisonFunction)
           ( ((Node const*)node1)->ivElement,
             ((Node const*)node2)->ivElement );
}

#pragma info (restore)
