/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IAVLKSS_H
#define _IAVLKSS_H

#include <iglobals.h>
#include <icursor.h>
#include <iiavlkss.h>
#include <ibexcept.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class Key, class ElementOps >
class IGAvlKeySortedSet;

template < class Element, class Key, class ElementOps >
class IGAvlKeySortedSetNode;

template < class Element, class Key, class ElementOps >
class IGAvlKeySortedSetCursor : public IOrderedCursor < Element > {
public:

           IGAvlKeySortedSetCursor
                      (IGAvlKeySortedSet
                        < Element, Key, ElementOps > const& c)
           : ivCollection (&c), ivNode (0)
           {
           }

  IBoolean setToFirst ();

  IBoolean setToNext  ();

  IBoolean isValid    () const;

  void     invalidate ();

  void     copy (ICursor const& cursor);

  IBoolean operator== (ICursor const& cursor) const;

  Element const&
           element      () const;

  IBoolean setToLast     ();

  IBoolean setToPrevious ()
           ;

  inline
  IBoolean operator== (IGAvlKeySortedSetCursor
                        < Element,
                          Key, ElementOps > const& cursor) const
           { return ivCollection == cursor.ivCollection &&
                    ivNode == cursor.ivNode;
           }

  inline
  IBoolean operator!= (IGAvlKeySortedSetCursor
                        < Element,
                          Key, ElementOps > const& cursor) const
           { return ! operator== (cursor);
           }

protected:

  IGAvlKeySortedSet < Element, Key, ElementOps > const*
           ivCollection;

  IAvlKeySortedSetImplNode *
           ivNode;

  IBoolean isFor (IGAvlKeySortedSet
                   < Element, Key, ElementOps > const& c) const
           { return ivCollection == &c;
           }

  friend class IGAvlKeySortedSet < Element, Key, ElementOps >;

};

template < class Element, class Key, class ElementOps >
class IGAvlKeySortedSetNode :
    public IAvlKeySortedSetImplNode {

    Element  ivElement;

             IGAvlKeySortedSetNode (Element const& element)
             : ivElement (element)
             {
             }

            ~IGAvlKeySortedSetNode ()
             {
             }

    friend class IGAvlKeySortedSet < Element, Key, ElementOps >;

    friend class IGAvlKeySortedSetCursor < Element, Key, ElementOps >;


};


template < class Element, class Key, class ElementOps >
class IGAvlKeySortedSet : protected IAvlKeySortedSetImpl {

public:


  typedef IGAvlKeySortedSetCursor < Element, Key, ElementOps > Cursor;

  inline   IGAvlKeySortedSet (INumber numberOfElements = 100);

  inline   IGAvlKeySortedSet (IGAvlKeySortedSet
                               < Element, Key, ElementOps > const&);

  inline  ~IGAvlKeySortedSet ();

  inline
  IBoolean add               (Element const&);

  inline
  IBoolean add               (Element const&, ICursor&);

  inline
  void     addAllFrom        (IGAvlKeySortedSet
                               < Element, Key, ElementOps > const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean allElementsDo     (IBoolean (*function) (Element&, void*),
                              void* additionalArgument = 0);

  inline
  IBoolean allElementsDo     (IIterator <Element>&);

  inline
  IBoolean allElementsDo     (IBoolean (*function)
                               (Element const&, void*),
                              void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo     (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement        () const;

  inline
  long     compare           (IGAvlKeySortedSet
                               < Element, Key, ElementOps > const&,
                              long (*comparisonFunction)
                               (Element const&, Element const&)) const;

  inline
  IBoolean containsElementWithKey (Key const&) const;

  inline
  IBoolean containsAllKeysFrom (IGAvlKeySortedSet
                                 < Element,
                                   Key, ElementOps > const&) const;

  inline
  Element const&
           elementAt         (ICursor const&) const;

  inline
  Element& elementAt         (ICursor const&);

  inline
  Element const&
           elementAtPosition (IPosition) const;

  inline
  Element const&
           elementWithKey    (Key const&) const;

  inline
  Element& elementWithKey    (Key const&);

  inline
  Element const&
           firstElement      () const;

  inline
  IBoolean isBounded         () const;

  inline
  IBoolean isEmpty           () const;

  inline
  IBoolean isFirst           (ICursor const&) const;

  inline
  IBoolean isFull            () const;

  inline
  IBoolean isLast            (ICursor const&) const;

  inline
  Key const&
           key               (Element const&) const;

  inline
  Element const&
           lastElement       () const;

  inline
  IBoolean locateElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateNextElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateOrAddElementWithKey (Element const&);

  inline
  IBoolean locateOrAddElementWithKey (Element const&, ICursor&);

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor         () const;

  inline
  INumber  numberOfElements  () const;

  inline
  IGAvlKeySortedSet < Element, Key, ElementOps >&
           operator =        (IGAvlKeySortedSet
                               < Element, Key, ElementOps > const&);

  inline
  void     removeAt          (ICursor&);

  inline
  void     removeAll         ();

  inline
  INumber  removeAll         (IBoolean (*property)
                               (Element const&, void*),
                              void* additionalArgument = 0);

  inline
  void     removeAtPosition  (IPosition);

  inline
  IBoolean removeElementWithKey (Key const&);

  inline
  void     removeFirst       ();

  inline
  void     removeLast        ();

  inline
  void     replaceAt         (ICursor const&, Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean setToFirst        (ICursor&) const;

  inline
  IBoolean setToNext         (ICursor&) const;

  inline
  IBoolean setToLast         (ICursor&) const;

  inline
  void     setToPosition     (IPosition, ICursor&) const;

#ifndef __BIN_COMPAT_BASE_43__
  inline
  IPosition     position     (ICursor const&) const;
#endif

  inline
  IBoolean setToPrevious     (ICursor&) const;

protected:

  typedef IGAvlKeySortedSetNode < Element, Key, ElementOps > Node;

  friend class IGAvlKeySortedSetCursor < Element, Key, ElementOps >;

public:

  inline
  IBoolean isConsistent      () const;

protected:

  ElementOps ivElementOps;

  IAvlKeySortedSetImplNode*
           newNodeFromElem     (Element const&) const;

  IAvlKeySortedSetImplNode*
           newNode             (void const* element) const;

  IAvlKeySortedSetImplNode*
           copyNode            (void const* node) const;

  void     deleteNode          (void* node) const;

  void*    elementAtNode       (void *node) const;

  void const*
           constElementAtNode  (void const* node) const;

  void const*
           keyAt               (void const* node) const;

  long     compareKeyToElement (void const* node,
                                void const* element) const;

  long     compareKeyToKey     (void const* node,
                                void const* key) const;

  IBoolean isEqualTo           (void const* node1,
                                void const* node2) const;

  IBoolean isEqualToElement    (void const* node,
                                void const* element) const;

  void     copyFrom            (void* node,
                                void const* element) const;

  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      void const* node) const;

  IBoolean functionIteration   (void *function,
                                void* env,
                                void* node) const;

  IBoolean constantIteratorIteration (void* iterator,
                                      void const* node) const;

  IBoolean iteratorIteration   (void* iterator, void* node) const;

  long     functionComparison  (void *compareFunction,
                                void const* node1,
                                void const* node2) const;

};

template < class Element, class Key >
class IAvlKeySortedSet :
  public IGAvlKeySortedSet < Element, Key, IKCOps < Element, Key > > {
public:


  inline   IAvlKeySortedSet (INumber n = 100);

};


#ifdef ICL_NO_INLINE
#undef inline
#else
#include <iavlkss.inl>
#endif

#if ! defined ( __TEMPINC__ )
#include <iavlkss.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
