/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

// ************************** IGAvlKeySortedSet ************************

template < class Element, class Key, class ElementOps >
inline IGAvlKeySortedSet < Element, Key, ElementOps >::
IGAvlKeySortedSet (INumber)
{
}

template < class Element, class Key, class ElementOps >
inline IGAvlKeySortedSet < Element, Key, ElementOps >::
IGAvlKeySortedSet (IGAvlKeySortedSet
                    < Element, Key, ElementOps > const& c)
: IAvlKeySortedSetImpl (c)
{ IAvlKeySortedSetImpl::operator= (c);
}

template < class Element, class Key, class ElementOps >
inline IGAvlKeySortedSet < Element, Key, ElementOps >&
IGAvlKeySortedSet < Element, Key, ElementOps >::
operator = (IGAvlKeySortedSet < Element, Key, ElementOps > const& c)
{ IAvlKeySortedSetImpl::operator= (c);
  return *this;
}

template < class Element, class Key, class ElementOps >
inline IGAvlKeySortedSet < Element, Key, ElementOps >::
~IGAvlKeySortedSet ()
{ removeAll ();
}

template < class Element, class Key, class ElementOps >
inline void IGAvlKeySortedSet < Element, Key, ElementOps >::
addAllFrom (IGAvlKeySortedSet < Element, Key, ElementOps > const& c)
{
#ifndef INO_CHECKS
  ICHECK (this != &c,
          IIdenticalCollectionException, IIdenticalCollectionText)
#endif
  IAvlKeySortedSetImpl::addAllFrom (c);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
add (Element const& e)
{ IAvlKeySortedSetImplNode *dummy;
  return IAvlKeySortedSetImpl::add (&e, dummy);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
add (Element const& e, ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IAvlKeySortedSetImpl::add (&e, ((Cursor&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline Element const& IGAvlKeySortedSet < Element, Key, ElementOps >::
elementAt (ICursor const& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IAvlKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return *(Element const*)
  IAvlKeySortedSetImpl::elementAt (((Cursor const&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline Element& IGAvlKeySortedSet < Element, Key, ElementOps >::
elementAt (ICursor const& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IAvlKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return *(Element*)
  IAvlKeySortedSetImpl::elementAt (((Cursor const&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline Element const& IGAvlKeySortedSet < Element, Key, ElementOps >::
anyElement () const
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  return *(Element const*) IAvlKeySortedSetImpl::firstElement ();
}

template < class Element, class Key, class ElementOps >
inline void IGAvlKeySortedSet < Element, Key, ElementOps >::
removeAt (ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IAvlKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  IAvlKeySortedSetImpl::removeAt (((Cursor&)c).ivNode);
  c.invalidate ();
}

template < class Element, class Key, class ElementOps >
inline INumber IGAvlKeySortedSet < Element, Key, ElementOps >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{ return IAvlKeySortedSetImpl::removeAll ((void*)predicate, env);
}

template < class Element, class Key, class ElementOps >
inline void IGAvlKeySortedSet < Element, Key, ElementOps >::
replaceAt (ICursor const& c, Element const& e)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IAvlKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
#ifndef INO_CHECKS
  ICHECK (ivElementOps.keyOps.compare
           (ivElementOps.key
             (((Node const*)((Cursor const&)c).ivNode)->ivElement),
            ivElementOps.key (e)) == 0,
          IInvalidReplacementException, IInvalidReplacementText)
#endif
  IAvlKeySortedSetImpl::replaceAt (((Cursor const&)c).ivNode, &e);
}

template < class Element, class Key, class ElementOps >
inline void IGAvlKeySortedSet < Element, Key, ElementOps >::
removeAll ()
{ IAvlKeySortedSetImpl::removeAll ();
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
isBounded () const
{ return IAvlKeySortedSetImpl::isBounded ();
}

template < class Element, class Key, class ElementOps >
inline INumber IGAvlKeySortedSet < Element, Key, ElementOps >::
maxNumberOfElements () const
{ ICHECK (False, INotBoundedException, INotBoundedText)
  return 0;
}

template < class Element, class Key, class ElementOps >
inline INumber IGAvlKeySortedSet < Element, Key, ElementOps >::
numberOfElements () const
{ return IAvlKeySortedSetImpl::numberOfElements ();
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
isEmpty () const
{ return IAvlKeySortedSetImpl::isEmpty ();
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
isFull () const
{ return IAvlKeySortedSetImpl::isFull ();
}

template < class Element, class Key, class ElementOps >
//inline IGAvlKeySortedSet < Element, Key, ElementOps >::Cursor*
inline IGAvlKeySortedSetCursor < Element, Key, ElementOps >*
IGAvlKeySortedSet < Element, Key, ElementOps >::
newCursor () const
{ Cursor* result = new Cursor (*this);
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
setToFirst (ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IAvlKeySortedSetImpl::setToFirst (((Cursor&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
setToNext (ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IAvlKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return IAvlKeySortedSetImpl::setToNext (((Cursor&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* env) const
{ return IAvlKeySortedSetImpl::allElementsDo
           ((void*)iterationFunction, env);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
allElementsDo (IBoolean (*iterationFunction) (Element&, void*),
               void* env)
{ return IAvlKeySortedSetImpl::allElementsDo
           ((void*)iterationFunction, env);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return IAvlKeySortedSetImpl::allElementsDo (&iterator);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
allElementsDo (IIterator <Element>& iterator)
{ return IAvlKeySortedSetImpl::allElementsDo (&iterator);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
isConsistent () const
{ return IAvlKeySortedSetImpl::isConsistent ();
}

template < class Element, class Key, class ElementOps >
inline Key const& IGAvlKeySortedSet < Element, Key, ElementOps >::
key (Element const& element) const
{ return ivElementOps.key (element);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
containsElementWithKey (Key const& k) const
{ return IAvlKeySortedSetImpl::containsElementWithKey (&k);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
containsAllKeysFrom (IGAvlKeySortedSet
                      < Element, Key, ElementOps > const& c) const
{ return IAvlKeySortedSetImpl::containsAllKeysFrom (c);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
locateElementWithKey (Key const& k, ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IAvlKeySortedSetImpl::locateElementWithKey
           (&k, ((Cursor&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
locateNextElementWithKey (Key const& k, ICursor& c) const
{ if (ivElementOps.keyOps.compare
       (k, ivElementOps.key (elementAt (c))) <= 0) {
    c.invalidate ();
    return False;
  }
  else
    return locateElementWithKey (k, c);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
replaceElementWithKey (Element const& e)
{ return IAvlKeySortedSetImpl::replaceElementWithKey (&e);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
replaceElementWithKey (Element const& e, ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IAvlKeySortedSetImpl::replaceElementWithKey
           (&e,((Cursor&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
addOrReplaceElementWithKey (Element const& e)
{ return IAvlKeySortedSetImpl::addOrReplaceElementWithKey (&e);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
addOrReplaceElementWithKey (Element const& e, ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IAvlKeySortedSetImpl::
           addOrReplaceElementWithKey (&e, ((Cursor&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
locateOrAddElementWithKey (Element const& e)
{ return IAvlKeySortedSetImpl::locateOrAddElementWithKey (&e);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
locateOrAddElementWithKey (Element const& e, ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IAvlKeySortedSetImpl::
           locateOrAddElementWithKey (&e, ((Cursor&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
removeElementWithKey (Key const& k)
{ return IAvlKeySortedSetImpl::removeElementWithKey (&k);
}

template < class Element, class Key, class ElementOps >
inline Element const& IGAvlKeySortedSet < Element, Key, ElementOps >::
elementWithKey (Key const& k) const
{ return *(Element const*) IAvlKeySortedSetImpl::elementWithKey (&k);
}

template < class Element, class Key, class ElementOps >
inline Element& IGAvlKeySortedSet < Element, Key, ElementOps >::
elementWithKey (Key const& k)
{ return *(Element*) IAvlKeySortedSetImpl::elementWithKey (&k);
}

template < class Element, class Key, class ElementOps >
inline void IGAvlKeySortedSet < Element, Key, ElementOps >::
removeFirst ()
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  IAvlKeySortedSetImpl::removeFirst ();
}

template < class Element, class Key, class ElementOps >
inline void IGAvlKeySortedSet < Element, Key, ElementOps >::
removeLast ()
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  IAvlKeySortedSetImpl::removeLast ();
}

template < class Element, class Key, class ElementOps >
inline void IGAvlKeySortedSet < Element, Key, ElementOps >::
removeAtPosition (IPosition position)
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
  IAvlKeySortedSetImpl::removeAtPosition (position);
}

template < class Element, class Key, class ElementOps >
inline Element const& IGAvlKeySortedSet < Element, Key, ElementOps >::
firstElement () const
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  return *(Element const*) IAvlKeySortedSetImpl::firstElement ();
}

template < class Element, class Key, class ElementOps >
inline Element const& IGAvlKeySortedSet < Element, Key, ElementOps >::
lastElement () const
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  return *(Element const*) IAvlKeySortedSetImpl::lastElement ();
}

template < class Element, class Key, class ElementOps >
inline Element const& IGAvlKeySortedSet < Element, Key, ElementOps >::
elementAtPosition (IPosition position) const
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
  return *(Element const*)
           IAvlKeySortedSetImpl::elementAtPosition (position);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
setToLast (ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IAvlKeySortedSetImpl::setToLast (((Cursor&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
setToPrevious (ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IAvlKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return IAvlKeySortedSetImpl::setToPrevious (((Cursor&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline void IGAvlKeySortedSet < Element, Key, ElementOps >::
setToPosition (IPosition position, ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  IAvlKeySortedSetImpl::setToPosition (position, ((Cursor&)c).ivNode);
}

#ifndef __BIN_COMPAT_BASE_43__
template < class Element, class Key, class ElementOps >
inline IPosition IGAvlKeySortedSet < Element, Key, ElementOps >::
position (ICursor const& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
  return IAvlKeySortedSetImpl::position (((Cursor&)c).ivNode);
}
#endif

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
isFirst (ICursor const& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IAvlKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return IAvlKeySortedSetImpl::isFirst (((Cursor&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGAvlKeySortedSet < Element, Key, ElementOps >::
isLast (ICursor const& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IAvlKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return IAvlKeySortedSetImpl::isLast (((Cursor&)c).ivNode);
}

template < class Element, class Key, class ElementOps >
inline long IGAvlKeySortedSet < Element, Key, ElementOps >::
compare (IGAvlKeySortedSet < Element,
                             Key, ElementOps > const& collection,
         long (*comparisonFunction)
           (Element const&, Element const&)) const
{ return IAvlKeySortedSetImpl::compare
    (collection, (void*)comparisonFunction);
}

// *************************** IAvlKeySortedSet ************************

template < class Element, class Key >
inline IAvlKeySortedSet < Element, Key >::
IAvlKeySortedSet (INumber n)
: IGAvlKeySortedSet < Element, Key, IKCOps < Element, Key > > (n)
{
}
