/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IBAGKS_H
#define _IBAGKS_H

#include <iglobals.h>
#include <icursor.h>
#include <ibexcept.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element >
class IBKeyElm {

public:

  Element ivElem;

  INumber ivOccurrences;

           IBKeyElm ()
           {
           }

           IBKeyElm (Element const& e)
           : ivElem (e), ivOccurrences (1)
           {
           }

           IBKeyElm (Element const& e, INumber occurrences)
           : ivElem (e), ivOccurrences (occurrences)
           {
           }

  friend inline
  Element const&
           key (IBKeyElm < Element > const& e)
           { return e.ivElem;
           }

};

template < class Element, class ElementOps, class Base >
class IWBagOnKeySet;


template < class Element, class ElementOps, class Base >
class IWBagOnKeySetCursor : public IElementCursor < Element > {
public:

           IWBagOnKeySetCursor
                      (IWBagOnKeySet
                        < Element, ElementOps, Base > const& c)
           : ivBaseCursor (c.ivBase)
           {
           }

  IBoolean setToFirst ();

  IBoolean setToNext  ();

  IBoolean isValid    () const;

  void     invalidate ();

  void     copy (ICursor const& cursor);

  IBoolean operator== (ICursor const& cursor) const;

  Element const&
           element      () const;

  inline
  IBoolean operator== (IWBagOnKeySetCursor
                        < Element,
                          ElementOps, Base > const& cursor) const
           { return
             ivBaseCursor.operator== (cursor.ivBaseCursor)
             && ivOccurrence == cursor.ivOccurrence;
           }

  inline
  IBoolean operator!= (IWBagOnKeySetCursor
                        < Element,
                          ElementOps, Base > const& cursor) const
           { return ! operator== (cursor);
           }

protected:

  INumber ivOccurrence;

  Base::Cursor ivBaseCursor;

  friend class IWBagOnKeySet < Element, ElementOps, Base >;

};



template < class Element, class ElementOps, class Base >
class IWBagOnKeySet {
protected:

  Base ivBase;

public:


  typedef IWBagOnKeySetCursor < Element, ElementOps, Base > Cursor;

  inline   IWBagOnKeySet   (INumber numberOfElements = 100);

  inline   IWBagOnKeySet   (IWBagOnKeySet < Element,
                                            ElementOps, Base > const&);


  inline  ~IWBagOnKeySet   ();

  inline
  IBoolean add             (Element const&);

  inline
  IBoolean add             (Element const&, ICursor&);

  inline
  void     addAllFrom      (IWBagOnKeySet < Element,
                                            ElementOps, Base > const&);

  inline
  void     addDifference   (IWBagOnKeySet < Element,
                                            ElementOps, Base > const&,
                            IWBagOnKeySet < Element,
                                            ElementOps, Base > const&);

  inline
  void     addIntersection (IWBagOnKeySet < Element,
                                            ElementOps, Base > const&,
                            IWBagOnKeySet < Element,
                                            ElementOps, Base > const&);

  inline
  void     addUnion        (IWBagOnKeySet < Element,
                                            ElementOps, Base > const&,
                            IWBagOnKeySet < Element,
                                            ElementOps, Base > const&);

  inline
  IBoolean allElementsDo   (IBoolean (*function) (Element&, void*),
                            void* additionalArgument = 0);

  inline
  IBoolean allElementsDo   (IIterator <Element>&);

  inline
  IBoolean allElementsDo   (IBoolean (*function)
                             (Element const&, void*),
                            void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo   (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement      () const;

  inline
  IBoolean contains        (Element const&) const;

  inline
  IBoolean containsAllFrom (IWBagOnKeySet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  void     differenceWith  (IWBagOnKeySet
                             < Element,
                               ElementOps, Base > const&);

  inline
  Element const&
           elementAt       (ICursor const&) const;

  inline
  Element& elementAt       (ICursor const&);

  inline
  void     intersectionWith (IWBagOnKeySet
                              < Element, ElementOps, Base > const&);

  inline
  IBoolean isBounded       () const;

  inline
  IBoolean isEmpty         () const;

  inline
  IBoolean isFull          () const;

  inline
  IBoolean locate          (Element const&, ICursor&) const;

  inline
  IBoolean locateOrAdd     (Element const&);

  inline
  IBoolean locateOrAdd     (Element const&, ICursor&);

  inline
  IBoolean locateNext      (Element const&, ICursor&) const;

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor       () const;

  inline
  INumber  numberOfDifferentElements () const;

  inline
  INumber  numberOfElements () const;

  inline
  INumber  numberOfOccurrences (Element const&) const;

  inline
  IWBagOnKeySet < Element, ElementOps, Base >&
           operator =      (IWBagOnKeySet
                             < Element, ElementOps, Base > const&);

  inline
  IBoolean operator ==     (IWBagOnKeySet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  IBoolean operator !=     (IWBagOnKeySet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  IBoolean remove          (Element const&);

  inline
  void     removeAll       ();

  inline
  INumber  removeAll       (IBoolean (*property)
                             (Element const&, void*),
                            void* additionalArgument = 0);

  inline
  INumber  removeAllOccurrences (Element const&);

  inline
  void     removeAt        (ICursor&);

  inline
  void     replaceAt       (ICursor const&, Element const&);

  inline
  IBoolean setToFirst      (ICursor&) const;

  inline
  IBoolean setToNext       (ICursor&) const;

  inline
  IBoolean setToNextDifferentElement (ICursor&) const;

  inline
  void     unionWith       (IWBagOnKeySet
                             < Element, ElementOps, Base > const&);

  friend class IWBagOnKeySetCursor < Element, ElementOps, Base > ;

  inline
  IBoolean isConsistent    () const;

protected:

  ElementOps ivElementOps;

  INumber ivNumberOfElements;

  inline
  IBoolean add (Element const&, INumber);

  inline
  IBoolean add (Element const&, ICursor&, INumber);

  inline
  INumber  numberOfOccurrencesAt (ICursor const&) const;

  struct RemoveEnv {
    IBoolean (*predicate) (Element const&, void*);
    void* env;
    INumber removeCount;
  };

  static
  IBoolean removePredicate (IBKeyElm < Element > const&, void*);

  static
  IBoolean cleanupPredicate (IBKeyElm < Element > const&, void*);

  // used for cfront support:
  inline
  IElementCursor < IBKeyElm < Element > >&
            baseCursor (Cursor&) const;

  inline
  IElementCursor < IBKeyElm < Element > > const&
            baseCursor (Cursor const&) const;

  inline
  IElementCursor < IBKeyElm < Element > >&
            baseCursor (ICursor&) const;

  inline
  IElementCursor < IBKeyElm < Element > > const&
            baseCursor (ICursor const&) const;

};

template < class Element, class ElementOps >
class IBKSOps {

  ElementOps elementOps;

public:

  void*    allocate   (size_t s
#ifdef __DEBUG_ALLOC__
                      , char const* fn, size_t ln
#endif
                      ) const
           { return elementOps.allocate (s
#ifdef __DEBUG_ALLOC__
                                        , fn, ln
#endif
                                        );
           }

  void     deallocate (void* p
#ifdef __DEBUG_ALLOC__
                      , char const* fn, size_t ln
#endif
                      ) const
           { elementOps.deallocate (p
#ifdef __DEBUG_ALLOC__
                                   , fn, ln
#endif
                                   );
           }

  void     assign (IBKeyElm < Element >& e1,
                   IBKeyElm < Element > const& e2) const
           { elementOps.assign (e1.ivElem, e2.ivElem);
             e1.ivOccurrences = e2.ivOccurrences;
           }

  Element const&
           key (IBKeyElm < Element > const& e) const
           { return e.ivElem;
           }

  ElementOps keyOps;

};


#define IDefineGBagOnGKeySet(GKeySet, GBag) \
template < class Element, class ElementOps > \
class GBag : \
  public IWBagOnKeySet \
           < Element, ElementOps, \
             GKeySet < IBKeyElm < Element >, Element, \
                       IBKSOps < Element, ElementOps > > > { \
public: \
  GBag (INumber n = 100) : \
    IWBagOnKeySet \
      < Element, ElementOps, \
        GKeySet < IBKeyElm < Element >, Element, \
                  IBKSOps < Element, ElementOps > > > (n) {} \
};



#ifdef ICL_NO_INLINE
#undef inline
#else
#include <ibagks.inl>
#endif

#if ! defined (__TEMPINC__)
#include <ibagks.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
