/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

// *************************** IWBagOnKeySet ***************************

template <class Element, class ElementOps, class Base>
inline IElementCursor < IBKeyElm < Element > >&
IWBagOnKeySet <Element, ElementOps, Base>::
baseCursor (Cursor& cursor) const
{
  return cursor.ivBaseCursor;
}

template <class Element, class ElementOps, class Base>
inline IElementCursor < IBKeyElm < Element > > const&
IWBagOnKeySet <Element, ElementOps, Base>::
baseCursor (Cursor const& cursor) const
{
  return cursor.ivBaseCursor;
}

template <class Element, class ElementOps, class Base>
inline IElementCursor < IBKeyElm < Element > >&
IWBagOnKeySet <Element, ElementOps, Base>::
baseCursor (ICursor& cursor) const
{ return baseCursor ((Cursor&) cursor);
}

template <class Element, class ElementOps, class Base>
inline IElementCursor < IBKeyElm < Element > > const&
IWBagOnKeySet <Element, ElementOps, Base>::
baseCursor (ICursor const& cursor) const
{ return baseCursor ((Cursor const&) cursor);
}

template < class Element, class ElementOps, class Base >
inline IWBagOnKeySet < Element, ElementOps, Base >::
IWBagOnKeySet (INumber n)
: ivBase (n)
, ivNumberOfElements (0)
{
}

template < class Element, class ElementOps, class Base >
inline IWBagOnKeySet < Element, ElementOps, Base >::
IWBagOnKeySet (IWBagOnKeySet < Element,
                               ElementOps, Base > const& collection)
: ivBase (collection.ivBase)
, ivNumberOfElements (collection.ivNumberOfElements)
{
}

template < class Element, class ElementOps, class Base >
inline IWBagOnKeySet < Element, ElementOps, Base >&
IWBagOnKeySet < Element, ElementOps, Base >::
operator= (IWBagOnKeySet < Element,
                           ElementOps, Base > const& collection)
{ ivBase.operator= (collection.ivBase);
  ivNumberOfElements = collection.ivNumberOfElements;
  return *this;
}

template < class Element, class ElementOps, class Base >
inline IWBagOnKeySet < Element, ElementOps, Base >::
~IWBagOnKeySet ()
{
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
add (Element const& element, ICursor &cursor, INumber n)
{ ((Cursor&) cursor).ivOccurrence = 1;
  IBKeyElm < Element > e (element, n);
  if (ivBase.locateOrAddElementWithKey (e, baseCursor (cursor)))
    ivBase.elementAt (baseCursor (cursor)).ivOccurrences += n;
  ivNumberOfElements += n;
  return True;
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
add (Element const& element, INumber n)
{ Cursor dummy (*this);
  return add (element, dummy, n);
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
add (Element const& element, ICursor &cursor)
{ return add (element, cursor, 1);
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
add (Element const& element)
{ Cursor dummy (*this);
  return add (element, dummy, 1);
}

template < class Element, class ElementOps, class Base >
inline void IWBagOnKeySet < Element, ElementOps, Base >::
addAllFrom (IWBagOnKeySet < Element,
                            ElementOps, Base > const& collection)
{
#ifndef INO_CHECKS
  ICHECK (this != &collection,
          IIdenticalCollectionException, IIdenticalCollectionText)
#endif
  Cursor collectionCursor (collection);
  for (IBoolean moreElements = collection.setToFirst (collectionCursor);
       moreElements;
       moreElements =
         collection.setToNextDifferentElement (collectionCursor)) {
    add (collectionCursor.element(),
         collection.numberOfOccurrencesAt (collectionCursor));
  }
}

template < class Element, class ElementOps, class Base >
inline Element const& IWBagOnKeySet < Element, ElementOps, Base >::
elementAt (ICursor const& cursor) const
{ return ivBase.elementAt (baseCursor (cursor)).ivElem;
}

template < class Element, class ElementOps, class Base >
inline Element& IWBagOnKeySet < Element, ElementOps, Base >::
elementAt (ICursor const& cursor)
{ return ivBase.elementAt (baseCursor (cursor)).ivElem;
}

template < class Element, class ElementOps, class Base >
inline Element const& IWBagOnKeySet < Element, ElementOps, Base >::
anyElement () const
{ return ivBase.anyElement ().ivElem;
}

template < class Element, class ElementOps, class Base >
inline void IWBagOnKeySet < Element, ElementOps, Base >::
removeAt (ICursor& cursor)
{ if (--ivBase.elementAt (baseCursor (cursor)).ivOccurrences == 0)
    ivBase.removeAt (baseCursor (cursor));
  ivNumberOfElements--;
  cursor.invalidate ();
}

template < class Element, class ElementOps, class Base >
inline INumber IWBagOnKeySet < Element, ElementOps, Base >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{ RemoveEnv re;
  re.predicate = predicate;
  re.removeCount = 0;
  re.env = env;
  ivBase.removeAll (removePredicate, &re);
  ivNumberOfElements -= re.removeCount;
  return re.removeCount;
}

template < class Element, class ElementOps, class Base >
inline void IWBagOnKeySet < Element, ElementOps, Base >::
replaceAt (ICursor const& cursor, Element const& element)
{ Element& toBeReplaced = ivBase.elementAt (baseCursor (cursor)).ivElem;
#ifndef INO_CHECKS
  ICHECK (ivElementOps.equal (toBeReplaced, element),
          IInvalidReplacementException, IInvalidReplacementText)
#endif
  ivElementOps.assign (toBeReplaced, element);
}

template < class Element, class ElementOps, class Base >
inline void IWBagOnKeySet < Element, ElementOps, Base >::
removeAll ()
{ ivBase.removeAll ();
  ivNumberOfElements = 0;
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
isBounded () const
{ return ivBase.isBounded ();
}

template < class Element, class ElementOps, class Base >
inline INumber IWBagOnKeySet < Element, ElementOps, Base >::
maxNumberOfElements () const
{ return ivBase.maxNumberOfElements ();
}

template < class Element, class ElementOps, class Base >
inline INumber IWBagOnKeySet < Element, ElementOps, Base >::
numberOfElements () const
{ return ivNumberOfElements;
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
isEmpty () const
{ return ivBase.isEmpty ();
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
isFull () const
{ return ivBase.isBounded () &&
  ivNumberOfElements == ivBase.maxNumberOfElements ();
}

template < class Element, class ElementOps, class Base >
inline IWBagOnKeySetCursor < Element, ElementOps, Base >*
IWBagOnKeySet < Element, ElementOps, Base >::
newCursor () const
{ Cursor* result = new Cursor (*this);
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
setToFirst (ICursor &icursor) const
{ Cursor& cursor = (Cursor&) icursor;
  cursor.ivOccurrence = 1;
  return ivBase.setToFirst (baseCursor (icursor));
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
setToNext (ICursor &icursor) const
{ Cursor& cursor = (Cursor&) icursor;
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
  if (cursor.ivOccurrence == numberOfOccurrencesAt (cursor)) {
    cursor.ivOccurrence = 1;
    return ivBase.setToNext (baseCursor (icursor));
  }
  else {
    cursor.ivOccurrence++;
    return True;
  }
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element &, void*),
               void* environment)
{ Cursor cursor (*this);
  forCursor (cursor)
    if (! (*iterationFunction)
            (ivBase.elementAt
              (baseCursor (cursor)).ivElem, environment))
      return False;
  return True;
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
allElementsDo (IIterator <Element>& iterator)
{ Cursor cursor (*this);
  forCursor (cursor)
    if (! iterator.applyTo
            (ivBase.elementAt (baseCursor (cursor)).ivElem))
      return False;
  return True;
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* environment) const
{ Cursor cursor (*this);
  forCursor (cursor)
    if (! (*iterationFunction)
            (ivBase.elementAt
              (baseCursor (cursor)).ivElem, environment))
      return False;
  return True;
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ Cursor cursor (*this);
  forCursor (cursor)
    if (! iterator.applyTo
            (ivBase.elementAt (baseCursor (cursor)).ivElem))
      return False;
  return True;
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
isConsistent () const
{ return ivBase.isConsistent ();
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
contains (Element const& element) const
{ return ivBase.containsElementWithKey (element);
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
containsAllFrom (IWBagOnKeySet
                  < Element, ElementOps, Base > const& collection) const
{ return ivBase.containsAllKeysFrom (collection.ivBase);
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
locate (Element const& element, ICursor &cursor) const
{ ((Cursor&)cursor).ivOccurrence = 1;
  return ivBase.locateElementWithKey (element, baseCursor (cursor));
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
locateOrAdd (Element const& element)
{ Cursor dummy (*this);
  return locateOrAdd (element, dummy);
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
locateOrAdd (Element const& element, ICursor &cursor)
{ ((Cursor&)cursor).ivOccurrence = 1;
  IBKeyElm < Element > e (element);
  IBoolean result =
             ivBase.locateOrAddElementWithKey (e, baseCursor (cursor));
  if (! result)
    ivNumberOfElements++;
  return result;
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
remove (Element const& element)
{ Cursor cursor (*this);
  if (locate (element, cursor)) {
    removeAt (cursor);
    return True;
  }
  else
    return False;
}

template < class Element, class ElementOps, class Base >
inline INumber IWBagOnKeySet < Element, ElementOps, Base >::
numberOfOccurrencesAt (ICursor const& cursor) const
{ return ivBase.elementAt (baseCursor (cursor)).ivOccurrences;
}

template < class Element, class ElementOps, class Base >
inline INumber IWBagOnKeySet < Element, ElementOps, Base >::
numberOfOccurrences (Element const& element) const
{ Cursor cursor (*this);
  if (locate (element, cursor))
    return numberOfOccurrencesAt (cursor);
  else
    return 0;
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
locateNext (Element const& element, ICursor& cursor) const
{ if (ivElementOps.equal (element, elementAt (cursor))) {
    if (ivBase.elementAt (baseCursor (cursor)).ivOccurrences ==
        ((Cursor&) cursor).ivOccurrence) {
      cursor.invalidate ();
      return False;
    } else {
      ((Cursor&) cursor).ivOccurrence++;
      return True;
    }
  }
  else {
    ((Cursor&) cursor).ivOccurrence = 1;
    return ivBase.locateElementWithKey (element, baseCursor (cursor));
  }
}

template < class Element, class ElementOps, class Base >
inline INumber IWBagOnKeySet < Element, ElementOps, Base >::
removeAllOccurrences (Element const& element)
{ Cursor cursor (*this);
  if (! ivBase.locateElementWithKey (element, baseCursor (cursor)))
    return 0;
  INumber result = ivBase.elementAt (baseCursor (cursor)).ivOccurrences;
  ivNumberOfElements -= result;
  ivBase.removeAt (baseCursor (cursor));
  return result;
}

template < class Element, class ElementOps, class Base >
inline INumber IWBagOnKeySet < Element, ElementOps, Base >::
numberOfDifferentElements () const
{ return ivBase.numberOfElements ();
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
setToNextDifferentElement (ICursor& cursor) const
{ ((Cursor&)cursor).ivOccurrence = 1;
  return ivBase.setToNext (baseCursor (cursor));
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
operator== (IWBagOnKeySet < Element,
                            ElementOps, Base > const& collection) const
{ if (ivNumberOfElements != collection.ivNumberOfElements)
    return False;
  Cursor cursor (*this);
  for (IBoolean moreElements = setToFirst (cursor);
       moreElements;
       moreElements = setToNextDifferentElement (cursor)) {
    if (collection.numberOfOccurrences (cursor.element()) !=
        numberOfOccurrencesAt (cursor))
      return False;
  }
  return True;
}

template < class Element, class ElementOps, class Base >
inline IBoolean IWBagOnKeySet < Element, ElementOps, Base >::
operator!= (IWBagOnKeySet < Element,
                            ElementOps, Base > const& collection) const
{ return ! operator== (collection);
}

template < class Element, class ElementOps, class Base >
inline void IWBagOnKeySet < Element, ElementOps, Base >::
unionWith (IWBagOnKeySet < Element,
                           ElementOps, Base > const& collection)
{ if (this != &collection)
    addAllFrom (collection);
  else
  { Cursor cursor (*this);
    for (IBoolean moreElements = setToFirst (cursor);
         moreElements;
         moreElements = setToNextDifferentElement (cursor)) {
      ivBase.elementAt (baseCursor (cursor)).ivOccurrences *= 2;
    }
    ivNumberOfElements *= 2;
  }
}

template < class Element, class ElementOps, class Base >
inline void IWBagOnKeySet < Element, ElementOps, Base >::
intersectionWith (IWBagOnKeySet < Element,
                                  ElementOps, Base > const& collection)
{ if (this != &collection)
  { Cursor cursor (*this);
    for (IBoolean moreElements = setToFirst (cursor);
         moreElements;
         moreElements = setToNextDifferentElement (cursor)) {
      INumber occ = collection.numberOfOccurrences (cursor.element());
      INumber& thisOcc =
        ivBase.elementAt (baseCursor (cursor)).ivOccurrences;
      if (occ < thisOcc) {
        ivNumberOfElements -= thisOcc - occ;
        thisOcc = occ;
      }
    }
    ivBase.removeAll (cleanupPredicate);
  }
}

template < class Element, class ElementOps, class Base >
inline void IWBagOnKeySet < Element, ElementOps, Base >::
differenceWith (IWBagOnKeySet < Element,
                                ElementOps, Base > const& collection)
{ if (this == &collection)
    removeAll ();
  else
  { Cursor cursor (*this);
    for (IBoolean moreElements = setToFirst (cursor);
         moreElements;
         moreElements = setToNextDifferentElement (cursor)) {
      INumber occ = collection.numberOfOccurrences (cursor.element ());
      INumber& thisOcc =
        ivBase.elementAt (baseCursor (cursor)).ivOccurrences;
      INumber n = (thisOcc < occ ? thisOcc : occ);
      thisOcc -= n;
      ivNumberOfElements -= n;
    }
    ivBase.removeAll (cleanupPredicate);
  }
}

template < class Element, class ElementOps, class Base >
inline void IWBagOnKeySet < Element, ElementOps, Base >::
addUnion (IWBagOnKeySet < Element,
                          ElementOps, Base > const& collection1,
          IWBagOnKeySet < Element,
                          ElementOps, Base > const& collection2)
{ if (this == &collection1) {
    if (this == &collection2) {
      Cursor cursor (*this);
      for (IBoolean moreElements = setToFirst (cursor);
           moreElements;
           moreElements = setToNextDifferentElement (cursor)) {
        ivBase.elementAt (baseCursor (cursor)).ivOccurrences *= 3;
      }
      ivNumberOfElements *= 3;
    }
    else {
      unionWith (collection1);
      unionWith (collection2);
    }
  }
  else {
    unionWith (collection2);
    unionWith (collection1);
  }
}

template < class Element, class ElementOps, class Base >
inline void IWBagOnKeySet < Element, ElementOps, Base >::
addIntersection (IWBagOnKeySet < Element,
                                 ElementOps, Base > const& collection1,
                 IWBagOnKeySet < Element,
                                 ElementOps, Base > const& collection2)
{ if (this == &collection1) {
    if (this == &collection2)
      unionWith (*this);
    else
      addIntersection (collection2, collection1);
  }
  else {
    Cursor cursor2 (collection2);
    for (IBoolean moreElements = collection2.setToFirst (cursor2);
         moreElements;
         moreElements =
           collection2.setToNextDifferentElement (cursor2)) {
      Element const& e = cursor2.element ();
      INumber n1, n2;
      n1 = collection1.numberOfOccurrences (e);
      n2 = collection2.numberOfOccurrencesAt (cursor2);
      INumber n = (n1 < n2 ? n1 : n2);
      if (n > 0) {
        Cursor cursor (*this);
        if (ivBase.locateOrAddElementWithKey (e, baseCursor (cursor)))
          ivBase.elementAt (baseCursor (cursor)).ivOccurrences += n;
        else
          ivBase.elementAt (baseCursor (cursor)).ivOccurrences = n;
        ivNumberOfElements += n;
      }
    }
  }
}

template < class Element, class ElementOps, class Base >
inline void IWBagOnKeySet < Element, ElementOps, Base >::
addDifference (IWBagOnKeySet < Element,
                               ElementOps, Base > const& collection1,
               IWBagOnKeySet < Element,
                               ElementOps, Base > const& collection2)
{ if (this == &collection1) {
    if (this == &collection2)
      return;
    else {
      unionWith (*this);
      Cursor cursor2 (collection2);
      for (IBoolean moreElements = collection2.setToFirst (cursor2);
           moreElements;
           moreElements =
             collection2.setToNextDifferentElement (cursor2)) {
        Element const& e = cursor2.element ();
        INumber n1, n2;
        n1 = numberOfOccurrences (e) / 2;
        n2 = collection2.numberOfOccurrencesAt (cursor2);
        INumber n = (n1 < n2 ? n1 : n2);
        if (n > 0) {
          Cursor cursor (*this);
          ivBase.locateElementWithKey (e, baseCursor (cursor));
          ivBase.elementAt (baseCursor (cursor)).ivOccurrences -= n;
          ivNumberOfElements -= n;
        }
      }
    }
  }
  else {
    Cursor cursor1 (collection1);
    for (IBoolean moreElements = collection1.setToFirst (cursor1);
         moreElements;
         moreElements =
           collection1.setToNextDifferentElement (cursor1)) {
      Element const& e = cursor1.element ();
      INumber n1, n2;
      n1 = collection1.numberOfOccurrencesAt (cursor1);
      n2 = collection2.numberOfOccurrences (e);
      INumber n = (n1 < n2 ? 0 : n1 - n2);
      if (n > 0) {
        Cursor cursor (*this);
        if (ivBase.locateOrAddElementWithKey (e, baseCursor (cursor)))
          ivBase.elementAt (baseCursor (cursor)).ivOccurrences += n;
        else
          ivBase.elementAt (baseCursor (cursor)).ivOccurrences = n;
        ivNumberOfElements += n;
      }
    }
  }
}
