/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IBAGKSS_H
#define _IBAGKSS_H

#include <iglobals.h>
#include <icursor.h>
#include <ibexcept.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element >
class IBKSElm {
public:

  Element ivElem;

  INumber ivOccurrences;

           IBKSElm ()
           {
           }

           IBKSElm (Element const& e)
           : ivElem (e), ivOccurrences (1)
           {
           }

           IBKSElm (Element const& e, INumber occurrences)
           : ivElem (e), ivOccurrences (occurrences)
           {
           }

  friend inline
  Element const&
           key (IBKSElm < Element > const& e)
           { return e.ivElem;
           }

};

template < class Element, class ElementOps, class Base >
class IWBagOnKSSet;

template < class Element, class ElementOps, class Base >
class IWBagOnKSSetCursor : public IElementCursor < Element > {
public:

           IWBagOnKSSetCursor
                      (IWBagOnKSSet
                        < Element, ElementOps, Base > const& c)
           : ivBaseCursor (c.ivBase)
           {
           }

  IBoolean setToFirst ();

  IBoolean setToNext  ();

  IBoolean isValid    () const;

  void     invalidate ();

  void     copy       (ICursor const& cursor);

  IBoolean operator== (ICursor const& cursor) const;

  Element const&
           element    () const;

  inline
  IBoolean operator== (IWBagOnKSSetCursor
                        < Element,
                          ElementOps, Base > const& cursor) const
           { return
             ivBaseCursor.operator== (cursor.ivBaseCursor)
             && ivOccurrence == cursor.ivOccurrence;
           }

  inline
  IBoolean operator!= (IWBagOnKSSetCursor
                        < Element,
                          ElementOps, Base > const& cursor) const
           { return ! operator== (cursor);
           }

  protected:

  INumber ivOccurrence;

  Base::Cursor ivBaseCursor;


    friend class IWBagOnKSSet < Element, ElementOps, Base >;

};


template < class Element, class ElementOps, class Base >
class IWBagOnKSSet {
protected:

  Base ivBase;

public:


  typedef IWBagOnKSSetCursor < Element, ElementOps, Base > Cursor;

  inline   IWBagOnKSSet   (INumber numberOfElements = 100);

  inline   IWBagOnKSSet   (IWBagOnKSSet < Element,
                                          ElementOps, Base > const&);


  inline  ~IWBagOnKSSet   ();

  inline
  IBoolean add             (Element const&);

  inline
  IBoolean add             (Element const&, ICursor&);

  inline
  void     addAllFrom      (IWBagOnKSSet < Element,
                                           ElementOps, Base > const&);

  inline
  void     addDifference   (IWBagOnKSSet < Element,
                                           ElementOps, Base > const&,
                            IWBagOnKSSet < Element,
                                           ElementOps, Base > const&);

  inline
  void     addIntersection (IWBagOnKSSet < Element,
                                           ElementOps, Base > const&,
                            IWBagOnKSSet < Element,
                                           ElementOps, Base > const&);

  inline
  void     addUnion        (IWBagOnKSSet < Element,
                                           ElementOps, Base > const&,
                            IWBagOnKSSet < Element,
                                           ElementOps, Base > const&);

  inline
  IBoolean allElementsDo   (IBoolean (*function) (Element&, void*),
                            void* additionalArgument = 0);

  inline
  IBoolean allElementsDo   (IIterator <Element>&);

  inline
  IBoolean allElementsDo   (IBoolean (*function)
                             (Element const&, void*),
                            void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo   (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement      () const;

  inline
  IBoolean contains        (Element const&) const;

  inline
  IBoolean containsAllFrom (IWBagOnKSSet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  void     differenceWith  (IWBagOnKSSet < Element,
                                           ElementOps, Base > const&);

  inline
  Element const&
           elementAt       (ICursor const&) const;

  inline
  Element& elementAt       (ICursor const&);

  inline
  void     intersectionWith (IWBagOnKSSet
                              < Element, ElementOps, Base > const&);

  inline
  IBoolean isBounded       () const;

  inline
  IBoolean isEmpty         () const;

  inline
  IBoolean isFull          () const;

  inline
  IBoolean locate          (Element const&, ICursor&) const;

  inline
  IBoolean locateOrAdd     (Element const&);

  inline
  IBoolean locateOrAdd     (Element const&, ICursor&);

  inline
  IBoolean locateNext      (Element const&, ICursor&) const;

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor       () const;

  inline
  INumber  numberOfDifferentElements () const;

  inline
  INumber  numberOfElements () const;

  inline
  INumber  numberOfOccurrences (Element const&) const;

  inline
  IWBagOnKSSet < Element, ElementOps, Base >&
           operator =      (IWBagOnKSSet < Element,
                                           ElementOps, Base > const&);

  inline
  IBoolean operator ==     (IWBagOnKSSet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  IBoolean operator !=     (IWBagOnKSSet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  IBoolean remove          (Element const&);

  inline
  void     removeAll       ();

  inline
  INumber  removeAll       (IBoolean (*property)
                             (Element const&, void*),
                            void* additionalArgument = 0);

  inline
  INumber  removeAllOccurrences (Element const&);

  inline
  void     removeAt        (ICursor&);

  inline
  void     replaceAt       (ICursor const&, Element const&);

  inline
  IBoolean setToFirst      (ICursor&) const;

  inline
  IBoolean setToNext       (ICursor&) const;

  inline
  IBoolean setToNextDifferentElement (ICursor&) const;

  inline
  void     unionWith       (IWBagOnKSSet
                             < Element, ElementOps, Base > const&);

  friend class IWBagOnKSSetCursor < Element, ElementOps, Base >;

  inline
  IBoolean isConsistent    () const;

protected:

  ElementOps ivElementOps;

  INumber ivNumberOfElements;

  inline
  IBoolean add (Element const&, INumber);

  inline
  IBoolean add (Element const&, ICursor&, INumber);

  inline
  INumber  numberOfOccurrencesAt (ICursor const&) const;

  struct RemoveEnv {
    IBoolean (*predicate) (Element const&, void*);
    void* env;
    INumber removeCount;
  };

  static
  IBoolean removePredicate (IBKSElm < Element > const&, void*);

  static
  IBoolean cleanupPredicate (IBKSElm < Element > const&, void*);

  // used for cfront support:
  inline
  IElementCursor < IBKSElm < Element > >&
           baseCursor (Cursor&) const;

  inline
  IElementCursor < IBKSElm < Element > > const&
           baseCursor (IWBagOnKSSetCursor
                        < Element, ElementOps, Base > const&) const;

  inline
  IElementCursor < IBKSElm < Element > >&
           baseCursor (ICursor&) const;

  inline
  IElementCursor < IBKSElm < Element > > const&
           baseCursor (ICursor const&) const;


};

template < class Element, class ElementOps >
class IBKSSOps {

  ElementOps elementOps;

public:

  void*    allocate   (size_t s
#ifdef __DEBUG_ALLOC__
                      , char const* fn, size_t ln
#endif
                      ) const
           { return elementOps.allocate (s
#ifdef __DEBUG_ALLOC__
                                        , fn, ln
#endif
                                        );
           }

  void     deallocate (void* p
#ifdef __DEBUG_ALLOC__
                      , char const* fn, size_t ln
#endif
                      ) const
           { elementOps.deallocate (p
#ifdef __DEBUG_ALLOC__
                                   , fn, ln
#endif
                                   );
           }

  void     assign (IBKSElm < Element >& e1,
                   IBKSElm < Element > const& e2) const
           { elementOps.assign (e1.ivElem, e2.ivElem);
             e1.ivOccurrences = e2.ivOccurrences;
           }

  Element const&
           key (IBKSElm < Element > const& e) const
           { return e.ivElem;
           }

  ElementOps keyOps;

};


#define IDefineGBagOnGKeySortedSet(GKSSet, GBag) \
template < class Element, class ElementOps > \
class GBag : \
  public IWBagOnKSSet \
           < Element, ElementOps, \
             GKSSet < IBKSElm < Element >, Element, \
                      IBKSSOps < Element, ElementOps > > > { \
public: \
  GBag (INumber n = 100) : \
    IWBagOnKSSet \
      < Element, ElementOps, \
        GKSSet < IBKSElm < Element >, Element, \
                 IBKSSOps < Element, ElementOps > > > (n) {} \
};



#ifdef ICL_NO_INLINE
#undef inline
#else
#include <ibagkss.inl>
#endif

#if ! defined ( __TEMPINC__ )
#include <ibagkss.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
