/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IBEXCEPT_H
#define _IBEXCEPT_H

#include <stdlib.h>
#include <iglobals.h>
#include <iexcbase.h>
#include <ibexctxt.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#if (INO_EXCEPTIONS_SUPPORT)

  #define ITHROW_LOC(exc,loc)\
     exc.addLocation(loc),\
     exc.logExceptionData(),\
     exc.terminate()

  #undef IRETHROW
  #define IRETHROW(exc)

  #define ITRY if (True)
  #define IENDTRY
  #define ICATCH(exc) if (False)
  #define IENDCATCH

#else

  #define ITHROW_LOC(exc,loc)\
     exc.addLocation(loc),\
     exc.logExceptionData(),\
     throw exc

  #define ITRY try
  #define IENDTRY
  #define ICATCH(exc) catch (exc)
  #define IENDCATCH

#endif

#define ICCEXCLASSDECLARE(child,parent) class child : public parent {\
public:\
  child(const char* a, unsigned long b = 0,\
        Severity c = IException::unrecoverable);\
  virtual const char* name() const;\
  virtual ~child();\
  child(const child &);\
  static void throwIt (unsigned msgId, \
                       const char* fileName, unsigned long lineNumber);\
private:\
  operator = ( const child & );\
}

#define ICCEXCLASSIMPLEMENT(child,parent)\
  child :: child(const char* a, unsigned long b,\
                 Severity c)\
    : parent(a, b, c)\
        { }\
  const char* child :: name() const\
    {\
     return ( # child);\
    }\
  child :: ~child() {;}\
  child :: child(const child & a) : parent(a) {;}\
  void child::throwIt (unsigned msgId, \
                       const char* fileName, unsigned long lineNumber)\
  { ICollectionExceptionText msgText (msgId);\
    child exc (msgText);\
    ITHROW_LOC (exc, IExceptionLocation(fileName, "", lineNumber));\
  }

#define ICHECK(test,exc,msgID) \
  if (! (test)) {\
    exc::throwIt (msgID, __FILE__, __LINE__);\
  }

ICCEXCLASSDECLARE(IPreconditionViolation,IException);
  ICCEXCLASSDECLARE(IFullException,IPreconditionViolation);
  ICCEXCLASSDECLARE(IEmptyException,IPreconditionViolation);
  ICCEXCLASSDECLARE(INotEmptyException,IPreconditionViolation);
  ICCEXCLASSDECLARE(INotBoundedException,IPreconditionViolation);
  ICCEXCLASSDECLARE(ICursorInvalidException,IPreconditionViolation);
  ICCEXCLASSDECLARE(IPositionInvalidException,IPreconditionViolation);
  ICCEXCLASSDECLARE(INotContainsKeyException,IPreconditionViolation);
  ICCEXCLASSDECLARE(IKeyAlreadyExistsException,IPreconditionViolation);
  ICCEXCLASSDECLARE(IRootAlreadyExistsException,IPreconditionViolation);
  ICCEXCLASSDECLARE(IChildAlreadyExistsException,
                    IPreconditionViolation);
  ICCEXCLASSDECLARE(IIdenticalCollectionException,
                    IPreconditionViolation);
  ICCEXCLASSDECLARE(IInvalidReplacementException,
                    IPreconditionViolation);
  ICCEXCLASSDECLARE(ICyclicAttachException,IPreconditionViolation);

  ICCEXCLASSDECLARE(IOutOfCollectionMemory,IOutOfMemory);


#pragma info (restore)
#pragma pack ()

#endif
