/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <new.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)

#ifdef ICL_NO_INLINE
#define inline
#include <ibstkss.inl>
#undef inline
#endif

// ********************** IWBSTKeySortedSet::Cursor ********************

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean
IWBSTKeySortedSetCursor < Element, Key, ElementOps, nodeSize >::
setToFirst ()
{ return ivCollection->setToFirst (*this);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean
IWBSTKeySortedSetCursor < Element, Key, ElementOps, nodeSize >::
setToNext ()
{ return ivCollection->setToNext (*this);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean
IWBSTKeySortedSetCursor < Element, Key, ElementOps, nodeSize >::
isValid () const
{ return ivNode != 0;
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
void IWBSTKeySortedSetCursor < Element, Key, ElementOps, nodeSize >::
invalidate ()
{ ivNode = 0;
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
void IWBSTKeySortedSetCursor < Element, Key, ElementOps, nodeSize >::
copy (ICursor const& cursor)
{ operator= ((IWBSTKeySortedSetCursor
               < Element, Key,
                 ElementOps, nodeSize > const&) cursor);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSetCursor < Element,
                                   Key, ElementOps, nodeSize >::
operator== (ICursor const& cursor) const
{ return operator== ((IWBSTKeySortedSetCursor
                       < Element, Key,
                         ElementOps, nodeSize > const&) cursor);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
Element const&
IWBSTKeySortedSetCursor < Element, Key, ElementOps, nodeSize >::
element      () const
{ return ivCollection->elementAt (*this);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean
IWBSTKeySortedSetCursor < Element, Key, ElementOps, nodeSize >::
setToLast     ()
{ return ivCollection->setToLast (*this);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean
IWBSTKeySortedSetCursor < Element, Key, ElementOps, nodeSize >::
setToPrevious ()
{ return ivCollection->setToPrevious (*this);
}

// ************************** IWBSTKeySortedSet ************************

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBSTKeySortedSetImplLnode*
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
newLnode (void const* element) const
{ void* nodeSpace = ivElementOps.allocate
                      (sizeof (Lnode)
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
  ICHECK (nodeSpace != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return new (nodeSpace) Lnode (*(Element const*) element);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBSTKeySortedSetImplLnode*
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
newLnodeFromLnode (void const* lnode) const
{ void* nodeSpace = ivElementOps.allocate
                      (sizeof (Lnode)
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
  ICHECK (nodeSpace != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return new (nodeSpace) Lnode (*(Lnode const*) lnode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBSTKeySortedSetImplInode*
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
newInode () const
{ void* nodeSpace = ivElementOps.allocate
                      (sizeof (Inode)
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
  ICHECK (nodeSpace != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return new (nodeSpace) Inode;
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBSTKeySortedSetImplInode*
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
newInode (void const* inode) const
{ Inode const* in = (Inode const*) inode;

  void* nodeSpace = ivElementOps.allocate
                      (sizeof (Inode)
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
  ICHECK (nodeSpace != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)

  Inode* result = new (nodeSpace) Inode (*in);

  for (IPosition i = 0; i < result->ivm; i++)
    result->ivNodeptrs[i] = in->ivNodeptrs[i];
  for (i = 0; i < result->ivm-1; i++)
    result->ivKeys [i] = in->ivKeys [i];
  return result;
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
void* IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
newKey () const
{ Key* result = new Key;
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
deleteLnode (void* lnode) const
{ ((Lnode*) lnode)->~Lnode ();
  ivElementOps.deallocate
                      (lnode
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
deleteInode (void* inode) const
{ ((Inode*) inode)->~Inode ();
  ivElementOps.deallocate
                      (inode
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
deleteKey (void* key) const
{ delete (Key*) key;
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isKeyLessThan (void const* key1, void const* key2) const
{ return 0 >  ivElementOps.keyOps.compare (
                * (Key*) key1,
                * (Key*) key2);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
copyKey (void *to, INumber toIndex,
         void const* from, INumber fromIndex) const
{ ((Key*)to) [toIndex] = ((Key*)from) [fromIndex];
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
void const* IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
getLnodeElement (IBSTKeySortedSetImplLnode const* lnode) const
{ return &((Lnode const*)lnode)->ivItem;
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
void* IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
getLnodeElement (IBSTKeySortedSetImplLnode* lnode)
{ return &((Lnode*)lnode)->ivItem;
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
replaceLnodeElement (IBSTKeySortedSetImplLnode* lnode,
                     void const* element)
{ ((Lnode*)lnode)->ivItem = *(Element const*) element;
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isLnodeKeyEqualTo (IBSTKeySortedSetImplLnode const* lnode1,
                   void const* lnode2) const
{ return 0 == ivElementOps.keyOps.compare (
                ivElementOps.key (((Lnode const*) lnode1)->ivItem),
                ivElementOps.key (((Lnode const*) lnode2)->ivItem));
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isLnodeKeyEqualToKeyOfElement (IBSTKeySortedSetImplLnode const* lnode,
                               void const* element) const
{ return 0 == ivElementOps.keyOps.compare (
                ivElementOps.key (((Lnode const*) lnode)->ivItem),
                ivElementOps.key (*(Element const*) element));
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isLnodeKeyEqualToKey (IBSTKeySortedSetImplLnode const* lnode,
                      void const* key) const
{ return 0 == ivElementOps.keyOps.compare (
                ivElementOps.key (((Lnode const*) lnode)->ivItem),
                *(Key const*) key);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isKeyLessThanLnode (void const* key, void const* lnode) const
{ return 0 >  ivElementOps.keyOps.compare (
                *(Key const*) key,
                ivElementOps.key (((Lnode const*) lnode)->ivItem));
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
copyKeyFromLnode (void* key, void const* lnode)
{ *(Key*)key = ivElementOps.key (((Lnode const*) lnode)->ivItem);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isKeyGreaterThanLnode (void const* key,
                       INumber index,
                       void const* lnode) const
{ return 0 <  ivElementOps.keyOps.compare (
                ((Key const*) key) [index],
                ivElementOps.key (((Lnode const*) lnode)->ivItem));
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isKeyGreaterThanKey (void const* key1,
                     INumber index,
                     void const* key2) const
{ return 0 <  ivElementOps.keyOps.compare (
                ((Key const*) key1) [index],
                *(Key const*) key2);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isKeyGreaterThanElement (void const* key,
                         INumber index,
                         void const* element) const
{ return 0 <  ivElementOps.keyOps.compare (
                ((Key const*) key) [index],
                ivElementOps.key (*(Element const*) element));
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
constantFunctionIteration (void *iterationFunction,
                           void* env,
                           void const* lnode) const
{ return (*(IBoolean (*) (Element const&, void*)) iterationFunction)
           (((Lnode const*)lnode)->ivItem, env);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
functionIteration (void *iterationFunction,
                   void* env, void* lnode) const
{ return (*(IBoolean (*) (Element&, void*)) iterationFunction)
           (((Lnode*)lnode)->ivItem, env);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
constantIteratorIteration (void* iterator, void const* lnode) const
{ return ((IConstantIterator < Element >*)iterator)->
           applyTo (((Lnode const*)lnode)->ivItem);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
IBoolean IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
iteratorIteration (void* iterator, void* lnode) const
{ return ((IIterator < Element >*)iterator)->
           applyTo (((Lnode*)lnode)->ivItem);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
long IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
functionComparison (void *comparisonFunction,
                    void const* lnode1,
                    void const* lnode2) const
{ return (*(long (*) (Element const&, Element const&) const)
            comparisonFunction)
           ( ((Lnode const*)lnode1)->ivItem,
             ((Lnode const*)lnode2)->ivItem );
}

#pragma info (restore)
