/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IBSTKSS_H
#define _IBSTKSS_H

#include <iglobals.h>
#include <icursor.h>
#include <iibstkss.h>
#include <stdlib.h>
#include <ibexcept.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class Key, class ElementOps, size_t nodeSize >
class IWBSTKeySortedSet;

template < class Element, class Key, class ElementOps, size_t nodeSize >
class IWBSTKeySortedSetCursor : public IOrderedCursor < Element > {
public:

           IWBSTKeySortedSetCursor (IWBSTKeySortedSet
                    < Element, Key, ElementOps, nodeSize > const& c)
           : ivCollection (&c), ivNode (0)
           {
           }

  IBoolean setToFirst ();

  IBoolean setToNext  ();

  IBoolean isValid    () const;

  void     invalidate ();

  void     copy (ICursor const& cursor);

  IBoolean operator== (ICursor const& cursor) const;

  Element const&
           element      () const;

  IBoolean setToLast     ();

  IBoolean setToPrevious ();

  inline
  IBoolean operator== (IWBSTKeySortedSetCursor
                        < Element, Key,
                          ElementOps, nodeSize >const& cursor) const
           { return ivCollection == cursor.ivCollection &&
                    ivNode == cursor.ivNode;
           }

  inline
  IBoolean operator!= (IWBSTKeySortedSetCursor
                        < Element, Key,
                          ElementOps, nodeSize >const& cursor) const
           { return ! operator== (cursor);
           }

protected:

  IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize > const*
           ivCollection;

  IBSTKeySortedSetImplLnode*
           ivNode;

  IBoolean isFor (IWBSTKeySortedSet
                   < Element, Key,
                     ElementOps, nodeSize > const& c) const
           { return ivCollection == &c;
           }

  friend class IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >;

};


template < class Element, class Key, class ElementOps, size_t nodeSize >
class IWBSTKeySortedSetLnode : public IBSTKeySortedSetImplLnode {
protected:

  Element ivItem;

  friend class IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >;

public:

           IWBSTKeySortedSetLnode (Element const& item)
           : ivItem (item)
           { ivIsLeaf = True;
           }

           IWBSTKeySortedSetLnode (IWBSTKeySortedSetLnode
                                    < Element, Key, ElementOps,
                                      nodeSize > const& lnode)

           : ivItem (lnode.ivItem)
           { ivIsLeaf = True;
           }

          ~IWBSTKeySortedSetLnode ()
           {
           }


};

#define IBSTMAX(a,b)  ((a) < (b) ? (b) : (a))

#define IBSTORDER \
  IBSTMAX (2, (nodeSize \
               - (sizeof(IBSTKeySortedSetImplInode) \
                  + sizeof(IBSTKeySortedSetImplNode*)) \
              ) / (sizeof(Key) + sizeof(IBSTKeySortedSetImplNode*)) + 1)

template < class Element, class Key, class ElementOps, size_t nodeSize >
class IWBSTKeySortedSetInode : public IBSTKeySortedSetImplInode {

protected:

  IBSTKeySortedSetImplNode*
            ivNodeptrs [IBSTORDER];

  Key       ivKeys [IBSTORDER - 1];

  friend class IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >;

public:

           IWBSTKeySortedSetInode ()
           { ivIsLeaf = False;
             IBSTKeySortedSetImplInode::ivPtrs = ivNodeptrs;
             IBSTKeySortedSetImplInode::ivKeys = ivKeys;
           }

           IWBSTKeySortedSetInode (IWBSTKeySortedSetInode
                                    < Element, Key, ElementOps,
                                      nodeSize > const& inode)
           : IBSTKeySortedSetImplInode (inode)
           { ivIsLeaf = False;
             IBSTKeySortedSetImplInode::ivPtrs = ivNodeptrs;
             IBSTKeySortedSetImplInode::ivKeys = ivKeys;
           }

          ~IWBSTKeySortedSetInode ()
           {
           }


};

template < class Element, class Key, class ElementOps, size_t nodeSize >
class IWBSTKeySortedSet : public IBSTKeySortedSetImpl {

public:


  typedef IWBSTKeySortedSetCursor < Element, Key,
                                    ElementOps, nodeSize > Cursor;

  inline   IWBSTKeySortedSet (INumber numberOfElements = 100);

  inline   IWBSTKeySortedSet (IWBSTKeySortedSet
                               < Element, Key,
                                 ElementOps, nodeSize > const&);

  inline  ~IWBSTKeySortedSet ();

  inline
  IBoolean add               (Element const&);

  inline
  IBoolean add               (Element const&, ICursor&);

  inline
  void     addAllFrom        (IWBSTKeySortedSet
                               < Element, Key,
                                 ElementOps, nodeSize > const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean allElementsDo     (IBoolean (*function) (Element&, void*),
                              void* additionalArgument = 0);

  inline
  IBoolean allElementsDo     (IIterator <Element>&);

  inline
  IBoolean allElementsDo     (IBoolean (*function)
                               (Element const&, void*),
                              void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo     (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement        () const;

  inline
  long     compare           (IWBSTKeySortedSet
                               < Element, Key,
                                 ElementOps, nodeSize > const&,
                              long (*comparisonFunction)
                               (Element const&, Element const&)) const;

  inline
  IBoolean containsAllKeysFrom (IWBSTKeySortedSet
                                 < Element, Key,
                                   ElementOps, nodeSize > const&) const;

  inline
  IBoolean containsElementWithKey (Key const&) const;

  inline
  Element const&
           elementAt         (ICursor const&) const;

  inline
  Element& elementAt         (ICursor const&);

  inline
  Element const&
           elementAtPosition (IPosition) const;

  inline
  Element const&
           elementWithKey    (Key const&) const;

  inline
  Element& elementWithKey    (Key const&);

  inline
  Element const&
           firstElement      () const;

  inline
  IBoolean isBounded         () const;

  inline
  IBoolean isEmpty           () const;

  inline
  IBoolean isFirst           (ICursor const&) const;

  inline
  IBoolean isFull            () const;

  inline
  IBoolean isLast            (ICursor const&) const;

  inline
  Key const&
           key               (Element const&) const;

  inline
  Element const&
           lastElement       () const;

  inline
  IBoolean locateElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateNextElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateOrAddElementWithKey (Element const&);

  inline
  IBoolean locateOrAddElementWithKey (Element const&, ICursor&);

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor         () const;

  inline
  INumber  numberOfElements  () const;

  inline
  IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >&
           operator =        (IWBSTKeySortedSet
                               < Element, Key,
                                 ElementOps, nodeSize > const&);

  inline
  void     removeAll         ();

  inline
  INumber  removeAll         (IBoolean (*property)
                               (Element const&, void*),
                              void* additionalArgument = 0);

  inline
  void     removeAt          (ICursor&);

  inline
  void     removeAtPosition  (IPosition);

  inline
  IBoolean removeElementWithKey (Key const&);

  inline
  void     removeFirst       ();

  inline
  void     removeLast        ();

  inline
  void     replaceAt         (ICursor const&, Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean setToFirst        (ICursor&) const;

  inline
  IBoolean setToLast         (ICursor&) const;

  inline
  IBoolean setToNext         (ICursor&) const;

  inline
  void     setToPosition     (IPosition, ICursor&) const;

#ifndef __BIN_COMPAT_BASE_43__
  inline
  IPosition     position     (ICursor const&) const;
#endif

  inline
  IBoolean setToPrevious     (ICursor&) const;

protected:

  ElementOps ivElementOps;

  typedef IWBSTKeySortedSetLnode < Element, Key,
                                   ElementOps, nodeSize > Lnode;

  friend class IWBSTKeySortedSetLnode < Element, Key,
                                        ElementOps, nodeSize >;

  typedef IWBSTKeySortedSetInode < Element, Key,
                                   ElementOps, nodeSize > Inode;

  friend class IWBSTKeySortedSetInode < Element, Key,
                                        ElementOps, nodeSize >;

public:

  friend class IWBSTKeySortedSetCursor < Element, Key,
                                         ElementOps, nodeSize >;


  inline
  IBoolean isConsistent      () const;

protected:

  virtual
  IBSTKeySortedSetImplLnode*
           newLnode          (void const* element) const;

  virtual
  IBSTKeySortedSetImplLnode*
           newLnodeFromLnode (void const* lnode) const;

  virtual
  IBSTKeySortedSetImplInode*
           newInode          () const;

  virtual
  IBSTKeySortedSetImplInode*
           newInode          (void const* inode) const;

  virtual
  void*    newKey            () const;

  virtual
  void     deleteLnode       (void* lnode) const;

  virtual
  void     deleteInode       (void* inode) const;

  virtual
  void     deleteKey         (void *key) const;

  virtual
  IBoolean isKeyLessThan     (void const* key1, void const* key2) const;

  virtual
  void     copyKey           (void* toKey,
                              INumber toIndex,
                              void const* fromKey,
                              INumber fromIndex) const;

  virtual
  void const*
           getLnodeElement   (IBSTKeySortedSetImplLnode const*) const;

  virtual
  void*    getLnodeElement   (IBSTKeySortedSetImplLnode*);

  virtual
  void     replaceLnodeElement (IBSTKeySortedSetImplLnode*,
                                void const* element);

  virtual
  IBoolean isLnodeKeyEqualTo (IBSTKeySortedSetImplLnode const*,
                              void const* lnode) const;

  virtual
  IBoolean isLnodeKeyEqualToKeyOfElement
                             (IBSTKeySortedSetImplLnode const*,
                              void const* element) const;

  virtual
  IBoolean isLnodeKeyEqualToKey (IBSTKeySortedSetImplLnode const*,
                                 void const* key) const;

  virtual
  IBoolean isKeyLessThanLnode (void const* key,
                               void const* lnode) const;

  virtual
  void     copyKeyFromLnode  (void* key, void const* lnode);

  virtual
  IBoolean isKeyGreaterThanLnode (void const* key,
                                  INumber index,
                                  void const* lnode) const;

  virtual
  IBoolean isKeyGreaterThanKey (void const* key1,
                                INumber index,
                                void const* key2) const;

  virtual
  IBoolean isKeyGreaterThanElement (void const* key,
                                    INumber index,
                                    void const* element) const;

  virtual
  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      void const* lnode) const;
  virtual
  IBoolean functionIteration (void *function,
                              void* env,
                              void* lnode) const;
  virtual
  IBoolean constantIteratorIteration (void* iterator,
                                      void const* lnode) const;
  virtual
  IBoolean iteratorIteration (void* iterator, void* lnode) const;

  virtual
  long     functionComparison (void *compareFunction,
                               void const* lnode1,
                               void const* lnode2) const;

};

#ifndef IBSTPAGESIZE
#define IBSTPAGESIZE ((size_t) 4096)
#endif

template < class Element, class Key, class ElementOps >
class IGBSTKeySortedSet :
  public IWBSTKeySortedSet < Element, Key, ElementOps, IBSTPAGESIZE > {
public:


  inline   IGBSTKeySortedSet (INumber n = 100);
};

template < class Element, class Key >
class IBSTKeySortedSet :
  public IGBSTKeySortedSet < Element, Key, IKCOps < Element, Key > > {
public:


  inline   IBSTKeySortedSet (INumber n = 100);

};


#ifdef ICL_NO_INLINE
#undef inline
#else
#include <ibstkss.inl>
#endif

#if ! defined (__TEMPINC__)
#include <ibstkss.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
