/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <ibexcept.h>

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
IWBSTKeySortedSet (INumber)
: IBSTKeySortedSetImpl (IBSTORDER)
{
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
IWBSTKeySortedSet (IWBSTKeySortedSet
                    < Element, Key, ElementOps, nodeSize > const& c)
: IBSTKeySortedSetImpl (c.ivTreeOrder)
{ if (c.ivRoot != 0) {
    ivRoot = copySubtree (c.ivRoot);
    ivHeight = c.ivHeight;
    ivNoElements = c.ivNoElements;
  }
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
~IWBSTKeySortedSet ()
{ IBSTKeySortedSetImpl::removeAll ();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean IWBSTKeySortedSet < Element, Key,
                                    ElementOps, nodeSize >::
add (Element const& e)
{ return IBSTKeySortedSetImpl::add(&e);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean IWBSTKeySortedSet < Element, Key,
                                    ElementOps, nodeSize >::
add (Element const& e, ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IBSTKeySortedSetImpl::add(&e, ((Cursor&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
addAllFrom (IWBSTKeySortedSet < Element, Key,
                                ElementOps, nodeSize > const& t)
{
#ifndef INO_CHECKS
  ICHECK (this != &t,
          IIdenticalCollectionException, IIdenticalCollectionText)
#endif
  IBSTKeySortedSetImpl::addAllFrom(t);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline Element const&
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
elementAt (ICursor const& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IBSTKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return *(Element const*)
         IBSTKeySortedSetImpl::elementAt(((Cursor const&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline Element&
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
elementAt (ICursor const& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IBSTKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return *(Element*)
         IBSTKeySortedSetImpl::elementAt(((Cursor const&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline Element const&
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
anyElement () const
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  return *(Element const*) IBSTKeySortedSetImpl::anyElement();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
removeAt (ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IBSTKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  IBSTKeySortedSetImpl::removeAt(((Cursor const&) c).ivNode);
  c.invalidate ();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline INumber
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{ return IBSTKeySortedSetImpl::removeAll (predicate, env);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
replaceAt (ICursor const& c, Element const& e)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IBSTKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
#ifndef INO_CHECKS
  ICHECK (ivElementOps.keyOps.compare
           (ivElementOps.key (elementAt (c)),
            ivElementOps.key (e)) == 0,
          IInvalidReplacementException, IInvalidReplacementText)
#endif
  IBSTKeySortedSetImpl::replaceAt(((Cursor const&) c).ivNode, &e);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
removeAll ()
{ IBSTKeySortedSetImpl::removeAll();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isBounded () const
{ return IBSTKeySortedSetImpl::isBounded();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline INumber
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
maxNumberOfElements () const
{ ICHECK (False, INotBoundedException,
          INotBoundedText)
  return IBSTKeySortedSetImpl::maxNumberOfElements();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline INumber
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
numberOfElements () const
{ return IBSTKeySortedSetImpl::numberOfElements();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isEmpty () const
{ return IBSTKeySortedSetImpl::isEmpty();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isFull () const
{ return IBSTKeySortedSetImpl::isFull();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IWBSTKeySortedSetCursor < Element, Key, ElementOps, nodeSize >*
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
newCursor () const
{ Cursor* result = new Cursor (*this);
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
setToFirst (ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IBSTKeySortedSetImpl::setToFirst(((Cursor&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
setToNext (ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IBSTKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return IBSTKeySortedSetImpl::setToNext(((Cursor&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* env) const
{ return IBSTKeySortedSetImpl::allElementsDo (iterationFunction, env);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
allElementsDo (IBoolean (*iterationFunction) (Element&, void*),
               void* env)
{ return IBSTKeySortedSetImpl::allElementsDo (iterationFunction, env);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return IBSTKeySortedSetImpl::allElementsDo (&iterator);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
allElementsDo (IIterator <Element>& iterator)
{ return IBSTKeySortedSetImpl::allElementsDo (&iterator);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isConsistent () const
{ return IBSTKeySortedSetImpl::isConsistent ();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline Key const&
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
key (Element const& element) const
{ return ivElementOps.key (element);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
containsAllKeysFrom (IWBSTKeySortedSet
                      < Element, Key,
                        ElementOps, nodeSize > const& t) const
{ return IBSTKeySortedSetImpl::containsAllKeysFrom(t);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
containsElementWithKey (Key const& k) const
{ return IBSTKeySortedSetImpl::containsElementWithKey(&k);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
locateElementWithKey (Key const& k, ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IBSTKeySortedSetImpl::locateElementWithKey
           (&k, ((Cursor&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
locateNextElementWithKey (Key const& k, ICursor& c) const
{ if (ivElementOps.keyOps.compare
        (k, ivElementOps.key (elementAt (c))) <= 0) {
    c.invalidate ();
    return False;
  }
  else
    return locateElementWithKey (k, c);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
replaceElementWithKey (Element const& e)
{ return IBSTKeySortedSetImpl::replaceElementWithKey(&e);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
replaceElementWithKey (Element const& e, ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IBSTKeySortedSetImpl::replaceElementWithKey
           (&e, ((Cursor&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
locateOrAddElementWithKey (Element const& e)
{ return ! add (e);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
locateOrAddElementWithKey (Element const& e, ICursor& c)
{ return ! add (e, c);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
addOrReplaceElementWithKey (Element const& e)
{ return IBSTKeySortedSetImpl::addOrReplaceElementWithKey(&e);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
addOrReplaceElementWithKey (Element const& e, ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IBSTKeySortedSetImpl::addOrReplaceElementWithKey
           (&e, ((Cursor&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
removeElementWithKey(Key const& k)
{ return IBSTKeySortedSetImpl::removeElementWithKey(&k);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline Element const&
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
elementWithKey (Key const& k) const
{ return *(Element const*) IBSTKeySortedSetImpl::elementWithKey(&k);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline Element&
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
elementWithKey (Key const& k)
{ return *(Element*) IBSTKeySortedSetImpl::elementWithKey(&k);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
removeFirst ()
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  IBSTKeySortedSetImpl::removeFirst();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
removeLast ()
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  IBSTKeySortedSetImpl::removeLast();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
removeAtPosition (IPosition position)
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
  IBSTKeySortedSetImpl::removeAtPosition(position);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline Element const&
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
firstElement () const
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  return *(Element const*) IBSTKeySortedSetImpl::firstElement();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline Element const&
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
lastElement () const
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  return *(Element const*) IBSTKeySortedSetImpl::lastElement();
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline Element const&
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
elementAtPosition (IPosition position) const
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
  return *(Element const*)
           IBSTKeySortedSetImpl::elementAtPosition(position);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
setToLast (ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IBSTKeySortedSetImpl::setToLast(((Cursor&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
setToPrevious (ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IBSTKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return IBSTKeySortedSetImpl::setToPrevious(((Cursor&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline void IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
setToPosition (IPosition position, ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
  IBSTKeySortedSetImpl::setToPosition(position, ((Cursor&) c).ivNode);
}

#ifndef __BIN_COMPAT_BASE_43__
template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IPosition IWBSTKeySortedSet < Element, Key, ElementOps,
                                     nodeSize >::
position (ICursor const& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
  return IBSTKeySortedSetImpl::position(((Cursor&) c).ivNode);
}
#endif

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isFirst (ICursor const& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IBSTKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return IBSTKeySortedSetImpl::isFirst(((Cursor const&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IBoolean
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
isLast (ICursor const& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IBSTKeySortedSetImpl::checkNode(((Cursor const&)c).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return IBSTKeySortedSetImpl::isLast(((Cursor const&) c).ivNode);
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >&
IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
operator= (IWBSTKeySortedSet < Element, Key,
                               ElementOps, nodeSize > const& t)
{ IBSTKeySortedSetImpl::operator = (t);
  return *this;
}

template < class Element, class Key, class ElementOps, size_t nodeSize >
inline long IWBSTKeySortedSet < Element, Key, ElementOps, nodeSize >::
compare (IWBSTKeySortedSet
          < Element, Key, ElementOps, nodeSize > const& collection,
         long (*comparisonFunction)
           (Element const&, Element const&)) const
{ return IBSTKeySortedSetImpl::compare (collection, comparisonFunction);
}

// ************************** IGBSTKeySortedSet ************************

template < class Element, class Key, class ElementOps >
inline IGBSTKeySortedSet < Element, Key, ElementOps >::
IGBSTKeySortedSet (INumber n)
: IWBSTKeySortedSet < Element, Key, ElementOps, IBSTPAGESIZE > (n)
{
}

// ************************** IBSTKeySortedSet *************************

template < class Element, class Key >
inline IBSTKeySortedSet < Element, Key >::
IBSTKeySortedSet (INumber n)
: IGBSTKeySortedSet < Element, Key, IKCOps < Element, Key > > (n)
{
}
