#ifndef _IDBCSBUF_
#define _IDBCSBUF_
/*******************************************************************************
* FILE NAME: idbcsbuf.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IDBCSBuffer                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <stdlib.h>
#include <limits.h>
#include <ibuffer.hpp>

#pragma pack(4)

class IDBCSBuffer : public IBuffer {
public:

/*------------------------- Subset -------------------------------------------*/
IBuffer
 *subString         ( unsigned startPos,
                      unsigned len,
                      char     padCharacter ) const;

/*------------------------- NLS Testing --------------------------------------*/
Boolean
  isMBCS            ( ) const,
  isDBCS            ( ) const,
  isSBCS            ( ) const,
  isValidMBCS       ( ) const,
  isValidDBCS       ( ) const,
  includesMBCS      ( ) const,
  includesDBCS      ( ) const,
  includesSBCS      ( ) const;



/*------------------------- Searches -----------------------------------------*/
unsigned
  indexOf           ( const IStringTest &aTest,
                      unsigned           startPos = 1 ) const,
  indexOf           ( const char        *pString,
                      unsigned           len,
                      unsigned           startPos = 1 ) const,
  indexOfAnyBut     ( const char        *pString,
                      unsigned           len,
                      unsigned           startPos = 1 ) const,
  indexOfAnyBut     ( const IStringTest &aTest,
                      unsigned           startPos = 1 ) const,
  indexOfAnyOf      ( const char        *pString,
                      unsigned           len,
                      unsigned           startPos = 1 ) const,
  indexOfAnyOf      ( const IStringTest &aTest,
                      unsigned           startPos = 1 ) const,
  lastIndexOf       ( const IStringTest &aTest,
                      unsigned           startPos = 1 ) const,
  lastIndexOf       ( const char        *pString,
                      unsigned           len,
                      unsigned           startPos = 0 ) const,
  lastIndexOfAnyBut ( const char        *pString,
                      unsigned           len,
                      unsigned           startPos = 0 ) const,
  lastIndexOfAnyBut ( const IStringTest &aTest,
                      unsigned           startPos = 0 ) const,
  lastIndexOfAnyOf  ( const char        *pString,
                      unsigned           len,
                      unsigned           startPos = 0 ) const,
  lastIndexOfAnyOf  ( const IStringTest &aTest,
                      unsigned           startPos = 0 ) const;


/*------------------------- Editing ------------------------------------------*/
IBuffer
 *center            ( unsigned newLen,
                      char     padCharacter ),
 *insert            ( const char *pInsert,
                      unsigned    insertLen,
                      unsigned    pos,
                      char        padCharacter ),
 *leftJustify       ( unsigned newLen,
                      char     padCharacter ),
 *lowerCase         ( ),
 *overlayWith       ( const char *overlay,
                      unsigned    len,
                      unsigned    pos,
                      char        padCharacter ),
 *remove            ( unsigned startPos,
                      unsigned numChars ),
 *reverse           ( ),
 *rightJustify      ( unsigned newLen,
                      char     padCharacter ),
 *strip             ( const char             *pChars,
                      unsigned                len,
                      IStringEnum::StripMode  mode ),
 *strip             ( const IStringTest      &aTest,
                      IStringEnum::StripMode  mode ),
 *translate         ( const char *pInputChars,
                      unsigned    inputLen,
                      const char *pOutputChars,
                      unsigned    outputLen,
                      char        padCharacter ),
 *upperCase         ( );

/*------------------------- Allocation ---------------------------------------*/
IBuffer
 *allocate          ( unsigned newLen ) const;

/*------------------------- Queries ------------------------------------------*/
IStringEnum::CharType
  charType          ( unsigned index ) const;

char
 *next              ( const char *prev );

const char
 *next              ( const char *prev ) const;

protected:

/*------------------------- Search Initialization ----------------------------*/
unsigned
  startSearch          ( unsigned startPos,
                         unsigned searchLen ) const,
  startBackwardsSearch ( unsigned startPos,
                         unsigned searchLen ) const;


/*------------------------- Protected Testing --------------------------------*/
Boolean
  isCharValid          ( unsigned    pos,
                         const char *pValidChars,
                         unsigned    numValidChars ) const;

Boolean
  isDBCS1              ( unsigned    pos ) const,
  isPrevDBCS           ( unsigned    pos ) const;

static Boolean
  isSBC                ( char const* );

/*------------------------- Protected Queries --------------------------------*/
const char
 *className            ( ) const;

static size_t
  charLength           ( char const* ),
  maxCharLength        ( );

size_t
  charLength           ( unsigned pos ) const,
  prevCharLength       ( unsigned pos ) const;


friend class IBuffer;

/*------------------------- Constructors -------------------------------------*/
  IDBCSBuffer          ( unsigned bufLength );
 ~IDBCSBuffer          ( );

}; // class IDBCSBuffer

#pragma pack()

#ifndef I_NO_INLINES
  #include <idbcsbuf.inl>
#endif

#endif // _IDBCSBUF_
