#ifndef _IDMIMAGE_
#define _IDMIMAGE_
/*******************************************************************************
* FILE NAME: idmimage.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IDMImage                                                                 *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ivbase.hpp>
#include <ipoint.hpp>
#include <ireslib.hpp>
#include <ibitflag.hpp>

struct _DRAGIMAGE;
class IDMImageData;

#pragma pack(4)


class IDMImage : public IVBase {
typedef IVBase
  Inherited;
public:

class Style;

/*------------------------------- Constructors -------------------------------*/
IDMImage    ( );
IDMImage    ( const IResourceId&    resourceId,
              Boolean               iconResource = true,
              const ISize&          pointerOffset = ISize(0,0),
              const ISize&          stretchSize = ISize(0,0),
              const Style&          style = defaultStyle() );

IDMImage    ( const IPointerHandle& icon,
              const ISize&          pointerOffset = ISize(0,0),
              const ISize&          stretchSize = ISize(0,0),
              const Style&          style = defaultStyle() );

IDMImage    ( const IBitmapHandle&  bitmap,
              const ISize&          pointerOffset = ISize(0,0),
              const ISize&          stretchSize = ISize(0,0),
              const Style&          style = defaultStyle() );

IDMImage    ( unsigned long         arrayOfPoints[],
              const unsigned long   numberOfPoints,
              const ISize&          pointerOffset = ISize(0,0),
              const Style&          style = defaultStyle() );

IDMImage    ( const IDMImage&       image );

virtual
  ~IDMImage ( );

IDMImage
 &operator = ( const IDMImage&      image );

/*---------------------------------- Styles ----------------------------------*/
INESTEDBITFLAGCLASSDEF0(Style, IDMImage);
static const Style
  classDefaultStyle,
  noStyle,
  ptr,
  bmp,
  polygon,
  stretch,
  transparent,
  closed;

static Style
  defaultStyle    ( );
static void
  setDefaultStyle ( const Style& style );

/*----------------------------- Image Processing -----------------------------*/
IPointerHandle
  pointer           ( ) const;

IBitmapHandle
  bitmap            ( ) const;

ISize
  pointerOffset     ( ) const,
  stretchSize       ( ) const;

Style
  style             ( );

unsigned long
  numberOfPoints    ( );

unsigned long
 *pointArray        ( );

IDMImage
 &setPointer        ( const IPointerHandle& pointerImage ),
 &setBitmap         ( const IBitmapHandle&  bitmapImage ),
 &setPointerOffset  ( const ISize&          pointerOffset ),
 &setStretchSize    ( const ISize&          stretchSize ),
 &setStyle          ( const Style&          style ),
 &setNumberOfPoints ( unsigned long         points ),
 &setPointArray     ( unsigned long         arrayOfPoints[],
                      unsigned long         numberOfPoints );

private:
/*--------------------------------- Private ----------------------------------*/
friend class IDMSourceOperation;

IPointerHandle
  pointerImage;

IBitmapHandle
  bitmapImage;

ISystemPointerHandle
  fadeImage;

unsigned long
 *pPointArray;

ISize
  ptrOffset,
  imageStretchSize;

static Style
  currentDefaultStyle;

Style
  imgStyle;

unsigned long
  points;

void
  asPMDragImage ( _DRAGIMAGE* pDragImage,
                   Boolean    useSystemIcons = false,
                   Boolean    fadeIcon = false );

IDMImageData
 *fDMImageData;

/*----------------------------- Obsolete Members -----------------------------*/
#if (IC_OBSOLETE <= IC_OBSOLETE_1)
public:
IDMImage    ( const IResourceId&    resourceId,
              const ISize&          pointerOffset = ISize(0,0),
              const ISize&          stretchSize = ISize(0,0),
              const Style&          style = defaultStyle(),
              Boolean               iconResource = true );
IDMImage    ( unsigned long         arrayOfPoints[],
              const ISize&          pointerOffset = ISize(0,0),
              const Style&          style = defaultStyle(),
              const unsigned long   points = 0 );
IDMImage
 &setPointArray ( unsigned long     arrayOfPoints[] );
#endif

}; // IDMImage


#pragma pack()

#ifndef I_NO_INLINES
  #include <idmimage.inl>
#endif

#endif //_IDMIMAGE_
