#ifndef _IDSCON_HPP_
#define _IDSCON_HPP_

/*******************************************************************************
* FILE NAME: idscon.hpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*     IDatastoreConnection - The context for a connection to a datastore       *
*                                                                              *
* The IDatastoreConnection allows a client to establish the context for        *
* connecting to a datastore and to identify the connection once it is opened.  *
*                                                                              *
* An IDatastoreConnection is established within the context of a single        *
* IUserEnvironment.                                                            *
*                                                                              *
*******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* IBM(R) VisualAge(TM) C++ for OS/2(R), Version 3                            */
/* (C) Copyright IBM Corp. 1991, 1995.                                        */
/*  - Licensed Material - Program-Property of IBM - All Rights Reserved.      */
/* US Government Users Restricted Rights - Use, duplication or disclosure     */
/* restricted by GSA ADP Schedule Contract with IBM Corp.                     */
/*                                                                            */
/******************************************************************************/

#include "idaxbs.h"
#include "dax.hpp"
#include <istdntfy.hpp>
#include "iany.hpp"
#include "idsexc.hpp"

class IString;
class IUserEnvironment;
class AnyOptionSet;
class IDatastoreMgrCli;
class DatastoreMgrCli_Proxy;
class IDatastoreMgrSQL;
class DatastoreMgrSQL_Proxy;
class IDSCursor;

#pragma pack(4)

class EXPORT IDatastoreConnection : public IStandardNotifier {

public:

  IDatastoreConnection ();                                  // Constructor
  IDatastoreConnection (IUserEnvironment &userEnv);         // Constructor

virtual
  ~IDatastoreConnection ();                                 // Destructor

void
  getOption (long option, IAny& value)
     throw(IOptionInvalid);
void
  setOption (long option, const IAny& value)
     throw (IValueTypeInvalid,
            IValueInvalid,
            IOptionInvalid);
void   // take this out later
  setUe(IUserEnvironment *userEnv);

static INotificationId
  setOptionId;

private:

/*--------------------------------- PRIVATE ----------------------------------*/

  Boolean addCursor     (IDSCursor *pDCur);
  Boolean removeCursor  (IDSCursor *pDCur);
  void*   getDSCHandle() {return pDSCHandle;}

  void invalidateCursors ();

AnyOptionSet* pIOptKeySet;            //option dictionary
void        *pUserEnvironmentHandle;  // UserEnvironment Handle
void        *pCursorList;             // Cursor List
void        *pDSCHandle;              // Datastore Connection Handle

// some options can not be set by users of the classes, but must be
// set by some other classes. This method doesn't check the value of
// option before setting it.

void
  _unprotectedSetOption(long option, const IAny& value);

friend class IDatastoreMgrCli;
friend class IDatastoreMgrSQL;
friend class IDSCursorCli;
friend class DatastoreMgrCli_Proxy;
friend class DatastoreMgrSQL_Proxy;

};

// class EXPORT IDatastoreConnectionFactory : public IStandardNotifier {

// public:

//   IDatastoreConnectionFactory ();              // Constructor

// virtual
//   ~IDatastoreConnectionFactory ();             // Destructor

// IDatastoreConnection*
//   createObject( IUserEnvironment *pUe);        // create a IDatastoreConnection

// };

#pragma pack()

#endif /* _IDSCON_HPP_ */
