#ifndef _IDSMCON_HPP_
#define _IDSMCON_HPP_

/*******************************************************************************
* FILE NAME: idsmcon.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*     IDatastore - Manage a datastore connection.                              *
*                                                                              *
* The IDatastore allows a client to establish and manage                       *
* datastore connection.                                                        *
*                                                                              *
*******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* IBM(R) VisualAge(TM) C++ for OS/2(R), Version 3                            */
/* (C) Copyright IBM Corp. 1991, 1995.                                        */
/*  - Licensed Material - Program-Property of IBM - All Rights Reserved.      */
/* US Government Users Restricted Rights - Use, duplication or disclosure     */
/* restricted by GSA ADP Schedule Contract with IBM Corp.                     */
/*                                                                            */
/******************************************************************************/

#include <istring.hpp>
#include <dax.hpp>
#include <istdntfy.hpp>
#include <idsexc.hpp>

#pragma pack(4)

#ifndef __NO_DEFAULT_LIBS__
   #ifdef __IMPORTLIB__
      #pragma library("CPPODS3I.LIB")
   #else
      #pragma library("CPPODS3.LIB")
      #pragma library("SQL_DYN.LIB")
      #pragma library("UPM.LIB")
   #endif
#endif

class EXPORT IDatastore : public IStandardNotifier
{

public:

  IDatastore ();                                  // default Constructor

  IDatastore ( const IString& datastoreName,
                            const IString& userName = "",
                            const IString& authentication = "" );

virtual
  ~IDatastore ();                                 // Destructor

virtual IDatastore&
   connect ()
      throw    (IConnectFailed,
                IDatastoreAccessError,
                IDatastoreConnectionInUse,
                IDatastoreLogoffFailed,
                IDatastoreLogonFailed );

virtual IDatastore&
   connect ( const IString& datastoreName,
             const IString& userName = "",
             const IString& authentication = "" )
      throw    (IConnectFailed,
                IDatastoreAccessError,
                IDatastoreConnectionInUse,
                IDatastoreLogoffFailed,
                IDatastoreLogonFailed );

virtual IDatastore&
   disconnect ()
      throw     (IDatastoreConnectionNotOpen,
                 IDatastoreLogoffFailed,
                 IDisconnectError);

virtual IDatastore&
   commit ()
    throw     (ICompletionTypeInvalid,
               IDatastoreAccessError,
               IDatastoreConnectionNotOpen);

virtual IDatastore&
   rollback ()
    throw     (ICompletionTypeInvalid,
               IDatastoreAccessError,
               IDatastoreConnectionNotOpen);

virtual Boolean
   isConnected ();

// returns true if Share Mode is Exclusive
virtual Boolean
   isShareModeExclusive ();

// enable Share Mode to Exclusive (or reset to Shared)
virtual IDatastore&
   enableShareModeExclusive(Boolean enable = true);

virtual IString
   asString() const;

virtual IString
   datastoreName() const;

virtual IDatastore&
   setDatastoreName(const IString& aDatastoreName);

virtual IString
   userName() const;

virtual IDatastore&
   setUserName(const IString& aUserName);

virtual IString
   authentication() const;

virtual IDatastore&
   setAuthentication(const IString& aAuthentication);

static INotificationId DatastoreNameId;
static INotificationId UserNameId;
static INotificationId AuthenticationId;
static INotificationId isConnectedId;
static INotificationId connectedId;
static INotificationId disconnectedId;
static INotificationId transactedId;
static INotificationId shareModeExclusiveId;

protected:
/*--------------------------------- PROTECTED --------------------------------*/
  // notify observers about connect state changes
  void NotifyAboutConnectState ( Boolean ConnectState );


private:
/*--------------------------------- PRIVATE ----------------------------------*/
  class IDatastoreMgrSQL* pDatastoreMgrSQL;
  class IUserEnvironment* pUserEnvironment;
  class IDatastoreConnection* pDatastoreConnection;
  IString iDatastoreType;
  IString iDatastoreName;
  IString iServerName;
  IString iUserName;
  IString iAuthentication;

};

#pragma pack()

#endif /* _IDSMCON_HPP_ */
