#ifndef _IFONTDLG_
#define _IFONTDLG_
/*******************************************************************************
* FILE NAME: IFONTDLG.HPP                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IFontDialog                                                              *
*     IFontDialog::Settings                                                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <iframe.hpp>
#include <istring.hpp>

class IResourceId;
class IPoint;
class IFont;
class IFontHandler;
class IFontDialogData;
class IFontDialogSettingsData;
struct _FONTDLG;

#pragma pack(4)

class IFontDialog : public IFrameWindow {
typedef IFrameWindow
  Inherited;
public:
class Style;
class Settings;
/*------------------------------ Constructors --------------------------------*/
 IFontDialog(IWindow*        parent,
             IWindow*        owner,
             const Style&    style = defaultStyle() );

 IFontDialog(IWindow*        parent,
             IWindow*        owner,
             const Settings& settings,
             const Style&    style = defaultStyle() );

 IFontDialog(IWindow*        parent,
             IWindow*        owner,
             const Style&    style,
             const Settings& settings );

 IFontDialog(IWindow*        parent,
             IWindow*        owner,
             IHandler*       handler,
             const Style&    style = defaultStyle(),
             const Settings& settings = Settings(0) );

virtual
 ~IFontDialog( );

/*---------------------------------- Styles ----------------------------------*/
  INESTEDBITFLAGCLASSDEF0(Style, IFontDialog);

static const Style
  classDefaultStyle,
  noStyle,
  applyButton,
  bitmapOnly,
  fixedWidthOnly,
  helpButton,
  modeless,
  noSynthesize,
  proportionalOnly,
  resetButton,
  vectorOnly;

static void
  setDefaultStyle        ( const Style& newDefault );
static Style
  defaultStyle           ( );

virtual unsigned long
  convertToGUIStyle ( const IBitFlag& style,
                      Boolean         extendedOnly = false ) const;

class Settings : public IBase {
public:
/*------------------------------- Constructors -------------------------------*/
  Settings   ( IFont*         font = 0 );
 ~Settings   ( );

/*--------------------------- Dialog Customization ---------------------------*/
Settings
 &setTitle            ( const char*        title       ),
 &setTitle            ( const IResourceId& text        ),
 &setPreviewText      ( const char*        previewText ),
 &setPosition         ( const IPoint&      position    );
Settings
 &setDialogTemplate   ( const IResourceId& templateId  );

/*------------------- Setting the Presentation Space -------------------------*/
Settings
 &setDisplayPS        ( const IPresSpaceHandle& presSpaceHandle ),
 &setPrinterPS        ( const IPresSpaceHandle& presSpaceHandle );

/*----------- Setting Information About the Initial Font ---------------------*/
Settings
 &setFont             ( IFont*        font       ),
 &setFamily           ( const char*   fontFamily ),
 &setPointSize        ( unsigned long pointSize  ),
 &setSizeList         ( const char*   sizeList   );

private:
/*------------------------------ Hidden Members ------------------------------*/
Settings    ( const Settings& settings );
Settings
 &operator= ( const Settings& settings );

/*--------------------------------- Private ----------------------------------*/
friend IFontDialog;

IFont
 *fApplyFont;

struct _FONTDLG
 *fData;

IFontDialogSettingsData
 *fSettingsData;
}; // IFontDialog::Settings


/*---------------- Getting Information About the Dialog ----------------------*/

static const unsigned long
  ok,
  cancel;

unsigned long
  buttonPressedId        ( ) const;

long
  returnValue            ( ) const;

Boolean
  isModeless             ( ) const,
  pressedOK              ( ) const;

/*---------------- Getting Information About the Chosen Font -----------------*/

IString
  fontFamily            ( ) const;

unsigned long
  fontWeight            ( ) const,
  fontWidth             ( ) const;

unsigned long
  emHeight              ( ) const,
  xHeight               ( ) const,
  externalLeading       ( ) const,
  pointSize             ( ) const,
  nominalPointSize      ( ) const;

private:
/*------------------------------ Hidden Members ------------------------------*/
IFontDialog   ( const IFontDialog& fontDialog );
IFontDialog
 &operator=   ( const IFontDialog& fontDialog );

/*--------------------------------- Private ----------------------------------*/
friend  void* _System
  _IFontDlgProc(unsigned long hwnd, unsigned long ulMsg, void* mp1, void* mp2);
friend IFont;
friend IFontHandler;

void IFontDialog  :: initialize (IWindow*        parent,
                                 IWindow*        owner,
                                 const Style&    style,
                                 IHandler*       handler,
                                 const Settings& settings);

static Style currentDefaultStyle;

IFont
 *fApplyFont;
IHandler*
  _handlerToAdd;

struct _FONTDLG
 *fData;

IFontDialogData
 *fFontDialogData;

}; // IFontDialog

#pragma pack()

#ifndef I_NO_INLINES
  #include <ifontdlg.inl>
#endif

#endif // _IFONTDLG_
