/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IHSHKB_H
#define _IHSHKB_H

#include <iglobals.h>
#include <icursor.h>
#include <stdlib.h>
#include <iihshkb.h>
#include <ibexcept.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class Key, class ElementOps >
class IGHashKeyBag;

template < class Element, class Key, class ElementOps >
class IGHashKeyBagNode;

template < class Element, class Key, class ElementOps >
class IGHashKeyBagCursor : public IElementCursor < Element > {
public:

           IGHashKeyBagCursor (IGHashKeyBag
                                < Element, Key, ElementOps > const& c)
           : ivCollection (&c)
           {
           }

  IBoolean setToFirst ();

  IBoolean setToNext  ();

  IBoolean isValid    () const;

  void     invalidate ();

  void     copy (ICursor const& cursor);

  IBoolean operator== (ICursor const& cursor) const;

  Element const&
           element    () const;

  inline
  IBoolean operator== (IGHashKeyBagCursor
                        < Element, Key,
                          ElementOps >const& cursor) const
           { return ivCollection == cursor.ivCollection &&
                    ivImpl == cursor.ivImpl;
           }

  inline
  IBoolean operator!= (IGHashKeyBagCursor
                        < Element, Key,
                          ElementOps >const& cursor) const
           { return ! operator== (cursor);
           }

protected:

  IGHashKeyBag < Element, Key, ElementOps > const*
           ivCollection;

  IHashKeyBagImplCursor
           ivImpl;

  IBoolean isFor (IGHashKeyBag
                   < Element, Key, ElementOps > const& c) const
           { return ivCollection == &c;
           }

  friend class IGHashKeyBag < Element, Key, ElementOps >;

};

template < class Element, class Key, class ElementOps >
class IGHashKeyBagNode : public IHashKeyBagImplNode {

    Element  ivElement;

             IGHashKeyBagNode (Element const& element)
             : ivElement (element)
             {
             }

            ~IGHashKeyBagNode ()
             {
             }

    friend class IGHashKeyBag < Element, Key, ElementOps >;

//  friend class IGHashKeyBagCursor < Element, Key, ElementOps >;


};


template < class Element, class Key, class ElementOps >
class IGHashKeyBag : protected IHashKeyBagImpl {

public:


  typedef IGHashKeyBagCursor < Element, Key, ElementOps > Cursor;

  inline   IGHashKeyBag     (INumber numberOfElements = 100);

  inline   IGHashKeyBag     (IGHashKeyBag < Element,
                                            Key, ElementOps > const&);

  inline  ~IGHashKeyBag     ();

  inline
  IBoolean add              (Element const&);

  inline
  IBoolean add              (Element const&, ICursor&);

  inline
  void     addAllFrom       (IGHashKeyBag < Element,
                                            Key, ElementOps > const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean allElementsDo    (IBoolean (*function) (Element&, void*),
                             void* additionalArgument = 0);

  inline
  IBoolean allElementsDo    (IIterator <Element>&);

  inline
  IBoolean allElementsDo    (IBoolean (*function)
                              (Element const&, void*),
                             void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo    (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement       () const;

  inline
  IBoolean containsElementWithKey (Key const&) const;

  inline
  IBoolean containsAllKeysFrom (IGHashKeyBag
                                 < Element, Key,
                                   ElementOps > const&) const;

  inline
  Element const&
           elementAt        (ICursor const&) const;

  inline
  Element& elementAt        (ICursor const&);

  inline
  Element const&
           elementWithKey   (Key const&) const;

  inline
  Element& elementWithKey   (Key const&);

  inline
  IBoolean isBounded        () const;

  inline
  IBoolean isEmpty          () const;

  inline
  IBoolean isFull           () const;

  inline
  Key const&
           key              (Element const&) const;

  inline
  IBoolean locateElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateNextElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateOrAddElementWithKey (Element const&);

  inline
  IBoolean locateOrAddElementWithKey (Element const&, ICursor&);

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor        () const;

  inline
  INumber  numberOfDifferentKeys () const;

  inline
  INumber  numberOfElements () const;

  inline
  INumber  numberOfElementsWithKey (Key const&) const;

  inline
  IGHashKeyBag < Element, Key, ElementOps >&
           operator =       (IGHashKeyBag < Element,
                                            Key, ElementOps > const&);

  inline
  void     removeAll        ();

  inline
  INumber  removeAll        (IBoolean (*property)
                              (Element const&, void*),
                             void* additionalArgument = 0);

  inline
  INumber  removeAllElementsWithKey (Key const&);

  inline
  void     removeAt         (ICursor&);

  inline
  IBoolean removeElementWithKey (Key const&);

  inline
  void     replaceAt        (ICursor const&, Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean setToFirst       (ICursor&) const;

  inline
  IBoolean setToNext        (ICursor&) const;

  inline
  IBoolean setToNextWithDifferentKey (ICursor&) const;

  friend class IGHashKeyBagCursor < Element, Key, ElementOps >;

  inline
  IBoolean isConsistent     () const;

protected:

  typedef IGHashKeyBagNode < Element, Key, ElementOps > Node;

  ElementOps ivElementOps;

  IHashKeyBagImplNode*
           newNode              (Element const&) const;

  IHashKeyBagImplNode*
           newNode              (void const* element) const;

  IHashKeyBagImplNode*
           copyNode             (void const* node) const;

  void     deleteNode           (void* node) const;

  void*    newNodeBlock         (INumber) const;

  void*    newCountBlock        (INumber) const;

  void     removeBlock          (void*) const;

  INumber  getHashvalue         (void const* key, INumber) const;

  void*    elementAt            (void *node) const;

  void const*
           keyAt                (void const* node) const;

  IBoolean isKeyEqualToElement  (void const* node,
                                 void const* element) const;

  IBoolean isKeyEqualToKey      (void const* node,
                                 void const* key) const;

  IBoolean isEqualTo            (void const* node1,
                                 void const* node2) const;

  IBoolean isEqualToElement     (void const* node,
                                 void const* element) const;

  void     copyFrom             (void* node, void const* element) const;


  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      void const* node) const;

  IBoolean functionIteration    (void *function,
                                 void* env, void* node) const;

  IBoolean constantIteratorIteration (void* iterator,
                                      void const* node) const;

  IBoolean iteratorIteration    (void* iterator, void* node) const;

};

template < class Element, class Key >
class IHashKeyBag :
  public IGHashKeyBag < Element, Key, IKEHOps < Element, Key > > {
public:


  inline   IHashKeyBag (INumber n = 100);
};


#ifdef ICL_NO_INLINE
#undef inline
#else
#include <ihshkb.inl>
#endif

#if ! defined (__TEMPINC__)
#include <ihshkb.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
