/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <new.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)


#ifdef ICL_NO_INLINE
#define inline
#include <ihshks.inl>
#undef inline
#endif

// ************************ IHashKeySet::Cursor ************************

template < class Element, class Key, class ElementOps >
IBoolean IGHashKeySetCursor < Element, Key, ElementOps >::
setToFirst ()
{ return ivCollection->setToFirst (*this);
}

template < class Element, class Key, class ElementOps >
IBoolean IGHashKeySetCursor < Element, Key, ElementOps >::
setToNext ()
{ return ivCollection->setToNext (*this);
}

template < class Element, class Key, class ElementOps >
IBoolean IGHashKeySetCursor < Element, Key, ElementOps >::
isValid () const
{ return ivImpl.ivNode != 0;
}

template < class Element, class Key, class ElementOps >
void IGHashKeySetCursor < Element, Key, ElementOps >::
invalidate ()
{ ivImpl.ivNode = 0;
}

template < class Element, class Key, class ElementOps >
void IGHashKeySetCursor < Element, Key, ElementOps >::
copy (ICursor const& cursor)
{ operator= ((IGHashKeySetCursor < Element,
                                   Key, ElementOps > const&) cursor);
}

template < class Element, class Key, class ElementOps >
IBoolean IGHashKeySetCursor < Element, Key, ElementOps >::
operator== (ICursor const& cursor) const
{ return operator== ((IGHashKeySetCursor
                       < Element, Key, ElementOps > const&) cursor);
}

template < class Element, class Key, class ElementOps >
Element const& IGHashKeySetCursor < Element, Key, ElementOps >::
element () const
{ return ivCollection->elementAt (*this);
}

// **************************** IHashKeySet ****************************

template < class Element, class Key, class ElementOps >
IHashKeyBagImplNode* IGHashKeySet < Element, Key, ElementOps >::
newNode (Element const& element) const
{ void* nodeSpace = ivElementOps.allocate
                      (sizeof (Node)
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
  ICHECK (nodeSpace != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return new (nodeSpace) Node (element);
}


template < class Element, class Key, class ElementOps >
IHashKeyBagImplNode* IGHashKeySet < Element, Key, ElementOps >::
newNode (void const* element) const
{ return newNode (*(Element const*) element);
}

template < class Element, class Key, class ElementOps >
IHashKeyBagImplNode* IGHashKeySet < Element, Key, ElementOps >::
copyNode (void const* node) const
{ return newNode (((Node const*) node)->ivElement);
}

template < class Element, class Key, class ElementOps >
void IGHashKeySet < Element, Key, ElementOps >::
deleteNode (void* node) const
{ ((Node*) node)->~Node ();
  ivElementOps.deallocate
                      (node
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
}

template < class Element, class Key, class ElementOps >
void* IGHashKeySet < Element, Key, ElementOps >::
newNodeBlock (INumber n) const
{ void* result = ivElementOps.allocate (n * sizeof (Node*)
#ifdef __DEBUG_ALLOC__
                                       , __FILE__, __LINE__
#endif
                                       );
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template < class Element, class Key, class ElementOps >
void* IGHashKeySet < Element, Key, ElementOps >::
newCountBlock (INumber n) const
{ void* result = ivElementOps.allocate (n * sizeof (unsigned long)
#ifdef __DEBUG_ALLOC__
                                       , __FILE__, __LINE__
#endif
                                       );
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template < class Element, class Key, class ElementOps >
void IGHashKeySet < Element, Key, ElementOps >::
removeBlock (void* p) const
{ ivElementOps.deallocate (p
#ifdef __DEBUG_ALLOC__
                           , __FILE__, __LINE__
#endif
                          );
}

template < class Element, class Key, class ElementOps >
INumber IGHashKeySet < Element, Key, ElementOps >::
getHashvalue (void const* key, INumber noEntries) const
{ return ivElementOps.keyOps.hash (*(Key const*) key, noEntries);
}

template < class Element, class Key, class ElementOps >
void* IGHashKeySet < Element, Key, ElementOps >::
elementAt (void* node) const
{ return &((Node*) node)->ivElement;
}

template < class Element, class Key, class ElementOps >
void const* IGHashKeySet < Element, Key, ElementOps >::
keyAt (void const* node) const
{ return &ivElementOps.key (((Node*) node)->ivElement);
}

template < class Element, class Key, class ElementOps >
IBoolean IGHashKeySet < Element, Key, ElementOps >::
isKeyEqualToElement (void const* node, void const* element) const
{ return ivElementOps.keyOps.equal
           (ivElementOps.key (((Node const*)node)->ivElement),
            ivElementOps.key (*(Element const*)element));
}

template < class Element, class Key, class ElementOps >
IBoolean IGHashKeySet < Element, Key, ElementOps >::
isKeyEqualToKey (void const* node, void const* key) const
{ return ivElementOps.keyOps.equal
           (ivElementOps.key (((Node const*)node)->ivElement),
            (*(Key const*)key));
}

template < class Element, class Key, class ElementOps >
void IGHashKeySet < Element, Key, ElementOps >::
copyFrom (void* node, void const* element) const
{ ivElementOps.assign
    (((Node*)node)->ivElement, *(Element const*)element);
}

template < class Element, class Key, class ElementOps >
IBoolean IGHashKeySet < Element, Key, ElementOps >::
constantFunctionIteration (void *iterationFunction,
                           void* env,
                           void const* node) const
{ return (*(IBoolean (*) (Element const&, void*)) iterationFunction)
           (((Node const*)node)->ivElement, env);
}

template < class Element, class Key, class ElementOps >
IBoolean IGHashKeySet < Element, Key, ElementOps >::
functionIteration (void *iterationFunction, void* env, void* node) const
{ return (*(IBoolean (*) (Element&, void*)) iterationFunction)
           (((Node*)node)->ivElement, env);
}

template < class Element, class Key, class ElementOps >
IBoolean IGHashKeySet < Element, Key, ElementOps >::
constantIteratorIteration (void* iterator, void const* node) const
{ return ((IConstantIterator < Element >*)iterator)->
           applyTo (((Node const*)node)->ivElement);
}

template < class Element, class Key, class ElementOps >
IBoolean IGHashKeySet < Element, Key, ElementOps >::
iteratorIteration (void* iterator, void* node) const
{ return ((IIterator < Element >*)iterator)->
           applyTo (((Node*)node)->ivElement);
}

template < class Element, class Key, class ElementOps >
void IGHashKeySet < Element, Key, ElementOps >::
addAllFrom (IGHashKeySet < Element, Key, ElementOps > const& h)
{
#ifndef INO_CHECKS
  ICHECK (this != &h,
          IIdenticalCollectionException, IIdenticalCollectionText)
#endif
  Cursor cursor (h);
  forCursor (cursor) add (h.elementAt (cursor));
}

#pragma info (restore)
