/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IHSHKS_H
#define _IHSHKS_H

#include <iglobals.h>
#include <icursor.h>
#include <stdlib.h>
#include <iihshkb.h>
#include <ibexcept.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class Key, class ElementOps >
class IGHashKeySet;

template < class Element, class Key, class ElementOps >
class IGHashKeySetNode;

template < class Element, class Key, class ElementOps >
class IGHashKeySetCursor : public IElementCursor < Element > {
public:

           IGHashKeySetCursor (IGHashKeySet
                                < Element, Key, ElementOps > const& c)
           : ivCollection (&c)
           {
           }

  IBoolean setToFirst ();

  IBoolean setToNext  ();

  IBoolean isValid    () const;

  void     invalidate ();

  void     copy (ICursor const& cursor);

  IBoolean operator== (ICursor const& cursor) const;

  Element const&
           element      () const;

  inline
  IBoolean operator== (IGHashKeySetCursor
                        < Element, Key,
                          ElementOps > const& cursor) const;

  inline
  IBoolean operator!= (IGHashKeySetCursor
                        < Element, Key,
                          ElementOps > const& cursor) const;

protected:

  IGHashKeySet < Element, Key, ElementOps > const*
           ivCollection;

  IHashKeyBagImplCursor
           ivImpl;

  IBoolean isFor (IGHashKeySet < Element,
                                 Key, ElementOps > const& c) const
           { return ivCollection == &c;
           }

  friend class IGHashKeySet < Element, Key, ElementOps >;

};

template < class Element, class Key, class ElementOps >
class IGHashKeySetNode : public IHashKeyBagImplNode {

  Element  ivElement;

           IGHashKeySetNode (Element const& element)
           : ivElement (element) {}

          ~IGHashKeySetNode ()
           {
           }

  friend class IGHashKeySet < Element, Key, ElementOps >;

// (BP) friend class IGHashKeyBagCursor;


};

template < class Element, class Key, class ElementOps >
class IGHashKeySet : protected IHashKeyBagImpl {

  friend class IGHashKeySetCursor < Element, Key, ElementOps >;

  typedef IGHashKeySetNode < Element, Key, ElementOps > Node;

public:

  typedef IGHashKeySetCursor < Element, Key, ElementOps > Cursor;


  inline   IGHashKeySet     (INumber numberOfElements = 100);

  inline   IGHashKeySet     (IGHashKeySet < Element,
                                            Key, ElementOps > const&);

  inline  ~IGHashKeySet     ();

  inline
  IBoolean add              (Element const&);

  inline
  IBoolean add              (Element const&, ICursor&);

  void     addAllFrom       (IGHashKeySet < Element,
                                            Key, ElementOps > const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean allElementsDo    (IBoolean (*function) (Element&, void*),
                             void* additionalArgument = 0);

  inline
  IBoolean allElementsDo    (IIterator <Element>&);

  inline
  IBoolean allElementsDo    (IBoolean (*function)
                              (Element const&, void*),
                             void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo    (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement       () const;

  inline
  IBoolean containsElementWithKey (Key const&) const;

  inline
  IBoolean containsAllKeysFrom (IGHashKeySet
                                 < Element,
                                   Key, ElementOps > const&) const;

  inline
  Element const&
           elementAt        (ICursor const&) const;

  inline
  Element& elementAt        (ICursor const&);

  inline
  Element const&
           elementWithKey   (Key const&) const;

  inline
  Element& elementWithKey   (Key const&);

  inline
  IBoolean isBounded        () const;

  inline
  IBoolean isEmpty          () const;

  inline
  IBoolean isFull           () const;

  inline
  Key const&
           key              (Element const&) const;

  inline
  IBoolean locateElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateOrAddElementWithKey (Element const&);

  inline
  IBoolean locateOrAddElementWithKey (Element const&, ICursor&);

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor        () const;

  inline
  INumber  numberOfElements () const;

  inline
  IGHashKeySet < Element, Key, ElementOps >&
           operator =       (IGHashKeySet < Element,
                                            Key, ElementOps > const&);

  inline
  void     removeAll        ();

  inline
  INumber  removeAll        (IBoolean (*property)
                              (Element const&, void*),
                             void* additionalArgument = 0);

  inline
  void     removeAt         (ICursor&);

  inline
  IBoolean removeElementWithKey (Key const&);

  inline
  void     replaceAt        (ICursor const&, Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean setToFirst       (ICursor&) const;

  inline
  IBoolean setToNext        (ICursor&) const;


  inline
  IBoolean isConsistent     () const;

protected:

  ElementOps ivElementOps;

  IHashKeyBagImplNode*
           newNode              (Element const&) const;

  IHashKeyBagImplNode*
           newNode              (void const* element) const;

  IHashKeyBagImplNode*
           copyNode             (void const* node) const;

  void     deleteNode           (void* node) const;

  void*    newNodeBlock         (INumber size) const;

  void*    newCountBlock        (INumber size) const;

  void     removeBlock          (void*) const;

  INumber  getHashvalue         (void const* key, INumber) const;

  void*    elementAt            (void *node) const;

  void const*
           keyAt                (void const* node) const;

  IBoolean isKeyEqualToElement  (void const* node,
                                 void const* element) const;

  IBoolean isKeyEqualToKey      (void const* node,
                                 void const* key) const;

  IBoolean isEqualTo            (void const* node1,
                                 void const* node2) const;

  IBoolean isEqualToElement     (void const* node,
                                 void const* element) const;

  void     copyFrom             (void* node, void const* element) const;


  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      void const* node) const;

  IBoolean functionIteration    (void *function,
                                 void* env, void* node) const;

  IBoolean constantIteratorIteration (void* iterator,
                                      void const* node) const;

  IBoolean iteratorIteration    (void* iterator, void* node) const;

};

template < class Element, class Key >
class IHashKeySet :
  public IGHashKeySet < Element, Key, IKEHOps < Element, Key > > {
public:


  inline   IHashKeySet (INumber n = 100);
};


#ifdef ICL_NO_INLINE
#undef inline
#else
#include <ihshks.inl>
#endif


#if ! defined (__TEMPINC__)
#include <ihshks.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
