/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

// **************************** IGHashKeySetCursor *********************

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySetCursor <Element, Key, ElementOps >::
operator== (IGHashKeySetCursor < Element, Key,
                                 ElementOps >const& cursor) const
{ return ivCollection == cursor.ivCollection &&
         ivImpl == cursor.ivImpl;
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySetCursor <Element, Key, ElementOps >::
operator!= (IGHashKeySetCursor < Element, Key,
                                 ElementOps > const& cursor) const
{ return ! operator== (cursor);
}


// **************************** IGHashKeySet ***************************

template < class Element, class Key, class ElementOps >
inline IGHashKeySet < Element, Key, ElementOps >::
IGHashKeySet (INumber n)
{ IHashKeyBagImpl::createHashtable (n);
}

template < class Element, class Key, class ElementOps >
inline IGHashKeySet < Element, Key, ElementOps >::
IGHashKeySet (IGHashKeySet < Element, Key, ElementOps > const& h)
{ IHashKeyBagImpl::createHashtable (h);
}

template < class Element, class Key, class ElementOps >
inline IGHashKeySet < Element, Key, ElementOps >::
~IGHashKeySet ()
{ IHashKeyBagImpl::removeAll ();
  IHashKeyBagImpl::destruct ();
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
add (Element const& e, ICursor& c)
{ return ! locateOrAddElementWithKey (e, c);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
add (Element const& e)
{ return ! locateOrAddElementWithKey (e);
}

template < class Element, class Key, class ElementOps >
inline IGHashKeySet < Element, Key, ElementOps >&
IGHashKeySet < Element, Key, ElementOps >::
operator = (IGHashKeySet < Element, Key, ElementOps > const& h)
{ IHashKeyBagImpl::operator = (h);
  return *this;
}

template < class Element, class Key, class ElementOps >
inline Element const& IGHashKeySet < Element, Key, ElementOps >::
elementAt (ICursor const& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IHashKeyBagImpl::checkCursor(((Cursor const&)c).ivImpl),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return *(Element const*)
         IHashKeyBagImpl::elementAt(((Cursor const&)c).ivImpl);
}

template < class Element, class Key, class ElementOps >
inline Element& IGHashKeySet < Element, Key, ElementOps >::
elementAt (ICursor const& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IHashKeyBagImpl::checkCursor(((Cursor const&)c).ivImpl),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return *(Element*)
           IHashKeyBagImpl::elementAt(((Cursor const&)c).ivImpl);
}

template < class Element, class Key, class ElementOps >
inline Element const& IGHashKeySet < Element, Key, ElementOps >::
anyElement () const
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  return *(Element const*) IHashKeyBagImpl::anyElement();
}

template < class Element, class Key, class ElementOps >
inline void IGHashKeySet < Element, Key, ElementOps >::
removeAt (ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IHashKeyBagImpl::checkCursor(((Cursor const&)c).ivImpl),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  IHashKeyBagImpl::removeAt(((Cursor&)c).ivImpl);
  c.invalidate ();
}

template < class Element, class Key, class ElementOps >
inline INumber IGHashKeySet < Element, Key, ElementOps >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{ return IHashKeyBagImpl::removeAll (predicate, env);
}

template < class Element, class Key, class ElementOps >
inline void IGHashKeySet < Element, Key, ElementOps >::
replaceAt (ICursor const& c, Element const& e)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IHashKeyBagImpl::checkCursor(((Cursor const&)c).ivImpl),
          ICursorInvalidException, ICursorNotContainedText)
#endif
#ifndef INO_CHECKS
  ICHECK (ivElementOps.keyOps.equal
           (ivElementOps.key
             (((Node const*)
               ((Cursor const&)c).ivImpl.ivNode)->ivElement),
            ivElementOps.key (e)),
          IInvalidReplacementException, IInvalidReplacementText)
#endif
  IHashKeyBagImpl::replaceAt(((Cursor const&)c).ivImpl, &e);
}

template < class Element, class Key, class ElementOps >
inline void IGHashKeySet < Element, Key, ElementOps >::
removeAll ()
{ IHashKeyBagImpl::removeAll();
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
isBounded () const
{ return IHashKeyBagImpl::isBounded();
}

template < class Element, class Key, class ElementOps >
inline INumber IGHashKeySet < Element, Key, ElementOps >::
maxNumberOfElements () const
{ ICHECK (False, INotBoundedException,
          INotBoundedText)
  return 0;
}

template < class Element, class Key, class ElementOps >
inline INumber IGHashKeySet < Element, Key, ElementOps >::
numberOfElements () const
{ return IHashKeyBagImpl::numberOfElements();
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
isEmpty () const
{ return IHashKeyBagImpl::isEmpty();
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
isFull () const
{ return IHashKeyBagImpl::isFull();
}

template < class Element, class Key, class ElementOps >
inline IGHashKeySetCursor < Element, Key, ElementOps >*
IGHashKeySet < Element, Key, ElementOps >::
newCursor () const
{ Cursor* result = new Cursor (*this);
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
setToFirst (ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IHashKeyBagImpl::setToFirst(((Cursor&)c).ivImpl);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
setToNext (ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
  ICHECK (c.isValid (), ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IHashKeyBagImpl::checkCursor(((Cursor const&)c).ivImpl),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return IHashKeyBagImpl::setToNext(((Cursor&)c).ivImpl);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* env) const {
  return IHashKeyBagImpl::allElementsDo (iterationFunction, env);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
allElementsDo (IBoolean (*iterationFunction) (Element&, void*),
               void* env)
{ return IHashKeyBagImpl::allElementsDo (iterationFunction, env);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return IHashKeyBagImpl::allElementsDo (&iterator);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
allElementsDo (IIterator <Element>& iterator)
{ return IHashKeyBagImpl::allElementsDo (&iterator);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
isConsistent () const
{ return IHashKeyBagImpl::isConsistent ();
}

template < class Element, class Key, class ElementOps >
inline Key const& IGHashKeySet < Element, Key, ElementOps >::
key (Element const& element) const
{ return ivElementOps.key (element);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
containsElementWithKey (Key const& k) const
{ return IHashKeyBagImpl::containsElementWithKey
           (&k,
            ivElementOps.keyOps.hash(k, IHashKeyBagImpl::noEntries())
           );
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
containsAllKeysFrom (IGHashKeySet < Element,
                                    Key, ElementOps > const& h) const
{ return IHashKeyBagImpl::containsAllKeysFrom(h);
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
locateElementWithKey (Key const& k, ICursor& c) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IHashKeyBagImpl::locateElementWithKey
           (&k,
            ivElementOps.keyOps.hash(k, IHashKeyBagImpl::noEntries()),
            ((Cursor&)c).ivImpl
           );
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
replaceElementWithKey (Element const& e, ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IHashKeyBagImpl::replaceElementWithKey
           (&e,
            ivElementOps.keyOps.hash ((ivElementOps.key(e)),
                                      IHashKeyBagImpl::noEntries()),
            ((Cursor&)c).ivImpl
           );
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
replaceElementWithKey (Element const& e)
{ return IHashKeyBagImpl::replaceElementWithKey
           (&e,
            ivElementOps.keyOps.hash ((ivElementOps.key(e)),
                                      IHashKeyBagImpl::noEntries())
           );
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
locateOrAddElementWithKey (Element const& e, ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IHashKeyBagImpl::locateOrAddElementWithKey
           (&e,
            ivElementOps.keyOps.hash ((ivElementOps.key(e)),
                                      IHashKeyBagImpl::noEntries()),
            ((Cursor&)c).ivImpl
           );
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
locateOrAddElementWithKey (Element const& e)
{ return IHashKeyBagImpl::locateOrAddElementWithKey
           (&e,
            ivElementOps.keyOps.hash ((ivElementOps.key(e)),
                                      IHashKeyBagImpl::noEntries())
           );
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
addOrReplaceElementWithKey (Element const& e, ICursor& c)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)c).isFor (*this), ICursorInvalidException,
          ICursorNotForThisText)
#endif
  return IHashKeyBagImpl::addOrReplaceElementWithKey
           (&e,
            ivElementOps.keyOps.hash ((ivElementOps.key(e)),
                                      IHashKeyBagImpl::noEntries()),
            ((Cursor&)c).ivImpl
           );
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
addOrReplaceElementWithKey (Element const& e)
{ return IHashKeyBagImpl::addOrReplaceElementWithKey
           (&e,
            ivElementOps.keyOps.hash ((ivElementOps.key(e)),
                                      IHashKeyBagImpl::noEntries())
           );
}

template < class Element, class Key, class ElementOps >
inline IBoolean IGHashKeySet < Element, Key, ElementOps >::
removeElementWithKey (Key const& k)
{ return IHashKeyBagImpl::removeElementWithKey
                (&k, ivElementOps.keyOps.hash
                  (k, IHashKeyBagImpl::noEntries()));
}

template < class Element, class Key, class ElementOps >
inline Element const& IGHashKeySet < Element, Key, ElementOps >::
elementWithKey (Key const& k) const
{ return *(Element const*)
         IHashKeyBagImpl::elementWithKey
           (&k, ivElementOps.keyOps.hash
             (k, IHashKeyBagImpl::noEntries()));
}

template < class Element, class Key, class ElementOps >
inline Element& IGHashKeySet < Element, Key, ElementOps >::
elementWithKey (Key const& k)
{ return *(Element*) IHashKeyBagImpl::elementWithKey
    (&k, ivElementOps.keyOps.hash(k, IHashKeyBagImpl::noEntries()));
}

// **************************** IHashKeySet ****************************

template < class Element, class Key >
inline IHashKeySet < Element, Key >::
IHashKeySet (INumber n)
: IGHashKeySet < Element, Key, IKEHOps < Element, Key > > (n)
{
}
