/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IIAVLKSS_H
#define _IIAVLKSS_H

#include <iglobals.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

class IAvlKeySortedSetImplNode {

  IAvlKeySortedSetImplNode*
           ivLeft;

  IAvlKeySortedSetImplNode*
           ivRight;

  IAvlKeySortedSetImplNode*
           ivParent;

  short    ivDeeperSubtree;

  IAvlKeySortedSetImplNode*&
           link (short dir);

  void     parent (IAvlKeySortedSetImplNode*& P, short& dir);

  void     connectChild (short dir, IAvlKeySortedSetImplNode*);

  void     connectChild0 (short dir, IAvlKeySortedSetImplNode*);

  friend class IAvlKeySortedSetImpl;

  public:

           IAvlKeySortedSetImplNode ();

};

class IAvlKeySortedSetImpl {
public:

  virtual
  IAvlKeySortedSetImplNode*
           newNode             (void const* element) const = 0;

  virtual
  IAvlKeySortedSetImplNode*
           copyNode            (void const* node) const = 0;

  virtual
  void     deleteNode          (void* node) const = 0;

  virtual
  void*    elementAtNode       (void* node) const = 0;

  virtual
  void const*
           constElementAtNode  (void const* node) const = 0;
  virtual
  void const*
           keyAt               (void const* node) const = 0;
  virtual
  long     compareKeyToElement (void const* node,
                                void const* element) const = 0;
  virtual
  long     compareKeyToKey     (void const* node,
                                void const* key) const = 0;
  virtual
  void     copyFrom            (void* node,
                                void const* element) const = 0;
  virtual
  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      void const* node) const = 0;
  virtual
  IBoolean functionIteration   (void *function,
                                void* env,
                                void* node) const = 0;
  virtual
  IBoolean constantIteratorIteration (void* iterator,
                                      void const* node) const = 0;
  virtual
  IBoolean iteratorIteration   (void* iterator,
                                void* node) const = 0;

  virtual
  long     functionComparison  (void *compareFunction,
                                void const* node1,
                                void const* node2) const = 0;
private :

  IAvlKeySortedSetImplNode*    ivRoot;

  INumber  ivNumberOfElements;

  void     replace              (IAvlKeySortedSetImplNode* oldNode,
                                 IAvlKeySortedSetImplNode* newNode);

  void     deleteTree           (IAvlKeySortedSetImplNode*);

  void     balance              (IAvlKeySortedSetImplNode*,
                                 short dir,
                                 IBoolean&);

  void     disconnectAndRemove  (IAvlKeySortedSetImplNode*, IBoolean&);

  IAvlKeySortedSetImplNode*
           copySubtree          (IAvlKeySortedSetImplNode const*);

  IBoolean locateElementWithKeyOfElement
                                (void const* element,
                                 IAvlKeySortedSetImplNode*&) const;
  IBoolean containsAllKeysFromSubtree
                                (IAvlKeySortedSetImplNode const*) const;

  void     unionWithSubtree     (IAvlKeySortedSetImplNode const*);

  IBoolean allElementsDoSubtree (void* function,
                                 void* environment,
                                 IAvlKeySortedSetImplNode const*) const;

  IBoolean allElementsDoSubtree (void* function,
                                 void* environment,
                                 IAvlKeySortedSetImplNode*);

  IBoolean allElementsDoSubtree (void* iterator,
                                 IAvlKeySortedSetImplNode const*) const;

  IBoolean allElementsDoSubtree (void* iterator,
                                 IAvlKeySortedSetImplNode*);

public:

           IAvlKeySortedSetImpl ();

           IAvlKeySortedSetImpl (IAvlKeySortedSetImpl const&);

  void     operator=            (IAvlKeySortedSetImpl const&);

  virtual
          ~IAvlKeySortedSetImpl ();

  IBoolean add                  (void const* element);

  IBoolean add                  (void const* element,
                                 IAvlKeySortedSetImplNode*&);

  void     addAllFrom           (IAvlKeySortedSetImpl const&);

  void const*
           elementAt            (IAvlKeySortedSetImplNode const*) const;

  void*    elementAt            (IAvlKeySortedSetImplNode*);

  void     removeAt             (IAvlKeySortedSetImplNode*);

  INumber  removeAll            (void* predicate, void* environment);

  void     replaceAt            (IAvlKeySortedSetImplNode*,
                                 void const* element);

  void     removeAll            ();

  IBoolean isBounded            () const;

  IPosition
           numberOfElements     () const
           { return ivNumberOfElements; }

  IBoolean isEmpty              () const
           { return ivRoot == 0; }

  IBoolean isFull               () const;

  IBoolean setToFirst           (IAvlKeySortedSetImplNode*&) const;

  IBoolean setToNext            (IAvlKeySortedSetImplNode*&) const;

  IBoolean allElementsDo        (void* function,
                                 void* environment) const;

  IBoolean allElementsDo        (void* function, void* environment);

  IBoolean allElementsDo        (void* iterator) const;

  IBoolean allElementsDo        (void* iterator);

  INumber  removeAllOccurences  (void const* element);

  IBoolean containsAllKeysFrom  (IAvlKeySortedSetImpl const&) const;

  IBoolean containsElementWithKey (void const* key) const;

  IBoolean locateElementWithKey (void const* key,
                                 IAvlKeySortedSetImplNode*&) const;

  IBoolean replaceElementWithKey (void const* element,
                                  IAvlKeySortedSetImplNode*&);

  IBoolean replaceElementWithKey (void const* element);

  IBoolean addOrReplaceElementWithKey (void const* element,
                                       IAvlKeySortedSetImplNode*&);

  IBoolean addOrReplaceElementWithKey (void const* element);

  IBoolean locateOrAddElementWithKey (void const* element,
                                      IAvlKeySortedSetImplNode*&);

  IBoolean locateOrAddElementWithKey (void const* element);

  IBoolean removeElementWithKey (void const* key);

  void const*
           elementWithKey       (void const* key) const;

  void*    elementWithKey       (void const* key);

  void     removeFirst          ();

  void     removeLast           ();

  void     removeAtPosition     (IPosition);

  void const*
           firstElement         () const;

  void const*
           lastElement          () const;

  void const*
           elementAtPosition    (IPosition) const;

  IBoolean setToLast            (IAvlKeySortedSetImplNode*&) const;

  IBoolean setToPrevious        (IAvlKeySortedSetImplNode*&) const;

  void     setToPosition        (IPosition,
                                 IAvlKeySortedSetImplNode*&) const;

#ifndef __BIN_COMPAT_BASE_43__
  IPosition     position        (IAvlKeySortedSetImplNode const*) const;
#endif

  IBoolean isFirst              (IAvlKeySortedSetImplNode const*) const;

  IBoolean isLast               (IAvlKeySortedSetImplNode const*) const;

  long     compare              (IAvlKeySortedSetImpl const&,
                                 void *function) const;

  IBoolean checkNode            (IAvlKeySortedSetImplNode const*,
                                 IAvlKeySortedSetImplNode*) const;

  IBoolean checkNode            (IAvlKeySortedSetImplNode const*) const;

  IBoolean isConsistent         () const;

protected:

  IBoolean isConsistent         (IAvlKeySortedSetImplNode const* n,
                                 int& depth, int& nrOfElm) const;
};

#pragma info (restore)
#pragma pack ()

#endif
