/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IIBSTKSS_H
#define _IIBSTKSS_H

#include <iglobals.h>

#ifdef ICL_NO_INLINE
#define inline
#endif

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

class IBSTKeySortedSetImplNode {
protected:

  IBoolean ivIsLeaf;

public:

  IBoolean isLeaf () const
           { return ivIsLeaf;
           }

};

class IBSTKeySortedSetImplLnode : public IBSTKeySortedSetImplNode {

};

class IBSTKeySortedSetImplInode : public IBSTKeySortedSetImplNode {

public:

  inline   IBSTKeySortedSetImplInode ();

  inline   IBSTKeySortedSetImplInode (IBSTKeySortedSetImplInode const&);

protected:

  void*    ivKeys;

  IBSTKeySortedSetImplNode** ivPtrs;

  INumber  ivm;

  friend class IBSTKeySortedSetImpl;

};

class IBSTKeySortedSetImpl {

public:

  virtual
  IBSTKeySortedSetImplLnode*
           newLnode             (void const* element) const = 0;

  virtual
  IBSTKeySortedSetImplLnode*
           newLnodeFromLnode    (void const* lnode) const = 0;

  virtual
  IBSTKeySortedSetImplInode*
           newInode             () const = 0;

  virtual
  IBSTKeySortedSetImplInode*
           newInode             (void const* inode) const = 0;

  virtual
  void*    newKey               () const = 0;

  virtual
  void     deleteLnode          (void* lnode) const = 0;

  virtual
  void     deleteInode          (void* inode) const = 0;

  virtual
  void     deleteKey            (void *key) const = 0;

  virtual
  IBoolean isKeyLessThan        (void const* key1,
                                 void const* key2) const = 0;

  virtual
  void     copyKey              (void* toKey,
                                 INumber toIndex,
                                 void const* fromKey,
                                 INumber fromIndex) const = 0;

  void     copyKey                (void* toKey, void const* fromKey)
           { copyKey (toKey, 0, fromKey, 0);
           }

  void     copyKey              (void* toKey,
                                 INumber toIndex,
                                 void const* fromKey)
           { copyKey (toKey, toIndex, fromKey, 0);
           }

  void     copyKey              (void* toKey,
                                 void const* fromKey,
                                 INumber fromIndex)
           { copyKey (toKey, 0, fromKey, fromIndex);
           }

  virtual
  void const*
           getLnodeElement      (IBSTKeySortedSetImplLnode const*)
                                const = 0;

  virtual
  void*    getLnodeElement      (IBSTKeySortedSetImplLnode*) = 0;

  virtual
  void     replaceLnodeElement  (IBSTKeySortedSetImplLnode*,
                                 void const* element) = 0;

  virtual
  IBoolean isLnodeKeyEqualTo    (IBSTKeySortedSetImplLnode const*,
                                 void const* lnode) const = 0;

  virtual
  IBoolean isLnodeKeyEqualToKeyOfElement
                                (IBSTKeySortedSetImplLnode const*,
                                 void const* element) const = 0;

  virtual
  IBoolean isLnodeKeyEqualToKey (IBSTKeySortedSetImplLnode const*,
                                 void const* key) const = 0;

  virtual
  IBoolean isKeyLessThanLnode   (void const* key,
                                 void const* lnode) const = 0;

  virtual
  void     copyKeyFromLnode     (void* key, void const* lnode) = 0;

  virtual
  IBoolean isKeyGreaterThanLnode (void const* key,
                                  INumber index,
                                  void const* lnode) const = 0;

  virtual
  IBoolean isKeyGreaterThanKey  (void const* key1,
                                 INumber index,
                                 void const* key2) const = 0;

  virtual
  IBoolean isKeyGreaterThanElement (void const* key,
                                    INumber index,
                                    void const* element) const = 0;

  virtual
  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      void const* lnode) const = 0;
  virtual
  IBoolean functionIteration    (void *function,
                                 void* env,
                                 void* lnode) const = 0;
  virtual
  IBoolean constantIteratorIteration (void* iterator,
                                      void const* lnode) const = 0;

  virtual
  IBoolean iteratorIteration    (void* iterator, void* lnode) const = 0;

  virtual
  long     functionComparison   (void *compareFunction,
                                 void const* lnode1,
                                 void const* lnode2) const = 0;

  inline   IBSTKeySortedSetImpl (INumber);

  inline
  IBoolean add                  (void const* element);

  IBoolean add                  (void const* element,
                                 IBSTKeySortedSetImplLnode*&);

  inline
  void     addAllFrom           (IBSTKeySortedSetImpl const&);

  inline
  void const*
           elementAt            (IBSTKeySortedSetImplLnode* const&)
                                const;

  inline
  void*    elementAt            (IBSTKeySortedSetImplLnode* const&);

  inline
  void const*
           anyElement           () const;

  void     removeAt             (IBSTKeySortedSetImplLnode* const&);

  INumber  removeAll            (void* predicate, void* environment);

  inline
  void     replaceAt            (IBSTKeySortedSetImplLnode* const&,
                                 void const* element);

  inline
  void     removeAll            ();

  inline
  IBoolean isBounded            () const;

  inline
  INumber  maxNumberOfElements  () const;

  inline
  INumber  numberOfElements     () const;

  inline
  IBoolean isEmpty              () const;

  inline
  IBoolean isFull               () const;

  IBoolean setToFirst           (IBSTKeySortedSetImplLnode*&) const;

  IBoolean setToNext            (IBSTKeySortedSetImplLnode*&) const;

  IBoolean allElementsDo        (void* function, void* environment)
                                const;

  IBoolean allElementsDo        (void* function, void* environment);

  IBoolean allElementsDo        (void* iterator) const;

  IBoolean allElementsDo        (void* iterator);

  IBoolean containsAllKeysFrom  (IBSTKeySortedSetImpl const&) const;

  IBoolean containsElementWithKey (void const* key) const;

  IBoolean locateElementWithKey (void const* key,
                                 IBSTKeySortedSetImplLnode*&) const;

  IBoolean replaceElementWithKey (void const* element);

  IBoolean replaceElementWithKey (void const* element,
                                  IBSTKeySortedSetImplLnode*&);

  IBoolean addOrReplaceElementWithKey (void const* element);

  IBoolean addOrReplaceElementWithKey (void const* element,
                                       IBSTKeySortedSetImplLnode*&);

  IBoolean removeElementWithKey (void const* key);

  void const*
           elementWithKey       (void const* key) const;

  void*    elementWithKey       (void const* key);

  void     removeFirst          ();

  void     removeLast           ();

  void     removeAtPosition     (IPosition);

  inline
  void const*
           firstElement         () const;

  void const*
           lastElement          () const;

  void const*
           elementAtPosition    (IPosition) const;

  IBoolean setToLast            (IBSTKeySortedSetImplLnode*&) const;

  IBoolean setToPrevious        (IBSTKeySortedSetImplLnode*&) const;

  void     setToPosition        (IPosition,
                                 IBSTKeySortedSetImplLnode*&) const;

#ifndef __BIN_COMPAT_BASE_43__
  IPosition     position        (IBSTKeySortedSetImplLnode* const&)
                                const;
#endif

  IBoolean isFirst              (IBSTKeySortedSetImplLnode* const&)
                                const;

  IBoolean isLast               (IBSTKeySortedSetImplLnode* const&)
                                const;

  IBSTKeySortedSetImpl&
           operator =           (IBSTKeySortedSetImpl const&);

  long     compare              (IBSTKeySortedSetImpl const&,
                                 void *function) const;

  IBoolean checkNode            (IBSTKeySortedSetImplLnode const*,
                                 IBSTKeySortedSetImplNode const*) const;

  IBoolean checkNode            (IBSTKeySortedSetImplLnode const*)
                                const;

  IBoolean isConsistent         () const;

protected:

  inline
  IBoolean isUnderflown         (IBSTKeySortedSetImplInode const*)
                                const;

  IBoolean allElementsDoSubtree (void* function,
                                 void* environment,
                                 IBSTKeySortedSetImplNode const*) const;

  IBoolean allElementsDoSubtree (void* function,
                                 void* environment,
                                 IBSTKeySortedSetImplNode*);

  IBoolean allElementsDoSubtree (void* iterator,
                                 IBSTKeySortedSetImplNode const*) const;

  IBoolean allElementsDoSubtree (void* iterator,
                                 IBSTKeySortedSetImplNode*);

  IPosition binarySearchInPage  (IBSTKeySortedSetImplInode const*,
                                 IBSTKeySortedSetImplLnode const*)
                                const;

  IPosition binarySearchInPageWithKey (IBSTKeySortedSetImplInode const*,
                                       void const* key) const;

  IPosition binarySearchInPageWithElement
                                (IBSTKeySortedSetImplInode const*,
                                 void const* element) const;

  IBoolean setToNextSubtree     (IBSTKeySortedSetImplLnode*&,
                                 IBSTKeySortedSetImplInode const*)
                                const;

  IBoolean setToPreviousSubtree (IBSTKeySortedSetImplLnode*&,
                                 IBSTKeySortedSetImplInode const*)
                                const;

  IBoolean addAid               (IBSTKeySortedSetImplNode*,
                                 IBSTKeySortedSetImplLnode*,
                                 IBSTKeySortedSetImplNode*&,
                                 void* key,
                                 IBSTKeySortedSetImplLnode*&);

  IBSTKeySortedSetImplLnode*
           minimumLeaf          (IBSTKeySortedSetImplNode*) const;

  IBSTKeySortedSetImplLnode*
           maximumLeaf          (IBSTKeySortedSetImplNode*) const;

  INumber  removeAllSubtree     (IBSTKeySortedSetImplNode*);

  void     addAllFromSubtree    (IBSTKeySortedSetImplNode const*,
                                 IBSTKeySortedSetImplLnode*&);

  void     removeAid            (IBSTKeySortedSetImplInode*,
                                 IBSTKeySortedSetImplLnode*);

  void     handleUnderflow      (IBSTKeySortedSetImplInode*,
                                 IBSTKeySortedSetImplInode*,
                                 IPosition);

  void     joinInodes           (IBSTKeySortedSetImplInode*,
                                 IBSTKeySortedSetImplInode*,
                                 IBSTKeySortedSetImplInode*,
                                 IPosition);

  IBSTKeySortedSetImplNode*
           copySubtree          (IBSTKeySortedSetImplNode const*);

  IBoolean locateWithKeySubtree (void const* key,
                                 IBSTKeySortedSetImplLnode*&,
                                 IBSTKeySortedSetImplNode*) const;

  IBoolean containsAllKeysFromSubtree (IBSTKeySortedSetImplNode const*,
                                       IBSTKeySortedSetImplLnode*&)
                                      const;

  IBoolean locateElementWithKeySubtree (void const* key,
                                        IBSTKeySortedSetImplLnode*&,
                                        IBSTKeySortedSetImplNode*)
                                       const;

  IBoolean locateElementWithKeyOfElement (void const* element,
                                          IBSTKeySortedSetImplLnode*&)
                                         const;

  IBoolean locateElementWithKeyOfElementSubtree
                                 (void const* element,
                                  IBSTKeySortedSetImplLnode*&,
                                  IBSTKeySortedSetImplNode*) const;

  IBoolean containsElementWithKey (IBSTKeySortedSetImplLnode const*)
                                  const;

  IBoolean locateElementWithKey (IBSTKeySortedSetImplLnode const*,
                                 IBSTKeySortedSetImplLnode*&) const;

  IBoolean locateElementWithKeySubtree
                                (IBSTKeySortedSetImplLnode const* lnode,
                                 IBSTKeySortedSetImplLnode*& cursor,
                                 IBSTKeySortedSetImplNode* node) const;

  IBSTKeySortedSetImplNode* ivRoot;

  INumber  ivHeight;

  INumber  ivTreeOrder;

  INumber  ivNoElements;

};

#ifdef ICL_NO_INLINE
#undef inline
#else
#include <iibstkss.inl>
#endif

#pragma info (restore)
#pragma pack ()


#endif
