/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IIHSHKB_H
#define _IIHSHKB_H

#include <stdlib.h>
#include <iglobals.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

class IHashKeyBagImplNode {
  friend class IHashKeyBagImpl;
  IHashKeyBagImplNode* ivNext;
};

class IHashKeyBagImpl;

class IHashKeyBagImplCursor {
  public:

    INumber ivEntryNumber;

    IHashKeyBagImplNode* ivNode;

    IHashKeyBagImplCursor () : ivNode (0) {}

    IBoolean operator== (IHashKeyBagImplCursor const& cursor) const
             { return ivNode == cursor.ivNode &&
                      ivEntryNumber == cursor.ivEntryNumber;
             }

    IBoolean operator!= (IHashKeyBagImplCursor const& cursor) const
             { return ! operator== (cursor);
             }

};


class IHashKeyBagImpl {
public:

  virtual
  IHashKeyBagImplNode*
           newNode              (void const* element) const = 0;

  virtual
  IHashKeyBagImplNode*
           copyNode             (void const* node) const = 0;

  virtual
  void     deleteNode           (void* node) const = 0;

  virtual
  void*    newNodeBlock         (INumber) const = 0;

  virtual
  void*    newCountBlock        (INumber) const = 0;

  virtual
  void     removeBlock          (void*) const = 0;

  virtual
  INumber  getHashvalue         (void const* key, INumber) const = 0;

  virtual
  void*    elementAt            (void* node) const = 0;

  virtual
  void const*
           keyAt                (void const* node) const = 0;

  virtual
  IBoolean isKeyEqualToElement  (void const* node,
                                 void const* element) const = 0;

  virtual
  IBoolean isKeyEqualToKey      (void const* node,
                                 void const* key) const = 0;

  virtual
  void     copyFrom             (void* node,
                                 void const* element) const = 0;

  virtual
  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      void const* node) const = 0;

  virtual
  IBoolean functionIteration    (void *function,
                                 void* env,
                                 void* node) const = 0;

  virtual
  IBoolean constantIteratorIteration (void* iterator,
                                      void const* node) const = 0;

  virtual
  IBoolean iteratorIteration    (void* iterator, void* node) const = 0;

private :

  typedef  IHashKeyBagImplCursor Cursor;

  INumber  ivNoEntries;

  INumber  ivNoElements;

  IHashKeyBagImplNode**   ivTable;

  INumber* ivCollList;

  void     copyHashtable        (IHashKeyBagImpl const&);

  void     resize               (IHashKeyBagImplNode*, INumber&);


public:

  void     createHashtable      (INumber);

  void     createHashtable      (IHashKeyBagImpl const&);

  void     destruct             ();

  IBoolean add                  (void const* element, INumber, Cursor&);

  IBoolean add                  (void const* element, INumber);

  void     addAllFrom           (IHashKeyBagImpl const&);

  void     operator=            (IHashKeyBagImpl const&);

  void const*
           elementAt            (Cursor const&) const;

  void*    elementAt            (Cursor const&);

  void const*
           anyElement           () const;

  void     removeAt             (Cursor const&);

  INumber  removeAll            (void* predicate, void* environment);

  void     replaceAt            (Cursor const&, void const* element);

  void     removeAll            ();

  IBoolean isBounded            () const;

  INumber  maxNumberOfElements  () const;

  INumber  numberOfElements     () const
           { return ivNoElements;
           }

  IBoolean isEmpty              () const
           { return ivNoElements == 0;
           }

  IBoolean isFull               () const;

  IBoolean setToFirst           (Cursor&) const;

  IBoolean setToNext            (Cursor&) const;

  IBoolean allElementsDo        (void* function,
                                 void* environment) const;

  IBoolean allElementsDo        (void* function, void* environment);

  IBoolean allElementsDo        (void* iterator) const;

  IBoolean allElementsDo        (void* iterator);

  IBoolean containsAllKeysFrom  (IHashKeyBagImpl const&) const;

  IBoolean containsElementWithKey (void const* key, INumber)const;

  INumber  numberOfElementsWithKey (void const* key, INumber) const;

  IBoolean locateElementWithKey (void const* key,
                                 INumber, Cursor&) const;

  IBoolean locateElementWithKeyOfElement (void const* element,
                                          INumber,
                                          Cursor&) const;

  IBoolean replaceElementWithKey (void const* element,
                                  INumber, Cursor&);

  IBoolean replaceElementWithKey (void const* element, INumber);

  IBoolean locateOrAddElementWithKey (void const* element,
                                      INumber, Cursor&);

  IBoolean locateOrAddElementWithKey (void const* element, INumber);

  IBoolean addOrReplaceElementWithKey (void const* element,
                                       INumber, Cursor&);

  IBoolean addOrReplaceElementWithKey (void const* element, INumber);

  IBoolean removeElementWithKey (void const* key, INumber);

  void const*
           elementWithKey       (void const* key, INumber) const;

  void*    elementWithKey       (void const* key, INumber);

  IBoolean locateNextElementWithKey (void const* key,
                                     INumber, Cursor&) const;

  INumber  removeAllElementsWithKey (void const* key, INumber);

  INumber  numberOfDifferentKeys () const;

  IBoolean setToNextWithDifferentKey (Cursor&) const;

  INumber  noEntries            () const;

  IBoolean checkCursor          (Cursor const&) const;

  IBoolean isConsistent         () const;

};

#pragma info (restore)
#pragma pack ()

#endif
