/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IILNSEQ_H
#define _IILNSEQ_H

#include <iglobals.h>
#include <icursor.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

class ILinkedSequenceImplNode {
protected:

  ILinkedSequenceImplNode* ivNext;

  ILinkedSequenceImplNode* ivPrevious;

  friend class ILinkedSequenceImpl;

};

class ILinkedSequenceImpl
{
public:

  virtual
  ILinkedSequenceImplNode*
           newNode          (void const* node) const = 0;

  virtual
  void     deleteNode       (void* node) const = 0;

  virtual
  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      void const* node) const = 0;

  virtual
  IBoolean functionIteration (void *function,
                              void* env,
                              void* node) const = 0;

  virtual
  IBoolean constantIteratorIteration (void* iterator,
                                      void const* node) const = 0;

  virtual
  IBoolean iteratorIteration (void* iterator, void* node) const = 0;

  virtual
  long     functionComparison (void *compareFunction,
                               void const* node1,
                               void const* node2) const = 0;

protected:

  ILinkedSequenceImplNode* ivFirst;

  ILinkedSequenceImplNode* ivLast;

  INumber ivNumberOfElements;

public:

           ILinkedSequenceImpl ();

           ILinkedSequenceImpl (ILinkedSequenceImpl const&);

          ~ILinkedSequenceImpl ();

  void     addAllFrom          (ILinkedSequenceImpl const&);

  ILinkedSequenceImplNode*
           addAsFirst          (ILinkedSequenceImplNode*);

  ILinkedSequenceImplNode*
           addAsLast           (ILinkedSequenceImplNode*);

  ILinkedSequenceImplNode*
           addAsNext           (ILinkedSequenceImplNode*,
                                ILinkedSequenceImplNode*);

  ILinkedSequenceImplNode*
           addAsPrevious       (ILinkedSequenceImplNode*,
                                ILinkedSequenceImplNode*);

  ILinkedSequenceImplNode*
           addAtPosition       (IPosition,
                                ILinkedSequenceImplNode*);

  IBoolean allElementsDo       (void* function, void* env);

  IBoolean allElementsDo       (void* function, void* env) const;

  IBoolean allElementsDo       (void* iterator);

  IBoolean allElementsDo       (void* iterator) const;

  long     compare             (ILinkedSequenceImpl const&,
                                void* comparisonFunction) const;

  IBoolean isEmpty             () const;

  IBoolean isFirst             (ILinkedSequenceImplNode const*) const;

  IBoolean isFull              () const;

  IBoolean isLast              (ILinkedSequenceImplNode const*) const;

  INumber  numberOfElements    () const;

  void     removeAll           ();

  INumber  removeAll           (void* predicate, void* environment);

  void     removeAt            (ILinkedSequenceImplNode*);

  void     removeAtPosition    (IPosition);

  void     removeFirst         ();

  void     removeLast          ();

  IBoolean setToFirst          (ILinkedSequenceImplNode*&) const;

  IBoolean setToNext           (ILinkedSequenceImplNode*&) const;

  IBoolean setToLast           (ILinkedSequenceImplNode*&) const;

  IBoolean setToPrevious       (ILinkedSequenceImplNode*&) const;

  void     setToPosition       (IPosition,
                                ILinkedSequenceImplNode*&) const;

#ifndef __BIN_COMPAT_BASE_43__
  IPosition     position       (ILinkedSequenceImplNode const*) const;
#endif


  void     sort                (void* comparisonFunction);

  IBoolean checkNode           (ILinkedSequenceImplNode const*) const;

  IBoolean isConsistent        () const;
};

#pragma info (restore)
#pragma pack ()

#endif
