/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IITBTRE_H
#define _IITBTRE_H

#include <iglobals.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

class ITabularTreeImplNode
{
  friend class ITabularTreeImpl;
public:
  ITabularTreeImplNode* ivParent;
  ITabularTreeImplNode* ivChildren [1];
  // ivChildren contains in fact numberOfChildren node pointers,
  // where numberOfChildren is the template argument of the
  // corresponding ITabularTree.
};

class ITabularTreeImpl
{
public:

  ITabularTreeImplNode* ivRoot;
  INumber ivNumberOfChildren;

  ITabularTreeImpl (INumber numberOfChildren)
  : ivRoot (0), ivNumberOfChildren (numberOfChildren)
  {}

  void    initNode                (ITabularTreeImplNode* node,
                                   ITabularTreeImplNode* parent) const;
  INumber numberOfSubtreeElements (ITabularTreeImplNode const*) const;
  INumber numberOfSubtreeLeaves   (ITabularTreeImplNode const*) const;
  IBoolean isLeaf                 (ITabularTreeImplNode const*) const;
  INumber position                (ITabularTreeImplNode const*) const;

  void    attachSubtreeAsRoot     (ITabularTreeImpl&,
                                   ITabularTreeImplNode*);
  void    attachSubtreeAsChild    (ITabularTreeImplNode* to,
                                   IPosition pos,
                                   ITabularTreeImpl&,
                                   ITabularTreeImplNode* from);

  IBoolean setToParent             (ITabularTreeImplNode*& node) const;
  IBoolean setToFirstExistingChild (ITabularTreeImplNode*& node) const;
  IBoolean setToNextExistingChild  (ITabularTreeImplNode*& node) const;
  IBoolean setToLastExistingChild  (ITabularTreeImplNode*& node) const;
  IBoolean setToPreviousExistingChild
                                   (ITabularTreeImplNode*& node) const;

  IBoolean setToFirst       (ITabularTreeImplNode*& node,
                             ITreeIterationOrder order) const;
  IBoolean setToNext        (ITabularTreeImplNode*& node,
                             ITreeIterationOrder order) const;
  IBoolean setToLast        (ITabularTreeImplNode*& node,
                             ITreeIterationOrder order) const;
  IBoolean setToPrevious    (ITabularTreeImplNode*& node,
                             ITreeIterationOrder order) const;

  IBoolean allElementsDo    (void* function,
                             ITreeIterationOrder order,
                             void* env,
                             IBoolean (*apply) (void *function,
                                                void* env,
                                                void* node),
                             ITabularTreeImplNode* subtreeRoot);

  IBoolean allElementsDo    (void* function,
                             ITreeIterationOrder order,
                             void* env,
                             IBoolean (*apply) (void *function,
                                                void* env,
                                                void const* node),
                             ITabularTreeImplNode* subtreeRoot) const;

  IBoolean allElementsDo    (void* iterator,
                             ITreeIterationOrder order,
                             IBoolean (*apply) (void* iterator,
                                               void* node),
                             ITabularTreeImplNode* subtreeRoot);

  IBoolean allElementsDo    (void* iterator,
                             ITreeIterationOrder order,
                             IBoolean (*apply) (void* iterator,
                                                void const* node),
                             ITabularTreeImplNode* subtreeRoot) const;

  IBoolean checkNode        (ITabularTreeImplNode const*) const;
  IBoolean checkNodeInSubtree
                            (ITabularTreeImplNode const*,
                             ITabularTreeImplNode const* subtree) const;

  IBoolean isConsistent     () const;

protected:

  IBoolean isConsistent     (ITabularTreeImplNode const*) const;
};

#pragma info (restore)
#pragma pack ()

#endif
