/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IITDSEQ_H
#define _IITDSEQ_H

#include <iglobals.h>
#include <icursor.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

class IDilutedSequenceImpl;

class IDilutedSequenceImplCursor {

  public:

    INumber  ivIndex;

    IBoolean ivIsValid;

             IDilutedSequenceImplCursor ()
             : ivIsValid (False)
             {
             }

    IBoolean operator== (IDilutedSequenceImplCursor const& cursor) const
             { return ivIsValid == cursor.ivIsValid &&
                      ivIndex == cursor.ivIndex;
             }

};

class IDilutedSequenceImplTable {
public:

  INumber  ivFirst;

  INumber  ivLast;

  INumber  ivNumberOfElements;

  INumber  ivAllocatedElements;

  void*    ivNodes;

  IBoolean canAdd (INumber n) const
           { return ivNumberOfElements+n <= ivAllocatedElements;
           }

  INumber  pos (INumber n) const
           { return n >= ivFirst ?
                     n - ivFirst % ivAllocatedElements :
                     ivAllocatedElements -
                       (ivFirst - n) % ivAllocatedElements;
           }

  INumber& decMod (INumber& n) const
           { if (n == 0) n = ivAllocatedElements;
             return --n;
           }

  INumber& incMod (INumber& n) const
           { if (++n == ivAllocatedElements) n = 0;
             return n;
           }

  INumber& addMod (INumber& n, INumber n1) const
           { n = (n + n1) % ivAllocatedElements;
             return n;
           }

};


class IDilutedSequenceImpl
{
public:

  typedef IDilutedSequenceImplCursor Cursor;

  typedef IDilutedSequenceImplTable Table;

  virtual
  void     swap                 (INumber index1,
                                 INumber index2) const = 0;

  virtual
  void     copyFrom             (INumber index,
                                 void const* element) const = 0;
  virtual
  void const*
           elementAt            (INumber index) const = 0;

  virtual
  IBoolean isRemoved            (INumber index) const = 0;

  virtual
  void     remove               (INumber index) = 0;

  virtual
  IBoolean setToNext            (INumber& index) const = 0;

  virtual
  IBoolean setToPrevious        (INumber& index) const = 0;

  virtual
  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      INumber index) const = 0;

  virtual
  IBoolean functionIteration    (void *function,
                                 void* env,
                                 INumber index) = 0;

  virtual
  IBoolean constantIteratorIteration (void* iterator,
                                      INumber index) const = 0;

  virtual
  IBoolean iteratorIteration    (void* iterator, INumber index) = 0;

  virtual
  long     functionComparison   (void *compareFunction,
                                 void const* element1,
                                 void const* element2) const = 0;

  virtual
  long     functionComparison   (void *compareFunction,
                                 INumber index1,
                                 INumber index2) const = 0;

  virtual
  void     blockLeft            (INumber, INumber) = 0;

  virtual
  void     blockRight           (INumber, INumber) = 0;

  virtual
  void     copy                 (Table const& from) = 0;

  virtual
  void     expand               (INumber) = 0;

  virtual
  void     makeFree             (INumber&) = 0;

protected:

  Table ivTable;

public:

  void     addAllFrom           (IDilutedSequenceImpl const& sequence);

  void     addAsFirst           (void const* element, Cursor& cursor);

  void     addAsLast            (void const* element, Cursor& cursor);

  void     addAsNext            (void const* element, Cursor& cursor);

  void     addAsPrevious        (void const* element, Cursor& cursor);

  void     addAtPosition        (IPosition position,
                                 void const* element,
                                 Cursor& cursor);

  IBoolean allElementsDo        (void* iterationFunction, void* env);

  IBoolean allElementsDo        (void *iterationFunction,
                                 void* env) const;

  IBoolean allElementsDo        (void* iterator);

  IBoolean allElementsDo        (void *iterator) const;

  long     compare              (IDilutedSequenceImpl const& sequence,
                                 void* comparisonFunction) const;

  void     operator=            (IDilutedSequenceImpl const& sequence);

  void     removeAt             (INumber& index);

  INumber  removeAll            (void* predicate, void* env);

  IBoolean setToFirst           (Cursor& cursor) const;

  IBoolean setToLast            (Cursor& cursor) const;

  IBoolean setToNext            (Cursor& cursor) const;

  IBoolean setToPrevious        (Cursor& cursor) const;

  void     setToPosition        (IPosition position,
                                 Cursor& cursor) const;

#ifndef __BIN_COMPAT_BASE_43__
  IPosition     position        (Cursor const& cursor) const;
#endif

  void     sort                 (void* comparisonFunction);

  IBoolean checkIndex           (INumber index) const;

  IBoolean isConsistent         () const;

};

#pragma info (restore)
#pragma pack ()

#endif
