#ifndef _IKEYEVT_
  #define _IKEYEVT_
/*******************************************************************************
* FILE NAME: ikeyevt.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IKeyboardEvent                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ievent.hpp>

class IString;
class IKeyboardEventPrivateData;

#pragma pack(4)

class IKeyboardEvent : public IEvent {
typedef IEvent
  Inherited;
public:
/*------------------------------ Constructors --------------------------------*/
  IKeyboardEvent     ( const IEvent& event );
virtual
 ~IKeyboardEvent     ( );

/*--------------------------------- Testing ----------------------------------*/
Boolean
  isCharacter        ( ) const,
  isScanCode         ( ) const,
  isVirtual          ( ) const,
  isUpTransition     ( ) const,
  isRepeat           ( ) const,
  isShiftDown        ( ) const,
  isCtrlDown         ( ) const,
  isAltDown          ( ) const,
  isForComposite     ( ) const;
Boolean
  isUncombined       ( ) const,
  isComposite        ( ) const,
  isInvalidComposite ( ) const;

/*-------------------------------- Accessors ---------------------------------*/
char
  character          ( ) const;
IString
  mixedCharacter     ( ) const;
unsigned long
  scanCode           ( ) const;

enum VirtualKey
{
  esc, tab, backTab, space, backSpace, enter, newLine,
  shift, ctrl, altGraf, insert, deleteKey, home, end,
  pageUp, pageDown, left, right, up, down,
  capsLock, numLock, scrollLock, pause, sysRq, breakKey,
  f2, f3, f4, f5, f6, f7, f8, f9, f11, f12,
  f13, f14, f15, f16, f17, f18, f19, f20, f21, f22, f23, f24,
  button1, button2, button3, other
  , endDrag, firstDBCS, lastDBCS, firstUser, lastUser
};

VirtualKey
  virtualKey         ( ) const;

unsigned long
  repeatCount        ( ) const;


protected:
/*------------------------------ Implementation ------------------------------*/
static const unsigned long
  ulCharacterFlag,
  ulScanCodeFlag,
  ulVirtualFlag,
  ulRepeatFlag,
  ulUncombinedFlag,
  ulShiftFlag,
  ulCtrlFlag,
  ulForCompositeFlag,
  ulCompositeFlag,
  ulInvalidCompositeFlag,
  ulUpTransitionFlag,
  ulAltFlag;

private:

IKeyboardEventPrivateData
 *pData;
}; // IKeyboardEvent

#pragma pack()

/*----------------------------- Inline Functions -----------------------------*/
#ifndef I_NO_INLINES
  #include <ikeyevt.inl>
#endif

#endif /* _IKEYEVT_ */
