/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IKSBSEQ_H
#define _IKSBSEQ_H

#include <iglobals.h>
#include <icursor.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class Key, class ElementOps, class Base >
class IWKSBagOnSeq;

template < class Element, class Key, class ElementOps, class Base >
class IWKSBagOnSeqCursor :
  public Base::Cursor
{
public:
           IWKSBagOnSeqCursor (IWKSBagOnSeq
                                < Element, Key,
                                  ElementOps, Base > const& c)
           : Base::Cursor (c.ivBase)
           {
           }


};

template < class Element, class Key, class ElementOps, class Base >
class IWKSBagOnSeq {

  Base ivBase;

public:


  typedef IWKSBagOnSeqCursor < Element, Key, ElementOps, Base > Cursor;

  inline   IWKSBagOnSeq     (INumber numberOfElements = 100);

  inline   IWKSBagOnSeq     (IWKSBagOnSeq
                              < Element,
                                Key, ElementOps, Base > const&);

  inline  ~IWKSBagOnSeq     ();

  inline
  IBoolean add              (Element const&);

  inline
  IBoolean add              (Element const&, ICursor&);

  inline
  void     addAllFrom       (IWKSBagOnSeq
                              < Element,
                                Key, ElementOps, Base > const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean allElementsDo    (IBoolean (*function) (Element&, void*),
                             void* additionalArgument = 0);

  inline
  IBoolean allElementsDo    (IIterator <Element>&);

  inline
  IBoolean allElementsDo    (IBoolean (*function)
                              (Element const&, void*),
                             void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo    (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement       () const;

  inline
  long     compare          (IWKSBagOnSeq
                              < Element, Key, ElementOps, Base > const&,
                             long (*comparisonFunction)
                              (Element const&, Element const&)) const;

  inline
  IBoolean containsElementWithKey (Key const&) const;

  inline
  IBoolean containsAllKeysFrom
                            (IWKSBagOnSeq
                              < Element,
                                Key, ElementOps, Base > const&) const;

  inline
  Element const&
           elementAt        (ICursor const&) const;

  inline
  Element& elementAt        (ICursor const&);

  inline
  Element const&
           elementAtPosition (IPosition) const;

  inline
  Element const&
           elementWithKey   (Key const&) const;

  inline
  Element& elementWithKey   (Key const&);

  inline
  Element const&
           firstElement     () const;

  inline
  IBoolean isBounded        () const;

  inline
  IBoolean isEmpty          () const;

  inline
  IBoolean isFirst          (ICursor const&) const;

  inline
  IBoolean isFull           () const;

  inline
  IBoolean isLast           (ICursor const&) const;

  inline
  Key const&
           key              (Element const&) const;

  inline
  Element const&
           lastElement      () const;

  inline
  IBoolean locateElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateNextElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateOrAddElementWithKey (Element const&);

  inline
  IBoolean locateOrAddElementWithKey (Element const&, ICursor&);

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor        () const;

  inline
  INumber  numberOfDifferentKeys () const;

  inline
  INumber  numberOfElements () const;

  inline
  INumber  numberOfElementsWithKey (Key const&) const;

  inline
  IWKSBagOnSeq < Element, Key, ElementOps, Base >&
           operator =       (IWKSBagOnSeq
                              < Element,
                                Key, ElementOps, Base > const&);

  inline
  INumber  removeAll        (IBoolean (*property)
                              (Element const&, void*),
                             void* additionalArgument = 0);

  inline
  void     removeAll        ();

  inline
  INumber  removeAllElementsWithKey (Key const&);

  inline
  void     removeAt         (ICursor&);

  inline
  void     removeAtPosition (IPosition);

  inline
  IBoolean removeElementWithKey (Key const&);

  inline
  void     removeFirst      ();

  inline
  void     removeLast       ();

  inline
  void     replaceAt        (ICursor const&, Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean setToFirst       (ICursor&) const;

  inline
  IBoolean setToLast        (ICursor&) const;

  inline
  IBoolean setToNext        (ICursor&) const;

  inline
  IBoolean setToNextWithDifferentKey (ICursor&) const;

  inline
  void     setToPosition    (IPosition, ICursor&) const;

#ifndef __BIN_COMPAT_BASE_43__
  inline
  IPosition     position    (ICursor const&) const;
#endif

  inline
  IBoolean setToPrevious    (ICursor&) const;

  friend class IWKSBagOnSeqCursor < Element, Key, ElementOps, Base >;

  inline
  IBoolean isConsistent     () const;

protected:

  struct EnvForRemovePredicate {
    ElementOps const* ops;
    Key const* key;
  };

  static IBoolean  removePredicate (Element const&, void*);

  inline
  IBoolean         locateKeyFwd    (Key const&, ICursor&) const;

  inline
  IBoolean         locateKey       (Key const&, ICursor&) const;

  ElementOps ivElementOps;

  // used for cfront support:
  inline
  IOrderedCursor < Element >&
            baseCursor (ICursor&) const;

  inline
  IOrderedCursor < Element > const&
            baseCursor (ICursor const&) const;

};


#define IDefineGKeySortedBagOnGSequence(GSeq, GKeySortedBag) \
template < class Element, class Key, class ElementOps > \
class GKeySortedBag : \
  public IWKSBagOnSeq < Element, Key, ElementOps, \
                        GSeq < Element, ElementOps > > { \
public: \
  GKeySortedBag (INumber n = 100) : \
    IWKSBagOnSeq < Element, Key, ElementOps, \
                   GSeq < Element, ElementOps > > (n) {} \
};



#ifdef ICL_NO_INLINE
#undef inline
#else
#include <iksbseq.inl>
#endif


#if ! defined (__TEMPINC__)
#include <iksbseq.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
