/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IKSKSS_H
#define _IKSKSS_H

#include <iglobals.h>
#include <icursor.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class Key, class ElementOps, class Base >
class IWKeySetOnKSSet;

template < class Element, class Key, class ElementOps, class Base >
class IWKeySetOnKSSetCursor :
    public Base::Cursor
{
public:

           IWKeySetOnKSSetCursor (IWKeySetOnKSSet
                    < Element, Key, ElementOps, Base > const& c)
           : Base::Cursor (c.ivBase) {}

};

template < class Element, class Key, class ElementOps, class Base >
class IWKeySetOnKSSet {

protected:

  Base ivBase;
  friend class IWKeySetOnKSSetCursor < Element, Key, ElementOps, Base >;

public:


  typedef IWKeySetOnKSSetCursor < Element, Key,
                                  ElementOps, Base > Cursor;

  inline   IWKeySetOnKSSet  (INumber numberOfElements = 100);

  inline   IWKeySetOnKSSet  (IWKeySetOnKSSet
                              < Element, Key,
                                ElementOps, Base > const&);

  inline  ~IWKeySetOnKSSet  ();

  inline
  IBoolean add              (Element const&);

  inline
  IBoolean add              (Element const&, ICursor&);

  inline
  void     addAllFrom       (IWKeySetOnKSSet
                              < Element, Key,
                                ElementOps, Base > const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean allElementsDo    (IBoolean (*function) (Element&, void*),
                             void* additionalArgument = 0);

  inline
  IBoolean allElementsDo    (IIterator <Element>&);

  inline
  IBoolean allElementsDo    (IBoolean (*function)
                              (Element const&, void*),
                             void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo    (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement       () const;

  inline
  IBoolean containsAllKeysFrom
                            (IWKeySetOnKSSet
                              < Element, Key,
                                ElementOps, Base > const&) const;

  inline
  IBoolean containsElementWithKey (Key const&) const;

  inline
  Element const&
           elementAt        (ICursor const&) const;

  inline
  Element& elementAt        (ICursor const&);

  inline
  Element const&
           elementWithKey   (Key const&) const;

  inline
  Element& elementWithKey   (Key const&);

  inline
  IBoolean isBounded        () const;

  inline
  IBoolean isEmpty          () const;

  inline
  IBoolean isFull           () const;

  inline
  Key const&
           key              (Element const&) const;

  inline
  IBoolean locateElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateOrAddElementWithKey (Element const&);

  inline
  IBoolean locateOrAddElementWithKey (Element const&, ICursor&);

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor        () const;

  inline
  INumber  numberOfElements () const;

  inline
  IWKeySetOnKSSet < Element, Key, ElementOps, Base >&
           operator =       (IWKeySetOnKSSet
                              < Element, Key,
                                ElementOps, Base > const&);

  inline
  void     removeAll        ();

  inline
  INumber  removeAll        (IBoolean (*property)
                              (Element const&, void*),
                             void* additionalArgument = 0);

  inline
  void     removeAt         (ICursor&);

  inline
  IBoolean removeElementWithKey (Key const&);

  inline
  void     replaceAt        (ICursor const&, Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean setToFirst       (ICursor&) const;

  inline
  IBoolean setToNext        (ICursor&) const;

  friend class IWKeySetOnKSSetCursor < Element, Key, ElementOps, Base >;

  inline
  IBoolean isConsistent     () const;

protected:

  // used for cfront support:
  inline
  IElementCursor < Element >&
            baseCursor (ICursor&) const;

  inline
  IElementCursor < Element > const&
            baseCursor (ICursor const&) const;

};


#define IDefineGKeySetOnGKeySortedSet(GKSSet, GKeySet) \
template < class Element, class Key, class ElementOps > \
class GKeySet : \
  public IWKeySetOnKSSet \
           < Element, Key, ElementOps, \
             GKSSet < Element, Key, ElementOps > > { \
public: \
  GKeySet (INumber n = 100) : \
    IWKeySetOnKSSet \
      < Element, Key, ElementOps, \
        GKSSet < Element, Key, ElementOps > > (n) {} \
};



#ifdef ICL_NO_INLINE
#undef inline
#else
#include <ikskss.inl>
#endif


#if ! defined (__TEMPINC__)
#include <ikskss.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
