/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <new.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)


#ifdef ICL_NO_INLINE
#define inline
#include <ilnseq.inl>
#undef inline
#endif

// ********************** IGLinkedSequenceCursor ***********************

template < class Element, class ElementOps >
IBoolean IGLinkedSequenceCursor < Element, ElementOps >::
setToFirst ()
{ return ivCollection->setToFirst (*this);
}

template < class Element, class ElementOps >
IBoolean IGLinkedSequenceCursor < Element, ElementOps >::
setToNext ()
{ return ivCollection->setToNext (*this);
}

template < class Element, class ElementOps >
void IGLinkedSequenceCursor < Element, ElementOps >::
copy (ICursor const& cursor)
{ operator= ((IGLinkedSequenceCursor
                < Element, ElementOps > const&) cursor);
}

template < class Element, class ElementOps >
IBoolean IGLinkedSequenceCursor < Element, ElementOps >::
operator== (ICursor const& cursor) const
{ return operator== ((IGLinkedSequenceCursor
                        < Element, ElementOps > const&) cursor);
}

template < class Element, class ElementOps >
Element const& IGLinkedSequenceCursor < Element, ElementOps >::
element () const
{ return ivCollection->elementAt (*this);
}

template < class Element, class ElementOps >
IBoolean IGLinkedSequenceCursor < Element, ElementOps >::
setToLast ()
{ return ivCollection->setToLast (*this);
}

template < class Element, class ElementOps >
IBoolean IGLinkedSequenceCursor < Element, ElementOps >::
setToPrevious ()
{ return ivCollection->setToPrevious (*this);
}

template < class Element, class ElementOps >
IBoolean IGLinkedSequenceCursor < Element, ElementOps >::
isValid () const
{ return ivNode != 0;
}

template < class Element, class ElementOps >
void IGLinkedSequenceCursor < Element, ElementOps >::
invalidate ()
{ ivNode = 0;
}

// ************************** IGLinkedSequence *************************

template < class Element, class ElementOps >
IGLinkedSequenceNode < Element, ElementOps >*
IGLinkedSequence < Element, ElementOps >::
newNodeFromElem (Element const& e) const
{ void* nodeSpace = ivElementOps.allocate
                      (sizeof (Node)
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
  ICHECK (nodeSpace != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return new (nodeSpace) Node (e);
}

template < class Element, class ElementOps >
ILinkedSequenceImplNode* IGLinkedSequence < Element, ElementOps >::
newNode (void const* node) const
{ return newNodeFromElem (((Node const*) node)->ivElement);
}

template < class Element, class ElementOps >
void IGLinkedSequence < Element, ElementOps >::
deleteNode (void* node) const
{ ((Node*) node)->~Node ();
  ivElementOps.deallocate
                      (node
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
}

template < class Element, class ElementOps >
IBoolean IGLinkedSequence < Element, ElementOps >::
constantFunctionIteration (void *iterationFunction,
                           void* env,
                           void const* node) const
{ return (*(IBoolean (*) (Element const&, void*)) iterationFunction)
           (((Node const*)node)->ivElement, env);
}

template < class Element, class ElementOps >
IBoolean IGLinkedSequence < Element, ElementOps >::
functionIteration (void *iterationFunction, void* env, void* node) const
{ return (*(IBoolean (*) (Element&, void*)) iterationFunction)
           (((Node*)node)->ivElement, env);
}

template < class Element, class ElementOps >
IBoolean IGLinkedSequence < Element, ElementOps >::
constantIteratorIteration (void* iterator, void const* node) const
{ return ((IConstantIterator < Element >*)iterator)->
           applyTo (((Node const*)node)->ivElement);
}

template < class Element, class ElementOps >
IBoolean IGLinkedSequence < Element, ElementOps >::
iteratorIteration (void* iterator, void* node) const
{ return ((IIterator < Element >*)iterator)->
           applyTo (((Node*)node)->ivElement);
}

template < class Element, class ElementOps >
long IGLinkedSequence < Element, ElementOps >::
functionComparison (void *comparisonFunction,
                    void const* node1,
                    void const* node2) const
{ return (*(long (*) (Element const&, Element const&) const)
            comparisonFunction)
           ( ((Node const*)node1)->ivElement,
             ((Node const*)node2)->ivElement );
}

#pragma info (restore)
