/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _ILNSEQ_H
#define _ILNSEQ_H

#include <iglobals.h>
#include <icursor.h>
#include <iilnseq.h>
#include <ibexcept.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class ElementOps >
class IGLinkedSequence;

template < class Element, class ElementOps >
class IGLinkedSequenceNode;

template < class Element, class ElementOps >
class IGLinkedSequenceCursor :
  public IOrderedCursor < Element > {
public:

           IGLinkedSequenceCursor (IGLinkedSequence
                                    < Element, ElementOps > const& c)
           : ivCollection (&c), ivNode (0)
           {
           }

  IBoolean setToFirst ();

  IBoolean setToNext ();

  IBoolean isValid () const;

  void     invalidate ();

  void     copy (ICursor const& cursor);

  IBoolean operator== (ICursor const& cursor) const;

  Element const&
           element () const;

  IBoolean setToLast ();

  IBoolean setToPrevious ();

  inline
  IBoolean operator== (IGLinkedSequenceCursor
                        < Element, ElementOps > const& cursor) const;

  inline
  IBoolean operator!= (IGLinkedSequenceCursor
                        < Element, ElementOps > const& cursor) const;

protected:

  IGLinkedSequence < Element, ElementOps > const*
           ivCollection;

  ILinkedSequenceImplNode*
           ivNode;

  IBoolean isFor (IGLinkedSequence < Element,
                                     ElementOps > const& c) const
           { return ivCollection == &c;
           }

  friend class IGLinkedSequence < Element, ElementOps >;

};


template < class Element, class ElementOps >
class IGLinkedSequenceNode : public ILinkedSequenceImplNode {

    Element  ivElement;

             IGLinkedSequenceNode (Element const& element)
             : ivElement (element)
             {
             }

            ~IGLinkedSequenceNode ()
             {
             }

    friend class IGLinkedSequence < Element, ElementOps >;

    friend class IGLinkedSequenceCursor < Element, ElementOps >;


};


template < class Element, class ElementOps >
class IGLinkedSequence : protected ILinkedSequenceImpl {

public:


  typedef IGLinkedSequenceCursor < Element, ElementOps > Cursor;

  inline   IGLinkedSequence (INumber numberOfElements = 100);

  inline   IGLinkedSequence (IGLinkedSequence < Element,
                                                ElementOps > const&);

  inline  ~IGLinkedSequence ();

  inline
  IBoolean add              (Element const&);

  inline
  IBoolean add              (Element const&, ICursor&);

  inline
  void     addAllFrom       (IGLinkedSequence < Element,
                                                ElementOps > const&);

  inline
  void     addAsFirst       (Element const&);

  inline
  void     addAsFirst       (Element const&, ICursor&);

  inline
  void     addAsLast        (Element const&);

  inline
  void     addAsLast        (Element const&, ICursor&);

  inline
  void     addAsNext        (Element const&, ICursor&);

  inline
  void     addAsPrevious    (Element const&, ICursor&);

  inline
  void     addAtPosition    (IPosition, Element const&);

  inline
  void     addAtPosition    (IPosition, Element const&, ICursor&);

  inline
  IBoolean allElementsDo    (IBoolean (*function) (Element&, void*),
                             void* additionalArgument = 0);

  inline
  IBoolean allElementsDo    (IIterator <Element>&);

  inline
  IBoolean allElementsDo    (IBoolean (*function)
                              (Element const&, void*),
                             void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo    (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement       () const;

  inline
  long     compare          (IGLinkedSequence < Element,
                                                ElementOps > const&,
                             long (*comparisonFunction)
                               (Element const&, Element const&)) const;

  inline
  Element const&
           elementAt        (ICursor const&) const;

  inline
  Element& elementAt        (ICursor const&);

  inline
  Element const&
           firstElement     () const;

  inline
  Element const&
           elementAtPosition (IPosition) const;

  inline
  IBoolean isBounded        () const;

  inline
  IBoolean isEmpty          () const;

  inline
  IBoolean isFirst          (ICursor const&) const;

  inline
  IBoolean isFull           () const;

  inline
  IBoolean isLast           (ICursor const&) const;

  inline
  Element const&
           lastElement      () const;

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor        () const;

  inline
  INumber  numberOfElements () const;

  inline
  IGLinkedSequence < Element, ElementOps >&
           operator =       (IGLinkedSequence < Element,
                                                ElementOps > const&);

  inline
  void     removeAll        ();

  inline
  INumber  removeAll        (IBoolean (*property)
                              (Element const&, void*),
                             void* additionalArgument = 0);

  inline
  void     removeAt         (ICursor&);

  inline
  void     removeAtPosition (IPosition);

  inline
  void     removeFirst      ();

  inline
  void     removeLast       ();

  inline
  void     replaceAt        (ICursor const&, Element const&);

  inline
  IBoolean setToFirst       (ICursor&) const;

  inline
  IBoolean setToNext        (ICursor&) const;

  inline
  IBoolean setToLast        (ICursor&) const;

  inline
  void     setToPosition    (IPosition, ICursor&) const;

#ifndef __BIN_COMPAT_BASE_43__
  inline
  IPosition     position    (ICursor const&) const;
#endif

  inline
  IBoolean setToPrevious    (ICursor&) const;

  inline
  void     sort             (long (*comparisonFunction)
                              (Element const&, Element const&));

protected:

  typedef IGLinkedSequenceNode < Element, ElementOps > Node;


  friend class IGLinkedSequenceCursor < Element, ElementOps>;

public:


  inline
  IBoolean isConsistent     () const;

protected:

  ElementOps ivElementOps;

  virtual
  ILinkedSequenceImplNode*
           newNode          (void const* node) const;

  virtual
  void     deleteNode       (void* node) const;

  virtual
  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      void const* node) const;
  virtual
  IBoolean functionIteration         (void *function,
                                      void* env,
                                      void* node) const;
  virtual
  IBoolean constantIteratorIteration (void* iterator,
                                      void const* node) const;
  virtual
  IBoolean iteratorIteration         (void* iterator,
                                      void* node) const;

  virtual
  long     functionComparison        (void *compareFunction,
                                      void const* node1,
                                      void const* node2) const;

  Node*    newNodeFromElem           (Element const& e) const;

};

template < class Element >
class ILinkedSequence :
  public IGLinkedSequence < Element, IStdOps < Element > > {
public:


  inline ILinkedSequence (INumber n = 100);

};


#ifdef ICL_NO_INLINE
#undef inline
#else
#include <ilnseq.inl>
#endif

#ifndef __TEMPINC__
#include <ilnseq.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
