/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

// ************************** IGLinkedSequenceCursor *******************

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequenceCursor<Element, ElementOps>::
operator== (IGLinkedSequenceCursor
             < Element, ElementOps> const& cursor) const
{ return ivCollection == cursor.ivCollection &&
         ivNode == cursor.ivNode;
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequenceCursor<Element, ElementOps>::
operator!= (IGLinkedSequenceCursor
             < Element, ElementOps> const& cursor) const
{ return ! operator== (cursor);
}

// ************************** IGLinkedSequence *************************

template <class Element, class ElementOps>
inline IGLinkedSequence <Element, ElementOps>::
IGLinkedSequence (INumber)
{
}

template <class Element, class ElementOps>
inline IGLinkedSequence <Element, ElementOps>::
IGLinkedSequence (IGLinkedSequence < Element,
                                     ElementOps > const& sequence)
: ILinkedSequenceImpl (sequence)
{ ILinkedSequenceImpl::addAllFrom (sequence);
}

template <class Element, class ElementOps>
inline IGLinkedSequence <Element, ElementOps>::
~IGLinkedSequence ()
{ ILinkedSequenceImpl::removeAll ();
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
addAllFrom (IGLinkedSequence < Element, ElementOps > const& sequence)
{
#ifndef INO_CHECKS
  ICHECK (this != &sequence,
          IIdenticalCollectionException, IIdenticalCollectionText)
#endif
  ILinkedSequenceImpl::addAllFrom (sequence);
}

template <class Element, class ElementOps>
inline Element const& IGLinkedSequence <Element, ElementOps>::
elementAt (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ILinkedSequenceImpl::checkNode
            (((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Node*) ((Cursor const&)cursor).ivNode)->ivElement;
}

template <class Element, class ElementOps>
inline Element& IGLinkedSequence <Element, ElementOps>::
elementAt (ICursor const& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ILinkedSequenceImpl::checkNode
            (((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Node*) ((Cursor const&)cursor).ivNode)->ivElement;
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
removeAt (ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ILinkedSequenceImpl::checkNode
            (((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ILinkedSequenceImpl::removeAt (((Cursor&)cursor).ivNode);
  cursor.invalidate ();
}

template < class Element, class ElementOps >
inline INumber IGLinkedSequence < Element, ElementOps >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{ return ILinkedSequenceImpl::removeAll (predicate, env);
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
replaceAt (ICursor const& cursor, Element const& element)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ILinkedSequenceImpl::checkNode
            (((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ivElementOps.assign
    (((Node*) ((Cursor const&)cursor).ivNode)->ivElement, element);
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
removeAll ()
{ ILinkedSequenceImpl::removeAll ();
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
isBounded () const
{ return False;
}

template <class Element, class ElementOps>
inline INumber IGLinkedSequence <Element, ElementOps>::
maxNumberOfElements () const
{ ICHECK (False, INotBoundedException,
          INotBoundedText)
  return 0;
}

template <class Element, class ElementOps>
inline INumber IGLinkedSequence <Element, ElementOps>::
numberOfElements () const
{ return ILinkedSequenceImpl::numberOfElements ();
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
isEmpty () const
{ return ILinkedSequenceImpl::isEmpty ();
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
isFull () const
{ return ILinkedSequenceImpl::isFull ();
}

template <class Element, class ElementOps>
inline IGLinkedSequenceCursor <Element, ElementOps>*
IGLinkedSequence <Element, ElementOps>::
newCursor () const
{ Cursor* result = new Cursor (*this);
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
setToFirst (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  return ILinkedSequenceImpl::setToFirst (((Cursor&)cursor).ivNode);
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
setToNext (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ILinkedSequenceImpl::checkNode
            (((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ILinkedSequenceImpl::setToNext (((Cursor&)cursor).ivNode);
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
allElementsDo (IBoolean (*function) (Element&, void*), void* env)
{ return ILinkedSequenceImpl::allElementsDo (function, env);
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
allElementsDo (IIterator < Element > & iterator)
{ return ILinkedSequenceImpl::allElementsDo (&iterator);
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
allElementsDo (IBoolean (*function) (Element const&, void*),
               void* env) const
{ return ILinkedSequenceImpl::allElementsDo (function, env);
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
allElementsDo (IConstantIterator < Element > & iterator) const
{ return ILinkedSequenceImpl::allElementsDo (&iterator);
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
isConsistent () const
{ return ILinkedSequenceImpl::isConsistent ();
}

template <class Element, class ElementOps>
inline IGLinkedSequence <Element, ElementOps>&
IGLinkedSequence <Element, ElementOps>::
operator = (IGLinkedSequence < Element, ElementOps > const& sequence)
{ if (&sequence != this) {
    removeAll ();
    addAllFrom (sequence);
  }
  return *this;
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
removeFirst ()
{
#ifndef INO_CHECKS
  ICHECK (! ILinkedSequenceImpl::isEmpty (),
          IEmptyException, ICollectionIsEmptyText)
#endif
  ILinkedSequenceImpl::removeFirst ();
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
removeLast ()
{
#ifndef INO_CHECKS
  ICHECK (! ILinkedSequenceImpl::isEmpty (),
          IEmptyException, ICollectionIsEmptyText)
#endif
  ILinkedSequenceImpl::removeLast ();
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
removeAtPosition (IPosition position)
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
  ILinkedSequenceImpl::removeAtPosition (position);
}

template <class Element, class ElementOps>
inline Element const& IGLinkedSequence <Element, ElementOps>::
firstElement () const
{
#ifndef INO_CHECKS
  ICHECK (! ILinkedSequenceImpl::isEmpty (),
          IEmptyException, ICollectionIsEmptyText)
#endif
  ILinkedSequenceImplNode *node;
  ILinkedSequenceImpl::setToFirst (node);
  return ((Node*)node)->ivElement;
}

template <class Element, class ElementOps>
inline Element const& IGLinkedSequence <Element, ElementOps>::
anyElement () const
{ return firstElement ();
}

template <class Element, class ElementOps>
inline Element const& IGLinkedSequence <Element, ElementOps>::
lastElement () const
{
#ifndef INO_CHECKS
  ICHECK (! ILinkedSequenceImpl::isEmpty (),
          IEmptyException, ICollectionIsEmptyText)
#endif
  ILinkedSequenceImplNode *node;
  ILinkedSequenceImpl::setToLast (node);
  return ((Node*)node)->ivElement;
}

template <class Element, class ElementOps>
inline Element const& IGLinkedSequence <Element, ElementOps>::
elementAtPosition (IPosition position) const
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
  ILinkedSequenceImplNode *node;
  ILinkedSequenceImpl::setToPosition (position, node);
  return ((Node*)node)->ivElement;
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
setToLast (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  return ILinkedSequenceImpl::setToLast (((Cursor&)cursor).ivNode);
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
setToPrevious (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ILinkedSequenceImpl::checkNode
            (((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ILinkedSequenceImpl::setToPrevious (((Cursor&)cursor).ivNode);
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
setToPosition (IPosition position, ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  ILinkedSequenceImpl::setToPosition
    (position, ((Cursor&)cursor).ivNode);
}

#ifndef __BIN_COMPAT_BASE_43__
template <class Element, class ElementOps>
inline IPosition IGLinkedSequence <Element, ElementOps>::
position (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
  return ILinkedSequenceImpl::position
    (((Cursor&)cursor).ivNode);
}
#endif

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
isFirst (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ILinkedSequenceImpl::checkNode
            (((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ILinkedSequenceImpl::isFirst (((Cursor const&)cursor).ivNode);
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
isLast (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ILinkedSequenceImpl::checkNode
            (((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ILinkedSequenceImpl::isLast (((Cursor const&)cursor).ivNode);
}

template <class Element, class ElementOps>
inline long IGLinkedSequence <Element, ElementOps>::
compare (IGLinkedSequence < Element, ElementOps > const& sequence,
          long (*comparisonFunction)
            (Element const&, Element const&)) const
{ return ILinkedSequenceImpl::compare (sequence, comparisonFunction);
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
addAsFirst (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  ((Cursor&)cursor).ivNode =
    ILinkedSequenceImpl::addAsFirst (newNodeFromElem (element));
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
addAsFirst (Element const& element)
{ ILinkedSequenceImpl::addAsFirst (newNodeFromElem (element));
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
addAsLast (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  ((Cursor&)cursor).ivNode =
    ILinkedSequenceImpl::addAsLast (newNodeFromElem (element));
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
addAsLast (Element const& element)
{ ILinkedSequenceImpl::addAsLast (newNodeFromElem (element));
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
add (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  ((Cursor&)cursor).ivNode =
    ILinkedSequenceImpl::addAsLast (newNodeFromElem (element));
  return True;
}

template <class Element, class ElementOps>
inline IBoolean IGLinkedSequence <Element, ElementOps>::
add (Element const& element)
{ ILinkedSequenceImpl::addAsLast (newNodeFromElem (element));
  return True;
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
addAsNext (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ILinkedSequenceImpl::checkNode
            (((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ((Cursor&)cursor).ivNode =
    ILinkedSequenceImpl::addAsNext
      (((Cursor&)cursor).ivNode, newNodeFromElem (element));
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
addAsPrevious (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ILinkedSequenceImpl::checkNode
            (((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ((Cursor&)cursor).ivNode =
    ILinkedSequenceImpl::addAsPrevious
      (((Cursor&)cursor).ivNode, newNodeFromElem (element));
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
addAtPosition (IPosition position,
               Element const& element,
               ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements() + 1,
          IPositionInvalidException, IInvalidPositionText)
#endif
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  ((Cursor&)cursor).ivNode =
    ILinkedSequenceImpl::addAtPosition
      (position, newNodeFromElem (element));
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
addAtPosition (IPosition position, Element const& element)
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements() + 1,
          IPositionInvalidException, IInvalidPositionText)
#endif
  ILinkedSequenceImpl::addAtPosition
    (position, newNodeFromElem (element));
}

template <class Element, class ElementOps>
inline void IGLinkedSequence <Element, ElementOps>::
sort (long (*comparisonFunction) (Element const&, Element const&))
{ ILinkedSequenceImpl::sort (comparisonFunction);
}

// ************************** ILinkedSequence **************************

template <class Element>
inline ILinkedSequence < Element >::
ILinkedSequence (INumber n)
: IGLinkedSequence < Element, IStdOps < Element > > (n)
{
}
