/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IMAPKS_H
#define _IMAPKS_H

#include <iglobals.h>
#include <icursor.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class Key, class ElementOps, class Base >
class IWMapOnKeySet;

template < class Element, class Key, class ElementOps, class Base >
class IWMapOnKeySetCursor :
  public Base::Cursor
{
public:

           IWMapOnKeySetCursor
                     (IWMapOnKeySet
                       < Element, Key, ElementOps, Base > const& c)
             : Base::Cursor (c.ivBase)
             {
             }

};

template < class Element, class Key, class ElementOps, class Base >
class IWMapOnKeySet {

protected:

  Base ivBase;

public:


  typedef IWMapOnKeySetCursor< Element, Key, ElementOps, Base > Cursor;


  inline   IWMapOnKeySet    (INumber numberOfElements = 100);

  inline   IWMapOnKeySet    (IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&);

  inline  ~IWMapOnKeySet    ();

  inline
  IBoolean add              (Element const&);

  inline
  IBoolean add              (Element const&, ICursor&);

  inline
  void     addAllFrom       (IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&);

  inline
  void     addDifference    (IWMapOnKeySet
                              < Element, Key, ElementOps, Base > const&,
                             IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&);

  inline
  void     addIntersection  (IWMapOnKeySet
                              < Element, Key, ElementOps, Base > const&,
                             IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&);

  inline
  IBoolean addOrReplaceElementWithKey (Element const&, ICursor&);

  inline
  void     addUnion         (IWMapOnKeySet
                              < Element, Key, ElementOps, Base > const&,
                             IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&);

  inline
  IBoolean allElementsDo    (IBoolean (*function) (Element&, void*),
                             void* additionalArgument = 0);

  inline
  IBoolean allElementsDo    (IIterator <Element>&);

  inline
  IBoolean allElementsDo    (IBoolean (*function)
                              (Element const&, void*),
                             void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo    (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement       () const;

  inline
  IBoolean contains         (Element const&) const;

  inline
  IBoolean containsAllFrom  (IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&) const;

  inline
  IBoolean containsAllKeysFrom
                            (IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&) const;

  inline
  IBoolean containsElementWithKey (Key const&) const;

  inline
  void     differenceWith   (IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&);

  inline
  Element const&
           elementAt        (ICursor const&) const;

  inline
  Element& elementAt        (ICursor const&);

  inline
  Element const&
           elementWithKey   (Key const&) const;

  inline
  Element& elementWithKey   (Key const&);

  inline
  void     intersectionWith (IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&);

  inline
  IBoolean isBounded        () const;

  inline
  IBoolean isEmpty          () const;

  inline
  IBoolean isFull           () const;

  inline
  Key const&
           key              (Element const&) const;

  inline
  IBoolean locate           (Element const&, ICursor&) const;

  inline
  IBoolean locateElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateOrAdd      (Element const&);

  inline
  IBoolean locateOrAdd      (Element const&, ICursor&);

  inline
  IBoolean locateOrAddElementWithKey (Element const&);

  inline
  IBoolean locateOrAddElementWithKey (Element const&, ICursor&);

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor        () const;

  inline
  INumber  numberOfElements () const;

  inline
  IWMapOnKeySet < Element, Key, ElementOps, Base >&
           operator =       (IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&);

  inline
  IBoolean operator ==      (IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&) const;

  inline
  IBoolean operator !=      (IWMapOnKeySet
                              < Element, Key,
                                ElementOps, Base > const&) const;

  inline
  IBoolean remove           (Element const&);

  inline
  void     removeAll        ();

  inline
  INumber  removeAll        (IBoolean (*property)
                              (Element const&, void*),
                             void* additionalArgument = 0);

  inline
  void     removeAt         (ICursor&);

  inline
  IBoolean removeElementWithKey (Key const&);

  inline
  void     replaceAt        (ICursor const&, Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&);

  inline
  IBoolean replaceElementWithKey (Element const&, ICursor&);

  inline
  IBoolean setToFirst       (ICursor&) const;

  inline
  IBoolean setToNext        (ICursor&) const;

  inline
  void     unionWith        (IWMapOnKeySet < Element, Key,
                                             ElementOps, Base > const&);

  friend class IWMapOnKeySetCursor < Element, Key,
                                     ElementOps, Base >;

  inline
  IBoolean isConsistent     () const;

protected:

  ElementOps ivElementOps;

  static
  IBoolean isContained    (Element const&, void* env);

  static
  IBoolean isNotContained (Element const&, void* env);

  // used for cfront support:
  inline
  IElementCursor < Element >&
            baseCursor (ICursor&) const;

  inline
  IElementCursor < Element > const&
            baseCursor (ICursor const&) const;

};


#define IDefineGMapOnGKeySet(GKeySet, GMap) \
template < class Element, class Key, class ElementOps > \
class GMap : \
  public IWMapOnKeySet \
           < Element, Key, ElementOps, \
             GKeySet < Element, Key, ElementOps > > { \
public: \
  GMap (INumber n = 100) : \
    IWMapOnKeySet \
      < Element, Key, ElementOps, \
        GKeySet < Element, Key, ElementOps > > (n) {} \
};



#ifdef ICL_NO_INLINE
#undef inline
#else
#include <imapks.inl>
#endif

#ifndef __TEMPINC__
#include <imapks.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
