/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <ibexcept.h>


// **************************** IWMapOnKSSet **************************

template <class Element, class Key, class ElementOps, class Base>
inline IElementCursor < Element >&
IWMapOnKSSet <Element, Key, ElementOps, Base>::
baseCursor (ICursor& cursor) const
{
  return (Cursor&) cursor;
}

template <class Element, class Key, class ElementOps, class Base>
inline IElementCursor < Element > const&
IWMapOnKSSet <Element, Key, ElementOps, Base>::
baseCursor (ICursor const& cursor) const
{
  return (Cursor&) cursor;
}

template < class Element, class Key, class ElementOps, class Base >
inline IWMapOnKSSet < Element, Key, ElementOps, Base >::
IWMapOnKSSet (INumber n)
: ivBase (n)
{
}

template < class Element, class Key, class ElementOps, class Base >
inline IWMapOnKSSet < Element, Key, ElementOps, Base >::
IWMapOnKSSet (IWMapOnKSSet < Element, Key,
                             ElementOps, Base > const& collection)
: ivBase (collection.ivBase)
{
}

template < class Element, class Key, class ElementOps, class Base >
inline IWMapOnKSSet < Element, Key, ElementOps, Base >&
IWMapOnKSSet < Element, Key, ElementOps, Base >::
operator= (IWMapOnKSSet < Element, Key,
                          ElementOps, Base > const& collection)
{ ivBase.operator= (collection.ivBase);
  return *this;
}

template < class Element, class Key, class ElementOps, class Base >
inline IWMapOnKSSet < Element, Key, ElementOps, Base >::
~IWMapOnKSSet ()
{
}

template < class Element, class Key, class ElementOps, class Base >
inline Element const& IWMapOnKSSet < Element, Key, ElementOps, Base >::
elementAt (ICursor const& cursor) const
{ return ivBase.elementAt (baseCursor (cursor));
}

template < class Element, class Key, class ElementOps, class Base >
inline Element& IWMapOnKSSet < Element, Key, ElementOps, Base >::
elementAt (ICursor const& cursor)
{ return ivBase.elementAt (baseCursor (cursor));
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
add (Element const& element, ICursor &cursor)
{ IBoolean hasBeenAdded = ivBase.add (element, baseCursor (cursor));
#ifndef INO_CHECKS
  if (! hasBeenAdded) {
    ICHECK (ivElementOps.equal (elementAt (cursor), element),
            IKeyAlreadyExistsException, IKeyAlreadyExistsText)
  }
#endif
  return hasBeenAdded;
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
add (Element const& element)
{ Cursor cursor (*this);
  return add (element, cursor);
}

template <class Element, class Key, class ElementOps, class Base>
inline void IWMapOnKSSet <Element, Key, ElementOps, Base>::
addAllFrom (IWMapOnKSSet < Element, Key, ElementOps, Base >
            const& collection)
{
#ifndef INO_CHECKS
  ICHECK (this != &collection,
          IIdenticalCollectionException, IIdenticalCollectionText)
#endif
  Cursor cursor (collection);
  forCursor (cursor) {
    add (collection.elementAt (cursor));
  }
}

template < class Element, class Key, class ElementOps, class Base >
inline Element const& IWMapOnKSSet < Element, Key, ElementOps, Base >::
anyElement () const
{ return ivBase.anyElement ();
}

template < class Element, class Key, class ElementOps, class Base >
inline void IWMapOnKSSet < Element, Key, ElementOps, Base >::
removeAt (ICursor& cursor)
{ ivBase.removeAt (baseCursor (cursor));
}

template < class Element, class Key, class ElementOps, class Base >
inline INumber IWMapOnKSSet < Element, Key, ElementOps, Base >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{ return ivBase.removeAll (predicate, env);
}

template < class Element, class Key, class ElementOps, class Base >
inline void IWMapOnKSSet < Element, Key, ElementOps, Base >::
replaceAt (ICursor const& cursor, Element const& element)
{ ivBase.replaceAt (baseCursor (cursor), element);
}

template < class Element, class Key, class ElementOps, class Base >
inline void IWMapOnKSSet < Element, Key, ElementOps, Base >::
removeAll ()
{ ivBase.removeAll ();
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
isBounded () const
{ return ivBase.isBounded ();
}

template < class Element, class Key, class ElementOps, class Base >
inline INumber IWMapOnKSSet < Element, Key, ElementOps, Base >::
maxNumberOfElements () const
{ return ivBase.maxNumberOfElements ();
}

template < class Element, class Key, class ElementOps, class Base >
inline INumber IWMapOnKSSet < Element, Key, ElementOps, Base >::
numberOfElements () const
{ return ivBase.numberOfElements ();
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
isEmpty () const
{ return ivBase.isEmpty ();
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
isFull () const
{ return ivBase.isFull ();
}

template < class Element, class Key, class ElementOps, class Base >
inline IWMapOnKSSetCursor < Element, Key, ElementOps, Base >*
IWMapOnKSSet < Element, Key, ElementOps, Base >::
newCursor () const
{ Cursor* result = new Cursor (*this);
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
setToFirst (ICursor &cursor) const
{ return ivBase.setToFirst (baseCursor (cursor));
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
setToNext (ICursor &cursor) const
{ return ivBase.setToNext (baseCursor (cursor));
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element &, void*),
               void* environment)
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
allElementsDo (IIterator <Element>& iterator)
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* environment) const
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
isConsistent () const
{ return ivBase.isConsistent ();
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
locateElementWithKey (Key const& key, ICursor &cursor) const
{ return ivBase.locateElementWithKey (key, baseCursor (cursor));
}

template <class Element, class Key, class ElementOps, class Base>
inline IBoolean IWMapOnKSSet <Element, Key, ElementOps, Base>::
contains (Element const& element) const
{ Cursor cursor (*this);
  if (locateElementWithKey (ivElementOps.key (element), cursor)) {
    return ivElementOps.equal (elementAt (cursor), element);
  }
  else {
    return False;
  }
}

template <class Element, class Key, class ElementOps, class Base>
inline IBoolean IWMapOnKSSet <Element, Key, ElementOps, Base>::
containsAllFrom (IWMapOnKSSet <Element, Key, ElementOps, Base>
                 const& collection) const
{ Cursor cursor (collection);
  IBoolean result = True;
  forCursor (cursor) {
    if (! contains (collection.elementAt (cursor))) {
      result = False;
      break;
    }
  }
  return result;
}

template <class Element, class Key, class ElementOps, class Base>
inline IBoolean IWMapOnKSSet <Element, Key, ElementOps, Base>::
locate (Element const& element, ICursor& cursor) const
{ IBoolean result = locateElementWithKey
                      (ivElementOps.key (element), cursor);
  if (result) {
    result = ivElementOps.equal (elementAt (cursor), element);
  }
  if (! result) {
    cursor.invalidate ();
  }
  return result;
}

template <class Element, class Key, class ElementOps, class Base>
inline IBoolean IWMapOnKSSet <Element, Key, ElementOps, Base>::
locateOrAdd (Element const& element)
{ IBoolean result = contains (element);
  if (! result) {
    add (element);
  }
  return result;
}

template <class Element, class Key, class ElementOps, class Base>
inline IBoolean IWMapOnKSSet <Element, Key, ElementOps, Base>::
locateOrAdd (Element const& element, ICursor& cursor)
{ IBoolean result = locate (element, cursor);
  if (! result) {
    add (element, cursor);
  }
  return result;
}

template <class Element, class Key, class ElementOps, class Base>
inline IBoolean IWMapOnKSSet <Element, Key, ElementOps, Base>::
operator == (IWMapOnKSSet <Element, Key, ElementOps, Base>
               const& collection) const
{ return numberOfElements() == collection.numberOfElements() &&
         containsAllFrom (collection);
}

template <class Element, class Key, class ElementOps, class Base>
inline IBoolean IWMapOnKSSet <Element, Key, ElementOps, Base>::
operator != (IWMapOnKSSet <Element, Key, ElementOps, Base>
               const& collection) const
{ return ! operator == (collection);
}

template <class Element, class Key, class ElementOps, class Base>
inline void IWMapOnKSSet <Element, Key, ElementOps, Base>::
unionWith (IWMapOnKSSet <Element, Key, ElementOps, Base>
             const& collection)
{ if (this != &collection)
    addAllFrom (collection);
}

template <class Element, class Key, class ElementOps, class Base>
inline void IWMapOnKSSet <Element, Key, ElementOps, Base>::
intersectionWith (IWMapOnKSSet <Element, Key, ElementOps, Base>
                    const& collection)
{ if (this != &collection)
  { IWMapOnKSSet < Element, Key,
                   ElementOps, Base > const* env = &collection;
    removeAll (isNotContained, &env);
  }
}

template <class Element, class Key, class ElementOps, class Base>
inline void IWMapOnKSSet <Element, Key, ElementOps, Base>::
differenceWith (IWMapOnKSSet <Element, Key, ElementOps, Base>
                  const& collection)
{ if (this == &collection)
    removeAll ();
  else
  { IWMapOnKSSet < Element, Key,
                   ElementOps, Base > const* env = &collection;
    removeAll (isContained, &env);
  }
}

template <class Element, class Key, class ElementOps, class Base>
inline void IWMapOnKSSet <Element, Key, ElementOps, Base>::
addUnion (IWMapOnKSSet <Element, Key, ElementOps, Base>
            const& collection1,
          IWMapOnKSSet <Element, Key, ElementOps, Base>
            const& collection2)
{ unionWith (collection1);
  unionWith (collection2);
}

template < class Element, class Key, class ElementOps, class Base >
inline Key const& IWMapOnKSSet < Element, Key, ElementOps, Base >::
key (Element const& element) const
{ return ivElementOps.key (element);
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
containsElementWithKey (Key const& key) const
{ return ivBase.containsElementWithKey (key);
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
containsAllKeysFrom (IWMapOnKSSet
                      < Element, Key, ElementOps, Base > const& c) const
{ return ivBase.containsAllKeysFrom (c.ivBase);
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
replaceElementWithKey (Element const& element)
{ return ivBase.replaceElementWithKey (element);
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
replaceElementWithKey (Element const& element, ICursor &cursor)
{ return ivBase.replaceElementWithKey (element, baseCursor (cursor));
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
locateOrAddElementWithKey (Element const& element)
{ return ivBase.locateOrAddElementWithKey (element);
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
locateOrAddElementWithKey (Element const& element, ICursor &cursor)
{ return ivBase.locateOrAddElementWithKey
           (element, baseCursor (cursor));
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
addOrReplaceElementWithKey (Element const& element)
{ return ivBase.addOrReplaceElementWithKey (element);
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
addOrReplaceElementWithKey (Element const& element, ICursor &cursor)
{ return ivBase.addOrReplaceElementWithKey
           (element, baseCursor (cursor));
}

template < class Element, class Key, class ElementOps, class Base >
inline IBoolean IWMapOnKSSet < Element, Key, ElementOps, Base >::
removeElementWithKey (Key const& key)
{ return ivBase.removeElementWithKey (key);
}

template < class Element, class Key, class ElementOps, class Base >
inline Element const& IWMapOnKSSet < Element, Key, ElementOps, Base >::
elementWithKey (Key const& key) const
{ return ivBase.elementWithKey (key);
}

template < class Element, class Key, class ElementOps, class Base >
inline Element& IWMapOnKSSet < Element, Key, ElementOps, Base >::
elementWithKey (Key const& key)
{ return ivBase.elementWithKey (key);
}

template <class Element, class Key, class ElementOps, class Base>
inline IBoolean IWMapOnKSSet <Element, Key, ElementOps, Base>::
remove (Element const& element)
{ Cursor cursor (*this);
  IBoolean result = locateElementWithKey
                      (ivElementOps.key (element), cursor);
  if (result) {
    result = ivElementOps.equal (elementAt (cursor), element);
  }
  if (result) {
    removeAt (cursor);
  }
  return result;
}

template <class Element, class Key, class ElementOps, class Base>
inline void IWMapOnKSSet <Element, Key, ElementOps, Base>::
addIntersection (IWMapOnKSSet <Element, Key, ElementOps, Base>
                   const& collection1,
                 IWMapOnKSSet <Element, Key, ElementOps, Base>
                   const& collection2)
{ if (this == &collection1 || this == &collection2)
    return;
  Cursor cursor (collection1);
  forCursor (cursor) {
    if (collection2.contains (collection1.elementAt (cursor)))
      add (collection1.elementAt (cursor));
  }
}

template <class Element, class Key, class ElementOps, class Base>
inline void IWMapOnKSSet <Element, Key, ElementOps, Base>::
addDifference (IWMapOnKSSet <Element, Key, ElementOps, Base>
                 const& collection1,
               IWMapOnKSSet <Element, Key, ElementOps, Base>
                 const& collection2)
{ if (this == &collection1)
    return;
  else if (this == &collection2)
    unionWith (collection1);
  else {
    Cursor cursor (collection1);
    forCursor (cursor) {
      if (! collection2.contains (collection1.elementAt (cursor)))
        add (collection1.elementAt (cursor));
    }
  }
}
