#ifndef _IMENU_
#define _IMENU_
/*******************************************************************************
* FILE NAME: imenu.hpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IMenu                                                                    *
*     IMenu::Cursor                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <iwindow.hpp>

class IMenuItem;
class IColor;
class IResourceId;
class IMenuData;
class IMenuCursorData;
struct _WidgetRec;

#pragma pack(4)

class IMenu : public IWindow {
typedef IWindow
  Inherited;
public:
/*------------------------------- Constructors -------------------------------*/
  IMenu ( const IMenuHandle& menuHandle );

virtual
 ~IMenu ( );

/*---------------------------------- Styles ----------------------------------*/
INESTEDBITFLAGCLASSDEF1(Style, IMenu, IWindow);

static const Style
  classDefaultStyle,
  noStyle;

static Style
  defaultStyle      ( );

static void
  setDefaultStyle   ( const Style&    aStyle );

virtual unsigned long
  convertToGUIStyle ( const IBitFlag& style,
                      Boolean         extendedOnly = false ) const;

/*------------------------------- Adding Items -------------------------------*/
virtual IMenu
 &addItem      ( IMenuItem&           menuItem,
                 unsigned long        intoSubmenuId = 0),
 &addText      ( unsigned long        newItemId,
                 const char*          itemText,
                 unsigned long        intoSubmenuId = 0),
 &addText      ( unsigned long        newItemId,
                 const IResourceId&   textResId,
                 unsigned long        intoSubmenuId = 0),
 &addBitmap    ( unsigned long        newItemId,
                 const IBitmapHandle& itemBitmap,
                 unsigned long        intoSubmenuId = 0),
 &addBitmap    ( unsigned long        newItemId,
                 const IResourceId&   bitmapResId,
                 unsigned long        intoSubmenuId = 0),
 &addBitmap    ( unsigned long        newItemId,
                 unsigned long        bitmapResId,
                 unsigned long        intoSubmenuId = 0),
 &addSeparator ( unsigned long        newItemId,
                 unsigned long        intoSubmenuId),
 &addSeparator ( unsigned long        intoSubmenuId = 0 ),
 &addSubmenu   ( unsigned long        itemId ),
 &addSubmenu   ( unsigned long        itemId,
                 const IResourceId&   submenuResId );

/*----------------------------- Replacing Items ------------------------------*/
virtual IMenu
 &setItem    ( const IMenuItem&     menuItem ),
 &setText    ( unsigned long        menuItemId,
               const char*          newText     ),
 &setText    ( unsigned long        menuItemId,
               const IResourceId&   newTextResId ),
 &setBitmap  ( unsigned long        menuItemId,
               const IBitmapHandle& bitmapHandle ),
 &setBitmap  ( unsigned long        menuItemId,
               const IResourceId&   newBitmapResId ),
 &setBitmap  ( unsigned long        menuItemId,
               unsigned long        newBitmapResId ),
 &setSubmenu ( unsigned long        itemId,
               const IResourceId&   submenuResId );

/*------------------------------ Removing Items ------------------------------*/
virtual IMenu
 &deleteItem    ( unsigned long itemId ),
 &removeSubmenu ( unsigned long itemWithSubmenuId );

/*-------------------------------- Selection ---------------------------------*/
virtual IMenu
 &checkItem     ( unsigned long itemId,
                  Boolean       check  = true ),
 &uncheckItem   ( unsigned long itemId ),
 &enableItem    ( unsigned long itemId,
                  Boolean       enable = true ),
 &disableItem   ( unsigned long itemId ),
 &selectItem    ( unsigned long itemId );

Boolean
  isItemChecked ( unsigned long itemId ) const,
  isItemEnabled ( unsigned long itemId ) const;

/*----------------------------- Item Properties ------------------------------*/
IMenuItem
  menuItem      ( unsigned long itemId ) const;
IRectangle
  itemRect      ( unsigned long itemId ) const;
unsigned long
  numberOfItems ( unsigned long forSubmenuId = 0 ) const;

virtual IMenuHandle
  menuHandle    ( ) const;

virtual IMenu
 &setConditionalCascade    ( unsigned long itemWithSubmenuId,
                             unsigned long defaultItemId ),
 &removeConditionalCascade ( unsigned long itemWithSubmenuId );

/*---------------------------------- Color -----------------------------------*/
virtual IColor
  foregroundColor         ( ) const,
  backgroundColor         ( ) const,
  disabledForegroundColor ( ) const,
  disabledBackgroundColor ( ) const,
  hiliteForegroundColor   ( ) const,
  hiliteBackgroundColor   ( ) const;

virtual IMenu
  &setForegroundColor         ( const IColor& color ),
  &setBackgroundColor         ( const IColor& color ),
  &setDisabledForegroundColor ( const IColor& color ),
  &setDisabledBackgroundColor ( const IColor& color ),
  &setHiliteForegroundColor   ( const IColor& color ),
  &setHiliteBackgroundColor   ( const IColor& color );

virtual IMenu
  &resetForegroundColor         ( ),
  &resetBackgroundColor         ( ),
  &resetDisabledForegroundColor ( ),
  &resetDisabledBackgroundColor ( ),
  &resetHiliteForegroundColor   ( ),
  &resetHiliteBackgroundColor   ( );

class Cursor : public IVBase {
public :
/*---------------------------- Constructors -----------------------------*/
  Cursor ( const IMenu& menu,
           unsigned long forSubmenuId = 0 );

virtual
 ~Cursor ( );

/*--------------------------- Cursor Movement ---------------------------*/
virtual Boolean
  setToFirst    ( ),
  setToNext     ( ),
  setToPrevious ( ),
  setToLast     ( );

/*-------------------------- Cursor Validation --------------------------*/
virtual Boolean
  isValid    ( ) const;
virtual void
  invalidate ( );

private:
/*--------------------------- Hidden Members ----------------------------*/
  Cursor ( const Cursor& cursor );

/*------------------------------- Private -------------------------------*/
friend class IMenu;

long
  lCurrent;
unsigned long
  ulSubmenu;
IMenu
 *pMenu;

IMenuCursorData
 *fCursorData;
}; // IMenu::Cursor

/*--------------------------- Cursored Operations ----------------------------*/
virtual IMenuItem
  elementAt       ( const Cursor& cursor ) const;

virtual IMenu
 &deleteAt        ( Cursor&       cursor ),
 &removeSubmenuAt ( Cursor&       cursor ),

 &add             ( IMenuItem&    menuItem,
                    Cursor&       cursor ),
 &addAsNext       ( IMenuItem&    menuItem,
                    Cursor&       cursor );

Cursor
  cursor          ( unsigned long itemId,
                    unsigned long inSubmenuId = 0 ) const;

/*-------------------------- Observer Notification ---------------------------*/
virtual IMenu
 &enableNotification ( Boolean enable = true );

protected:
/*------------------------------- Constructors -------------------------------*/
  IMenu ( );

private:
/*------------------------------ Hidden Members ------------------------------*/
  IMenu      ( const IMenu& menu );
IMenu
 &operator = ( const IMenu& menu );

/*--------------------------------- Private ----------------------------------*/

unsigned long
  menuItemId     ( const Cursor&        cursor ) const;
void
  refCountBitmap ( const IBitmapHandle& handle );
IMenuHandle
  submenuHandle  ( unsigned long        submenuId ) const;

static Style
  currentDefaultStyle;
void
 *pBitmapList;

IMenuData
 *fMenuData;

/*----------------------------- Obsolete Members -----------------------------*/
#if (IC_OBSOLETE <= IC_OBSOLETE_1)
public:
enum ColorArea {
  foreground,
  background,
  disabledForeground,
  disabledBackground,
  highlightForeground,
  highlightBackground,
  border };
IMenu
 &setColor        ( ColorArea     value,
                    const IColor& color );
IColor
  color           ( ColorArea     value ) const;
static const Style
  verticalFlip;
IMenu
 &setVerticalFlip ( Boolean       flip = true );

Boolean
  isVerticalFlip  ( ) const,
  isItemDisabled  ( unsigned long itemId ) const;
virtual IMenu
 &addAt           ( IMenuItem&    menuItem,
                    Cursor&       cursor   ),
 &addNextAt       ( IMenuItem&    menuItem,
                    Cursor&       cursor   );
#endif // IC_OBSOLETE
}; // IMenu

INESTEDBITFLAGCLASSFUNCS(Style, IMenu);

#pragma pack()

#ifndef I_NO_INLINES
  #include <imenu.inl>
#endif

#endif  /* _IMENU_ */
