#ifndef _IMSGBOX_
#define _IMSGBOX_
/*******************************************************************************
* FILE NAME: imsgbox.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IMessageBox                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ivbase.hpp>
#include <ibitflag.hpp>
#include <istring.hpp>

class IErrorInfo;
class IException;
class IMessageBoxData;
class IResourceId;
class IWindow;

#pragma pack(4)

class IMessageBox : public IVBase {
typedef IVBase
  Inherited;
public:
/*------------------------------- Constructors -------------------------------*/
  IMessageBox ( IWindow* owner );

virtual
 ~IMessageBox ( );

/*---------------------------------- Styles ----------------------------------*/
INESTEDBITFLAGCLASSDEF0(Style, IMessageBox);
static const Style
  okButton,
  okCancelButton,
#ifndef IC_WIN_FLAGNOP
  cancelButton,
  enterButton,
  enterCancelButton,
#endif
  retryCancelButton,
  abortRetryIgnoreButton,
  yesNoButton,
  yesNoCancelButton,
  defButton1,
  defButton2,
  defButton3,
  noIcon,
  informationIcon,
  queryIcon,
  warningIcon,
  errorIcon,
  applicationModal,
  systemModal
#ifndef IC_WIN_FLAGNOP
  ,
  moveable
#endif
  ;

/*-------------------------------- Title Text --------------------------------*/
virtual IMessageBox
 &setTitle( const char*        title ),
 &setTitle( const IResourceId& title );

/*------------------------- Showing the Message Box --------------------------*/
enum Response {
#ifndef IC_WIN_FLAGNOP
  enter,
#endif
  ok,
  cancel,
  abort,
  retry,
  ignore,
  yes,
  no,
  unknown
  };

enum Severity {
  information,
  warning,
  action,
  critical,
  catastrophic
  };

virtual IMessageBox::Response
  show ( const char*        message,
         Severity           severity,
         unsigned long      helpId = 0 ),

  show ( const IResourceId& message,
         Severity           severity,
         unsigned long      helpId = 0 ),

  show ( const IResourceId& message,
         const Style&       style,
         unsigned long      helpId = 0 ),

  show ( const char*        message,
         const Style&       style,
         unsigned long      helpId = 0 ),

  show ( const IException&  exception,
         unsigned long      helpId = 0 ),

  show ( const IErrorInfo&  error,
         unsigned long      helpId = 0 );

private:
/*------------------------------ Hidden Members ------------------------------*/
  IMessageBox ( const IMessageBox& messageBox );
IMessageBox
 &operator=   ( const IMessageBox& messageBox );

/*--------------------------------- Private ----------------------------------*/
unsigned long
  showMessageBox  ( const char*   message,
                    const char*   title,
                    unsigned long helpPanelId,
                    const Style&  style );
static Style
  convertSeverity ( Severity      severity );
static IMessageBox::Response
  convertResponse ( unsigned long buttonId );
IString
  strClTitle;
IWindow
 *pwndClOwner;

IMessageBoxData
 *fMessageBoxData;
}; // IMessageBox

#pragma pack()

#endif  /* _IMSGBOX_  */
