#ifndef _IOUTLBOX_
  #define _IOUTLBOX_
/*******************************************************************************
* FILE NAME: ioutlbox.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IOutlineBox                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <icontrol.hpp>
#include <irect.hpp>

class ISize;
class IWindowHandle;
class IColor;
class IPoint;
class IOutlineBoxData;

#pragma pack(4)

class IOutlineBox : public IControl  {
typedef IControl
  Inherited;
public:
class Style;
/*------------------------ Constructors --------------------------------------*/
  IOutlineBox(unsigned long     id,
              IWindow*          parent,
              IWindow*          owner,
              const IRectangle& initial= IRectangle(),
              const Style&      style = defaultStyle() );

  IOutlineBox(unsigned long id,
              IWindow*      parent );

  IOutlineBox(const IWindowHandle& handle);

virtual
  ~IOutlineBox ( );

/*---------------------------------- Styles ----------------------------------*/
INESTEDBITFLAGCLASSDEF1(Style, IOutlineBox, IWindow);
                                  // style class definition
static const Style
  classDefaultStyle,
  backgroundFrame,
  foregroundFrame;

static const Style
  halftoneFrame;

static Style
  defaultStyle      ( );

static void
  setDefaultStyle   ( const Style&    style );

virtual unsigned long
  convertToGUIStyle ( const IBitFlag& style,
                      Boolean         extendedOnly = false ) const;

/*------------------------- Outline Types ------------------------------------*/
enum OutlineType {
  foreground,
  background,
  halftone };

IOutlineBox&
  setOutlineType(const OutlineType& type);
OutlineType
  outlineType() const;

/*----------------------------- Color Functions ------------------------------*/
virtual IColor
  foregroundColor     ( ) const;



/*------------------------------ Canvas Support ------------------------------*/
virtual IRectangle
  visibleRectangle   ( ) const;

private:
/*--------------------------------- Private ----------------------------------*/
static Style
  currentDefaultStyle;

/*----------------------------Hidden Functions -------------------------------*/
  IOutlineBox(const IOutlineBox&);
  IOutlineBox& operator=(const IOutlineBox&);

IOutlineBoxData
 *fOutlineBoxData;

public:
#if (IC_OBSOLETE <= IC_OBSOLETE_1)
/*---------------------- Obsolete data and Functions -------------------------*/
enum ColorArea {
  fillRegion};
IOutlineBox
  &setColor ( ColorArea     value,
              const IColor& color);
IColor
  color     ( ColorArea     value ) const;
#endif
};  // class IOutlineBox

INESTEDBITFLAGCLASSFUNCS(Style, IOutlineBox);
                                  // global style functions

#pragma pack()

#endif  // _IOUTLBOX_
