/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/


#ifndef _IPARTCCL_H_
#define _IPARTCCL_H_

#include <istdntfy.hpp>
#include <icursor.h>
#include <iaorder.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

template < class Element >
class IPartCollectionEventData {

  ICursor const& c;

public:

           IPartCollectionEventData (ICursor const& aCursor)
           : c(aCursor)
           {
           }

  ICursor const& cursor() const { return c; };

};

template < class Element >
class IPartCollectionAddedEventData : public virtual
      IPartCollectionEventData < Element > {

public:

           IPartCollectionAddedEventData (ICursor const& aCursor)
           : IPartCollectionEventData < Element > (aCursor)
           {
           }

};

template < class Element >
class IPartCollectionRemovedEventData : public virtual
      IPartCollectionEventData < Element > {

public:

           IPartCollectionRemovedEventData (ICursor const& aCursor)
           : IPartCollectionEventData < Element > (aCursor)
           {
           }

};

template < class Element >
class IPartCollectionReplacedEventData : public virtual
      IPartCollectionEventData < Element > {

      Element& e;

public:

        IPartCollectionReplacedEventData (ICursor const& aCursor,
              Element& anElement)
        : IPartCollectionEventData < Element > (aCursor), e(anElement)
        {
        }

  Element& replacedElement() { return e; };

};

template < class Element >
class IPartCollection : public IStandardNotifier {

  typedef IStandardNotifier Inherited;

public:

  inline IPartCollection();

  virtual inline ~IPartCollection ();

protected:

  inline IStandardNotifier& addObserver(IObserver&,const IEventData&);
  inline IStandardNotifier& removeObserver(IObserver&);
  inline IStandardNotifier& removeAllObservers();
  inline IBoolean hasObservers() const;
  inline IBoolean hasObserversOrShouldSendNotification() const;
  inline void disableAllNotifications();
  inline void enableAllNotifications();

  inline void notifyModified();
  inline void notifyAdded(ICursor const&);
  inline void notifyRemoved(ICursor const&);
  inline void notifyReplaced(ICursor const&, Element&);

private:

   unsigned long obsCount;
   unsigned long disableCount;

};

template < class Element >
class IPartOrderedCollection : public virtual IAOrderedCollection < Element >
                      , public IPartCollection < Element > {
public:

  virtual inline ~IPartOrderedCollection ();

};

#include <ipartccl.inl>

#pragma info (restore)
#pragma pack ()

#endif
