/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/


#include <istdntfy.hpp>
#include <inotifev.hpp>
#include <icursor.h>
#include <ipartccn.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)

template < class Element >
IPartOrderedCollection < Element > ::~IPartOrderedCollection()
{
}

template < class Element >
IPartCollection < Element > ::IPartCollection() : obsCount(0) , disableCount(0)
{
}

template < class Element >
IStandardNotifier& IPartCollection < Element >::
addObserver(IObserver& anObserver,const IEventData& userData)
{
   obsCount++;
   return Inherited::addObserver(anObserver, userData);
}

template < class Element >
IStandardNotifier& IPartCollection < Element >::
removeObserver(IObserver& anObserver)
{
   obsCount--;
   return Inherited::removeObserver(anObserver);
}

template < class Element >
IStandardNotifier& IPartCollection < Element >::
removeAllObservers()
{
   obsCount = 0;
   return Inherited::removeAllObservers();
}

template < class Element >
IBoolean IPartCollection < Element >::
hasObservers() const
{
   return obsCount != 0;
}

template < class Element >
void IPartCollection < Element >::
disableAllNotifications()
{
   disableCount++;
}

template < class Element >
void IPartCollection < Element >::
enableAllNotifications()
{
   disableCount--;
}

template < class Element >
IBoolean IPartCollection < Element >::
hasObserversOrShouldSendNotification() const
{
   return !((obsCount == 0) || (disableCount > 0));
}

template < class Element >
IPartCollection < Element >::~IPartCollection()
{
}

template < class Element >
void IPartCollection < Element > :: notifyModified()
{
   INotificationEvent
      event(IPartCollectionNotification::modifiedId,*this,true);
   notifyObservers (event);
}

template < class Element >
void IPartCollection < Element > ::
   notifyAdded (ICursor const& cursor)
{
   IPartCollectionAddedEventData <Element> eventData(cursor);
   INotificationEvent
      event(IPartCollectionNotification::addedId,*this,true,
         (void*)&eventData);
   notifyObservers (event);
}

template < class Element >
void IPartCollection < Element> ::
   notifyRemoved (ICursor const& cursor)
{
   IPartCollectionRemovedEventData <Element> eventData(cursor);
   INotificationEvent
      event(IPartCollectionNotification::removedId,*this,true,
         (void*)&eventData);
   notifyObservers (event);
}

template < class Element >
void IPartCollection < Element> ::
   notifyReplaced (ICursor const& cursor, Element& element)
{
   IPartCollectionReplacedEventData <Element>
      eventData(cursor, element);
   INotificationEvent
      event(IPartCollectionNotification::replacedId,*this,true,
         (void*)&eventData);
   notifyObservers (event);
}

#pragma info (restore)
