#ifndef _IPERSIST_
  #define _IPERSIST_

/*******************************************************************************
* FILE NAME: ipersist.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*    IPersistentObject - Persistent Object Class                               *
*    IPOManager        - Persistent Object Factory Class                       *
*                                                                              *
*******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* IBM(R) VisualAge(TM) C++ for OS/2(R), Version 3                            */
/* (C) Copyright IBM Corp. 1991, 1995.                                        */
/*  - Licensed Material - Program-Property of IBM - All Rights Reserved.      */
/* US Government Users Restricted Rights - Use, duplication or disclosure     */
/* restricted by GSA ADP Schedule Contract with IBM Corp.                     */
/*                                                                            */
/******************************************************************************/

#ifndef _ISTRING_
  #include <istring.hpp>
#endif

#ifndef _IDSEXC_HPP
  #include <idsexc.hpp>
#endif

#ifndef _ISTDNTFY_
  #include <istdntfy.hpp>
#endif

#include <ivseq.h>

/*----------------------------------------------------------------------------*/
/* Align classes on four byte boundary.                                       */
/*----------------------------------------------------------------------------*/
#pragma pack(4)

/******************************************************************************/
/* Class: IPersistentObject (Abstract)                                        */
/* Description: This abstract class defines the standard interface for a data */
/*              access class. This class provides data acess to a row of      */
/*              a table.                                                      */
/******************************************************************************/
class _Export IPersistentObject : public IStandardNotifier
{
public:

/*----------------------------------------------------------------------------*/
/* Constructor and destructor                                                 */
/*----------------------------------------------------------------------------*/
  IPersistentObject ();
  IPersistentObject (const IPersistentObject& partCopy);
  virtual ~IPersistentObject ();

/*----------------------------------------------------------------------------*/
/* Operator functions                                                         */
/*----------------------------------------------------------------------------*/
  IPersistentObject& operator= (const IPersistentObject& aIPersistentObject);

/*----------------------------------------------------------------------------*/
/* Data Access functions: add, update, del, and retrieve.                     */
/* Add:      This function adds a row to a table.                             */
/* update:   This function updates a row.                                     */
/* del:      This function deletes a row.                                     */
/* retrieve: This function retrieves a row.                                   */
/*----------------------------------------------------------------------------*/
  virtual IPersistentObject &add () = 0;
  virtual IPersistentObject &update() = 0;
  virtual IPersistentObject &del() = 0;
  virtual IPersistentObject &retrieve () = 0;

/*----------------------------------------------------------------------------*/
/* Functions for read only support:                                           */
/* isReadOnly():        returns true if the object is set to read only.       */
/* isDefaultReadOnly(): returns true if the object by default is read only.   */
/* setReadOnly(Boolean flag=true): if flag is true, the object is read only.  */
/*                       If flag is false, the object is read write.          */
/*----------------------------------------------------------------------------*/
  virtual Boolean isReadOnly();
  virtual Boolean isDefaultReadOnly();
  virtual IPersistentObject& setReadOnly(Boolean flag = true);

  static INotificationId currentReadOnlyId;
protected:
/*----------------------------------------------------------------------------*/
/* Protected data members for read-only support. The currentReadOnly and      */
/* the defaultReadOnly field should be initialized to the same value.         */
/*----------------------------------------------------------------------------*/
  const Boolean defaultReadOnly;// true if the object is read-only by default.
  Boolean currentReadOnly;      // true if the set to read-only.
  IPersistentObject(Boolean defReadOnly);
private:
};


/*----------------------------------------------------------------------------*/
/* Class: IPOManager (Abstract)                                               */
/* Description: This abstract class defines the standard interface for a data */
/*              access class. This class provides data acess to a collection  */
/*              of rows from a table.                                         */
/*----------------------------------------------------------------------------*/
class _Export IPOManager: public IStandardNotifier
{
public:
/*----------------------------------------------------------------------------*/
/* Constructor and destructor                                                 */
/*----------------------------------------------------------------------------*/
  IPOManager();
  IPOManager(const IPOManager& partCopy);
  virtual ~IPOManager();

/*----------------------------------------------------------------------------*/
/* Operator functions                                                         */
/*----------------------------------------------------------------------------*/
  IPOManager& operator= (const IPOManager& aIPOManager);

/*----------------------------------------------------------------------------*/
/* Data Access functions: refresh and select.                                 */
/* refresh:  This function retrieves all the rows from a table.               */
/* select:   This function retrieves selected rows from a table based on      */
/*           the conditions described in the clause string.                   */
/* items:    This function returns an IVSequence<concrete_class*>* that is    */
/*           used to access data retrieved by the refresh/select calls        */
/*----------------------------------------------------------------------------*/
  virtual IPOManager &refresh () = 0;
  virtual IPOManager &select (const IString& clause) = 0;

protected:

private:
};

/*----------------------------------------------------------------------------*/
/* Resume compiler default packing.                                           */
/*----------------------------------------------------------------------------*/
#pragma pack()

#endif
