/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IPQUKSB_H
#define _IPQUKSB_H

#include <iglobals.h>
#include <icursor.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class Key, class ElementOps, class Base >
class IWPQueOnKSBag;

template < class Element, class Key, class ElementOps, class Base >
class IWPQueOnKSBagCursor :
  public Base::Cursor
{
public:

           IWPQueOnKSBagCursor
                     (IWPQueOnKSBag
                       < Element, Key, ElementOps, Base > const& c)
           : Base::Cursor (c.ivBase)
           {
           }

};

template < class Element, class Key, class ElementOps, class Base >
class IWPQueOnKSBag {

  Base ivBase;

public:


  typedef IWPQueOnKSBagCursor < Element, Key, ElementOps, Base > Cursor;

  friend class IWPQueOnKSBagCursor < Element, Key, ElementOps, Base >;

  inline   IWPQueOnKSBag    (INumber numberOfElements = 100);

  inline   IWPQueOnKSBag    (IWPQueOnKSBag
                              < Element, Key,
                                ElementOps, Base > const&);

  inline  ~IWPQueOnKSBag    ();

  inline
  IBoolean add              (Element const&);

  inline
  IBoolean add              (Element const&, ICursor&);

  inline
  void     addAllFrom       (IWPQueOnKSBag
                              < Element, Key,
                                ElementOps, Base > const&);

  inline
  IBoolean allElementsDo    (IBoolean (*function)
                              (Element const&, void*),
                             void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo    (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement       () const;

  inline
  long     compare          (IWPQueOnKSBag
                              < Element, Key, ElementOps, Base > const&,
                             long (*comparisonFunction)
                              (Element const&, Element const&)) const;

  inline
  IBoolean containsElementWithKey (Key const&) const;

  inline
  IBoolean containsAllKeysFrom
                             (IWPQueOnKSBag
                               < Element, Key,
                                 ElementOps, Base > const&) const;

  inline
  void     dequeue          ();

  inline
  void     dequeue          (Element&);

  inline
  Element const&
           elementAt        (ICursor const&) const;

  inline
  Element const&
           elementAtPosition (IPosition) const;

  inline
  Element const&
           elementWithKey   (Key const&) const;

  inline
  void     enqueue          (Element const&);

  inline
  void     enqueue          (Element const&, ICursor&);

  inline
  Element const&
           firstElement     () const;

  inline
  void     removeAll        ();

  inline
  IBoolean isBounded        () const;

  inline
  IBoolean isEmpty          () const;

  inline
  IBoolean isFirst          (ICursor const&) const;

  inline
  IBoolean isFull           () const;

  inline
  IBoolean isLast           (ICursor const&) const;

  inline
  Key const&
           key              (Element const&) const;

  inline
  Element const&
           lastElement      () const;

  inline
  IBoolean locateElementWithKey (Key const&, ICursor&) const;

  inline
  IBoolean locateOrAddElementWithKey (Element const&);

  inline
  IBoolean locateOrAddElementWithKey (Element const&, ICursor&);

  inline
  IBoolean locateNextElementWithKey (Key const&, ICursor&) const;

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor        () const;

  inline
  INumber  numberOfDifferentKeys () const;

  inline
  INumber  numberOfElements () const;

  inline
  INumber  numberOfElementsWithKey (Key const&) const;

  inline
  IWPQueOnKSBag < Element, Key, ElementOps, Base >&
           operator =       (IWPQueOnKSBag
                              < Element, Key,
                                ElementOps, Base > const&);

  inline
  void     removeFirst      ();

  inline
  IBoolean setToFirst       (ICursor&) const;

  inline
  IBoolean setToLast        (ICursor&) const;

  inline
  IBoolean setToNext        (ICursor&) const;

  inline
  IBoolean setToNextWithDifferentKey (ICursor&) const;

  inline
  void     setToPosition    (IPosition, ICursor&) const;

#ifndef __BIN_COMPAT_BASE_43__
  inline
  IPosition     position    (ICursor const&) const;
#endif

  inline
  IBoolean setToPrevious    (ICursor&) const;

  inline
  IBoolean isConsistent     () const;

protected:

  // used for cfront support:
  inline
  IOrderedCursor < Element >&
            baseCursor (ICursor&) const;

  inline
  IOrderedCursor < Element > const&
            baseCursor (ICursor const&) const;

};


#define IDefineGPriorityQueueOnGKeySortedBag(GKSBag, GPrioQue) \
template < class Element, class Key, class ElementOps > \
class GPrioQue : \
  public IWPQueOnKSBag \
           < Element, Key, \
             ElementOps, GKSBag < Element, Key, ElementOps > > { \
public: \
  GPrioQue (INumber n = 100) : \
    IWPQueOnKSBag < Element, Key, ElementOps, \
                    GKSBag < Element, Key, ElementOps > > (n) {} \
};



#ifdef ICL_NO_INLINE
#undef inline
#else
#include <ipquksb.inl>
#endif

#ifndef __TEMPINC__
#include <ipquksb.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
