/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IPTR_H
#define _IPTR_H

#include <iglobals.h>
#include <istdops.h>
#include <iinit.h>
#include <icompat.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICLCC_COMPAT_PTR
#define IPtr IElemPointer
#define IMgPtr IMngElemPointer
#endif

template < class Element >
class IElemPointer {
public:

           IElemPointer (Element* ptr, IExplicitInit = IINIT)
           : ivPtr (ptr)
           {
           }

           IElemPointer ()
           : ivPtr (0)
           {
           }

  Element& operator * () const
           { return *ivPtr;
           }

  Element* operator-> () const
           { return ivPtr;
           }

           operator Element* () const
           { return ivPtr;
           }

  friend inline
  Element& elementForOps (IElemPointer < Element >& ptr)
           { return *ptr.ivPtr;
           }

  friend inline
  Element const&
           elementForOps (IElemPointer < Element > const& ptr)
           { return *ptr.ivPtr;
           }

protected:

  Element *ivPtr;

};


template < class Element >
class IMngPointer {

  struct PtrRc;

public:

           IMngPointer (Element *ptr, IExplicitInit)
           : ivPtrRc (new PtrRc (ptr))
           {
           }

           IMngPointer ()
           : ivPtrRc (0)
           {
           }

           IMngPointer (IMngPointer < Element > const& ptr)
           { ivPtrRc = ptr.ivPtrRc;
             if (ivPtrRc != 0) ivPtrRc->ivRc++;
           }

          ~IMngPointer ()
           { if (ivPtrRc != 0 && --ivPtrRc->ivRc == 0) delete ivPtrRc;
           }

  IMngPointer < Element >&
           operator= (IMngPointer < Element > const& ptr)
           { if (ivPtrRc != ptr.ivPtrRc) {
               if (ivPtrRc && --ivPtrRc->ivRc == 0) delete ivPtrRc;
               ivPtrRc = ptr.ivPtrRc;
               if (ivPtrRc != 0) ivPtrRc->ivRc++;
             }
             return *this;
           }

  Element& operator * () const
           { return *ivPtrRc->ivPtr;
           }

  Element* operator-> () const
           { return ivPtrRc->ivPtr;
           }

           operator Element* () const
           { if (ivPtrRc == 0) return 0;
             else return ivPtrRc->ivPtr;
           }

protected:

  struct PtrRc
  { Element *ivPtr;

    unsigned int ivRc;

             PtrRc (Element *ptr)
             : ivPtr (ptr), ivRc (1)
             {
             }

            ~PtrRc ()
             { delete ivPtr; }
  } *ivPtrRc;

};


template < class Element >
class IMngElemPointer : private IMngPointer < Element > {
public:

           IMngElemPointer (Element *ptr, IExplicitInit)
           : IMngPointer < Element > (ptr, IINIT)
           {
           }

           IMngElemPointer ()
           {
           }

  Element& operator * () const
           { return IMngPointer < Element >::operator*();
           }

  Element* operator-> () const
           { return IMngPointer < Element >::operator->();
           }

           operator Element* () const
           { return (IMngPointer < Element > const&)*this;
           }

  friend inline
  Element& elementForOps (IMngElemPointer < Element >& ptr)
           { return *(Element*)ptr;
           }

  friend inline
  Element const&
           elementForOps (IMngElemPointer < Element > const& ptr)
           { return *(Element*)ptr;
           }

#ifdef ICLCC_COMPAT_PTR
           IMngElemPointer (Element const& e)
           : IMngPointer < Element > (new Element (e))
           {
           }

           IMngElemPointer (Element *ptr)
           : IMngPointer < Element > (ptr)
           {
           }
#endif

};


template < class Element >
class IAutoPointer {
public:

           IAutoPointer (Element *ptr, IExplicitInit)
           : ivPtr (ptr)
           {
           }

           IAutoPointer ()
           : ivPtr (0)
           {
           }

           IAutoPointer (IAutoPointer < Element > const& ptr)
           { ivPtr = ptr.ivPtr;
             ((IAutoPointer < Element >&) ptr).ivPtr = 0;
           }

          ~IAutoPointer ()
           { delete ivPtr;
           }

  void     operator = (IAutoPointer < Element > const& ptr)
           { delete ivPtr;
             ivPtr = ptr.ivPtr;
             ((IAutoPointer < Element >&) ptr).ivPtr = 0;
           }

           operator Element* () const
           { return ivPtr;
           }

  Element& operator * () const
           { return *ivPtr;
           }

  Element* operator-> () const
           { return ivPtr;
           }

protected:

  Element* ivPtr;

};


template < class Element >
class IAutoElemPointer : private IAutoPointer < Element > {
public:

           IAutoElemPointer (Element *ptr, IExplicitInit)
           : IAutoPointer < Element > (ptr, IINIT)
           {
           }

           IAutoElemPointer ()
           {
           }

  Element& operator * () const
           { return IAutoPointer < Element >::operator*();
           }

  Element* operator-> () const
           { return IAutoPointer < Element >::operator->();
           }

           operator Element* () const
           { return IAutoPointer < Element >::operator Element*();
           }

  friend inline
  Element& elementForOps (IAutoElemPointer < Element >& ptr)
           { return *(Element*)ptr;
           }

  friend inline
  Element const&
           elementForOps (IAutoElemPointer < Element > const& ptr)
           { return *(Element*)ptr;
           }

};



#pragma info (restore)
#pragma pack ()

#endif
