/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IQUESEQ_H
#define _IQUESEQ_H

#include <iglobals.h>
#include <icursor.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class ElementOps, class Base >
class IWQueueOnSeq;

template < class Element, class ElementOps, class Base >
class IWQueueOnSeqCursor :
  public Base::Cursor
{
public:

           IWQueueOnSeqCursor
                     (IWQueueOnSeq
                       < Element, ElementOps, Base > const& c)
           : Base::Cursor (c.ivBase)
           {
           }

};

template < class Element, class ElementOps, class Base >
class IWQueueOnSeq {
protected:

  Base ivBase;

public:


  typedef IWQueueOnSeqCursor < Element, ElementOps, Base > Cursor;

  inline   IWQueueOnSeq     (INumber numberOfElements = 100);

  inline   IWQueueOnSeq     (IWQueueOnSeq
                              < Element, ElementOps, Base > const&);

  inline  ~IWQueueOnSeq             ();

  inline
  IBoolean add              (Element const&);

  inline
  IBoolean add              (Element const&, ICursor&);

  inline
  void     addAllFrom       (IWQueueOnSeq
                              < Element, ElementOps, Base > const&);

  inline
  void     addAsLast        (Element const&);

  inline
  void     addAsLast        (Element const&, ICursor&);

  inline
  IBoolean allElementsDo    (IBoolean (*function)
                              (Element const&, void*),
                             void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo    (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement       () const;

  inline
  long     compare          (IWQueueOnSeq
                              < Element, ElementOps, Base > const&,
                             long (*comparisonFunction)
                              (Element const&, Element const&)) const;

  inline
  void     dequeue          ();

  inline
  void     dequeue          (Element&);

  inline
  Element const&
           elementAt        (ICursor const&) const;

  inline
  Element const&
           elementAtPosition (IPosition) const;

  inline
  void     enqueue          (Element const&);

  inline
  void     enqueue          (Element const&, ICursor&);

  inline
  Element const&
           firstElement     () const;

  inline
  IBoolean isBounded        () const;

  inline
  IBoolean isEmpty          () const;

  inline
  IBoolean isFirst          (ICursor const&) const;

  inline
  IBoolean isLast           (ICursor const&) const;

  inline
  IBoolean isFull           () const;

  inline
  Element const&
           lastElement      () const;

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor        () const;

  inline
  INumber  numberOfElements () const;

  inline
  IWQueueOnSeq < Element, ElementOps, Base >&
           operator =       (IWQueueOnSeq
                              < Element, ElementOps, Base > const&);

  inline
  void     removeAll        ();

  inline
  void     removeFirst      ();

  inline
  IBoolean setToFirst       (ICursor&) const;

  inline
  IBoolean setToLast        (ICursor&) const;

  inline
  IBoolean setToNext        (ICursor&) const;

  inline
  void     setToPosition    (IPosition, ICursor&) const;

#ifndef __BIN_COMPAT_BASE_43__
  inline
  IPosition     position    (ICursor const&) const;
#endif

  inline
  IBoolean setToPrevious    (ICursor&) const;

  friend class IWQueueOnSeqCursor < Element, ElementOps, Base >;

  inline
  IBoolean isConsistent     () const;

protected:

  // used for cfront support:
  inline
  IOrderedCursor < Element >&
            baseCursor (ICursor&) const;

  inline
  IOrderedCursor < Element > const&
            baseCursor (ICursor const&) const;

};


#define IDefineGQueueOnGSequence(GSeq, GQueue) \
template < class Element, class ElementOps > \
class GQueue : \
  public IWQueueOnSeq \
           < Element, ElementOps, GSeq < Element, ElementOps > > { \
public: \
  GQueue (INumber n = 100) : \
    IWQueueOnSeq < Element, ElementOps, \
                   GSeq < Element, ElementOps > > (n) {} \
};



#ifdef ICL_NO_INLINE
#undef inline
#else
#include <iqueseq.inl>
#endif

#ifndef __TEMPINC__
#include <iqueseq.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
