/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <iptr.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)

template < class Element, class Base >
void* IRBag < Element, Base >::
identity () const
{ return (void*)&ivBase;
}

template < class Element, class Base >
IRBag < Element, Base >::
~IRBag ()
{
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
add (Element const& element)
{
   if (hasObserversOrShouldSendNotification()) {
      return addNotify(element);
      }
   else
      return ivBase.add (element);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
add (Element const& element, ICursor &cursor)
{
if (hasObserversOrShouldSendNotification()) {
      IBoolean result = ivBase.add(element, cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }
   else
      return ivBase.add (element, cursor);
}

template < class Element, class Base >
Element const& IRBag < Element, Base >::
elementAt (ICursor const& cursor) const
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element& IRBag < Element, Base >::
elementAt (ICursor const& cursor)
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element const& IRBag < Element, Base >::
anyElement () const
{ return ivBase.anyElement ();
}

template <class Element , class Base >
void IRBag < Element , Base >::
addAllFrom (IACollection < Element > const& collection)
{ ICHECK (! isIdentical (collection),
          IIdenticalCollectionException, IIdenticalCollectionText)
   if (hasObserversOrShouldSendNotification()) {
      addAllFromNotify(collection);
      }
   else
      Inherited::addAllFrom(collection);
}

template <class Element , class Base >
void IRBag < Element , Base >::
copy (IACollection < Element > const& collection)
{ if (hasObserversOrShouldSendNotification()) {
     if (! isIdentical (collection)) {
       ivBase.removeAll ();
       IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
       forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
        }
       notifyModified();
     }
  }
  else
    Inherited::copy(collection);
}

template < class Element, class Base >
void IRBag < Element, Base >::
removeAt (ICursor& cursor)
{
   if (hasObserversOrShouldSendNotification()) {
      notifyRemoved(cursor);
      };
ivBase.removeAt (cursor);
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{
   if (hasObserversOrShouldSendNotification()) {
      INumber result =ivBase.removeAll (predicate, env);
      if (result > 0)
         notifyModified();
      return result;
      }
   else
      return ivBase.removeAll (predicate, env);
}

template < class Element, class Base >
void IRBag < Element, Base >::
replaceAt (ICursor const& cursor, Element const& element)
{
   if (hasObserversOrShouldSendNotification()) {
      Element oldElement = ivBase.elementAt(cursor);
      ivBase.replaceAt (cursor, element);
      notifyReplaced(cursor,oldElement);
      }
   else
      ivBase.replaceAt (cursor, element);
}

template < class Element, class Base >
void IRBag < Element, Base >::
removeAll ()
{ ivBase.removeAll ();
   if (hasObserversOrShouldSendNotification()) {
      notifyModified();
      };
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
isBounded () const
{ return ivBase.isBounded ();
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
maxNumberOfElements () const
{ return ivBase.maxNumberOfElements ();
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
numberOfElements () const
{ return ivBase.numberOfElements ();
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
isEmpty () const
{ return ivBase.isEmpty ();
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
isFull () const
{ return ivBase.isFull ();
}

template < class Element, class Base >
ICursor* IRBag < Element, Base >::
newCursor () const
{ return ivBase.newCursor ();
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
setToFirst (ICursor &cursor) const
{ return ivBase.setToFirst (cursor);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
setToNext (ICursor &cursor) const
{ return ivBase.setToNext (cursor);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element &, void*),
               void* environment)
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
allElementsDo (IIterator <Element>& iterator)
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* environment) const
{ return ((Base const&)ivBase).allElementsDo
           (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
isConsistent () const
{ return ivBase.isConsistent ();
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
contains (Element const& element) const
{ return ivBase.contains (element);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
locate (Element const& element, ICursor &cursor) const
{ return ivBase.locate (element, cursor);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
locateOrAdd (Element const& element)
{
   if (hasObserversOrShouldSendNotification()) {
      return locateOrAddNotify(element);
      }
   else
      return ivBase.locateOrAdd (element);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
locateOrAdd (Element const& element, ICursor &cursor)
{
   if (hasObserversOrShouldSendNotification()) {
      IBoolean result = ivBase.locateOrAdd(element,cursor);
      if (!result) {
         notifyAdded(cursor);
         }
      return result;
      }
   else
      return ivBase.locateOrAdd (element, cursor);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
remove (Element const& element)
{
   if (hasObserversOrShouldSendNotification()) {
      removeNotify(element);
      }
   return ivBase.remove (element);
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
numberOfOccurrences (Element const& element) const
{ return ivBase.numberOfOccurrences (element);
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
locateNext (Element const& element, ICursor& cursor) const
{ return ivBase.locateNext (element, cursor);
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
removeAllOccurrences (Element const& element)
{
   if (hasObserversOrShouldSendNotification()) {
      INumber number = ivBase.removeAllOccurrences (element);
      if (number>0)
         notifyModified();
      return number;
      }
   else
      return ivBase.removeAllOccurrences (element);
}

template < class Element, class Base >
INumber IRBag < Element, Base >::
numberOfDifferentElements () const
{ return ivBase.numberOfDifferentElements ();
}

template < class Element, class Base >
IBoolean IRBag < Element, Base >::
setToNextDifferentElement (ICursor& cursor) const
{ return ivBase.setToNextDifferentElement (cursor);
}

template < class Element, class Base >
void IRBag < Element, Base >::
unionWith (IABag < Element > const& collection)
{
disableAllNotifications();
if (isIdentical (collection)) {
    ivBase.unionWith (ivBase);
    }
  else
    addAllFrom (collection);
enableAllNotifications();
if (hasObserversOrShouldSendNotification())
   notifyModified();
}

template < class Element, class Base >
void IRBag < Element, Base >::
intersectionWith (IABag < Element > const& collection)
{
disableAllNotifications();
if (! isIdentical (collection))
  { IAEqualityCollection < Element > const* env = &collection;
    removeAll (IAEqualityCollection < Element >::isNotContained, &env);

    IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
    for (cursor->setToFirst ();
         cursor->isValid ();
         collection.setToNextDifferentElement (*cursor)) {
      Element const& e = collection.elementAt (*cursor);
      INumber n = numberOfOccurrences (e);
      if (n > 0) {
        INumber n1 = collection.numberOfOccurrences (e);
        for (; n > n1; n--)
          remove (e);
      }
    }
  }
enableAllNotifications();
if (hasObserversOrShouldSendNotification())
   notifyModified();
}

template < class Element, class Base >
void IRBag < Element, Base >::
differenceWith (IABag < Element > const& collection)
{
disableAllNotifications();
if (! isIdentical (collection)) {
    IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
    forCursor (*cursor)
      remove (collection.elementAt (*cursor));
  }
  else
    removeAll ();
enableAllNotifications();
if (hasObserversOrShouldSendNotification())
  notifyModified();
}

template < class Element, class Base >
void IRBag < Element, Base >::
addUnion (IABag < Element > const& collection1,
          IABag < Element > const& collection2)
{
disableAllNotifications();
if (isIdentical (collection1)) {
    if (isIdentical (collection2))
      ivBase.addUnion (ivBase, ivBase);
    else {
      unionWith (collection1);
      unionWith (collection2);
    }
  }
  else {
    unionWith (collection2);
    unionWith (collection1);
  }
enableAllNotifications();
if (hasObserversOrShouldSendNotification())
  notifyModified();
}

template < class Element, class Base >
void IRBag < Element, Base >::
addIntersection (IABag < Element > const& collection1,
                 IABag < Element > const& collection2)
{
disableAllNotifications();
if (isIdentical (collection1)) {
    if (isIdentical (collection2))
      ivBase.unionWith (ivBase);
    else
      addIntersection (collection2, collection1);
  }
  else {
    IAutoPointer < ICursor > cursor2 (collection2.newCursor (), IINIT);
    for (cursor2->setToFirst ();
         cursor2->isValid ();
         collection2.setToNextDifferentElement (*cursor2)) {
      Element const& e = collection2.elementAt (*cursor2);
      INumber n1, n2;
      n1 = collection1.numberOfOccurrences (e);
      n2 = collection2.numberOfOccurrences (e);
      INumber n = (n1 < n2 ? n1 : n2);
      for ( ; n > 0 ; n--)
        add (e);
    }
  }
enableAllNotifications();
if (hasObserversOrShouldSendNotification())
  notifyModified();
}

template < class Element, class Base >
void IRBag < Element, Base >::
addDifference (IABag < Element > const& collection1,
               IABag < Element > const& collection2)
{ if (isIdentical (collection1)) {
    if (isIdentical (collection2))
      return;
    else {
      disableAllNotifications();
      ivBase.unionWith (ivBase);
      IAutoPointer < ICursor >
        cursor2 (collection2.newCursor (), IINIT);
      for (cursor2->setToFirst ();
           cursor2->isValid ();
           collection2.setToNextDifferentElement (*cursor2)) {
        Element const& e = collection2.elementAt (*cursor2);
        INumber n1, n2;
        n1 = numberOfOccurrences (e) / 2;
        n2 = collection2.numberOfOccurrences (e);
        INumber n = (n1 < n2 ? n1 : n2);
        for ( ; n > 0 ; n--)
          remove (e);
      }
    }
  }
  else {
    disableAllNotifications();
    IAutoPointer < ICursor > cursor1 (collection1.newCursor (), IINIT);
    for (cursor1->setToFirst ();
         cursor1->isValid ();
         collection1.setToNextDifferentElement (*cursor1)) {
      Element const& e = collection1.elementAt (*cursor1);
      INumber n1, n2;
      n1 = collection1.numberOfOccurrences (e);
      n2 = collection2.numberOfOccurrences (e);
      INumber n = (n1 < n2 ? 0 : n1 - n2);
      for ( ; n > 0 ; n--)
        add (e);
    }
  }
enableAllNotifications();
if (hasObserversOrShouldSendNotification())
  notifyModified();
}

#pragma info (restore)
