/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IREQSEQ_H
#define _IREQSEQ_H

#include <iaeqseq.h>
#include <istdntfy.hpp>
#include <ipartccl.h>
#include <iptr.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

template < class Element, class Base >
class IREqualitySequence :
  public virtual IAEqualitySequence < Element >
                 ,  public IPartOrderedCollection < Element >
                                          {

  typedef IAEqualitySequence < Element > Inherited;

  Base &ivBase;
public:

           IREqualitySequence   (Base& base)
           : ivBase (base)
           {
           }

          ~IREqualitySequence   ();

private:
  inline IBoolean locateOrAddNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.locateOrAdd(element,cursor);
      if (!result) {
         notifyAdded(cursor);
         }
      return result;
      }

  inline IBoolean addNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.add(element,cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }

  inline void addAllFromNotify
     (IACollection <Element> const& collection) {
      IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
      forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
         }
      notifyModified();
      }

  inline IBoolean removeNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.locate(element,cursor);
      if (result) {
         notifyRemoved(cursor);
         }
      return result;
      }

  inline void removeFirstNotify     () {
      if (ivBase.numberOfElements() > 0) {
         Base::Cursor cursor(ivBase);
         ivBase.setToFirst(cursor);
         notifyRemoved(cursor);
         }
      }

  inline void removeLastNotify      () {
      if (ivBase.numberOfElements() > 0) {
         Base::Cursor cursor(ivBase);
         ivBase.setToLast(cursor);
         notifyRemoved(cursor);
         }
      }

  inline void removeAtPositionNotify (IPosition position) {
      Base::Cursor cursor(ivBase);
      ivBase.setToPosition(position,cursor);
      notifyRemoved(cursor);
      }

  inline void addAsFirstNotify      (Element const& element) {
      Base::Cursor cursor(ivBase);
      ivBase.addAsFirst (element,cursor);
      notifyAdded(cursor);
      }

  inline void addAsLastNotify       (Element const& element) {
      Base::Cursor cursor(ivBase);
      ivBase.addAsLast (element,cursor);
      notifyAdded(cursor);
      }

  inline void addAtPositionNotify
     (IPosition position, Element const& element) {
      Base::Cursor cursor(ivBase);
      ivBase.addAtPosition (position, element, cursor);
      notifyAdded(cursor);
      }
public:

  IBoolean add                  (Element const&);

  IBoolean add                  (Element const&, ICursor&);

  void     addAllFrom           (IACollection <Element> const&);

  void     addAsFirst           (Element const&);

  void     addAsFirst           (Element const&, ICursor&);

  void     addAsLast            (Element const&);

  void     addAsLast            (Element const&, ICursor&);

  void     addAsNext            (Element const&, ICursor&);

  void     addAsPrevious        (Element const&, ICursor&);

  void     addAtPosition        (IPosition, Element const&);

  void     addAtPosition        (IPosition, Element const&, ICursor&);

  IBoolean allElementsDo        (IBoolean (*function) (Element&, void*),
                                 void* additionalArgument = 0);

  IBoolean allElementsDo        (IIterator <Element>&);

  IBoolean allElementsDo        (IBoolean (*function)
                                  (Element const&, void*),
                                 void* additionalArgument = 0) const;

  IBoolean allElementsDo        (IConstantIterator <Element>&) const;

  Element const&
           anyElement           () const;

  IBoolean contains             (Element const&) const;

  Element const&
           elementAt            (ICursor const&) const;

  Element& elementAt            (ICursor const&);

  Element const&
           elementAtPosition    (IPosition) const;

  void
           copy                 (IACollection <Element> const&);

  Element const&
           firstElement         () const;

  IBoolean isBounded            () const;

  IBoolean isEmpty              () const;

  IBoolean isFirst              (ICursor const&) const;

  IBoolean isFull               () const;

  IBoolean isLast               (ICursor const&) const;

  Element const&
           lastElement          () const;

  IBoolean locate               (Element const&, ICursor&) const;

  IBoolean locateFirst          (Element const&, ICursor&) const;

  IBoolean locateLast           (Element const&, ICursor&) const;

  IBoolean locateNext           (Element const&, ICursor&) const;

  IBoolean locateOrAdd          (Element const&);

  IBoolean locateOrAdd          (Element const&, ICursor&);

  IBoolean locatePrevious       (Element const&, ICursor&) const;

  INumber  maxNumberOfElements  () const;

  ICursor* newCursor            () const;

  INumber  numberOfElements     () const;

  INumber  numberOfOccurrences  (Element const&) const;

  IBoolean remove               (Element const&);

  void     removeAll            ();

  INumber  removeAll            (IBoolean (*property)
                                  (Element const&, void*),
                                 void* additionalArgument = 0);

  INumber  removeAllOccurrences (Element const&);

  void     removeAt             (ICursor&);

  void     removeAtPosition     (IPosition);

  void     removeFirst          ();

  void     removeLast           ();

  void     replaceAt            (ICursor const&, Element const&);

  IBoolean setToFirst           (ICursor&) const;

  IBoolean setToLast            (ICursor&) const;

  IBoolean setToNext            (ICursor&) const;

  void     setToPosition        (IPosition, ICursor&) const;

#ifndef __BIN_COMPAT_BASE_43__
  IPosition     position        (ICursor const&) const;
#endif

  IBoolean setToPrevious        (ICursor&) const;

  void     sort                 (long (*comparisonFunction)
                                  (Element const&, Element const&));

  IBoolean isConsistent         () const;

protected:

  void*    identity () const;

};

#ifndef __TEMPINC__
#include <ireqseq.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
