/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)

template < class Element, class Base >
void* IRHeap < Element, Base >::
identity () const
{ return (void*)&ivBase;
}

template < class Element, class Base >
IRHeap < Element, Base >::
~IRHeap ()
{
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
add (Element const& element)
{
   if (hasObservers()) {
      return addNotify(element);
      }
   else
      return ivBase.add (element);
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
add (Element const& element, ICursor &cursor)
{
if (hasObservers()) {
      IBoolean result = ivBase.add(element, cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }
   else
      return ivBase.add (element, cursor);
}

template <class Element , class Base >
void IRHeap < Element , Base >::
addAllFrom (IACollection < Element > const& collection)
{ ICHECK (! isIdentical (collection),
          IIdenticalCollectionException, IIdenticalCollectionText)
   if (hasObservers()) {
      addAllFromNotify(collection);
      }
   else
      Inherited::addAllFrom(collection);
}

template <class Element , class Base >
void IRHeap < Element , Base >::
copy (IACollection < Element > const& collection)
{ if (hasObservers()) {
     if (! isIdentical (collection)) {
       ivBase.removeAll ();
       IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
       forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
        }
       notifyModified();
     }
  }
  else
    Inherited::copy(collection);
}

template < class Element, class Base >
Element const& IRHeap < Element, Base >::
elementAt (ICursor const& cursor) const
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element& IRHeap < Element, Base >::
elementAt (ICursor const& cursor)
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element const& IRHeap < Element, Base >::
anyElement () const
{ return ivBase.anyElement ();
}

template < class Element, class Base >
void IRHeap < Element, Base >::
removeAt (ICursor& cursor)
{
   if (hasObservers()) {
      notifyRemoved(cursor);
      };
   ivBase.removeAt (cursor);
}

template < class Element, class Base >
INumber IRHeap < Element, Base >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{
   if (hasObservers()) {
      INumber result =ivBase.removeAll (predicate, env);
      if (result > 0)
         notifyModified();
      return result;
      }
   else
      return ivBase.removeAll (predicate, env);
}

template < class Element, class Base >
void IRHeap < Element, Base >::
replaceAt (ICursor const& cursor, Element const& element)
{
   if (hasObservers()) {
      Element oldElement = ivBase.elementAt(cursor);
      ivBase.replaceAt (cursor, element);
      notifyReplaced(cursor,oldElement);
      }
   else
      ivBase.replaceAt (cursor, element);
}

template < class Element, class Base >
void IRHeap < Element, Base >::
removeAll ()
{ ivBase.removeAll ();
   if (hasObservers()) {
      notifyModified();
      };
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
isBounded () const
{ return ivBase.isBounded ();
}

template < class Element, class Base >
INumber IRHeap < Element, Base >::
maxNumberOfElements () const
{ return ivBase.maxNumberOfElements ();
}

template < class Element, class Base >
INumber IRHeap < Element, Base >::
numberOfElements () const
{ return ivBase.numberOfElements ();
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
isEmpty () const
{ return ivBase.isEmpty ();
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
isFull () const
{ return ivBase.isFull ();
}

template < class Element, class Base >
ICursor* IRHeap < Element, Base >::
newCursor () const
{ return ivBase.newCursor ();
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
setToFirst (ICursor &cursor) const
{ return ivBase.setToFirst (cursor);
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
setToNext (ICursor &cursor) const
{ return ivBase.setToNext (cursor);
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element &, void*),
               void* environment)
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
allElementsDo (IIterator <Element>& iterator)
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* environment) const
{ return ((Base const&)ivBase).allElementsDo
           (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRHeap < Element, Base >::
isConsistent () const
{ return ivBase.isConsistent ();
}

#pragma info (restore)
