/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IRHEAP_H
#define _IRHEAP_H

#include <iaheap.h>
#include <istdntfy.hpp>
#include <ipartccl.h>
#include <iptr.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

template < class Element, class Base >
class IRHeap : public virtual IAHeap < Element >
                 ,  public IPartCollection < Element >
                                          {
  typedef IAHeap < Element > Inherited;

  Base &ivBase;
public:

           IRHeap               (Base& base)
           : ivBase (base)
           {
           }

          ~IRHeap               ();

private:
  inline IBoolean addNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.add(element,cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }

  inline void addAllFromNotify
     (IACollection <Element> const& collection) {
      IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
      forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
         }
      notifyModified();
      }
public:

  IBoolean add                  (Element const&);

  IBoolean add                  (Element const&, ICursor&);

  void     addAllFrom           (IACollection <Element> const&);

  IBoolean allElementsDo        (IBoolean (*function) (Element&, void*),
                                 void* additionalArgument = 0);

  IBoolean allElementsDo        (IIterator <Element>&);

  IBoolean allElementsDo        (IBoolean (*function)
                                  (Element const&, void*),
                                 void* additionalArgument = 0) const;

  IBoolean allElementsDo        (IConstantIterator <Element>&) const;

  Element const&
           anyElement           () const;

  void
           copy                 (IACollection <Element> const&);

  Element const&
           elementAt            (ICursor const&) const;

  Element& elementAt            (ICursor const&);

  IBoolean isBounded            () const;

  IBoolean isEmpty              () const;

  IBoolean isFull               () const;

  INumber  maxNumberOfElements  () const;

  ICursor* newCursor            () const;

  INumber  numberOfElements     () const;

  void     removeAll            ();

  INumber  removeAll            (IBoolean (*property)
                                  (Element const&, void*),
                                 void* additionalArgument = 0);

  void     removeAt             (ICursor&);

  void     replaceAt            (ICursor const&, Element const&);

  IBoolean setToFirst           (ICursor&) const;

  IBoolean setToNext            (ICursor&) const;

  IBoolean isConsistent         () const;

protected:

  void*    identity () const;

};

#ifndef __TEMPINC__
#include <irheap.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
