/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)

template < class Element, class Key, class Base >
void* IRKeyBag < Element, Key, Base >::
identity () const
{ return (void*)&ivBase;
}

template < class Element, class Key, class Base >
IRKeyBag < Element, Key, Base >::
~IRKeyBag ()
{
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
add (Element const& element)
{
   if (hasObservers()) {
      return addNotify(element);
      }
   else
      return ivBase.add (element);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
add (Element const& element, ICursor &cursor)
{
if (hasObservers()) {
      IBoolean result = ivBase.add(element, cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }
   else
      return ivBase.add (element, cursor);
}

template < class Element, class Key, class Base >
void IRKeyBag < Element , Key , Base >::
addAllFrom (IACollection < Element > const& collection)
{ ICHECK (! isIdentical (collection),
          IIdenticalCollectionException, IIdenticalCollectionText)
   if (hasObservers()) {
      addAllFromNotify(collection);
      }
   else
      Inherited::addAllFrom(collection);
}

template < class Element, class Key, class Base >
void IRKeyBag < Element , Key , Base >::
copy (IACollection < Element > const& collection)
{ if (hasObservers()) {
     if (! isIdentical (collection)) {
       ivBase.removeAll ();
       IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
       forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
        }
       notifyModified();
     }
  }
  else
    Inherited::copy(collection);
}

template < class Element, class Key, class Base >
Element const& IRKeyBag < Element, Key, Base >::
elementAt (ICursor const& cursor) const
{ return ivBase.elementAt (cursor);
}

template < class Element, class Key, class Base >
Element& IRKeyBag < Element, Key, Base >::
elementAt (ICursor const& cursor)
{ return ivBase.elementAt (cursor);
}

template < class Element, class Key, class Base >
Element const& IRKeyBag < Element, Key, Base >::
anyElement () const
{ return ivBase.anyElement ();
}

template < class Element, class Key, class Base >
void IRKeyBag < Element, Key, Base >::
removeAt (ICursor& cursor)
{
   if (hasObservers()) {
      notifyRemoved(cursor);
      };
      ivBase.removeAt (cursor);
}

template < class Element, class Key, class Base >
INumber IRKeyBag < Element, Key, Base >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{
   if (hasObservers()) {
      INumber result =ivBase.removeAll (predicate, env);
      if (result > 0)
         notifyModified();
      return result;
      }
   else
      return ivBase.removeAll (predicate, env);
}

template < class Element, class Key, class Base >
void IRKeyBag < Element, Key, Base >::
replaceAt (ICursor const& cursor, Element const& element)
{
   if (hasObservers()) {
      Element oldElement = ivBase.elementAt(cursor);
      ivBase.replaceAt (cursor, element);
      notifyReplaced(cursor,oldElement);
      }
   else
      ivBase.replaceAt (cursor, element);
}

template < class Element, class Key, class Base >
void IRKeyBag < Element, Key, Base >::
removeAll ()
{ ivBase.removeAll ();
   if (hasObservers()) {
      notifyModified();
      };
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
isBounded () const
{ return ivBase.isBounded ();
}

template < class Element, class Key, class Base >
INumber IRKeyBag < Element, Key, Base >::
maxNumberOfElements () const
{ return ivBase.maxNumberOfElements ();
}

template < class Element, class Key, class Base >
INumber IRKeyBag < Element, Key, Base >::
numberOfElements () const
{ return ivBase.numberOfElements ();
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
isEmpty () const
{ return ivBase.isEmpty ();
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
isFull () const
{ return ivBase.isFull ();
}

template < class Element, class Key, class Base >
ICursor* IRKeyBag < Element, Key, Base >::
newCursor () const
{ return ivBase.newCursor ();
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
setToFirst (ICursor &cursor) const
{ return ivBase.setToFirst (cursor);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
setToNext (ICursor &cursor) const
{ return ivBase.setToNext (cursor);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element &, void*),
               void* environment)
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
allElementsDo (IIterator <Element>& iterator)
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* environment) const
{ return ((Base const&)ivBase).allElementsDo
           (iterationFunction, environment);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
isConsistent () const
{ return ivBase.isConsistent ();
}

template < class Element, class Key, class Base >
Key const& IRKeyBag < Element, Key, Base >::
key (Element const& element) const
{ return ivBase.key (element);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
containsElementWithKey (Key const& key) const
{ return ivBase.containsElementWithKey (key);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
locateElementWithKey (Key const& key, ICursor &cursor) const
{ return ivBase.locateElementWithKey (key, cursor);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
replaceElementWithKey (Element const& element)
{
   if (hasObservers()) {
      return replaceElementWithKeyNotify(element);
      }
   else
      return ivBase.replaceElementWithKey (element);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
replaceElementWithKey (Element const& element, ICursor &cursor)
{
   if (hasObservers()) {
      return replaceElementWithKeyNotify(element,cursor);
      }
   else
      return ivBase.replaceElementWithKey (element, cursor);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
locateOrAddElementWithKey (Element const& element)
{
   if (hasObservers()) {
      return locateOrAddElementWithKeyNotify(element);
      }
   else
      return ivBase.locateOrAddElementWithKey (element);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
locateOrAddElementWithKey (Element const& element, ICursor &cursor)
{
   if (hasObservers()) {
      IBoolean result
         = ivBase.locateOrAddElementWithKey(element,cursor);
      if (!result) {
         notifyAdded(cursor);
         }
      return result;
      }
   else
      return ivBase.locateOrAddElementWithKey (element, cursor);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
addOrReplaceElementWithKey (Element const& element)
{
   if (hasObservers()) {
      return addOrReplaceElementWithKeyNotify(element);
      }
   else
      return ivBase.addOrReplaceElementWithKey (element);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
addOrReplaceElementWithKey (Element const& element, ICursor &cursor)
{
   if (hasObservers()) {
      return addOrReplaceElementWithKeyNotify(element,cursor);
      }
   else
      return ivBase.addOrReplaceElementWithKey (element, cursor);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
removeElementWithKey (Key const& key)
{
   if (hasObservers()) {
      return removeElementWithKeyNotify(key);
      }
   else
      return ivBase.removeElementWithKey (key);
}

template < class Element, class Key, class Base >
Element const& IRKeyBag < Element, Key, Base >::
elementWithKey (Key const& key) const
{ return ivBase.elementWithKey (key);
}

template < class Element, class Key, class Base >
Element& IRKeyBag < Element, Key, Base >::
elementWithKey (Key const& key)
{ return ivBase.elementWithKey (key);
}

template < class Element, class Key, class Base >
INumber IRKeyBag < Element, Key, Base >::
numberOfElementsWithKey (Key const& key) const
{ return ivBase.numberOfElementsWithKey (key);
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
locateNextElementWithKey (Key const& key, ICursor& cursor) const
{ return ivBase.locateNextElementWithKey (key, cursor);
}

template < class Element, class Key, class Base >
INumber IRKeyBag < Element, Key, Base >::
removeAllElementsWithKey (Key const& key)
{
   if (hasObservers()) {
      INumber result =ivBase.removeAllElementsWithKey (key);
      if (result > 0)
         notifyModified();
      return result;
      }
   else
   return ivBase.removeAllElementsWithKey (key);
}

template < class Element, class Key, class Base >
INumber IRKeyBag < Element, Key, Base >::
numberOfDifferentKeys () const
{ return ivBase.numberOfDifferentKeys ();
}

template < class Element, class Key, class Base >
IBoolean IRKeyBag < Element, Key, Base >::
setToNextWithDifferentKey (ICursor& cursor) const
{ return ivBase.setToNextWithDifferentKey (cursor);
}

#pragma info (restore)
