/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)

template < class Element, class Key, class Base >
void* IRPriorityQueue < Element, Key, Base >::
identity () const
{ return (void*)&ivBase;
}

template < class Element, class Key, class Base >
IRPriorityQueue < Element, Key, Base >::
~IRPriorityQueue ()
{
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
add (Element const& element)
{
   if (hasObservers()) {
      return addNotify(element);
      }
   else
      return ivBase.add (element);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
add (Element const& element, ICursor &cursor)
{
if (hasObservers()) {
      IBoolean result = ivBase.add(element, cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }
   else
      return ivBase.add (element, cursor);
}

template <class Element , class Key, class Base >
void IRPriorityQueue < Element , Key, Base >::
addAllFrom (IAPriorityQueue < Element , Key> const& collection)
{ ICHECK (! isIdentical (collection),
          IIdenticalCollectionException, IIdenticalCollectionText)
   if (hasObservers()) {
      addAllFromNotify(collection);
      }
   else
      Inherited::addAllFrom(collection);
}

template <class Element , class Key, class Base >
void IRPriorityQueue < Element , Key, Base >::
copy (IAPriorityQueue < Element , Key > const& collection)
{ if (hasObservers()) {
     if (! isIdentical (collection)) {
       ivBase.removeAll ();
       IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
       forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
        }
       notifyModified();
     }
  }
  else
    Inherited::copy(collection);
}

template < class Element, class Key, class Base >
Element const& IRPriorityQueue < Element, Key, Base >::
elementAt (ICursor const& cursor) const
{ return ivBase.elementAt (cursor);
}

template < class Element, class Key, class Base >
Element const& IRPriorityQueue < Element, Key, Base >::
anyElement () const
{ return ivBase.anyElement ();
}

template < class Element, class Key, class Base >
void IRPriorityQueue < Element, Key, Base >::
removeAll ()
{ ivBase.removeAll ();
   if (hasObservers()) {
      notifyModified();
      };
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
isBounded () const
{ return ivBase.isBounded ();
}

template < class Element, class Key, class Base >
INumber IRPriorityQueue < Element, Key, Base >::
maxNumberOfElements () const
{ return ivBase.maxNumberOfElements ();
}

template < class Element, class Key, class Base >
INumber IRPriorityQueue < Element, Key, Base >::
numberOfElements () const
{ return ivBase.numberOfElements ();
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
isEmpty () const
{ return ivBase.isEmpty ();
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
isFull () const
{ return ivBase.isFull ();
}

template < class Element, class Key, class Base >
ICursor* IRPriorityQueue < Element, Key, Base >::
newCursor () const
{ return ivBase.newCursor ();
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
setToFirst (ICursor &cursor) const
{ return ivBase.setToFirst (cursor);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
setToNext (ICursor &cursor) const
{ return ivBase.setToNext (cursor);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* environment) const
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
isConsistent () const
{ return ivBase.isConsistent ();
}

template < class Element, class Key, class Base >
Key const& IRPriorityQueue < Element, Key, Base >::
key (Element const& element) const
{ return ivBase.key (element);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
containsElementWithKey (Key const& key) const
{ return ivBase.containsElementWithKey (key);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
locateElementWithKey (Key const& key, ICursor &cursor) const
{ return ivBase.locateElementWithKey (key, cursor);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
locateOrAddElementWithKey (Element const& element)
{
   if (hasObservers()) {
      return locateOrAddNotify(element);
      }
   else
      return ivBase.locateOrAddElementWithKey (element);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
locateOrAddElementWithKey (Element const& element, ICursor &cursor)
{
if (hasObservers()) {
      IBoolean result
         = ivBase.locateOrAddElementWithKey(element, cursor);
      if (!result) {
         notifyAdded(cursor);
         }
      return result;
      }
   else
      return ivBase.locateOrAddElementWithKey (element, cursor);
}

template < class Element, class Key, class Base >
Element const& IRPriorityQueue < Element, Key, Base >::
elementWithKey (Key const& key) const
{ return ivBase.elementWithKey (key);
}

template < class Element, class Key, class Base >
INumber IRPriorityQueue < Element, Key, Base >::
numberOfElementsWithKey (Key const& key) const
{ return ivBase.numberOfElementsWithKey (key);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
locateNextElementWithKey (Key const& key, ICursor& cursor) const
{ return ivBase.locateNextElementWithKey (key, cursor);
}

template < class Element, class Key, class Base >
INumber IRPriorityQueue < Element, Key, Base >::
numberOfDifferentKeys () const
{ return ivBase.numberOfDifferentKeys ();
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
setToNextWithDifferentKey (ICursor& cursor) const
{ return ivBase.setToNextWithDifferentKey (cursor);
}

template < class Element, class Key, class Base >
void IRPriorityQueue < Element, Key, Base >::
removeFirst ()
{
   if (hasObservers()) {
      removeFirstNotify();
      }
ivBase.removeFirst ();
}

template < class Element, class Key, class Base >
Element const& IRPriorityQueue < Element, Key, Base >::
firstElement () const
{ return ivBase.firstElement ();
}

template < class Element, class Key, class Base >
Element const& IRPriorityQueue < Element, Key, Base >::
lastElement () const
{ return ivBase.lastElement ();
}

template < class Element, class Key, class Base >
Element const& IRPriorityQueue < Element, Key, Base >::
elementAtPosition (IPosition position) const
{ return ivBase.elementAtPosition (position);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
setToLast (ICursor &cursor) const
{ return ivBase.setToLast (cursor);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
setToPrevious (ICursor &cursor) const
{ return ivBase.setToPrevious (cursor);
}

template < class Element, class Key, class Base >
void IRPriorityQueue < Element, Key, Base >::
setToPosition (IPosition position, ICursor &cursor) const
{ ivBase.setToPosition (position, cursor);
}

#ifndef __BIN_COMPAT_BASE_43__
template < class Element, class Key, class Base >
IPosition IRPriorityQueue < Element, Key, Base >::
position (ICursor const& cursor) const
{ return ivBase.position (cursor);
}
#endif

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
isFirst (ICursor const& cursor) const
{ return ivBase.isFirst (cursor);
}

template < class Element, class Key, class Base >
IBoolean IRPriorityQueue < Element, Key, Base >::
isLast (ICursor const& cursor) const
{ return ivBase.isLast (cursor);
}

template < class Element, class Key, class Base >
void IRPriorityQueue < Element, Key, Base >::
enqueue (Element const& element)
{
   if (hasObservers()) {
      addNotify(element);
      }
   else
      ivBase.add (element);
}

template < class Element, class Key, class Base >
void IRPriorityQueue < Element, Key, Base >::
enqueue (Element const& element, ICursor& cursor)
{
if (hasObservers()) {
      IBoolean result = ivBase.add(element, cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return;
      }
   else
      ivBase.add (element, cursor);
}

template < class Element, class Key, class Base >
void IRPriorityQueue < Element, Key, Base >::
dequeue ()
{
   if (hasObservers()) {
      removeFirstNotify();
      }
ivBase.removeFirst ();
}

template < class Element, class Key, class Base >
void IRPriorityQueue < Element, Key, Base >::
dequeue (Element& element)
{ element = ivBase.firstElement ();
   if (hasObservers()) {
      removeFirstNotify();
      }
  ivBase.removeFirst ();
}

#pragma info (restore)
