/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IRPRIOQU_H
#define _IRPRIOQU_H

#include <iaprioqu.h>
#include <istdntfy.hpp>
#include <ipartccl.h>
#include <iptr.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

template < class Element, class Key, class Base >
class IRPriorityQueue :
  public virtual IAPriorityQueue < Element, Key >
                 ,  public IPartCollection < Element >
                                          {
  typedef IAPriorityQueue < Element , Key > Inherited;

  Base &ivBase;
public:

           IRPriorityQueue      (Base& base)
           : ivBase (base)
           {
           }

          ~IRPriorityQueue      ();

private:
  inline IBoolean addNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.add(element,cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }

  inline IBoolean locateOrAddNotify (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result
         = ivBase.locateOrAddElementWithKey(element,cursor);
      if (!result) {
         notifyAdded(cursor);
         }
      return result;
      }

  inline void addAllFromNotify
     (IAPriorityQueue <Element , Key> const& collection) {
      IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
      forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
         }
      notifyModified();
      }

  inline void removeFirstNotify     () {
      if (ivBase.numberOfElements() > 0) {
         Base::Cursor cursor(ivBase);
         ivBase.setToFirst(cursor);
         notifyRemoved(cursor);
         }
      }

  inline void removeLastNotify      () {
      if (ivBase.numberOfElements() > 0) {
         Base::Cursor cursor(ivBase);
         ivBase.setToLast(cursor);
         notifyRemoved(cursor);
         }
      }
public:

  IBoolean add                  (Element const&);

  IBoolean add                  (Element const&, ICursor&);

  void     addAllFrom           (IAPriorityQueue <Element,Key> const&);

  IBoolean allElementsDo        (IBoolean (*function)
                                  (Element const&, void*),
                                 void* additionalArgument = 0) const;

  IBoolean allElementsDo        (IConstantIterator <Element>&) const;

  Element const&
           anyElement           () const;

  IBoolean containsElementWithKey (Key const&) const;

  void
           copy                 (IAPriorityQueue <Element,Key> const&);

  void     dequeue              ();

  void     dequeue              (Element&);

  Element const&
           elementAt            (ICursor const&) const;

  Element const&
           elementAtPosition    (IPosition) const;

  Element const&
           elementWithKey       (Key const&) const;

  void     enqueue              (Element const&);

  void     enqueue              (Element const&, ICursor&);

  Element const&
           firstElement         () const;

  IBoolean isBounded            () const;

  IBoolean isEmpty              () const;

  IBoolean isFirst              (ICursor const&) const;

  IBoolean isFull               () const;

  IBoolean isLast               (ICursor const&) const;

  Key const&
           key                  (Element const&) const;

  Element const&
           lastElement          () const;

  IBoolean locateElementWithKey (Key const&, ICursor&) const;

  IBoolean locateNextElementWithKey (Key const&, ICursor&) const;

  IBoolean locateOrAddElementWithKey (Element const&);

  IBoolean locateOrAddElementWithKey (Element const&, ICursor&);

  INumber  maxNumberOfElements  () const;

  ICursor* newCursor            () const;

  INumber  numberOfDifferentKeys () const;

  INumber  numberOfElements     () const;

  INumber  numberOfElementsWithKey (Key const&) const;

  void     removeAll            ();

  void     removeFirst          ();

  IBoolean setToFirst           (ICursor&) const;

  IBoolean setToLast            (ICursor&) const;

  IBoolean setToNext            (ICursor&) const;

  IBoolean setToNextWithDifferentKey (ICursor&) const;

  void     setToPosition        (IPosition, ICursor&) const;

#ifndef __BIN_COMPAT_BASE_43__
  IPosition     position        (ICursor const&) const;
#endif

  IBoolean setToPrevious        (ICursor&) const;

  IBoolean isConsistent         () const;

protected:

  void*    identity () const;

};

#ifndef __TEMPINC__
#include <irprioqu.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
