/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)

template < class Element, class Base >
void* IRQueue < Element, Base >::
identity () const
{ return (void*)&ivBase;
}

template < class Element, class Base >
IRQueue < Element, Base >::
~IRQueue ()
{
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
add (Element const& element)
{
   if (hasObservers()) {
      return addNotify(element);
      }
   else
      return ivBase.add (element);
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
add (Element const& element, ICursor &cursor)
{
if (hasObservers()) {
      IBoolean result = ivBase.add(element, cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }
   else
      return ivBase.add (element, cursor);
}

template <class Element , class Base >
void IRQueue < Element , Base >::
addAllFrom (IAQueue < Element > const& collection)
{ ICHECK (! isIdentical (collection),
          IIdenticalCollectionException, IIdenticalCollectionText)
   if (hasObservers()) {
      addAllFromNotify(collection);
      }
   else
      Inherited::addAllFrom(collection);
}

template <class Element , class Base >
void IRQueue < Element , Base >::
copy (IAQueue < Element > const& collection)
{ if (hasObservers()) {
     if (! isIdentical (collection)) {
       ivBase.removeAll ();
       IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
       forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
        }
       notifyModified();
     }
  }
  else
    Inherited::copy(collection);
}

template < class Element, class Base >
Element const& IRQueue < Element, Base >::
elementAt (ICursor const& cursor) const
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element const& IRQueue < Element, Base >::
anyElement () const
{ return ivBase.anyElement ();
}

template < class Element, class Base >
void IRQueue < Element, Base >::
removeAll ()
{ ivBase.removeAll ();
   if (hasObservers()) {
      notifyModified();
      };
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
isBounded () const
{ return ivBase.isBounded ();
}

template < class Element, class Base >
INumber IRQueue < Element, Base >::
maxNumberOfElements () const
{ return ivBase.maxNumberOfElements ();
}

template < class Element, class Base >
INumber IRQueue < Element, Base >::
numberOfElements () const
{ return ivBase.numberOfElements ();
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
isEmpty () const
{ return ivBase.isEmpty ();
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
isFull () const
{ return ivBase.isFull ();
}

template < class Element, class Base >
ICursor* IRQueue < Element, Base >::
newCursor () const
{ return ivBase.newCursor ();
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
setToFirst (ICursor &cursor) const
{ return ivBase.setToFirst (cursor);
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
setToNext (ICursor &cursor) const
{ return ivBase.setToNext (cursor);
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* environment) const
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
isConsistent () const
{ return ivBase.isConsistent ();
}

template < class Element, class Base >
void IRQueue < Element, Base >::
removeFirst ()
{
   if (hasObservers()) {
      removeFirstNotify();
      }
   ivBase.removeFirst ();
}

template < class Element, class Base >
Element const& IRQueue < Element, Base >::
firstElement () const
{ return ivBase.firstElement ();
}

template < class Element, class Base >
Element const& IRQueue < Element, Base >::
lastElement () const
{ return ivBase.lastElement ();
}

template < class Element, class Base >
Element const& IRQueue < Element, Base >::
elementAtPosition (IPosition position) const
{ return ivBase.elementAtPosition (position);
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
setToLast (ICursor &cursor) const
{ return ivBase.setToLast (cursor);
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
setToPrevious (ICursor &cursor) const
{ return ivBase.setToPrevious (cursor);
}

template < class Element, class Base >
void IRQueue < Element, Base >::
setToPosition (IPosition position, ICursor &cursor) const
{ ivBase.setToPosition (position, cursor);
}

#ifndef __BIN_COMPAT_BASE_43__
template < class Element, class Base >
IPosition IRQueue < Element, Base >::
position (ICursor const& cursor) const
{ return ivBase.position (cursor);
}
#endif

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
isFirst (ICursor const& cursor) const
{ return ivBase.isFirst (cursor);
}

template < class Element, class Base >
IBoolean IRQueue < Element, Base >::
isLast (ICursor const& cursor) const
{ return ivBase.isLast (cursor);
}

template < class Element, class Base >
void IRQueue < Element, Base >::
addAsLast (Element const& element)
{
   if (hasObservers()) {
      addAsLastNotify(element);
      }
   else
      ivBase.addAsLast (element);
}

template < class Element, class Base >
void IRQueue < Element, Base >::
addAsLast (Element const& element, ICursor &cursor)
{ ivBase.addAsLast (element, cursor);
   if (hasObservers()) {
      notifyAdded(cursor);
      }
}

template < class Element, class Base >
void IRQueue < Element, Base >::
enqueue (Element const& element)
{
   if (hasObservers()) {
      addNotify(element);
      }
   else
      ivBase.add (element);
}

template < class Element, class Base >
void IRQueue < Element, Base >::
enqueue (Element const& element, ICursor& cursor)
{ ivBase.add (element, cursor);
   if (hasObservers()) {
      notifyAdded(cursor);
      }
}

template < class Element, class Base >
void IRQueue < Element, Base >::
dequeue ()
{
   if (hasObservers()) {
      removeFirstNotify();
      }
ivBase.removeFirst ();
}

template < class Element, class Base >
void IRQueue < Element, Base >::
dequeue (Element& element)
{ element = ivBase.firstElement ();
   if (hasObservers()) {
      removeFirstNotify();
      }
  ivBase.removeFirst ();
}

#pragma info (restore)
