/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IRREL_H
#define _IRREL_H

#include <iarel.h>
#include <istdntfy.hpp>
#include <ipartccl.h>
#include <iptr.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

template < class Element, class Key, class Base >
class IRRelation : public virtual IARelation < Element, Key >
                 ,  public IPartCollection < Element >
                                          {
  typedef IARelation < Element , Key > Inherited;

  Base &ivBase;
public:

           IRRelation           (Base& base)
           : ivBase (base)
           {
           }

          ~IRRelation           ();

private:
  inline IBoolean addNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.add(element,cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }

  inline void addAllFromNotify
     (IACollection <Element> const& collection) {
      IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
      forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
         }
      notifyModified();
      }

  inline IBoolean replaceElementWithKeyNotify
     (Element const& element) {
         Base::Cursor cursor(ivBase);
         Element oldElement;
         IBoolean found
            = ivBase.locateElementWithKey(key(element),cursor);
         IBoolean result = False;
         if (found) {
            Element oldElement = ivBase.elementAt(cursor);
            result = ivBase.replaceElementWithKey(element);
            notifyReplaced(cursor,oldElement);
         }
      return result;
      }

  inline IBoolean replaceElementWithKeyNotify (Element const& element,
                             ICursor& cursor) {
         Element oldElement;
         IBoolean found
            = ivBase.locateElementWithKey(key(element),cursor);
         IBoolean result = False;
         if (found) {
            Element oldElement = ivBase.elementAt(cursor);
            result = ivBase.replaceElementWithKey(element,cursor);
            notifyReplaced(cursor,oldElement);
         }
      return result;
      }

  inline IBoolean addOrReplaceElementWithKeyNotify
     (Element const& element) {
         Base::Cursor cursor(ivBase);
         return addOrReplaceElementWithKeyNotify(element,cursor);
      }

  inline IBoolean addOrReplaceElementWithKeyNotify
     (Element const& element, ICursor &cursor) {
         Element oldElement;
         IBoolean found
            = ivBase.locateElementWithKey(key(element),cursor);
         IBoolean result;
         if (found) {
            Element oldElement = ivBase.elementAt(cursor);
            result = ivBase.addOrReplaceElementWithKey(element,cursor);
            notifyReplaced(cursor,oldElement);
            }
         else {
            result = ivBase.addOrReplaceElementWithKey(element,cursor);
            notifyAdded(cursor);
            }
      return result;
      }

  inline IBoolean locateOrAddElementWithKeyNotify
     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result
         = ivBase.locateOrAddElementWithKey(element,cursor);
      if (!result) {
         notifyAdded(cursor);
         }
      return result;
      }

  inline IBoolean removeElementWithKeyNotify (Key const& key) {
      Base::Cursor cursor(ivBase);
      IBoolean found = ivBase.locateElementWithKey(key, cursor);
      if (found) {
         notifyRemoved(cursor);
         }
      return ivBase.removeElementWithKey (key);
      }

  inline void removeNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.locate(element,cursor);
      if (result)
         notifyRemoved(cursor);
      }

  inline IBoolean locateOrAddNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.locateOrAdd(element,cursor);
      if (!result) {
         notifyAdded(cursor);
         }
      return result;
      }
public:

  IBoolean add                  (Element const&);

  IBoolean add                  (Element const&, ICursor&);

  void     addAllFrom           (IACollection <Element> const&);

  void
           copy                 (IACollection <Element> const&);

  Element const&
           elementAt            (ICursor const&) const;

  Element& elementAt            (ICursor const&);

  Element const&
           anyElement           () const;

  void     removeAt             (ICursor&);

  INumber  removeAll            (IBoolean (*property)
                                  (Element const&, void*),
                                 void* additionalArgument = 0);

  INumber  removeAllOccurrences (Element const&);

  void     replaceAt            (ICursor const&, Element const&);

  void     removeAll            ();

  IBoolean isBounded            () const;

  INumber  maxNumberOfElements  () const;

  INumber  numberOfElements     () const;

  IBoolean isEmpty              () const;

  IBoolean isFull               () const;

  ICursor* newCursor            () const;

  IBoolean setToFirst           (ICursor&) const;

  IBoolean setToNext            (ICursor&) const;

  IBoolean allElementsDo        (IBoolean (*function) (Element&, void*),
                                 void* additionalArgument = 0);

  IBoolean allElementsDo        (IIterator <Element>&);

  IBoolean allElementsDo        (IBoolean (*function)
                                  (Element const&, void*),
                                 void* additionalArgument = 0) const;

  IBoolean allElementsDo        (IConstantIterator <Element>&) const;

  IBoolean isConsistent         () const;

  IBoolean contains             (Element const&) const;

  IBoolean locate               (Element const&, ICursor&) const;

  IBoolean locateOrAdd          (Element const&);

  IBoolean locateOrAdd          (Element const&, ICursor&);

  IBoolean remove               (Element const&);

  void     unionWith            (IARelation < Element, Key > const&);

  void     intersectionWith     (IARelation < Element, Key > const&);

  void     differenceWith       (IARelation < Element, Key > const&);

  void     addUnion             (IARelation < Element, Key > const&,
                                 IARelation < Element, Key > const&);

  void     addIntersection      (IARelation < Element, Key > const&,
                                 IARelation < Element, Key > const&);

  void     addDifference        (IARelation < Element, Key > const&,
                                 IARelation < Element, Key > const&);

  Key const&
           key                  (Element const&) const;

  IBoolean containsElementWithKey (Key const&) const;

  IBoolean locateElementWithKey (Key const&, ICursor&) const;

  IBoolean replaceElementWithKey (Element const&);

  IBoolean replaceElementWithKey (Element const&, ICursor&);

  IBoolean locateOrAddElementWithKey (Element const&);

  IBoolean locateOrAddElementWithKey (Element const&, ICursor&);

  IBoolean addOrReplaceElementWithKey (Element const&);

  IBoolean addOrReplaceElementWithKey (Element const&, ICursor&);

  IBoolean removeElementWithKey (Key const&);

  Element const&
           elementWithKey       (Key const&) const;

  Element& elementWithKey       (Key const&);

  INumber  numberOfElementsWithKey (Key const&) const;

  IBoolean locateNextElementWithKey (Key const&, ICursor&) const;

  INumber  removeAllElementsWithKey (Key const&);

  INumber  numberOfDifferentKeys () const;

  IBoolean setToNextWithDifferentKey (ICursor&) const;

protected:

  void*    identity () const;

};

#ifndef __TEMPINC__
#include <irrel.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
