/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <iexcbase.hpp>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)

template < class Element, class Base >
void* IRSequence < Element, Base >::
identity () const
{ return (void*)&ivBase;
}

template < class Element, class Base >
IRSequence < Element, Base >::
~IRSequence ()
{
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
add (Element const& element)
{
   if (hasObservers()) {
      return addNotify(element);
      }
   else
      return ivBase.add(element);
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
add (Element const& element, ICursor &cursor)
{
if (hasObservers()) {
      IBoolean result = ivBase.add(element, cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }
   else
      return ivBase.add(element, cursor);
}

template <class Element , class Base >
void IRSequence < Element , Base >::
addAllFrom (IACollection < Element > const& collection)
{ ICHECK (! isIdentical (collection),
          IIdenticalCollectionException, IIdenticalCollectionText)
   if (hasObservers()) {
      addAllFromNotify(collection);
      }
   else
      Inherited::addAllFrom(collection);
}

template <class Element , class Base >
void IRSequence < Element , Base >::
copy (IACollection < Element > const& collection)
{ if (hasObservers()) {
     if (! isIdentical (collection)) {
       ivBase.removeAll ();
       IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
       forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
        }
       notifyModified();
     }
  }
  else
    Inherited::copy(collection);
}

template < class Element, class Base >
Element const& IRSequence < Element, Base >::
elementAt (ICursor const& cursor) const
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element& IRSequence < Element, Base >::
elementAt (ICursor const& cursor)
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element const& IRSequence < Element, Base >::
anyElement () const
{ return ivBase.anyElement ();
}

template < class Element, class Base >
void IRSequence < Element, Base >::
removeAt (ICursor& cursor)
{
   if (hasObservers()) {
      notifyRemoved(cursor);
      };
   ivBase.removeAt (cursor);
}

template < class Element, class Base >
INumber IRSequence < Element, Base >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{
   if (hasObservers()) {
      INumber result =ivBase.removeAll (predicate, env);
      if (result > 0)
         notifyModified();
      return result;
      }
   else
      return ivBase.removeAll (predicate, env);
}

template < class Element, class Base >
void IRSequence < Element, Base >::
replaceAt (ICursor const& cursor, Element const& element)
{
   if (hasObservers()) {
      Element oldElement = ivBase.elementAt(cursor);
      ivBase.replaceAt (cursor, element);
      notifyReplaced(cursor,oldElement);
      }
   else
      ivBase.replaceAt (cursor, element);
}

template < class Element, class Base >
void IRSequence < Element, Base >::
removeAll ()
{  ivBase.removeAll ();
   if (hasObservers()) {
      notifyModified();
      };
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
isBounded () const
{ return ivBase.isBounded ();
}

template < class Element, class Base >
INumber IRSequence < Element, Base >::
maxNumberOfElements () const
{ return ivBase.maxNumberOfElements ();
}

template < class Element, class Base >
INumber IRSequence < Element, Base >::
numberOfElements () const
{ return ivBase.numberOfElements ();
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
isEmpty () const
{ return ivBase.isEmpty ();
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
isFull () const
{ return ivBase.isFull ();
}

template < class Element, class Base >
ICursor* IRSequence < Element, Base >::
newCursor () const
{ return ivBase.newCursor ();
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
setToFirst (ICursor &cursor) const
{ return ivBase.setToFirst (cursor);
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
setToNext (ICursor &cursor) const
{ return ivBase.setToNext (cursor);
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element &, void*),
               void* environment)
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
allElementsDo (IIterator <Element>& iterator)
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* environment) const
{ return ((Base const&)ivBase).allElementsDo
           (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
isConsistent () const
{ return ivBase.isConsistent ();
}

template < class Element, class Base >
void IRSequence < Element, Base >::
removeFirst ()
{
   if (hasObservers()) {
      removeFirstNotify();
      }
   ivBase.removeFirst ();
}

template < class Element, class Base >
void IRSequence < Element, Base >::
removeLast ()
{
   if (hasObservers()) {
      removeLastNotify();
      };
   ivBase.removeLast ();
}

template < class Element, class Base >
void IRSequence < Element, Base >::
removeAtPosition (IPosition position)
{
   if (hasObservers()) {
      removeAtPositionNotify(position);
      };
   ivBase.removeAtPosition (position);
}

template < class Element, class Base >
Element const& IRSequence < Element, Base >::
firstElement () const
{ return ivBase.firstElement ();
}

template < class Element, class Base >
Element const& IRSequence < Element, Base >::
lastElement () const
{ return ivBase.lastElement ();
}

template < class Element, class Base >
Element const& IRSequence < Element, Base >::
elementAtPosition (IPosition position) const
{ return ivBase.elementAtPosition (position);
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
setToLast (ICursor &cursor) const
{ return ivBase.setToLast (cursor);
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
setToPrevious (ICursor &cursor) const
{ return ivBase.setToPrevious (cursor);
}

template < class Element, class Base >
void IRSequence < Element, Base >::
setToPosition (IPosition position, ICursor &cursor) const
{ ivBase.setToPosition (position, cursor);
}

#ifndef __BIN_COMPAT_BASE_43__
template < class Element, class Base >
IPosition IRSequence < Element, Base >::
position (ICursor const& cursor) const
{ return ivBase.position (cursor);
}
#endif

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
isFirst (ICursor const& cursor) const
{ return ivBase.isFirst (cursor);
}

template < class Element, class Base >
IBoolean IRSequence < Element, Base >::
isLast (ICursor const& cursor) const
{ return ivBase.isLast (cursor);
}

template < class Element, class Base >
void IRSequence < Element, Base >::
addAsFirst (Element const& element)
{
   if (hasObservers()) {
      addAsFirstNotify(element);
      }
   else
      ivBase.addAsFirst(element);
}

template < class Element, class Base >
void IRSequence < Element, Base >::
addAsFirst (Element const& element, ICursor &cursor)
{  ivBase.addAsFirst (element, cursor);
   if (hasObservers()) {
      notifyAdded(cursor);
      }
}

template < class Element, class Base >
void IRSequence < Element, Base >::
addAsLast (Element const& element)
{
   if (hasObservers()) {
      addAsLastNotify(element);
      }
   else
      ivBase.addAsLast(element);
}

template < class Element, class Base >
void IRSequence < Element, Base >::
addAsLast (Element const& element, ICursor &cursor)
{  ivBase.addAsLast (element, cursor);
   if (hasObservers()) {
      notifyAdded(cursor);
      }
}

template < class Element, class Base >
void IRSequence < Element, Base >::
addAsNext (Element const& element, ICursor &cursor)
{  ivBase.addAsNext (element, cursor);
   if (hasObservers()) {
      notifyAdded(cursor);
      }
}

template < class Element, class Base >
void IRSequence < Element, Base >::
addAsPrevious (Element const& element, ICursor &cursor)
{  ivBase.addAsPrevious (element, cursor);
   if (hasObservers()) {
      notifyAdded(cursor);
      }
}

template < class Element, class Base >
void IRSequence < Element, Base >::
addAtPosition (IPosition position, Element const& element)
{
   if (hasObservers()) {
      addAtPositionNotify(position, element);
      }
   else
      ivBase.addAtPosition(position,element);
}

template < class Element, class Base >
void IRSequence < Element, Base >::
addAtPosition (IPosition position,
               Element const& element,
               ICursor &cursor)
{  ivBase.addAtPosition (position, element, cursor);
   if (hasObservers()) {
      notifyAdded(cursor);
      }
}

template < class Element, class Base >
void IRSequence < Element, Base >::
sort (long (*comparisonFunction) (Element const&, Element const&))
{  ivBase.sort (comparisonFunction);
   if (hasObservers())
      notifyModified();
}

#pragma info (restore)
