/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <iptr.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)

template < class Element, class Base >
void* IRSet < Element, Base >::
identity () const
{ return (void*)&ivBase;
}

template < class Element, class Base >
IRSet < Element, Base >::
~IRSet ()
{
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
add (Element const& element)
{
   if (hasObserversOrShouldSendNotification()) {
      return addNotify(element);
      }
   else
      return ivBase.add (element);
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
add (Element const& element, ICursor &cursor)
{
if (hasObserversOrShouldSendNotification()) {
      IBoolean result = ivBase.add(element, cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }
   else
      return ivBase.add (element, cursor);
}

template <class Element , class Base >
void IRSet < Element , Base >::
addAllFrom (IACollection < Element > const& collection)
{ ICHECK (! isIdentical (collection),
          IIdenticalCollectionException, IIdenticalCollectionText)
   if (hasObserversOrShouldSendNotification()) {
      addAllFromNotify(collection);
      }
   else
      Inherited::addAllFrom(collection);
}

template <class Element , class Base >
void IRSet < Element , Base >::
copy (IACollection < Element > const& collection)
{ if (hasObserversOrShouldSendNotification()) {
     if (! isIdentical (collection)) {
       ivBase.removeAll ();
       IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
       forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
        }
       notifyModified();
     }
  }
  else
    Inherited::copy(collection);
}

template < class Element, class Base >
Element const& IRSet < Element, Base >::
elementAt (ICursor const& cursor) const
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element& IRSet < Element, Base >::
elementAt (ICursor const& cursor)
{ return ivBase.elementAt (cursor);
}

template < class Element, class Base >
Element const& IRSet < Element, Base >::
anyElement () const
{ return ivBase.anyElement ();
}

template < class Element, class Base >
void IRSet < Element, Base >::
removeAt (ICursor& cursor)
{
   if (hasObserversOrShouldSendNotification()) {
      notifyRemoved(cursor);
      };
ivBase.removeAt (cursor);
}

template < class Element, class Base >
INumber IRSet < Element, Base >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{
   if (hasObserversOrShouldSendNotification()) {
      INumber result =ivBase.removeAll (predicate, env);
      if (result > 0)
         notifyModified();
      return result;
      }
   else
return ivBase.removeAll (predicate, env);
}

template < class Element, class Base >
void IRSet < Element, Base >::
replaceAt (ICursor const& cursor, Element const& element)
{
   if (hasObserversOrShouldSendNotification()) {
      Element oldElement = ivBase.elementAt(cursor);
      ivBase.replaceAt (cursor, element);
      notifyReplaced(cursor,oldElement);
      }
   else
      ivBase.replaceAt (cursor, element);
}

template < class Element, class Base >
void IRSet < Element, Base >::
removeAll ()
{ ivBase.removeAll ();
   if (hasObserversOrShouldSendNotification()) {
      notifyModified();
      };
}

template < class Element, class Base >
INumber IRSet < Element, Base >::
removeAllOccurrences (Element const& element)
{
   disableAllNotifications();
   if (remove (element)) {
      enableAllNotifications();
      if (hasObserversOrShouldSendNotification()) {
         notifyModified();
        }
      return 1;
      }
   else {
      enableAllNotifications();
      return 0;
      }
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
isBounded () const
{ return ivBase.isBounded ();
}

template < class Element, class Base >
INumber IRSet < Element, Base >::
maxNumberOfElements () const
{ return ivBase.maxNumberOfElements ();
}

template < class Element, class Base >
INumber IRSet < Element, Base >::
numberOfElements () const
{ return ivBase.numberOfElements ();
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
isEmpty () const
{ return ivBase.isEmpty ();
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
isFull () const
{ return ivBase.isFull ();
}

template < class Element, class Base >
ICursor* IRSet < Element, Base >::
newCursor () const
{ return ivBase.newCursor ();
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
setToFirst (ICursor &cursor) const
{ return ivBase.setToFirst (cursor);
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
setToNext (ICursor &cursor) const
{ return ivBase.setToNext (cursor);
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element &, void*),
               void* environment)
{ return ivBase.allElementsDo (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
allElementsDo (IIterator <Element>& iterator)
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
allElementsDo (IBoolean (*iterationFunction) (Element const&, void*),
               void* environment) const
{ return ((Base const&)ivBase).allElementsDo
           (iterationFunction, environment);
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
allElementsDo (IConstantIterator <Element>& iterator) const
{ return ivBase.allElementsDo (iterator);
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
isConsistent () const
{ return ivBase.isConsistent ();
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
contains (Element const& element) const
{ return ivBase.contains (element);
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
locate (Element const& element, ICursor &cursor) const
{ return ivBase.locate (element, cursor);
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
locateOrAdd (Element const& element)
{
   if (hasObserversOrShouldSendNotification()) {
      return locateOrAddNotify(element);
      }
   else
      return ivBase.locateOrAdd (element);
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
locateOrAdd (Element const& element, ICursor &cursor)
{
   if (hasObserversOrShouldSendNotification()) {
      IBoolean result = ivBase.locateOrAdd(element,cursor);
      if (!result) {
         notifyAdded(cursor);
         }
      return result;
      }
   else
      return ivBase.locateOrAdd (element, cursor);
}

template < class Element, class Base >
IBoolean IRSet < Element, Base >::
remove (Element const& element)
{
   if (hasObserversOrShouldSendNotification()) {
      removeNotify(element);
      }
   return ivBase.remove (element);
}

template < class Element, class Base >
void IRSet < Element, Base >::
unionWith (IASet < Element > const& collection)
{
disableAllNotifications();
if (isIdentical (collection))
    ivBase.unionWith (ivBase);
  else
    addAllFrom (collection);
enableAllNotifications();
if (hasObserversOrShouldSendNotification())
   notifyModified();
}

template < class Element, class Base >
void IRSet < Element, Base >::
intersectionWith (IASet < Element > const& collection)
{ if (! isIdentical (collection))
  {
  disableAllNotifications();
  IAEqualityCollection < Element > const* env = &collection;
    removeAll (IAEqualityCollection < Element >::isNotContained, &env);
  enableAllNotifications();
  if (hasObserversOrShouldSendNotification())
      notifyModified();

  }
}

template < class Element, class Base >
void IRSet < Element, Base >::
differenceWith (IASet < Element > const& collection)
{
  disableAllNotifications();
if (! isIdentical (collection)) {
    IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
    forCursor (*cursor)
      remove (collection.elementAt (*cursor));
  }
  else
    removeAll ();
enableAllNotifications();
if (hasObserversOrShouldSendNotification())
   notifyModified();
}

template < class Element, class Base >
void IRSet < Element, Base >::
addUnion (IASet < Element > const& collection1,
          IASet < Element > const& collection2)
{
  disableAllNotifications();
unionWith (collection1);
  unionWith (collection2);
enableAllNotifications();
if (hasObserversOrShouldSendNotification())
   notifyModified();
}

template < class Element, class Base >
void IRSet < Element, Base >::
addIntersection (IASet < Element > const& collection1,
                 IASet < Element > const& collection2)
{ if (isIdentical (collection1) || isIdentical (collection2))
    return;
  else {
  disableAllNotifications();
    IAutoPointer < ICursor > cursor1 (collection1.newCursor (), IINIT);
    forCursor (*cursor1) {
      if (collection2.contains (collection1.elementAt (*cursor1)))
        add (collection1.elementAt (*cursor1));
    }
  }
  enableAllNotifications();
  if (hasObserversOrShouldSendNotification())
      notifyModified();
}

template < class Element, class Base >
void IRSet < Element, Base >::
addDifference (IASet < Element > const& collection1,
               IASet < Element > const& collection2)
{ if (isIdentical (collection1))
    return;
  else if (isIdentical (collection2)) {
     disableAllNotifications();
    unionWith (collection1);
    }
  else {
    disableAllNotifications();
    IAutoPointer < ICursor > cursor1 (collection1.newCursor (), IINIT);
    forCursor (*cursor1) {
      if (! collection2.contains (collection1.elementAt (*cursor1)))
        add (collection1.elementAt (*cursor1));
    }
  }
  enableAllNotifications();
  if (hasObserversOrShouldSendNotification())
      notifyModified();
}

#pragma info (restore)
