/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IRSET_H
#define _IRSET_H

#include <iaset.h>
#include <istdntfy.hpp>
#include <ipartccl.h>
#include <iptr.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

template < class Element, class Base >
class IRSet : public virtual IASet < Element >
                 ,  public IPartCollection < Element >
                                          {
  typedef IASet < Element > Inherited;

  Base &ivBase;
public:

           IRSet                (Base& base)
           : ivBase (base)
           {
           }

          ~IRSet                ();

private:
  inline IBoolean addNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.add(element,cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }

  inline void addAllFromNotify
     (IACollection <Element> const& collection) {
      IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
      forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
         }
      notifyModified();
      }

  inline void removeNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.locate(element,cursor);
      if (result)
         notifyRemoved(cursor);
      }

  inline IBoolean locateOrAddNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.locateOrAdd(element,cursor);
      if (!result) {
         notifyAdded(cursor);
         }
      return result;
      }
public:

  IBoolean add                  (Element const&);

  IBoolean add                  (Element const&, ICursor&);

  void     addAllFrom           (IACollection <Element> const&);

  void     addDifference        (IASet < Element > const&,
                                 IASet < Element > const&);

  void     addIntersection      (IASet < Element > const&,
                                 IASet < Element > const&);

  void     addUnion             (IASet < Element > const&,
                                 IASet < Element > const&);

  IBoolean allElementsDo        (IBoolean (*function) (Element&, void*),
                                 void* additionalArgument = 0);

  IBoolean allElementsDo        (IIterator <Element>&);

  IBoolean allElementsDo        (IBoolean (*function)
                                  (Element const&, void*),
                                 void* additionalArgument = 0) const;

  IBoolean allElementsDo        (IConstantIterator <Element>&) const;

  Element const&
           anyElement           () const;

  IBoolean contains             (Element const&) const;

  void
           copy                 (IACollection <Element> const&);

  void     differenceWith       (IASet < Element > const&);

  Element const&
           elementAt            (ICursor const&) const;

  Element& elementAt            (ICursor const&);

  void     intersectionWith     (IASet < Element > const&);

  IBoolean isBounded            () const;

  IBoolean isEmpty              () const;

  IBoolean isFull               () const;

  IBoolean locate               (Element const&, ICursor&) const;

  IBoolean locateOrAdd          (Element const&);

  IBoolean locateOrAdd          (Element const&, ICursor&);

  INumber  maxNumberOfElements  () const;

  ICursor* newCursor            () const;

  INumber  numberOfElements     () const;

  IBoolean remove               (Element const&);

  void     removeAll            ();

  INumber  removeAll            (IBoolean (*property)
                                  (Element const&, void*),
                                 void* additionalArgument = 0);

  INumber  removeAllOccurrences (Element const&);

  void     removeAt             (ICursor&);

  void     replaceAt            (ICursor const&, Element const&);

  IBoolean setToFirst           (ICursor&) const;

  IBoolean setToNext            (ICursor&) const;

  void     unionWith            (IASet < Element > const&);

  IBoolean isConsistent         () const;

protected:

  void* identity () const;

};

#ifndef __TEMPINC__
#include <irset.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
