/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _IRSTACK_H
#define _IRSTACK_H

#include <iastack.h>
#include <istdntfy.hpp>
#include <ipartccl.h>
#include <iptr.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

template < class Element, class Base >
class IRStack : public virtual IAStack < Element >
                 ,  public IPartCollection < Element >
                                          {
  typedef IAStack < Element > Inherited;

  Base &ivBase;
public:

           IRStack              (Base& base)
           : ivBase (base)
           {
           }

          ~IRStack              ();

private:
  inline IBoolean addNotify     (Element const& element) {
      Base::Cursor cursor(ivBase);
      IBoolean result = ivBase.add(element,cursor);
      if (result) {
         notifyAdded(cursor);
         }
      return result;
      }

  inline void addAsLastNotify       (Element const& element) {
      Base::Cursor cursor(ivBase);
      ivBase.addAsLast (element,cursor);
      notifyAdded(cursor);
      }

  inline void removeFirstNotify     () {
      if (ivBase.numberOfElements() > 0) {
         Base::Cursor cursor(ivBase);
         ivBase.setToFirst(cursor);
         notifyRemoved(cursor);
         }
      }

  inline void removeLastNotify      () {
      if (ivBase.numberOfElements() > 0) {
         Base::Cursor cursor(ivBase);
         ivBase.setToLast(cursor);
         notifyRemoved(cursor);
         }
      }

  inline void addAllFromNotify
     (IAStack <Element> const& collection) {
      IAutoPointer < ICursor > cursor (collection.newCursor (), IINIT);
      forCursor (*cursor) {
         ivBase.add (collection.elementAt (*cursor));
         }
      notifyModified();
      }
public:

  IBoolean add                  (Element const&);

  IBoolean add                  (Element const&, ICursor&);

  void     addAllFrom           (IAStack <Element> const&);

  void     addAsLast            (Element const&);

  void     addAsLast            (Element const&, ICursor&);

  IBoolean allElementsDo        (IBoolean (*function)
                                  (Element const&, void*),
                                 void* additionalArgument = 0) const;

  IBoolean allElementsDo        (IConstantIterator <Element>&) const;

  Element const&
           anyElement           () const;

  void
           copy                 (IAStack <Element> const&);

  Element const&
           elementAt            (ICursor const&) const;

  Element const&
           elementAtPosition    (IPosition) const;

  Element const&
           firstElement         () const;

  IBoolean isBounded            () const;

  IBoolean isEmpty              () const;

  IBoolean isFirst              (ICursor const&) const;

  IBoolean isFull               () const;

  IBoolean isLast               (ICursor const&) const;

  Element const&
           lastElement          () const;

  INumber  maxNumberOfElements  () const;

  ICursor* newCursor            () const;

  INumber  numberOfElements     () const;

  void     pop                  ();

  void     pop                  (Element&);

  void     push                 (Element const&);

  void     push                 (Element const&, ICursor&);

  void     removeAll            ();

  void     removeLast           ();

  IBoolean setToFirst           (ICursor&) const;

  IBoolean setToLast            (ICursor&) const;

  IBoolean setToNext            (ICursor&) const;

  void     setToPosition        (IPosition, ICursor&) const;

#ifndef __BIN_COMPAT_BASE_43__
  IPosition     position        (ICursor const&) const;
#endif

  IBoolean setToPrevious        (ICursor&) const;

  Element const&
           top                  () const;

  IBoolean isConsistent         () const;

protected:

  void*    identity () const;

};

#ifndef __TEMPINC__
#include <irstack.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
