/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _ISBKSS_H
#define _ISBKSS_H

#include <iglobals.h>
#include <icursor.h>
#include <ibexcept.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element >
class ISBKSElm {

public:

  Element ivElem;

  INumber ivOccurrences;

           ISBKSElm ()
           {
           }

           ISBKSElm (Element const& e)
           : ivElem (e), ivOccurrences (1)
           {
           }

           ISBKSElm (Element const& e, INumber occurrences)
           : ivElem (e), ivOccurrences (occurrences)
           {
           }

  friend inline
  Element const&
           key (ISBKSElm < Element > const& e)
           { return e.ivElem;
           }

};

template < class Element, class ElementOps, class Base >
class IWSrtBagOnKSSet;

template < class Element, class ElementOps, class Base >
class IWSrtBagOnKSSetCursor : public IOrderedCursor < Element > {
public:

           IWSrtBagOnKSSetCursor
             (IWSrtBagOnKSSet < Element, ElementOps, Base > const& c)
           : ivBaseCursor (c.ivBase)
           {
           }

  IBoolean setToFirst ();

  IBoolean setToNext  ();

  IBoolean setToLast ();

  IBoolean setToPrevious ();

  IBoolean isValid    () const;

  void     invalidate ();

  void     copy       (ICursor const& cursor);

  IBoolean operator== (ICursor const& cursor) const;

  Element const&
           element    () const;

  inline
  IBoolean operator== (IWSrtBagOnKSSetCursor
                        < Element,
                          ElementOps, Base > const& cursor) const;

  inline
  IBoolean operator!= (IWSrtBagOnKSSetCursor
                        < Element,
                          ElementOps, Base > const& cursor) const;

protected:

  INumber ivOccurrence;

  Base::Cursor ivBaseCursor;

  friend class IWSrtBagOnKSSet < Element, ElementOps, Base >;

};

template < class Element, class ElementOps, class Base >
class IWSrtBagOnKSSet {
protected:

  Base ivBase;

public:


  typedef IWSrtBagOnKSSetCursor < Element, ElementOps, Base > Cursor;

  inline   IWSrtBagOnKSSet (INumber numberOfElements = 100);

  inline   IWSrtBagOnKSSet (IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&);


  inline  ~IWSrtBagOnKSSet ();

  inline
  IBoolean add             (Element const&);

  inline
  IBoolean add             (Element const&, ICursor&);

  inline
  void     addAllFrom      (IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&);

  inline
  void     addDifference   (IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&,
                            IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&);

  inline
  void     addIntersection (IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&,
                            IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&);

  inline
  void     addUnion        (IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&,
                            IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&);

  inline
  IBoolean allElementsDo   (IBoolean (*function) (Element&, void*),
                            void* additionalArgument = 0);

  inline
  IBoolean allElementsDo   (IIterator <Element>&);

  inline
  IBoolean allElementsDo   (IBoolean (*function)
                             (Element const&, void*),
                            void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo   (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement      () const;

  inline
  long     compare         (IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&,
                            long (*comparisonFunction)
                             (Element const&, Element const&)) const;

  inline
  IBoolean contains        (Element const&) const;

  inline
  IBoolean containsAllFrom (IWSrtBagOnKSSet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  void     differenceWith  (IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&);

  inline
  Element const&
           elementAt       (ICursor const&) const;

  inline
  Element& elementAt       (ICursor const&);

  inline
  Element const&
           elementAtPosition (IPosition) const;

  inline
  Element const&
           firstElement    () const;

  inline
  void     intersectionWith (IWSrtBagOnKSSet
                              < Element, ElementOps, Base > const&);

  inline
  IBoolean isBounded       () const;

  inline
  IBoolean isEmpty         () const;

  inline
  IBoolean isFirst         (ICursor const&) const;

  inline
  IBoolean isFull          () const;

  inline
  IBoolean isLast          (ICursor const&) const;

  inline
  Element const&
           lastElement     () const;

  inline
  IBoolean locate          (Element const&, ICursor&) const;

  inline
  IBoolean locateOrAdd     (Element const&);

  inline
  IBoolean locateOrAdd     (Element const&, ICursor&);

  inline
  IBoolean locateNext      (Element const&, ICursor&) const;

  inline
  INumber  maxNumberOfElements () const;

  inline
  Cursor*  newCursor       () const;

  inline
  INumber  numberOfDifferentElements () const;

  inline
  INumber  numberOfElements () const;

  inline
  INumber  numberOfOccurrences (Element const&) const;

  inline
  IWSrtBagOnKSSet < Element, ElementOps, Base >&
           operator =      (IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&);

  inline
  IBoolean operator ==     (IWSrtBagOnKSSet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  IBoolean operator !=     (IWSrtBagOnKSSet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  IBoolean remove          (Element const&);

  inline
  void     removeAll       ();

  inline
  INumber  removeAll       (IBoolean (*property)
                             (Element const&, void*),
                            void* additionalArgument = 0);

  inline
  INumber  removeAllOccurrences (Element const&);

  inline
  void     removeAt        (ICursor&);

  inline
  void     removeAtPosition (IPosition);

  inline
  void     removeFirst     ();

  inline
  void     removeLast      ();

  inline
  void     replaceAt       (ICursor const&, Element const&);

  inline
  IBoolean setToFirst      (ICursor&) const;

  inline
  IBoolean setToLast       (ICursor&) const;

  inline
  IBoolean setToNext       (ICursor&) const;

  inline
  void     setToPosition   (IPosition, ICursor&) const;

#ifndef __BIN_COMPAT_BASE_43__
  inline
  IPosition     position   (ICursor const&) const;
#endif

  inline
  IBoolean setToPrevious   (ICursor&) const;

  inline
  IBoolean setToNextDifferentElement (ICursor&) const;

  inline
  void     unionWith       (IWSrtBagOnKSSet
                             < Element, ElementOps, Base > const&);

  friend class IWSrtBagOnKSSetCursor < Element, ElementOps, Base >;

  inline
  IBoolean isConsistent    () const;

protected:

  inline
  IBoolean add (Element const&, INumber);

  inline
  IBoolean add (Element const&, ICursor&, INumber);

  inline
  INumber  numberOfOccurrencesAt (ICursor const&) const;

  ElementOps ivElementOps;

  INumber ivNumberOfElements;

  struct RemoveEnv {
    IBoolean (*predicate) (Element const&, void*);
    void* env;
    INumber removeCount;
  };

  static
  IBoolean removePredicate (ISBKSElm < Element > const&, void*);

  static
  IBoolean cleanupPredicate (ISBKSElm < Element > const&, void*);

  // used for cfront support:
  inline
  IOrderedCursor < ISBKSElm < Element > >&
            baseCursor (Cursor&) const;

  inline
  IOrderedCursor < ISBKSElm < Element > > const&
            baseCursor (Cursor const&) const;

  inline
  IOrderedCursor < ISBKSElm < Element > >&
            baseCursor (ICursor&) const;

  inline
  IOrderedCursor < ISBKSElm < Element > > const&
            baseCursor (ICursor const&) const;

};

template < class Element, class ElementOps >
class ISBKSSOps {

  ElementOps elementOps;

public:

  void*    allocate   (size_t s
#ifdef __DEBUG_ALLOC__
                      , char const* fn, size_t ln
#endif
                      ) const
           { return elementOps.allocate (s
#ifdef __DEBUG_ALLOC__
                                        , fn, ln
#endif
                                        );
           }

  void     deallocate (void* p
#ifdef __DEBUG_ALLOC__
                      , char const* fn, size_t ln
#endif
                      ) const
           { elementOps.deallocate (p
#ifdef __DEBUG_ALLOC__
                                   , fn, ln
#endif
                                   );
           }

  void     assign (ISBKSElm < Element >& e1,
                   ISBKSElm < Element > const& e2) const
           { elementOps.assign (e1.ivElem, e2.ivElem);
             e1.ivOccurrences = e2.ivOccurrences;
           }

  Element const&
           key (ISBKSElm < Element > const& e) const
           { return e.ivElem;
           }

  ElementOps keyOps;

};


#define IDefineGSortedBagOnGKeySortedSet(GKSSet, GSBag) \
template < class Element, class ElementOps > \
class GSBag : \
  public IWSrtBagOnKSSet \
           < Element, ElementOps, \
             GKSSet < ISBKSElm < Element >, Element, \
                      ISBKSSOps < Element, ElementOps > > > { \
public: \
  GSBag (INumber n = 100) : \
    IWSrtBagOnKSSet \
      < Element, ElementOps, \
        GKSSet < ISBKSElm < Element >, Element, \
                 ISBKSSOps < Element, ElementOps > > > (n) {} \
};



#ifdef ICL_NO_INLINE
#undef inline
#else
#include <isbkss.inl>
#endif

#ifndef __TEMPINC__
#include <isbkss.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
