/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _ISETKS_H
#define _ISETKS_H

#include <iglobals.h>
#include <icursor.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class ElementOps, class Base >
class IWSetOnKeySet;

template < class Element, class ElementOps, class Base >
class IWSetOnKeySetCursor :
    public Base::Cursor
{
public:

           IWSetOnKeySetCursor
                     (IWSetOnKeySet
                       < Element, ElementOps, Base > const& c)
           : Base::Cursor (c.ivBase)
           {
           }

};

template < class Element, class ElementOps, class Base >
class IWSetOnKeySet {
protected:

  Base ivBase;

public:


  typedef IWSetOnKeySetCursor < Element, ElementOps, Base > Cursor;

  inline   IWSetOnKeySet   (INumber numberOfElements = 100);

  inline   IWSetOnKeySet   (IWSetOnKeySet
                             < Element, ElementOps, Base > const&);

  inline  ~IWSetOnKeySet   ();

  inline
  IBoolean add             (Element const&);

  inline
  IBoolean add             (Element const&, ICursor&);

  inline
  void     addAllFrom      (IWSetOnKeySet
                             < Element, ElementOps, Base > const&);

  inline
  void     addDifference   (IWSetOnKeySet
                             < Element, ElementOps, Base > const&,
                            IWSetOnKeySet
                             < Element, ElementOps, Base > const&);

  inline
  void     addIntersection (IWSetOnKeySet
                             < Element, ElementOps, Base > const&,
                            IWSetOnKeySet
                              < Element, ElementOps, Base > const&);

  inline
  void     addUnion        (IWSetOnKeySet
                             < Element, ElementOps, Base > const&,
                            IWSetOnKeySet
                              < Element, ElementOps, Base > const&);

  inline
  IBoolean allElementsDo   (IBoolean (*function) (Element&, void*),
                            void* additionalArgument = 0);

  inline
  IBoolean allElementsDo   (IIterator <Element>&);

  inline
  IBoolean allElementsDo   (IBoolean (*function)
                             (Element const&, void*),
                            void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo   (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement      () const;

  inline
  IBoolean contains        (Element const&) const;

  inline
  IBoolean containsAllFrom (IWSetOnKeySet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  void     differenceWith  (IWSetOnKeySet
                             < Element, ElementOps, Base > const&);

  inline
  Element const&
           elementAt       (ICursor const&) const;

  inline
  Element& elementAt       (ICursor const&);

  inline
  void     intersectionWith(IWSetOnKeySet
                             < Element, ElementOps, Base > const&);

  inline
  IBoolean isBounded       () const;

  inline
  IBoolean isEmpty         () const;

  inline
  IBoolean isFull          () const;

  inline
  IBoolean locate          (Element const&, ICursor&) const;

  inline
  IBoolean locateOrAdd     (Element const&);

  inline
  IBoolean locateOrAdd     (Element const&, ICursor&);

  inline
  INumber  maxNumberOfElements () const;

  inline
  INumber  numberOfElements () const;

  inline
  Cursor*  newCursor       () const;

  inline
  IWSetOnKeySet < Element, ElementOps, Base >&
           operator =      (IWSetOnKeySet
                             < Element, ElementOps, Base > const&);

  inline
  IBoolean operator ==     (IWSetOnKeySet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  IBoolean operator !=     (IWSetOnKeySet
                             < Element,
                               ElementOps, Base > const&) const;

  inline
  IBoolean remove          (Element const&);

  inline
  void     removeAll       ();

  inline
  INumber  removeAll       (IBoolean (*property)
                             (Element const&, void*),
                            void* additionalArgument = 0);

  inline
  void     removeAt        (ICursor&);

  inline
  void     replaceAt       (ICursor const&, Element const&);

  inline
  IBoolean setToFirst      (ICursor&) const;

  inline
  IBoolean setToNext       (ICursor&) const;

  inline
  void     unionWith       (IWSetOnKeySet
                             < Element, ElementOps, Base > const&);

  friend class IWSetOnKeySetCursor < Element, ElementOps, Base >;

  inline
  IBoolean isConsistent     () const;

protected:

  static
  IBoolean isContained      (Element const&, void* env);

  static
  IBoolean isNotContained   (Element const&, void* env);

  // used for cfront support:
  inline
  IElementCursor < Element >&
            baseCursor (ICursor&) const;

  inline
  IElementCursor < Element > const&
            baseCursor (ICursor const&) const;

};


#define IDefineGSetOnGKeySet(GKeySet, GSet) \
template < class Element, class ElementOps > \
class GSet : \
  public IWSetOnKeySet \
           < Element, ElementOps, \
             GKeySet < Element, Element, \
                       IOpsWithKey < Element, ElementOps > > > { \
public: \
  GSet (INumber n = 100) : \
    IWSetOnKeySet \
      < Element, ElementOps, \
        GKeySet < Element, Element, \
                  IOpsWithKey < Element, ElementOps > > > (n) {} \
};



#ifdef ICL_NO_INLINE
#undef inline
#else
#include <isetks.inl>
#endif

#ifndef __TEMPINC__
#include <isetks.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
