#ifndef _ISPINBT_
#define _ISPINBT_
/*******************************************************************************
* FILE NAME: ispinbt.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ISpinButton                                                              *
*                                                                              *
*     NOTE:  This class is obsolete and may be removed from the library in a   *
*            future release.  It has been replaced by:                         *
*                 IBaseSpinButton    - ispinbas.hpp                            *
*                 ITextSpinButton    - ispintxt.hpp                            *
*                 INumericSpinButton - ispinnum.hpp                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <icontrol.hpp>
#include <irect.hpp>
#include <istring.hpp>

class IColor;
class IRange;
class IResourceId;
class ISpinButtonData;
class ISpinCursorData;

#pragma pack(4)

#if (IC_OBSOLETE <= IC_OBSOLETE_1)

class ISpinButton : public IControl {
typedef IControl
  Inherited;
public:
/*----------------------------- Styles ---------------------------------------*/
INESTEDBITFLAGCLASSDEF2(Style, ISpinButton, IWindow, IControl);

static const Style
  classDefaultStyle,
  master,
  servant,
  allCharacters,
  numericOnly,
  readOnly,
  noBorder,
  fastSpin,
  padWithZeros;
static const Style
  leftAlign,
  centerAlign,
  rightAlign;

/*------------------------------ Related Types -------------------------------*/
enum Type {
  alphanumeric,
  numeric
};

enum Alignment {
  left,
  right,
  center
};

/*-------------------------- Style Functions ---------------------------------*/
Alignment
  alignment  ( ) const;

Type
  inputTypeAllowed ( ) const;

virtual ISpinButton
 &setReadOnly   ( Boolean   readOnly  = true ),
 &setFastSpin   ( Boolean   fast      = true ),
 &setInputType  ( Type      inputType = alphanumeric ),
 &enableBorder  ( Boolean   enable    = true ),
 &disableBorder ( );
virtual ISpinButton
 &setAlignment  ( Alignment alignment = left );

Boolean
  isMaster   ( ) const,
  isServant  ( ) const,
  isReadOnly ( ) const,
  isFastSpin ( ) const,
  isBorder   ( ) const;

static Style
  defaultStyle    ( );
static void
  setDefaultStyle ( const Style& style );

/*------------------------ Constructors --------------------------------------*/
  ISpinButton ( unsigned long     id,
                IWindow*          parent,
                IWindow*          owner,
                const IRectangle& initial = IRectangle(),
                const Style&      style   = defaultStyle() );

  ISpinButton ( unsigned long id,
                IWindow*      parent );

  ISpinButton ( const IWindowHandle& handle );

virtual
 ~ISpinButton ( );

/*----------------------------- Color Functions ------------------------------*/
virtual IColor
  foregroundColor ( ) const;

/*----------------------------- Limit and Spin -------------------------------*/
virtual ISpinButton
 &setMaster ( ISpinButton&  master ),
 &setLimit  ( unsigned long aNumber ),
 &spinDown  ( unsigned long spinBy  = 1 ),
 &spinUp    ( unsigned long spinBy  = 1 );

virtual Boolean
  isValid   ( Boolean updateContents = false ) const;

/*------------------------- Number Range Operations --------------------------*/
virtual ISpinButton
 &setCurrent ( long          aValue ),
 &spinTo     ( long          aValue ),
 &setRange   ( const IRange& aNewRange,
               Boolean       override = false );

virtual IRange
  range ( ) const;

virtual long
  value ( ) const;

virtual ISpinButton
 &setValue ( long aValue );

/*-------------------- Text Query, Validate and Remove -----------------------*/
virtual IString
  text ( ) const;

virtual ISpinButton
 &setText    ( const char*        string ),
 &setText    ( const IResourceId& item ),
 &removeAll  ( );


class Cursor : public IVBase {
public:
/*------------------ Constructors -------------------------------*/
  Cursor ( const ISpinButton& spinButton );

virtual
 ~Cursor ( );

/*------------------------ Cursor Movement ----------------------*/
virtual Boolean
  setToFirst    ( ),
  setToNext     ( ),
  setToPrevious ( ),
  setToLast     ( );

/*--------------- Cursor Validation -----------------------------*/
virtual Boolean
  isValid    ( ) const;
virtual void
  invalidate ( );

private:
/*--------------------------- Hidden Members ----------------------------*/
  Cursor    ( const Cursor& cursor );
Cursor
  &operator= ( const Cursor& cursor );

/*-------------------------- Private ----------------------------*/
friend class ISpinButton;


long
  lCurrent;
unsigned long
  ulCurCount;
const ISpinButton
 &rSpBut;
}; // ISpinButton::Cursor

/*-------------------------- Text List Operations ----------------------------*/
virtual IString
  elementAt   ( const Cursor& cursor ) const;

virtual ISpinButton
 &refreshText ( ),
 &removeAt    ( Cursor&            cursor,
                Boolean            immediateUpdate = false ),
 &replaceAt   ( const char*        newString,
                Cursor&            cursor,
                Boolean            immediateUpdate = false ),
 &replaceAt   ( const IResourceId& item,
                Cursor&            cursor,
                Boolean            immediateUpdate = false ),
 &add         ( const char*        string,
                Cursor&            cursor,
                Boolean            immediateUpdate = false ),
 &add         ( const IResourceId& item,
                Cursor&            cursor,
                Boolean            immediateUpdate = false ),
 &addAsNext   ( const char*        string,
                Cursor&            cursor,
                Boolean            immediateUpdate = false ),
 &addAsNext   ( const IResourceId& item,
                Cursor&            cursor,
                Boolean            immediateUpdate = false ),
 &addAsFirst  ( const char*        string,
                Cursor&            cursor,
                Boolean            immediateUpdate = false ),
 &addAsFirst  ( const IResourceId& item,
                Cursor&            cursor,
                Boolean            immediateUpdate = false ),
 &addAsLast   ( const char*        string,
                Cursor&            cursor,
                Boolean            immediateUpdate = false ),
 &addAsLast   ( const IResourceId& item,
                Cursor&            cursor,
                Boolean            immediateUpdate = false ),
 &setCurrent  ( const Cursor&      cursor ),
 &spinTo      ( const Cursor&      cursor );

/*--------------------------------- Override ---------------------------------*/
virtual ISpinButton
 &enable ( Boolean enableControl = true );

protected:
/*----------------------------- Layout Size ----------------------------------*/
virtual ISize
  calcMinimumSize ( ) const;

/*------------------------- Event Handling Implementation --------------------*/

private:
/*--------------------------------- Hidden Functions -------------------------*/
  ISpinButton ( const ISpinButton& );
ISpinButton
 &operator=   ( const ISpinButton& );

/*--------------------------------- Private ----------------------------------*/
friend class ISpinButton::Cursor;
friend class ISpinCursorData;
friend class ISpinDefaultHandler;

static Style
  currentDefaultStyle;
unsigned long
  findStringIndex ( const Cursor& cursor ) const;


unsigned long
  limit ( ) const;

Boolean
  bDelayedItems;
IString
  spinText;
unsigned long
  lCount,
  textLimit;
char
 **pList;

public:
/*--------------------- Obsolete Data and Functions --------------------------*/
enum ColorArea {
  foreground,
  disabledForeground,
  highlightForeground,
  border
  };
ISpinButton
 &setColor  ( ColorArea     value,
              const IColor& color );
IColor
  color     ( ColorArea value ) const;
};

INESTEDBITFLAGCLASSFUNCS( Style, ISpinButton);

#endif // IC_OBSOLETE

#pragma pack()

#endif // _ISPINBT_
