#ifndef _ISTATTXT_
  #define _ISTATTXT_
/*******************************************************************************
* FILE NAME: istattxt.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IStaticText                                                              *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <itextctl.hpp>
#include <icolor.hpp>
#include <irect.hpp>

#pragma pack(4)

class ISize;
class IStaticTextHandler;
class IStaticTextData;

class IStaticText : public ITextControl {
typedef ITextControl
  Inherited;
public:
class Style;
/*------------------------------- Constructors -------------------------------*/
  IStaticText ( unsigned long        id,
                IWindow*             parent,
                IWindow*             owner,
                const IRectangle&    initial = IRectangle(),
                const Style&         style   = defaultStyle() );

  IStaticText ( unsigned long        id,
                IWindow*             parent );

  IStaticText ( const IWindowHandle& handle );

virtual
 ~IStaticText ( );

/*---------------------------------- Styles ----------------------------------*/
INESTEDBITFLAGCLASSDEF2 ( Style,
                          IStaticText,
                          IWindow,
                          IControl );
static const Style
  classDefaultStyle,
  left,
  center,
  right,
  wordBreak,
  mnemonic,
  top,
  vertCenter,
  bottom,
  halftone,
  underscore,
  strikeout,
  fillBackground;

static Style
  defaultStyle      ( );

static void
  setDefaultStyle   ( const Style&    style );

virtual unsigned long
  convertToGUIStyle ( const IBitFlag& style,
                      Boolean         extendedOnly = false ) const;

/*------------------------------- Alignment ----------------------------------*/
enum Alignment {
  topLeft,
  topLeftWrapped,
  topCenter,
  topRight,
  centerLeft,
  centerCenter,
  centerRight,
  bottomLeft,
  bottomCenter,
  bottomRight
};

virtual IStaticText
 &setAlignment ( Alignment alignment = IStaticText::topLeft );

Alignment
  alignment    ( ) const;

/*------------------------- Style Functions ----------------------------------*/
virtual IStaticText
 &enableStrikeout       ( Boolean enable = true ),
 &disableStrikeout      ( ),
 &enableUnderscore      ( Boolean enable = true ),
 &disableUnderscore     ( ),
 &enableHalftone        ( Boolean enable = true ),
 &disableHalftone       ( ),
 &enableFillBackground  ( Boolean enable = true ),
 &disableFillBackground ( );

Boolean
  isStrikeout           ( ) const,
  isUnderscore          ( ) const,
  isHalftone            ( ) const,
  hasFillBackground     ( ) const;

/*---------------------------------- Colors ----------------------------------*/
virtual IColor
  foregroundColor ( ) const,
  backgroundColor ( ) const;

virtual IColor
  fillColor       ( ) const;

virtual IStaticText
 &setFillColor    ( const IColor& color ),
 &resetFillColor  ( );

/*-------------------------- Minimum Size Adjustment -------------------------*/
virtual IStaticText
 &setLimit ( unsigned long limit = 0 );

unsigned long
  limit    ( ) const;

/*------------------------------ Text Processing -----------------------------*/
virtual IStaticText
 &setText ( const char*        text ),
 &setText ( const IResourceId& text );

/*----------------------- Notification Event Descriptions --------------------*/
static INotificationId const
  strikeoutId,
  underscoreId,
  halftoneId,
  fillColorId,
  fillBackgroundId,
  limitId;

/*----------------------------- Window Positioning ---------------------------*/


protected:
/*------------------------------ Layout Support ------------------------------*/
virtual ISize
  calcMinimumSize ( ) const,
  calcLimitSize   ( ) const;

/*---------------------- Event-Handling Implementation -----------------------*/

private:
/*------------------------------ Hidden Members ------------------------------*/
  IStaticText ( const IStaticText& );
IStaticText
 &operator=   ( const IStaticText& );

void
  notify      ( INotificationId nId,
                Boolean enabled );

/*--------------------------------- Private ----------------------------------*/
friend class IStaticTextHandler;

static Style
  currentDefaultStyle;

unsigned long
  ulClLimit;

IStaticTextData
 *fStaticTextData;

#if (IC_OBSOLETE <= IC_OBSOLETE_1)
public:
/*----------------------------- Obsolete Members -----------------------------*/
enum ColorArea {
  foreground,
  background,
  fill
};

IStaticText
 &setColor ( ColorArea value, const IColor& color );

IColor
  color    ( ColorArea value ) const;
#endif
}; // class IStaticText

INESTEDBITFLAGCLASSFUNCS(Style, IStaticText);
                                  // global style functions

#pragma pack()

/*--------------------------------- Inlines ----------------------------------*/
#ifndef I_NO_INLINES
  #include <istattxt.inl>
#endif

#endif  // _ISTATTXT_
