/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _ISTDOPS_H
#define _ISTDOPS_H

#include <isynonym.h>
#include <inumpos.h>
#include <string.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

// indirection for function/operator application


template < class Element >
inline Element& elementForOps (Element& e)
{ return e; }

template < class Element >
inline Element const& elementForOps (Element const& e)
{ return e; }


// memory management

class IStdMemOps {
public:

  void* allocate   (size_t s
#ifdef __DEBUG_ALLOC__
                   , char const* fn, size_t ln
#endif
                   ) const
        {
            return ::operator new (s
#ifdef __DEBUG_ALLOC__
                                  , fn, ln
#endif
                                  );
        }

  void  deallocate (void* p
#ifdef __DEBUG_ALLOC__
                   ,char const* fn, size_t ln
#endif
                   ) const
        { ::operator delete (p
#ifdef __DEBUG_ALLOC__
                            , fn, ln
#endif
                            );
        }
};

// assignment

template < class Element >
inline void assign (Element& e1, Element const& e2)
{ e1 = e2;
}

template < class Element >
class IStdAsOps {
public:
  inline void assign (Element& e1, Element const& e2) const;
};

template < class Element >
inline void IStdAsOps < Element >::
assign (Element& e1, Element const& e2) const
{ ::assign (e1, e2);
}

// equality

template < class Element >
inline IBoolean equal (Element const& e1, Element const& e2)
{ IBoolean result = (e1 == e2);
  return result;
}

inline IBoolean equal (char const* e1, char const* e2)
{ IBoolean result = (strcmp (e1, e2) == 0);
  return result;
}

template < class Element >
class IStdEqOps {
public:
  inline IBoolean equal (Element const& e1, Element const& e2) const;
};

template < class Element >
inline IBoolean IStdEqOps < Element >::
equal (Element const& e1, Element const& e2) const
{ IBoolean result = ::equal (elementForOps (e1), elementForOps (e2));
  return result;
}

// ordering relation

template < class Element >
inline long compare (Element const& e1, Element const& e2)
{ if      (e1 < e2) return -1;
  else if (e2 < e1) return 1;
  else              return 0;
}

inline long compare (int e1, int e2)
{ long result = e1 - e2;
  return result;
}

inline long compare (char const* e1, char const* e2)
{ long result = strcmp (e1, e2);
  return result;
}

template < class Element >
class IStdCmpOps {
public:
  inline long compare (Element const& e1, Element const& e2) const;
};

template < class Element >
inline long IStdCmpOps < Element >::
compare (Element const& e1, Element const& e2) const
{ long result = ::compare (elementForOps (e1), elementForOps (e2));
  return result;
}

// hash function

inline unsigned long hash (int e, unsigned long n)
{ unsigned long result = (e % n);
  return result;
}

inline unsigned long hash (char const* e, unsigned long n)
{ size_t len = strlen (e);
  if (len == 0)
    return 0;
  else {
    unsigned long result = ((e [0] + e [len-1] + e [len/2]) % n);
    return result;
  }
}

template < class Element >
class IStdHshOps {
public:
  inline unsigned long hash (Element const& e, unsigned long n) const;
};

template < class Element >
inline unsigned long IStdHshOps < Element >::
hash (Element const& e, unsigned long n) const
{ unsigned long result = (::hash (elementForOps (e), n));
  return result;
}

// key selection

template < class Element, class Key >
class IStdKeyOps {
public:
  inline Key const& key (Element const& e) const;
};

template < class Element, class Key >
inline Key const& IStdKeyOps < Element, Key >::
key (Element const& e) const
{ return ::key (elementForOps (e));
}

// standard operation combinations

template < class Element >
class IStdOps : public IStdMemOps,
                public IStdAsOps < Element > {
};

template < class Element >
class IEOps : public IStdOps < Element >,
              public IStdEqOps < Element > {
};

template < class Element >
class IECOps : public IEOps < Element >,
               public IStdCmpOps < Element > {
};

template < class Element >
class IEHOps : public IEOps < Element >,
               public IStdHshOps < Element > {
};

template < class Key >
class IKCKeyOps : public IStdCmpOps < Key > {
public:
   IKCKeyOps() {};
};


template < class Element, class Key >
class IKCOps : public IStdOps < Element >,
               public IStdKeyOps < Element, Key > {
public:
  IKCOps () : keyOps() {};
  IKCKeyOps< Key > keyOps;
};

template < class Key >
class IKEHKeyOps : public IStdEqOps < Key >,
                   public IStdHshOps < Key > {
public:
  IKEHKeyOps () {};
};

template < class Element, class Key >
class IKEHOps : public IStdOps < Element >,
                public IStdKeyOps < Element, Key > {
public:
  friend class IKEHKeyOps<Key>;
  IKEHOps () : keyOps() {};
  IKEHKeyOps< Key > keyOps;
};

template < class Key >
class IEKCKeyOps : public IStdCmpOps < Key > {
public:
  IEKCKeyOps () {};
};

template < class Element, class Key >
class IEKCOps : public IEOps < Element >,
                public IStdKeyOps < Element, Key > {
public:
  friend class IEKCKeyOps<Key>;
  IEKCOps () : keyOps() {};
  IEKCKeyOps< Key > keyOps;
};

template < class Key >
class IEKEHKeyOps : public IStdEqOps < Key >,
                    public IStdHshOps < Key > {
public:
  IEKEHKeyOps () {};
};


template < class Element, class Key >
class IEKEHOps : public IEOps < Element >,
                 public IStdKeyOps < Element, Key > {
public:
  friend class IEKEHKeyOps<Key>;
  IEKEHOps () : keyOps() {};
  IEKEHKeyOps< Key > keyOps;
};

// definition macor for templates with given operation class


#define IDefineCollectionWithOps(IGCollection, Ops, ICollection) \
template < class Element > \
class ICollection : \
  public IGCollection < Element, Ops < Element > > { \
public: \
  ICollection (INumber n = 100) : \
    IGCollection < Element, Ops < Element > > (n) {} \
};

#define IDefineKeyCollectionWithOps(IGCollection, Ops, ICollection) \
template < class Element, class Key > \
class ICollection : \
  public IGCollection < Element, Key, Ops < Element, Key > > { \
public: \
  ICollection (INumber n = 100) : \
    IGCollection < Element, Key, Ops < Element, Key > > (n) {} \
};


#define IDefineStdCollectionWithOps(Collection) \
  IDefineCollectionWithOps \
  (IG ## Collection, I ## Collection ## Ops, I ## Collection)

#define IDefineStdKeyCollectionWithOps(Collection) \
  IDefineKeyCollectionWithOps \
  (IG ## Collection, I ## Collection ## Ops, I ## Collection)

// key extension

template < class Element, class ElementOps >
class IOpsWithKey {
  ElementOps elementOps;
public:


  void*          allocate   (size_t s
#ifdef __DEBUG_ALLOC__
                            , char const* fn, size_t ln
#endif
                            ) const
  { return elementOps.allocate (s
#ifdef __DEBUG_ALLOC__
                            , fn, ln
#endif
                            );
  }
  void           deallocate (void* p
#ifdef __DEBUG_ALLOC__
                            , char const* fn, size_t ln
#endif
                            ) const
  { elementOps.deallocate (p
#ifdef __DEBUG_ALLOC__
                          , fn, ln
#endif
                          );
  }

  void           assign (Element& e1, Element const& e2) const
  { elementOps.assign (e1, e2);
  }

  Element const& key (Element const& e) const
  { return e;
  }
  ElementOps keyOps;
};

#pragma info (restore)
#pragma pack ()

#endif
