/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <new.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)


#ifdef ICL_NO_INLINE
#define inline
#include <itbseq.inl>
#undef inline
#endif

// ********************** IGTabularSequence::Cursor ********************

template <class Element, class ElementOps>
IBoolean IGTabularSequenceCursor <Element, ElementOps>::setToFirst ()
{ return ivCollection->setToFirst (*this);
}

template <class Element, class ElementOps>
IBoolean IGTabularSequenceCursor <Element, ElementOps>::setToNext ()
{ return ivCollection->setToNext (*this);
}

template <class Element, class ElementOps>
IBoolean IGTabularSequenceCursor <Element, ElementOps>::isValid () const
{ return ivImpl.ivIsValid;
}

template <class Element, class ElementOps>
void IGTabularSequenceCursor <Element, ElementOps>::invalidate ()
{ ivImpl.ivIsValid = False;
}

template <class Element, class ElementOps>
void IGTabularSequenceCursor <Element, ElementOps>::
copy (ICursor const& cursor)
{ operator= ((IGTabularSequenceCursor
                < Element, ElementOps> const&) cursor);
}

template <class Element, class ElementOps>
IBoolean IGTabularSequenceCursor <Element, ElementOps>::
operator== (ICursor const& cursor) const
{ return operator== ((IGTabularSequenceCursor
                        < Element, ElementOps> const&) cursor);
}

template <class Element, class ElementOps>
Element const& IGTabularSequenceCursor <Element, ElementOps>::
element () const
{ return ivCollection->elementAt (*this);
}

template <class Element, class ElementOps>
IBoolean IGTabularSequenceCursor <Element, ElementOps>::setToLast ()
{ return ivCollection->setToLast (*this);
}

template <class Element, class ElementOps>
IBoolean IGTabularSequenceCursor <Element, ElementOps>::setToPrevious ()
{ return ivCollection->setToPrevious (*this);
}

// ************************** IGTabularSequence ************************

template < class Element, class ElementOps >
void IGTabularSequence < Element, ElementOps >::
copyFrom (INumber index, void const* element) const
{ ivElementOps.assign
    (((Node*)ivTable.ivNodes)
      [index].ivElement, *(Element const*)element);
}

template < class Element, class ElementOps >
void const* IGTabularSequence < Element, ElementOps >::
elementAt (INumber index) const
{ return &((Node*)ivTable.ivNodes) [index].ivElement;
}

template < class Element, class ElementOps >
void IGTabularSequence < Element, ElementOps >::
swap (INumber index1, INumber index2) const
{ Element e;
  ivElementOps.assign (e, ((Node*)ivTable.ivNodes) [index1].ivElement);
  ivElementOps.assign
    (((Node*)ivTable.ivNodes) [index1].ivElement,
     ((Node*)ivTable.ivNodes) [index2].ivElement);
  ivElementOps.assign (((Node*)ivTable.ivNodes) [index2].ivElement, e);
}

template < class Element, class ElementOps >
IBoolean IGTabularSequence < Element, ElementOps >::
constantFunctionIteration (void *iterationFunction,
                           void* env,
                           INumber index) const
{ return (*(IBoolean (*) (Element const&, void*)) iterationFunction)
           (((Node*)ivTable.ivNodes) [index].ivElement, env);
}

template < class Element, class ElementOps >
IBoolean IGTabularSequence < Element, ElementOps >::
functionIteration (void *iterationFunction, void* env, INumber index)
{ return (*(IBoolean (*) (Element&, void*)) iterationFunction)
           (((Node*)ivTable.ivNodes) [index].ivElement, env);
}

template < class Element, class ElementOps >
IBoolean IGTabularSequence < Element, ElementOps >::
constantIteratorIteration (void* iterator, INumber index) const
{ return ((IConstantIterator < Element >*)iterator)->
           applyTo (((Node*)ivTable.ivNodes) [index].ivElement);
}

template < class Element, class ElementOps >
IBoolean IGTabularSequence < Element, ElementOps >::
iteratorIteration (void* iterator, INumber index)
{ return ((IIterator < Element >*)iterator)->
           applyTo (((Node*)ivTable.ivNodes) [index].ivElement);
}

template < class Element, class ElementOps >
long IGTabularSequence < Element, ElementOps >::
functionComparison (void *comparisonFunction,
                    void const* element1,
                    void const* element2) const
{ return (*(long (*) (Element const&, Element const&) const)
            comparisonFunction)
           (*(Element*)element1, *(Element*)element2);
}

template < class Element, class ElementOps >
long IGTabularSequence < Element, ElementOps >::
functionComparison (void *comparisonFunction,
                    INumber index1,
                    INumber index2) const
{ return (*(long (*) (Element const&, Element const&) const)
            comparisonFunction)
           (((Node*)ivTable.ivNodes) [index1].ivElement,
            ((Node*)ivTable.ivNodes) [index2].ivElement);
}

template < class Element, class ElementOps >
void IGTabularSequence < Element, ElementOps >::
blockLeft (INumber lTarget, INumber lMoveCount)
{ for (INumber i = lTarget; lMoveCount > 0; i++, lMoveCount--)
    ((Node*)ivTable.ivNodes) [i] = ((Node*)ivTable.ivNodes) [i+1];
}

template < class Element, class ElementOps >
void IGTabularSequence < Element, ElementOps >::
blockRight (INumber lTarget, INumber lMoveCount)
{ for (INumber i = lTarget + lMoveCount - 1;
       lMoveCount > 0;
       i--, lMoveCount--)
    ((Node*)ivTable.ivNodes) [i] = ((Node*)ivTable.ivNodes) [i-1];
}

template < class Element, class ElementOps >
void IGTabularSequence < Element, ElementOps >::
copy (ITabularSequenceImpl::Table const& from)
{ INumber toIndex = ivTable.ivLast;
  INumber fromIndex = from.ivFirst;
  for (INumber i = 0; i < from.ivNumberOfElements; i++) {
    ivTable.incMod (toIndex);
    ((Node*)ivTable.ivNodes) [toIndex] =
      ((Node*)from.ivNodes) [fromIndex];
    from.incMod (fromIndex);
  }
  ivTable.ivNumberOfElements += from.ivNumberOfElements;
  ivTable.ivLast = toIndex;
}

template < class Element, class ElementOps >
void IGTabularSequence < Element, ElementOps >::
expand (INumber n)
{ n += ivTable.ivNumberOfElements;

  ITabularSequenceImpl::Table tab = ivTable;

  INumber newAllocatedElements = ivTable.ivAllocatedElements;
  while (newAllocatedElements < n)
    newAllocatedElements *= 2;

  newNodes (newAllocatedElements);

  ivTable.ivFirst = 0;
  ivTable.ivLast = ivTable.ivAllocatedElements - 1;
  ivTable.ivNumberOfElements = 0;
  copy (tab);
  for (INumber i = 0; i < tab.ivAllocatedElements; i++)
    ((Node*) tab.ivNodes) [i].~Node ();
  ivElementOps.deallocate
                      (tab.ivNodes
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
}

template < class Element, class ElementOps >
void IGTabularSequence < Element, ElementOps >::
newNodes (INumber n)
{ void* nodeSpace = ivElementOps.allocate
                      (sizeof (Node) * n
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
  ICHECK (nodeSpace != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  ivTable.ivAllocatedElements = n;
  (Node*&)ivTable.ivNodes = (Node*)nodeSpace;
  for (INumber i = 0; i < n; i++)
    new (&((Node*)nodeSpace) [i]) Node;
}

template < class Element, class ElementOps >
void IGTabularSequence < Element, ElementOps >::
deleteNodes ()
{ for (INumber i = 0; i < ivTable.ivAllocatedElements; i++)
    ((Node*) ivTable.ivNodes) [i].~Node ();
  ivElementOps.deallocate
                      (ivTable.ivNodes
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
}

#pragma info (restore)
