/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _ITBSEQ_H
#define _ITBSEQ_H

#include <iglobals.h>
#include <icursor.h>
#include <iitbseq.h>
#include <ibexcept.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class ElementOps >
class IGTabularSequence;

template < class Element, class ElementOps >
class IGTabularSequenceNode;

template < class Element, class ElementOps >
class IGTabularSequenceCursor :
public IOrderedCursor < Element >
{
public:

           IGTabularSequenceCursor (IGTabularSequence
           < Element, ElementOps > const& c)
           : ivCollection (&c)
           {
           }

  IBoolean setToFirst ();

  IBoolean setToNext ();

  IBoolean isValid () const;

  void     invalidate ();

  void     copy (ICursor const& cursor);

  IBoolean operator== (ICursor const& cursor) const;

  Element const&
           element () const;

  IBoolean setToLast ();

  IBoolean setToPrevious ();

  inline
  IBoolean operator== (IGTabularSequenceCursor
                        < Element, ElementOps > const& cursor) const
           { return ivCollection == cursor.ivCollection &&
                    ivImpl == cursor.ivImpl;
           }

  inline
  IBoolean operator!= (IGTabularSequenceCursor
                        < Element, ElementOps > const& cursor) const
           { return ! operator== (cursor);
           }

protected:

  IGTabularSequence < Element, ElementOps > const*
           ivCollection;

  ITabularSequenceImplCursor
           ivImpl;

  IBoolean isFor (IGTabularSequence < Element,
                                      ElementOps > const& c) const
           { return ivCollection == &c;
           }

  friend class IGTabularSequence < Element, ElementOps >;

};

template < class Element, class ElementOps >
class IGTabularSequenceNode {

    Element  ivElement;

             IGTabularSequenceNode ()
             {
             }

            ~IGTabularSequenceNode ()
             {
             }

    friend class IGTabularSequence < Element, ElementOps >;

    friend class IGTabularSequenceCursor < Element, ElementOps >;


};


template < class Element, class ElementOps >
class IGTabularSequence : protected ITabularSequenceImpl {

public:


  typedef IGTabularSequenceCursor < Element, ElementOps > Cursor;

  inline   IGTabularSequence (INumber numberOfElements = 100);

  inline   IGTabularSequence (IGTabularSequence < Element,
                                                  ElementOps > const&);

  inline  ~IGTabularSequence ();

  inline
  IBoolean add               (Element const&);

  inline
  IBoolean add               (Element const&, ICursor&);

  inline
  void     addAllFrom        (IGTabularSequence < Element,
                                                  ElementOps > const&);

  inline
  void     addAsFirst        (Element const&);

  inline
  void     addAsFirst        (Element const&, ICursor&);

  inline
  void     addAsLast         (Element const&);

  inline
  void     addAsLast         (Element const&, ICursor&);

  inline
  void     addAsNext         (Element const&, ICursor&);

  inline
  void     addAsPrevious     (Element const&, ICursor&);

  inline
  void     addAtPosition     (IPosition, Element const&);

  inline
  void     addAtPosition     (IPosition, Element const&, ICursor&);

  inline
  IBoolean allElementsDo     (IBoolean (*function) (Element&, void*),
                              void* additionalArgument = 0);

  inline
  IBoolean allElementsDo     (IIterator <Element>&);

  inline
  IBoolean allElementsDo     (IBoolean (*function)
                               (Element const&, void*),
                              void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo     (IConstantIterator <Element>&) const;

  inline
  Element const&
           anyElement        () const;

  inline
  long     compare           (IGTabularSequence < Element,
                                                  ElementOps > const&,
                              long (*comparisonFunction)
                               (Element const&, Element const&)) const;

  inline
  Element const&
           elementAt         (ICursor const&) const;

  inline
  Element& elementAt         (ICursor const&);

  inline
  Element const&
           elementAtPosition (IPosition) const;

  inline
  Element const&
           firstElement      () const;

  inline
  IBoolean isBounded         () const;

  inline
  IBoolean isEmpty           () const;

  inline
  IBoolean isFirst           (ICursor const&) const;

  inline
  IBoolean isFull            () const;

  inline
  IBoolean isLast            (ICursor const&) const;

  inline
  Element const&
           lastElement       () const;

  inline
  INumber  maxNumberOfElements () const;

  inline
  INumber  numberOfElements  () const;

  inline
  Cursor*  newCursor         () const;

  inline
  IGTabularSequence < Element, ElementOps >&
           operator =        (IGTabularSequence < Element,
                                                  ElementOps > const&);

  inline
  void     removeAll         ();

  inline
  INumber  removeAll         (IBoolean (*property)
                               (Element const&, void*),
                              void* additionalArgument = 0);

  inline
  void     removeAt          (ICursor&);

  inline
  void     removeAtPosition  (IPosition);

  inline
  void     removeFirst       ();

  inline
  void     removeLast        ();

  inline
  void     replaceAt         (ICursor const&, Element const&);

  inline
  IBoolean setToFirst        (ICursor&) const;

  inline
  IBoolean setToLast         (ICursor&) const;

  inline
  IBoolean setToNext         (ICursor&) const;

  inline
  void     setToPosition     (IPosition, ICursor&) const;

#ifndef __BIN_COMPAT_BASE_43__
  inline
  IPosition     position     (ICursor const&) const;
#endif

  inline
  IBoolean setToPrevious     (ICursor&) const;

  inline
  void     sort              (long (*comparisonFunction)
                               (Element const&, Element const&));

  friend class IGTabularSequenceCursor < Element, ElementOps >;

  inline
  IBoolean isConsistent      () const;

protected:

  typedef IGTabularSequenceNode < Element, ElementOps > Node;

  ElementOps ivElementOps;

  void     swap              (INumber index1, INumber index2) const;

  void     copyFrom          (INumber index, void const* element) const;

  void const*
           elementAt         (INumber index) const;

  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      INumber index) const;

  IBoolean functionIteration (void *function, void* env, INumber index);

  IBoolean constantIteratorIteration (void* iterator,
                                      INumber index) const;

  IBoolean iteratorIteration (void* iterator, INumber index);

  long     functionComparison (void *compareFunction,
                               void const* element1,
                               void const* element2) const;

  long     functionComparison (void *compareFunction,
                               INumber index1,
                               INumber index2) const;

  void     blockLeft         (INumber target, INumber moveCount);

  void     blockRight        (INumber target, INumber moveCount);

  void     copy              (ITabularSequenceImpl::Table const&);

  void     expand            (INumber n);

  void     newNodes          (INumber n);

  void     deleteNodes       ();

};

template < class Element >
class ITabularSequence :
  public IGTabularSequence < Element, IStdOps < Element > > {
public:


  inline   ITabularSequence (INumber n = 100);
};


#ifdef ICL_NO_INLINE
#undef inline
#else
#include <itbseq.inl>
#endif

#ifndef __TEMPINC__
#include <itbseq.c>
#endif

#pragma info (restore)
#pragma pack ()

#endif
