/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

// ************************** IGTabularSequence ************************

template <class Element, class ElementOps>
inline IGTabularSequence <Element, ElementOps>::
IGTabularSequence (INumber n)
{ if (n == 0) n++;
  newNodes (n);
  ivTable.ivLast = ivTable.ivAllocatedElements - 1;
  ivTable.ivFirst = ivTable.ivNumberOfElements = 0;
}

template <class Element, class ElementOps>
inline IGTabularSequence <Element, ElementOps>::
IGTabularSequence (IGTabularSequence < Element,
                                       ElementOps > const& sequence)
{ newNodes (sequence.ivTable.ivAllocatedElements);
  ivTable.ivLast = ivTable.ivAllocatedElements - 1;
  ivTable.ivFirst = ivTable.ivNumberOfElements = 0;
  copy (sequence.ivTable);
}

template <class Element, class ElementOps>
inline IGTabularSequence <Element, ElementOps>::
~IGTabularSequence ()
{ deleteNodes ();
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
addAsLast (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  ITabularSequenceImpl::addAsLast (&element, ((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
addAsLast (Element const& element)
{ Cursor cursor (*this);
  addAsLast (element, cursor);
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
add (Element const& element, ICursor& cursor)
{ addAsLast (element, cursor);
  return True;
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
add (Element const& element)
{ Cursor cursor (*this);
  return add (element, cursor);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
addAllFrom (IGTabularSequence < Element, ElementOps > const& sequence)
{
#ifndef INO_CHECKS
  ICHECK (this != &sequence,
          IIdenticalCollectionException, IIdenticalCollectionText)
#endif
  ITabularSequenceImpl::addAllFrom (sequence);
}

template <class Element, class ElementOps>
inline Element const& IGTabularSequence <Element, ElementOps>::
elementAt (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ITabularSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Node*)ivTable.ivNodes)
           [((Cursor const&)cursor).ivImpl.ivIndex].ivElement;
}

template <class Element, class ElementOps>
inline Element& IGTabularSequence <Element, ElementOps>::
elementAt (ICursor const& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ITabularSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Node*) ivTable.ivNodes)
           [((Cursor const&)cursor).ivImpl.ivIndex].ivElement;
}

template <class Element, class ElementOps>
inline Element const& IGTabularSequence <Element, ElementOps>::
anyElement () const
{ return firstElement ();
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
removeAt (ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ITabularSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  INumber index = ((Cursor&)cursor).ivImpl.ivIndex;
  ITabularSequenceImpl::removeAt (index);
  cursor.invalidate ();
}

template < class Element, class ElementOps >
inline INumber IGTabularSequence < Element, ElementOps >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{ return ITabularSequenceImpl::removeAll (predicate, env);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
replaceAt (ICursor const& cursor, Element const& element)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ITabularSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ivElementOps.assign
    (((Node*) ivTable.ivNodes)
       [((Cursor const&)cursor).ivImpl.ivIndex].ivElement,
     element);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
removeAll ()
{ ivTable.ivLast = ivTable.ivAllocatedElements - 1;
  ivTable.ivFirst = ivTable.ivNumberOfElements = 0;
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
isBounded () const
{ return False;
}

template <class Element, class ElementOps>
inline INumber IGTabularSequence <Element, ElementOps>::
maxNumberOfElements () const
{ ICHECK (False, INotBoundedException,
          INotBoundedText)
  return 0;
}

template <class Element, class ElementOps>
inline INumber IGTabularSequence <Element, ElementOps>::
numberOfElements () const
{ return ivTable.ivNumberOfElements;
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
isEmpty () const
{ return ivTable.ivNumberOfElements == 0;
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
isFull () const
{ return False;
}

template <class Element, class ElementOps>
inline IGTabularSequenceCursor <Element, ElementOps>*
IGTabularSequence <Element, ElementOps>::
newCursor () const
{ Cursor* result = new Cursor (*this);
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
setToFirst (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  return ITabularSequenceImpl::setToFirst (((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
setToNext (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ITabularSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularSequenceImpl::setToNext (((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
allElementsDo (IBoolean (*function) (Element&, void*), void* env)
{ return ITabularSequenceImpl::allElementsDo (function, env);
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
allElementsDo (IIterator < Element > & iterator)
{ return ITabularSequenceImpl::allElementsDo (&iterator);
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
allElementsDo (IBoolean (*function) (Element const&, void*),
               void* env) const
{ return ITabularSequenceImpl::allElementsDo (function, env);
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
allElementsDo (IConstantIterator < Element > & iterator) const
{ return ITabularSequenceImpl::allElementsDo (&iterator);
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
isConsistent () const
{ return ITabularSequenceImpl::isConsistent ();
}

template <class Element, class ElementOps>
inline IGTabularSequence <Element, ElementOps>&
IGTabularSequence <Element, ElementOps>::
operator = (IGTabularSequence < Element, ElementOps > const& sequence)
{ ITabularSequenceImpl::operator= (sequence);
  return *this;
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
removeFirst ()
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  INumber index = ivTable.ivFirst;
  ITabularSequenceImpl::removeAt (index);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
removeLast ()
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  INumber index = ivTable.ivLast;
  ITabularSequenceImpl::removeAt (index);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
removeAtPosition (IPosition position)
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
  ITabularSequenceImpl::Cursor cursor;
  ITabularSequenceImpl::setToPosition (position, cursor);
  ITabularSequenceImpl::removeAt (cursor.ivIndex);
}

template <class Element, class ElementOps>
inline Element const& IGTabularSequence <Element, ElementOps>::
firstElement () const
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  return ((Node*) ivTable.ivNodes) [ivTable.ivFirst].ivElement;
}

template <class Element, class ElementOps>
inline Element const& IGTabularSequence <Element, ElementOps>::
lastElement () const
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  return ((Node*) ivTable.ivNodes) [ivTable.ivLast].ivElement;
}

template <class Element, class ElementOps>
inline Element const& IGTabularSequence <Element, ElementOps>::
elementAtPosition (IPosition position) const
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
  ITabularSequenceImpl::Cursor cursor;
  ITabularSequenceImpl::setToPosition (position, cursor);
  return ((Node*) ivTable.ivNodes) [cursor.ivIndex].ivElement;
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
setToLast (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  return ITabularSequenceImpl::setToLast (((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
setToPrevious (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ITabularSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularSequenceImpl::setToPrevious (((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
setToPosition (IPosition position, ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  ITabularSequenceImpl::setToPosition
    (position, ((Cursor&)cursor).ivImpl);
}

#ifndef __BIN_COMPAT_BASE_43__
template <class Element, class ElementOps>
inline IPosition IGTabularSequence <Element, ElementOps>::
position (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
  return ITabularSequenceImpl::position
    (((Cursor&)cursor).ivImpl);
}
#endif

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
isFirst (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ITabularSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Cursor const&)cursor).ivImpl.ivIndex == ivTable.ivFirst;
}

template <class Element, class ElementOps>
inline IBoolean IGTabularSequence <Element, ElementOps>::
isLast (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ITabularSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Cursor const&)cursor).ivImpl.ivIndex == ivTable.ivLast;
}

template <class Element, class ElementOps>
inline long IGTabularSequence <Element, ElementOps>::
compare (IGTabularSequence < Element, ElementOps > const& sequence,
         long (*comparisonFunction)
           (Element const&, Element const&)) const
{
  return ITabularSequenceImpl::compare (sequence, comparisonFunction);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
addAsFirst (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  ITabularSequenceImpl::addAsFirst (&element, ((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
addAsFirst (Element const& element)
{ Cursor cursor (*this);
  addAsFirst (element, cursor);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
addAsNext (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ITabularSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ITabularSequenceImpl::addAsNext (&element, ((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
addAsPrevious (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (ITabularSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ITabularSequenceImpl::addAsPrevious
    (&element, ((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
addAtPosition (IPosition position, Element const& element)
{ Cursor cursor (*this);
  addAtPosition (position, element, cursor);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
addAtPosition (IPosition position,
               Element const& element,
               ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements() + 1,
          IPositionInvalidException, IInvalidPositionText)
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  ITabularSequenceImpl::addAtPosition
    (position, &element, ((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGTabularSequence <Element, ElementOps>::
sort (long (*comparisonFunction) (Element const&, Element const&))
{ ITabularSequenceImpl::sort (comparisonFunction);
}

// ************************** ITabularSequence *************************

template <class Element>
inline ITabularSequence < Element >::
ITabularSequence (INumber n)
: IGTabularSequence < Element, IStdOps < Element > > (n)
{
}
