/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <new.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)


#ifdef ICL_NO_INLINE
#define inline
#include <itbtre.inl>
#undef inline
#endif

// *********************** IGTabularTree::Cursor **********************

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTreeCursor < Element, ElementOps, numOfChildren >::
setToChild (IPosition position)
{ return ivCollection->setToChild (position, *this);
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTreeCursor < Element, ElementOps, numOfChildren >::
setToParent ()
{ return ivCollection->setToParent (*this);
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTreeCursor < Element, ElementOps, numOfChildren >::
setToRoot ()
{ return ivCollection->setToRoot (*this);
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTreeCursor < Element, ElementOps, numOfChildren >::
setToFirstExistingChild ()
{ return ivCollection->setToFirstExistingChild (*this);
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTreeCursor < Element, ElementOps, numOfChildren >::
setToLastExistingChild ()
{ return ivCollection->setToLastExistingChild (*this);
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTreeCursor < Element, ElementOps, numOfChildren >::
setToNextExistingChild ()
{ return ivCollection->setToNextExistingChild (*this);
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTreeCursor < Element, ElementOps, numOfChildren >::
setToPreviousExistingChild ()
{ return ivCollection->setToPreviousExistingChild (*this);
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTreeCursor < Element, ElementOps, numOfChildren >::
isValid () const
{ return ivNode != 0;
}

template < class Element, class ElementOps, INumber numOfChildren >
void IGTabularTreeCursor < Element, ElementOps, numOfChildren >::
invalidate ()
{ ivNode = 0;
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTreeCursor < Element, ElementOps, numOfChildren >::
operator== (ITreeCursor const& cursor) const
{ return operator== ((IGTabularTreeCursor
                        < Element,
                          ElementOps, numOfChildren >const&) cursor);
}

template < class Element, class ElementOps, INumber numOfChildren >
Element const&
IGTabularTreeCursor < Element, ElementOps, numOfChildren >::
element () const
{ return ivCollection->elementAt (*this);
}

template < class Element, class ElementOps, INumber numOfChildren >
void IGTabularTree < Element, ElementOps, numOfChildren >::
copySubtree (Node*& to, Node* from)
{ IASSERT (from != 0);
  to = newNode (from->ivElement);
  for (INumber n = 0; n < numOfChildren; n++) {
    if (from->ivChildren [n] == 0)
      to->ivChildren [n] = 0;
    else {
      copySubtree (to->ivChildren [n], from->ivChildren [n]);
      to->ivChildren [n]->ivParent = to;
    }
  }
}

template < class Element, class ElementOps, INumber numOfChildren >
INumber IGTabularTree < Element, ElementOps, numOfChildren >::
removeSubtree (Node* node)
{ IASSERT (node != 0);
  INumber result = 0;
  for (INumber n = 0; n < numOfChildren; n++) {
    if (node->ivChildren [n] != 0)
      result += removeSubtree (node->ivChildren [n]);
  }
  node->~Node ();
  ivElementOps.deallocate
                      (node
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
  return result + 1;
}

template < class Element, class ElementOps, INumber numOfChildren >
IGTabularTreeNode < Element, ElementOps, numOfChildren >*
IGTabularTree < Element, ElementOps, numOfChildren >::
newNode (Element const& e) const
{ void* nodeSpace = ivElementOps.allocate
                      (sizeof (Node)
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
  ICHECK (nodeSpace != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return new (nodeSpace) Node (e);
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
constantFunctionIteration (void *iterationFunction,
                           void* env,
                           void const* node)
{ return (*(IBoolean (*) (Element const&, void*)) iterationFunction)
           (((Node const*)node)->ivElement, env);
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
functionIteration (void *iterationFunction, void* env, void* node)
{ return (*(IBoolean (*) (Element&, void*)) iterationFunction)
           (((Node*)node)->ivElement, env);
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
constantIteratorIteration (void* iterator, void const* node)
{ return ((IConstantIterator < Element >*)iterator)->
           applyTo (((Node const*)node)->ivElement);
}

template < class Element, class ElementOps, INumber numOfChildren >
IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
iteratorIteration (void* iterator, void* node)
{ return ((IIterator < Element >*)iterator)->
           applyTo (((Node*)node)->ivElement);
}

#pragma info (restore)
