/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#ifndef _ITBTRE_H
#define _ITBTRE_H

#include <iglobals.h>
#include <itcursor.h>
#include <iitbtre.h>
#include <ibexcept.h>


#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#ifdef ICL_NO_INLINE
#define inline
#endif

template < class Element, class ElementOps, INumber numOfChildren >
class IGTabularTree;

template < class Element, class ElementOps, INumber numOfChildren >
class IGTabularTreeNode;

template < class Element, class ElementOps, INumber numOfChildren >
class IGTabularTreeCursor : public IElementTreeCursor < Element >
{
public:

           IGTabularTreeCursor
                      (IGTabularTree
                        < Element, ElementOps,
                          numOfChildren > const& c)
           : ivCollection (&c), ivNode (0)
           {
           }

  IBoolean setToChild (IPosition position);

  IBoolean setToParent ();

  IBoolean setToRoot ();

  IBoolean setToFirstExistingChild ();

  IBoolean setToLastExistingChild ();

  IBoolean setToNextExistingChild ();

  IBoolean setToPreviousExistingChild ();

  IBoolean isValid () const;

  void     invalidate ();

  IBoolean operator== (ITreeCursor const& cursor) const;

  Element const&
           element () const;

  IBoolean operator== (IGTabularTreeCursor
                        < Element, ElementOps,
                          numOfChildren > const& cursor) const
           { return ivCollection == cursor.ivCollection &&
                    ivNode == cursor.ivNode;
           }

  IBoolean operator!= (IGTabularTreeCursor
                        < Element, ElementOps,
                          numOfChildren > const& cursor) const
           { return ! operator== (cursor);
           }

protected:

  IGTabularTree < Element, ElementOps, numOfChildren > const*
           ivCollection;

  ITabularTreeImplNode *
           ivNode;

  IBoolean isFor (IGTabularTree
                   < Element, ElementOps,
                     numOfChildren > const& c) const
           { return ivCollection == &c;
           }

  friend class IGTabularTree < Element, ElementOps, numOfChildren >;

};


template < class Element, class ElementOps, INumber numOfChildren >
class IGTabularTreeNode  {
    // must be layout compatible with ITabularTreeImplNode
    // cannot derive because of unknown numOfChildren in impl

   IGTabularTreeNode< Element, ElementOps, numOfChildren >*
                          ivParent;

   IGTabularTreeNode< Element, ElementOps, numOfChildren >*
                          ivChildren [numOfChildren];

   Element   ivElement;

             IGTabularTreeNode (Element const& element)
             : ivElement (element)
             {
             }

            ~IGTabularTreeNode ()
             {
             }

   friend class IGTabularTree < Element, ElementOps, numOfChildren >;

   friend class IGTabularTreeCursor < Element,
                                      ElementOps, numOfChildren >;


};

template < class Element, class ElementOps, INumber numOfChildren >
class IGTabularTree : protected ITabularTreeImpl {
public :


  typedef IGTabularTreeCursor < Element,
                                ElementOps, numOfChildren > Cursor;

  inline
           IGTabularTree    ();

  inline
           IGTabularTree    (IGTabularTree
                              < Element,
                                ElementOps, numOfChildren > const&);

  inline
          ~IGTabularTree    ();

  inline
  void     addAsChild       (ITreeCursor const&,
                             IPosition,
                             Element const&);

  inline
  void     addAsRoot        (Element const&);

  inline
  IBoolean allElementsDo    (IBoolean (*function) (Element&, void*),
                             ITreeIterationOrder,
                             void* additionalArgument = 0);

  inline
  IBoolean allElementsDo    (IIterator <Element>&, ITreeIterationOrder);

  inline
  IBoolean allElementsDo    (IBoolean (*function)
                              (Element const&, void*),
                             ITreeIterationOrder,
                             void* additionalArgument = 0) const;

  inline
  IBoolean allElementsDo    (IConstantIterator <Element>&,
                             ITreeIterationOrder) const;

  inline
  IBoolean allSubtreeElementsDo (ITreeCursor const&,
                                 IBoolean (*function) (Element&, void*),
                                 ITreeIterationOrder,
                                 void* additionalArgument = 0);

  inline
  IBoolean allSubtreeElementsDo (ITreeCursor const&,
                                 IIterator <Element>&,
                                 ITreeIterationOrder);

  inline
  IBoolean allSubtreeElementsDo (ITreeCursor const&,
                                 IBoolean (*function)
                                  (Element const&, void*),
                                 ITreeIterationOrder,
                                 void* additionalArgument = 0) const;

  inline
  IBoolean allSubtreeElementsDo (ITreeCursor const&,
                                 IConstantIterator <Element>&,
                                 ITreeIterationOrder) const;

  inline
  void     attachAsChild    (ITreeCursor const&,
                             IPosition,
                             IGTabularTree
                              < Element, ElementOps, numOfChildren >&);

  inline
  void     attachAsRoot     (IGTabularTree
                              < Element, ElementOps, numOfChildren >&);

  inline
  void     attachSubtreeAsChild (ITreeCursor const&,
                                 IPosition,
                                 IGTabularTree
                                  < Element,
                                    ElementOps, numOfChildren >&,
                                 ITreeCursor const&);

  inline
  void     attachSubtreeAsRoot (IGTabularTree
                                 < Element,
                                   ElementOps, numOfChildren >&,
                                ITreeCursor const&);

  inline
  void     copy             (IGTabularTree
                              < Element,
                                ElementOps, numOfChildren > const&);

  inline
  void     copySubtree      (IGTabularTree
                              < Element,
                                ElementOps, numOfChildren > const&,
                             ITreeCursor const&);

  inline
  Element const&
           elementAt        (ITreeCursor const&) const;

  inline
  Element& elementAt        (ITreeCursor const&);

  inline
  IBoolean hasChild         (IPosition, ITreeCursor const&) const;

  inline
  IBoolean isEmpty          () const;

  inline
  IBoolean isRoot           (ITreeCursor const&) const;

  inline
  IBoolean isLeaf           (ITreeCursor const&) const;

  inline
  Cursor*  newCursor        () const;

  inline
  INumber  numberOfChildren () const;

  inline
  INumber  numberOfElements () const;

  inline
  INumber  numberOfLeaves   () const;

  inline
  INumber  numberOfSubtreeElements (ITreeCursor const&) const;

  inline
  INumber  numberOfSubtreeLeaves (ITreeCursor const&) const;

  inline
  IGTabularTree < Element, ElementOps, numOfChildren >&
           operator=        (IGTabularTree
                              < Element,
                                ElementOps, numOfChildren > const&);

  inline
  INumber  position         (ITreeCursor const&) const;

  inline
  INumber  removeAll        ();

  inline
  INumber  removeSubtree    (ITreeCursor const&);

  inline
  void     replaceAt        (ITreeCursor const&, Element const&);

  inline
  IBoolean setToChild       (IPosition, ITreeCursor&) const;

  inline
  IBoolean setToFirst       (ITreeCursor&, ITreeIterationOrder) const;

  inline
  IBoolean setToFirstExistingChild (ITreeCursor&) const;

  inline
  IBoolean setToLast        (ITreeCursor&, ITreeIterationOrder) const;

  inline
  IBoolean setToLastExistingChild (ITreeCursor&) const;

  inline
  IBoolean setToNext        (ITreeCursor&, ITreeIterationOrder) const;

  inline
  IBoolean setToNextExistingChild (ITreeCursor&) const;

  inline
  IBoolean setToParent      (ITreeCursor&) const;

  inline
  IBoolean setToPrevious    (ITreeCursor&, ITreeIterationOrder) const;

  inline
  IBoolean setToPreviousExistingChild (ITreeCursor&) const;

  inline
  IBoolean setToRoot        (ITreeCursor&) const;

protected:

  typedef IGTabularTreeNode < Element, ElementOps, numOfChildren > Node;

public:

  inline
  IBoolean isConsistent     () const;

protected:

  ElementOps ivElementOps;


  inline
  void     copySubtree      (Node* from);

  void     copySubtree      (Node*& to, Node* from);

  INumber  removeSubtree    (Node*);

  Node*    newNode          (Element const&) const;

  static
  IBoolean constantFunctionIteration (void *function,
                                      void* env,
                                      void const* node);

  static
  IBoolean functionIteration (void *function,
                              void* env,
                              void* node);

  static
  IBoolean constantIteratorIteration (void* iterator, void const* node);

  static
  IBoolean iteratorIteration (void* iterator, void* node);

};

template < class Element, INumber numOfChildren >
class ITabularTree :
  public IGTabularTree < Element, IStdOps < Element >, numOfChildren > {
public:


};

#ifdef ICL_NO_INLINE
#undef inline
#else
#include <itbtre.inl>
#endif

#ifndef __TEMPINC__
#include <itbtre.c>
#endif

#pragma info (restore)
#pragma pack ()


#endif
