/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

template < class Element, class ElementOps, INumber numOfChildren >
inline void IGTabularTree < Element, ElementOps, numOfChildren >::
copySubtree (Node* from)
{ if (from == 0)
    ivRoot = 0;
  else {
    copySubtree ((Node*&)ivRoot, from);
    ivRoot->ivParent = 0;
  }
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IGTabularTree < Element, ElementOps, numOfChildren >::
IGTabularTree ()
: ITabularTreeImpl (numOfChildren)
{
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IGTabularTree < Element, ElementOps, numOfChildren >::
IGTabularTree (IGTabularTree < Element,
                               ElementOps,
                               numOfChildren > const& tabularTree)
: ITabularTreeImpl (numOfChildren)
{ copySubtree ((Node*)tabularTree.ivRoot);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IGTabularTree < Element, ElementOps, numOfChildren >::
~IGTabularTree ()
{ removeAll ();
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IGTabularTree < Element, ElementOps, numOfChildren >&
IGTabularTree < Element, ElementOps, numOfChildren >::
operator= (IGTabularTree < Element,
                           ElementOps, numOfChildren > const& tree)
{ copy (tree);
  return *this;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline void IGTabularTree < Element, ElementOps, numOfChildren >::
copy (IGTabularTree < Element, ElementOps, numOfChildren > const& tree)
{ if (&tree != this)
  { removeAll ();
    copySubtree ((Node*)tree.ivRoot);
  }
}

template < class Element, class ElementOps, INumber numOfChildren >
inline void IGTabularTree < Element, ElementOps, numOfChildren >::
copySubtree (IGTabularTree < Element,
                             ElementOps,
                             numOfChildren > const& tree,
             ITreeCursor const& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).ivCollection == &tree,
          ICursorInvalidException, ICursorNotForGivenText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (tree.checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  if (&tree != this) {
    removeAll ();
    copySubtree ((Node*)((Cursor const&)cursor).ivNode);
  }
  else {
    IGTabularTree < Element, ElementOps, numOfChildren > t;
    t.attachSubtreeAsRoot (*this, cursor);
    removeAll ();
    attachAsRoot (t);
  }
}

template < class Element, class ElementOps, INumber numOfChildren >
inline void IGTabularTree < Element, ElementOps, numOfChildren >::
addAsRoot (Element const& element)
{
#ifndef INO_CHECKS
  ICHECK (ivRoot == 0,
          IRootAlreadyExistsException, IRootAlreadyExistsText)
#endif
  ivRoot = (ITabularTreeImplNode*) newNode (element);
  initNode (ivRoot, 0);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline void IGTabularTree < Element, ElementOps, numOfChildren >::
addAsChild (ITreeCursor const& cursor,
            IPosition position,
            Element const& element)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numOfChildren,
          IPositionInvalidException, IInvalidPositionText)
#endif
  ITabularTreeImplNode* parent = ((Cursor const&)cursor).ivNode;
  ITabularTreeImplNode*& child = parent->ivChildren [position-1];
#ifndef INO_CHECKS
  ICHECK (child == 0,
          IChildAlreadyExistsException, IChildAlreadyExistsText)
#endif
  child = (ITabularTreeImplNode*) newNode (element);
  initNode (child, parent);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline void IGTabularTree < Element, ElementOps, numOfChildren >::
attachAsRoot (IGTabularTree < Element,
                              ElementOps, numOfChildren >& tree)
{
#ifndef INO_CHECKS
  ICHECK (ivRoot == 0,
          IRootAlreadyExistsException, IRootAlreadyExistsText)
#endif
  ivRoot = tree.ivRoot;
  tree.ivRoot = 0;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline void IGTabularTree < Element, ElementOps, numOfChildren >::
attachAsChild (ITreeCursor const& cursor, IPosition position,
               IGTabularTree < Element,
                               ElementOps, numOfChildren >& tree)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
  ICHECK (1 <= position && position <= numOfChildren,
          IPositionInvalidException, IInvalidPositionText)
  ICHECK (this != &tree,
          ICyclicAttachException, ICyclicAttachText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ITabularTreeImplNode* parent = ((Cursor const&)cursor).ivNode;
  ITabularTreeImplNode*& child = parent->ivChildren [position-1];
#ifndef INO_CHECKS
  ICHECK (child == 0,
          IChildAlreadyExistsException, IChildAlreadyExistsText)
#endif
  child = tree.ivRoot;
  if (child != 0)
    child->ivParent = parent;
  tree.ivRoot = 0;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline void IGTabularTree < Element, ElementOps, numOfChildren >::
attachSubtreeAsRoot (IGTabularTree < Element,
                                     ElementOps,
                                     numOfChildren >& tree,
                     ITreeCursor const& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).ivCollection == &tree,
          ICursorInvalidException, ICursorNotForGivenText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
  ICHECK (ivRoot == 0,
          IRootAlreadyExistsException, IRootAlreadyExistsText)
#endif
#ifdef IALL_CHECKS
  ICHECK (tree.checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ITabularTreeImplNode* node = ((Cursor const&)cursor).ivNode;
  ITabularTreeImpl::attachSubtreeAsRoot (tree, node);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline void IGTabularTree < Element, ElementOps, numOfChildren >::
attachSubtreeAsChild (ITreeCursor const& cursorTo,
                      IPosition pos,
                      IGTabularTree < Element,
                                      ElementOps,
                                      numOfChildren >& tree,
                      ITreeCursor const& cursorFrom)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursorFrom).ivCollection == &tree,
          ICursorInvalidException, ICursorNotForGivenText)
  ICHECK (cursorFrom.isValid (),
          ICursorInvalidException, IInvalidCursorText)
  ICHECK (((Cursor const&)cursorTo).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursorTo.isValid (),
          ICursorInvalidException, IInvalidCursorText)
  ICHECK (1 <= pos && pos <= numOfChildren,
          IPositionInvalidException, IInvalidPositionText)
  ICHECK (this != &tree ||
          ! checkNodeInSubtree (((Cursor const&)cursorTo).ivNode,
                                ((Cursor const&)cursorFrom).ivNode),
          ICyclicAttachException, ICyclicAttachText)
#endif
#ifdef IALL_CHECKS
  ICHECK (tree.checkNode(((Cursor const&)cursorFrom).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
  ICHECK (checkNode(((Cursor const&)cursorTo).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ITabularTreeImplNode* parent = ((Cursor const&)cursorTo).ivNode;
  ITabularTreeImplNode*& child = parent->ivChildren [pos-1];
#ifndef INO_CHECKS
  ICHECK (child == 0,
          IChildAlreadyExistsException, IChildAlreadyExistsText)
#endif
  ITabularTreeImplNode* nodeFrom = ((Cursor const&)cursorFrom).ivNode;
  ITabularTreeImpl::attachSubtreeAsChild (parent, pos, tree, nodeFrom);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline INumber IGTabularTree < Element, ElementOps, numOfChildren >::
removeAll ()
{ INumber result;
  if (ivRoot != 0) {
    result = removeSubtree ((Node*)ivRoot);
    ivRoot = 0;
  }
  else
    result = 0;
  return result;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline INumber IGTabularTree < Element, ElementOps, numOfChildren >::
removeSubtree (ITreeCursor const& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ITabularTreeImplNode* node = ((Cursor const&)cursor).ivNode;
  if (node->ivParent == 0)
    ivRoot = 0;
  else
    node->ivParent->ivChildren [position (cursor) - 1] = 0;
  return removeSubtree ((Node*)node);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline Element const&
IGTabularTree < Element, ElementOps, numOfChildren >::
elementAt (ITreeCursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Node*)((Cursor const&)cursor).ivNode)->ivElement;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline Element& IGTabularTree < Element, ElementOps, numOfChildren >::
elementAt (ITreeCursor const& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Node*)((Cursor const&)cursor).ivNode)->ivElement;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline void IGTabularTree < Element, ElementOps, numOfChildren >::
replaceAt (ITreeCursor const& cursor, Element const& element)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ivElementOps.assign
    (((Node*)((Cursor const&)cursor).ivNode)->ivElement, element);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline INumber IGTabularTree < Element, ElementOps, numOfChildren >::
numberOfChildren () const
{ return numOfChildren;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline INumber IGTabularTree < Element, ElementOps, numOfChildren >::
numberOfElements () const
{ if (ivRoot == 0)
    return 0;
  else
    return ITabularTreeImpl::numberOfSubtreeElements (ivRoot);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline INumber IGTabularTree < Element, ElementOps, numOfChildren >::
numberOfSubtreeElements (ITreeCursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::numberOfSubtreeElements
           (((Cursor const&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline INumber IGTabularTree < Element, ElementOps, numOfChildren >::
numberOfLeaves () const
{ if (ivRoot == 0)
    return 0;
  else
    return ITabularTreeImpl::numberOfSubtreeLeaves (ivRoot);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline INumber IGTabularTree < Element, ElementOps, numOfChildren >::
numberOfSubtreeLeaves (ITreeCursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::numberOfSubtreeLeaves
           (((Cursor const&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
isEmpty () const
{ return ivRoot == 0;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IGTabularTreeCursor < Element, ElementOps, numOfChildren >*
IGTabularTree < Element, ElementOps, numOfChildren >::
newCursor () const
{ Cursor* result = new Cursor (*this);
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
isRoot (ITreeCursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ivRoot == ((Cursor const&)cursor).ivNode;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
isLeaf (ITreeCursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::isLeaf (((Cursor const&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline INumber IGTabularTree < Element, ElementOps, numOfChildren >::
position (ITreeCursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::position (((Cursor const&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
hasChild (IPosition position, ITreeCursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numOfChildren,
          IPositionInvalidException, IInvalidPositionText)
#endif
  return (((Cursor const&)cursor).ivNode)->ivChildren [position-1] != 0;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
setToRoot (ITreeCursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  ((Cursor&)cursor).ivNode = (ITabularTreeImplNode*)ivRoot;
  return ivRoot != 0;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
setToChild (IPosition position, ITreeCursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numOfChildren,
          IPositionInvalidException, IInvalidPositionText)
#endif
  ITabularTreeImplNode* node = ((Cursor const&)cursor).ivNode;
  ((Cursor&)cursor).ivNode = node->ivChildren [position-1];
  return ((Cursor const&)cursor).ivNode != 0;
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
setToParent (ITreeCursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::setToParent (((Cursor&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
setToFirstExistingChild (ITreeCursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::setToFirstExistingChild
           (((Cursor&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
setToNextExistingChild (ITreeCursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::setToNextExistingChild
           (((Cursor&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
setToLastExistingChild (ITreeCursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::setToLastExistingChild
           (((Cursor&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
setToPreviousExistingChild (ITreeCursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::setToPreviousExistingChild
           (((Cursor&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
setToFirst (ITreeCursor& cursor, ITreeIterationOrder order) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  return ITabularTreeImpl::setToFirst (((Cursor&)cursor).ivNode, order);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
setToNext (ITreeCursor& cursor, ITreeIterationOrder order) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  return ITabularTreeImpl::setToNext (((Cursor&)cursor).ivNode, order);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
setToLast (ITreeCursor& cursor, ITreeIterationOrder order) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  return ITabularTreeImpl::setToLast (((Cursor&)cursor).ivNode, order);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
setToPrevious (ITreeCursor& cursor, ITreeIterationOrder order) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  return ITabularTreeImpl::setToPrevious
           (((Cursor&)cursor).ivNode, order);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
allElementsDo (IBoolean (*function) (Element&, void*),
               ITreeIterationOrder order,
               void* additionalArgument)
{ if (ivRoot == 0)
    return True;
  else
    return ITabularTreeImpl::allElementsDo
             (function, order, additionalArgument,
                                            functionIteration, ivRoot);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
allElementsDo (IIterator <Element>& iterator, ITreeIterationOrder order)
{ if (ivRoot == 0)
    return True;
  else
    return ITabularTreeImpl::allElementsDo (&iterator, order,
                                            iteratorIteration, ivRoot);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
allElementsDo (IBoolean (*function) (Element const&, void*),
               ITreeIterationOrder order,
               void* additionalArgument) const
{ if (ivRoot == 0)
    return True;
  else
    return ITabularTreeImpl::allElementsDo
             (function, order, additionalArgument,
              constantFunctionIteration, ivRoot);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
allElementsDo (IConstantIterator <Element>& iterator,
               ITreeIterationOrder order) const
{ if (ivRoot == 0)
    return True;
  else
    return ITabularTreeImpl::allElementsDo
             (&iterator, order, constantIteratorIteration, ivRoot);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
isConsistent () const
{ return ITabularTreeImpl::isConsistent ();
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
allSubtreeElementsDo (ITreeCursor const& cursor,
                      IBoolean (*function) (Element&, void*),
                      ITreeIterationOrder order,
                      void* additionalArgument)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::allElementsDo
           (function, order, additionalArgument,
            functionIteration, ((Cursor const&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
allSubtreeElementsDo (ITreeCursor const& cursor,
                      IIterator <Element>& iterator,
                      ITreeIterationOrder order)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::allElementsDo
           (&iterator, order, iteratorIteration,
            ((Cursor const&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
allSubtreeElementsDo (ITreeCursor const& cursor,
                      IBoolean (*function) (Element const&, void*),
                      ITreeIterationOrder order,
                      void* additionalArgument) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::allElementsDo
           (function, order, additionalArgument,
            constantFunctionIteration, ((Cursor const&)cursor).ivNode);
}

template < class Element, class ElementOps, INumber numOfChildren >
inline IBoolean IGTabularTree < Element, ElementOps, numOfChildren >::
allSubtreeElementsDo (ITreeCursor const& cursor,
                      IConstantIterator <Element>& iterator,
                      ITreeIterationOrder order) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (checkNode(((Cursor const&)cursor).ivNode),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ITabularTreeImpl::allElementsDo
           (&iterator, order, constantIteratorIteration,
            ((Cursor const&)cursor).ivNode);
}
