/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

#include <new.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)


#ifdef ICL_NO_INLINE
#define inline
#include <itdseq.inl>
#undef inline
#endif

// ********************** IGDilutedSequence::Cursor ********************

template <class Element, class ElementOps>
IBoolean IGDilutedSequenceCursor <Element, ElementOps>::setToFirst ()
{ return ivCollection->setToFirst (*this);
}

template <class Element, class ElementOps>
IBoolean IGDilutedSequenceCursor <Element, ElementOps>::setToNext ()
{ return ivCollection->setToNext (*this);
}

template <class Element, class ElementOps>
IBoolean IGDilutedSequenceCursor <Element, ElementOps>::isValid () const
{ return ivImpl.ivIsValid;
}

template <class Element, class ElementOps>
void IGDilutedSequenceCursor <Element, ElementOps>::invalidate ()
{ ivImpl.ivIsValid = False;
}

template <class Element, class ElementOps>
void IGDilutedSequenceCursor <Element, ElementOps>::
copy (ICursor const& cursor)
{ operator= ((IGDilutedSequenceCursor
                < Element, ElementOps> const&) cursor);
}

template <class Element, class ElementOps>
IBoolean IGDilutedSequenceCursor <Element, ElementOps>::
operator== (ICursor const& cursor) const
{ return operator== ((IGDilutedSequenceCursor
                        < Element, ElementOps> const&) cursor);
}

template <class Element, class ElementOps>
Element const& IGDilutedSequenceCursor <Element, ElementOps>::
element () const
{ return ivCollection->elementAt (*this);
}

template <class Element, class ElementOps>
IBoolean IGDilutedSequenceCursor <Element, ElementOps>::setToLast ()
{ return ivCollection->setToLast (*this);
}

template <class Element, class ElementOps>
IBoolean IGDilutedSequenceCursor <Element, ElementOps>::setToPrevious ()
{ return ivCollection->setToPrevious (*this);
}

// ************************** IGDilutedSequence ************************

template < class Element, class ElementOps >
void IGDilutedSequence < Element, ElementOps >::
copyFrom (INumber index, void const* element) const
{ ivElementOps.assign
    (((Node*)ivTable.ivNodes)
      [index].ivElement, *(Element const*)element);
  ((Node*)ivTable.ivNodes) [index].ivIsRemoved = False;
}

template < class Element, class ElementOps >
void const* IGDilutedSequence < Element, ElementOps >::
elementAt (INumber index) const
{ return &((Node*)ivTable.ivNodes) [index].ivElement;
}

template < class Element, class ElementOps >
IBoolean IGDilutedSequence < Element, ElementOps >::
isRemoved (INumber index) const
{ return ((Node*)ivTable.ivNodes) [index].ivIsRemoved;
}

template < class Element, class ElementOps >
void IGDilutedSequence < Element, ElementOps >::
remove (INumber index)
{ ((Node*)ivTable.ivNodes) [index].ivIsRemoved = True;
  if (--ivTable.ivNumberOfElements > 0) {
    if (index == ivTable.ivFirst)
      setToNext (ivTable.ivFirst);
    else if (index == ivTable.ivLast)
      setToPrevious (ivTable.ivLast);
  }
  else
    ivTable.decMod (ivTable.ivLast);
}

template < class Element, class ElementOps >
IBoolean IGDilutedSequence < Element, ElementOps >::
setToNext (INumber& index) const
{ if (index == ivTable.ivLast)
    return False;
  do
    ivTable.incMod (index);
  while (((Node*)ivTable.ivNodes) [index].ivIsRemoved);
  return True;
}

template < class Element, class ElementOps >
IBoolean IGDilutedSequence < Element, ElementOps >::
setToPrevious (INumber& index) const
{ if (index == ivTable.ivFirst)
    return False;
  do
    ivTable.decMod (index);
  while (((Node*)ivTable.ivNodes) [index].ivIsRemoved);
  return True;
}

template < class Element, class ElementOps >
void IGDilutedSequence < Element, ElementOps >::
swap (INumber index1, INumber index2) const
{ Element e;
  ivElementOps.assign (e, ((Node*)ivTable.ivNodes) [index1].ivElement);
  ivElementOps.assign
    (((Node*)ivTable.ivNodes) [index1].ivElement,
     ((Node*)ivTable.ivNodes) [index2].ivElement);
  ivElementOps.assign (((Node*)ivTable.ivNodes) [index2].ivElement, e);
}

template < class Element, class ElementOps >
IBoolean IGDilutedSequence < Element, ElementOps >::
constantFunctionIteration (void *iterationFunction,
                           void* env,
                           INumber index) const
{ return (*(IBoolean (*) (Element const&, void*)) iterationFunction)
           (((Node*)ivTable.ivNodes) [index].ivElement, env);
}

template < class Element, class ElementOps >
IBoolean IGDilutedSequence < Element, ElementOps >::
functionIteration (void *iterationFunction, void* env, INumber index)
{ return (*(IBoolean (*) (Element&, void*)) iterationFunction)
           (((Node*)ivTable.ivNodes) [index].ivElement, env);
}

template < class Element, class ElementOps >
IBoolean IGDilutedSequence < Element, ElementOps >::
constantIteratorIteration (void* iterator, INumber index) const
{ return ((IConstantIterator < Element >*)iterator)->
           applyTo (((Node*)ivTable.ivNodes) [index].ivElement);
}

template < class Element, class ElementOps >
IBoolean IGDilutedSequence < Element, ElementOps >::
iteratorIteration (void* iterator, INumber index)
{ return ((IIterator < Element >*)iterator)->
           applyTo (((Node*)ivTable.ivNodes) [index].ivElement);
}

template < class Element, class ElementOps >
long IGDilutedSequence < Element, ElementOps >::
functionComparison (void *comparisonFunction,
                    void const* element1,
                    void const* element2) const
{ return (*(long (*) (Element const&, Element const&) const)
            comparisonFunction)
           (*(Element*)element1, *(Element*)element2);
}

template < class Element, class ElementOps >
long IGDilutedSequence < Element, ElementOps >::
functionComparison (void *comparisonFunction,
                    INumber index1,
                    INumber index2) const
{ return (*(long (*) (Element const&, Element const&) const)
            comparisonFunction)
           (((Node*)ivTable.ivNodes) [index1].ivElement,
            ((Node*)ivTable.ivNodes) [index2].ivElement);
}

template < class Element, class ElementOps >
void IGDilutedSequence < Element, ElementOps >::
blockLeft (INumber lTarget, INumber lMoveCount)
{ for (INumber i = lTarget; lMoveCount > 0; i++, lMoveCount--)
    ((Node*)ivTable.ivNodes) [i] = ((Node*)ivTable.ivNodes) [i+1];
}

template < class Element, class ElementOps >
void IGDilutedSequence < Element, ElementOps >::
blockRight (INumber lTarget, INumber lMoveCount)
{ for (INumber i = lTarget + lMoveCount - 1;
       lMoveCount > 0;
       i--, lMoveCount--)
    ((Node*)ivTable.ivNodes) [i] = ((Node*)ivTable.ivNodes) [i-1];
}

template < class Element, class ElementOps >
void IGDilutedSequence < Element, ElementOps >::
compress ()
{ INumber from = ivTable.ivFirst, to = ivTable.ivFirst;
  for (;;) {
    if (from != to)
      ((Node*)ivTable.ivNodes) [to] = ((Node*)ivTable.ivNodes) [from];
    if (! setToNext (from))
      break;
    ivTable.incMod (to);
  }
  ivTable.ivLast = to;
}

template < class Element, class ElementOps >
void IGDilutedSequence < Element, ElementOps >::
copy (IDilutedSequenceImpl::Table const& from)
{ if (from.ivNumberOfElements == 0)
    return;
  if (ivTable.ivNumberOfElements > 0 &&
      (ivTable.ivFirst > ivTable.ivLast ?
        (ivTable.ivFirst - ivTable.ivLast - 1) :
        (ivTable.ivFirst + ivTable.ivAllocatedElements
                         - ivTable.ivLast - 1))
        < from.ivNumberOfElements)
    compress ();
  INumber toIndex = ivTable.ivLast;
  INumber fromIndex = from.ivFirst;
  for (;;) {
    ivTable.incMod (toIndex);
    ((Node*)ivTable.ivNodes) [toIndex] =
      ((Node*)from.ivNodes) [fromIndex];
    if (fromIndex == from.ivLast)
      break;
    do
      from.incMod (fromIndex);
    while (((Node*)from.ivNodes) [fromIndex].ivIsRemoved);
  }
  ivTable.ivNumberOfElements += from.ivNumberOfElements;
  ivTable.ivLast = toIndex;
}

template < class Element, class ElementOps >
void IGDilutedSequence < Element, ElementOps >::
expand (INumber n)
{ n += ivTable.ivNumberOfElements;

  IDilutedSequenceImpl::Table tab = ivTable;

  INumber newAllocatedElements = ivTable.ivAllocatedElements;
  while (newAllocatedElements < n)
    newAllocatedElements *= 2;
  newNodes (newAllocatedElements);

  ivTable.ivFirst = 0;
  ivTable.ivLast = ivTable.ivAllocatedElements - 1;
  ivTable.ivNumberOfElements = 0;
  copy (tab);
  for (INumber i = 0; i < tab.ivAllocatedElements; i++)
    ((Node*) &tab.ivNodes) [i].~Node ();
  ivElementOps.deallocate
                      (tab.ivNodes
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
}

template < class Element, class ElementOps >
void IGDilutedSequence < Element, ElementOps >::
newNodes (INumber n)
{ void* nodeSpace = ivElementOps.allocate
                      (sizeof (Node) * n
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
  ICHECK (nodeSpace != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  ivTable.ivAllocatedElements = n;
  ivTable.ivNodes = nodeSpace;
  for (INumber i = 0; i < n; i++)
    new (&((Node*)nodeSpace) [i]) Node;
}

template < class Element, class ElementOps >
void IGDilutedSequence < Element, ElementOps >::
deleteNodes ()
{ for (INumber i = 0; i < ivTable.ivAllocatedElements; i++)
    ((Node*) ivTable.ivNodes) [i].~Node ();
  ivElementOps.deallocate
                      (ivTable.ivNodes
#ifdef __DEBUG_ALLOC__
                      ,__FILE__, __LINE__
#endif
                      );
}

template < class Element, class ElementOps >
void IGDilutedSequence < Element, ElementOps >::
makeFree (INumber& index)
{ INumber leftFree = index;
  while (leftFree > 0 &&
         ! ((Node*)ivTable.ivNodes) [leftFree].ivIsRemoved) {
    if (leftFree == ivTable.ivFirst && leftFree - 1 != ivTable.ivLast)
      ((Node*)ivTable.ivNodes) [leftFree - 1].ivIsRemoved = True;
    leftFree--;
  }
  if (index - leftFree <= 1 &&
      ((Node*)ivTable.ivNodes) [leftFree].ivIsRemoved) {
    index = leftFree;
    return;
  }
  INumber rightFree = index;
  while (rightFree < ivTable.ivAllocatedElements - 1 &&
         ! ((Node*)ivTable.ivNodes) [rightFree].ivIsRemoved) {
    if (rightFree == ivTable.ivLast && rightFree + 1 != ivTable.ivFirst)
      ((Node*)ivTable.ivNodes) [rightFree + 1].ivIsRemoved = True;
    rightFree++;
  }
  if (! ((Node*)ivTable.ivNodes) [leftFree].ivIsRemoved ||
      (index - leftFree > rightFree - index &&
       ((Node*)ivTable.ivNodes) [rightFree].ivIsRemoved)) {
    blockRight (index + 1, rightFree - (index + 1) + 1);
    if (index <= ivTable.ivFirst && ivTable.ivFirst < rightFree)
      ivTable.ivFirst++;
    if (index <= ivTable.ivLast && ivTable.ivLast < rightFree)
      ivTable.ivLast++;
  }
  else {
    blockLeft (leftFree, (index - 1) - (leftFree + 1) + 1);
    if (leftFree < ivTable.ivFirst && ivTable.ivFirst < index)
      ivTable.ivFirst--;
    if (leftFree < ivTable.ivLast && ivTable.ivLast < index)
      ivTable.ivLast--;
    index--;
  }
}

#pragma info (restore)
