/*************************************************************************
  IBM C/C++ Tools Version 3.00 - Collection Class Library
 (C) Copyright IBM Corporation 1992 ,1995, Licensed Program-Property of
 IBM.  All Rights Reserved.  US Government Users Restricted Rights - Use,
 duplication or disclosure restricted by GSA ADP Schedule Contract with
 IBM Corp.
 *************************************************************************/

// ************************** IGDilutedSequenceCursor ******************

template <class Element, class ElementOps>
inline IBoolean
IGDilutedSequenceCursor < Element, ElementOps >::
operator== (IGDilutedSequenceCursor < Element,
                                      ElementOps > const& cursor) const
{ return ivCollection == cursor.ivCollection &&
         ivImpl == cursor.ivImpl;
}

template <class Element, class ElementOps>
inline IBoolean
IGDilutedSequenceCursor < Element, ElementOps >::
operator!= (IGDilutedSequenceCursor < Element,
                                      ElementOps > const& cursor) const
{ return ! operator== (cursor);
}


// ************************** IGDilutedSequence ************************

template <class Element, class ElementOps>
inline IGDilutedSequence <Element, ElementOps>::
IGDilutedSequence (INumber n)
{ if (n == 0) n++;
  newNodes (n);
  ivTable.ivLast = ivTable.ivAllocatedElements - 1;
  ivTable.ivFirst = ivTable.ivNumberOfElements = 0;
}

template <class Element, class ElementOps>
inline IGDilutedSequence <Element, ElementOps>::
IGDilutedSequence (IGDilutedSequence < Element,
                                       ElementOps > const& sequence)
{ newNodes (sequence.ivTable.ivAllocatedElements);
  ivTable.ivLast = ivTable.ivAllocatedElements - 1;
  ivTable.ivFirst = ivTable.ivNumberOfElements = 0;
  copy (sequence.ivTable);
}

template <class Element, class ElementOps>
inline IGDilutedSequence <Element, ElementOps>::
~IGDilutedSequence ()
{ deleteNodes ();
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
addAsLast (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  IDilutedSequenceImpl::addAsLast (&element, ((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
addAsLast (Element const& element)
{ Cursor cursor (*this);
  addAsLast (element, cursor);
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
add (Element const& element, ICursor& cursor)
{ addAsLast (element, cursor);
  return True;
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
add (Element const& element)
{ Cursor cursor (*this);
  return add (element, cursor);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
addAllFrom (IGDilutedSequence < Element, ElementOps > const& sequence)
{
#ifndef INO_CHECKS
  ICHECK (this != &sequence,
          IIdenticalCollectionException, IIdenticalCollectionText)
#endif
  IDilutedSequenceImpl::addAllFrom (sequence);
}

template <class Element, class ElementOps>
inline Element const& IGDilutedSequence <Element, ElementOps>::
elementAt (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IDilutedSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Node*)ivTable.ivNodes)
           [((Cursor const&)cursor).ivImpl.ivIndex].ivElement;
}

template <class Element, class ElementOps>
inline Element& IGDilutedSequence <Element, ElementOps>::
elementAt (ICursor const& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IDilutedSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Node*) ivTable.ivNodes)
           [((Cursor const&)cursor).ivImpl.ivIndex].ivElement;
}

template <class Element, class ElementOps>
inline Element const& IGDilutedSequence <Element, ElementOps>::
anyElement () const
{ return firstElement ();
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
removeAt (ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IDilutedSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  INumber index = ((Cursor&)cursor).ivImpl.ivIndex;
  IDilutedSequenceImpl::removeAt (index);
  cursor.invalidate ();
}

template < class Element, class ElementOps >
inline INumber IGDilutedSequence < Element, ElementOps >::
removeAll (IBoolean (*predicate) (Element const&, void*), void* env)
{ return IDilutedSequenceImpl::removeAll (predicate, env);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
replaceAt (ICursor const& cursor, Element const& element)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IDilutedSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  ivElementOps.assign
    (((Node*) ivTable.ivNodes)
       [((Cursor const&)cursor).ivImpl.ivIndex].ivElement,
     element);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
removeAll ()
{ ivTable.ivLast = ivTable.ivAllocatedElements - 1;
  ivTable.ivFirst = ivTable.ivNumberOfElements = 0;
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
isBounded () const
{ return False;
}

template <class Element, class ElementOps>
inline INumber IGDilutedSequence <Element, ElementOps>::
maxNumberOfElements () const
{ ICHECK (False, INotBoundedException,
          INotBoundedText)
  return 0;
}

template <class Element, class ElementOps>
inline INumber IGDilutedSequence <Element, ElementOps>::
numberOfElements () const
{ return ivTable.ivNumberOfElements;
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
isEmpty () const
{ return ivTable.ivNumberOfElements == 0;
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
isFull () const
{ return False;
}

template <class Element, class ElementOps>
inline IGDilutedSequenceCursor <Element, ElementOps>*
IGDilutedSequence <Element, ElementOps>::
newCursor () const
{ Cursor* result = new Cursor (*this);
  ICHECK (result != 0,
          IOutOfCollectionMemory, IOutOfCollectionMemoryText)
  return result;
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
setToFirst (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  return IDilutedSequenceImpl::setToFirst (((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
setToNext (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IDilutedSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return IDilutedSequenceImpl::setToNext (((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
allElementsDo (IBoolean (*function) (Element&, void*), void* env)
{ return IDilutedSequenceImpl::allElementsDo (function, env);
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
allElementsDo (IIterator < Element > & iterator)
{ return IDilutedSequenceImpl::allElementsDo (&iterator);
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
allElementsDo (IBoolean (*function) (Element const&, void*),
               void* env) const
{ return IDilutedSequenceImpl::allElementsDo (function, env);
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
allElementsDo (IConstantIterator < Element > & iterator) const
{ return IDilutedSequenceImpl::allElementsDo (&iterator);
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
isConsistent () const
{ return IDilutedSequenceImpl::isConsistent ();
}

template <class Element, class ElementOps>
inline IGDilutedSequence <Element, ElementOps>&
IGDilutedSequence <Element, ElementOps>::
operator = (IGDilutedSequence < Element, ElementOps > const& sequence)
{ IDilutedSequenceImpl::operator= (sequence);
  return *this;
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
removeFirst ()
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  INumber index = ivTable.ivFirst;
  IDilutedSequenceImpl::removeAt (index);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
removeLast ()
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  INumber index = ivTable.ivLast;
  IDilutedSequenceImpl::removeAt (index);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
removeAtPosition (IPosition position)
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
  IDilutedSequenceImpl::Cursor cursor;
  IDilutedSequenceImpl::setToPosition (position, cursor);
  IDilutedSequenceImpl::removeAt (cursor.ivIndex);
}

template <class Element, class ElementOps>
inline Element const& IGDilutedSequence <Element, ElementOps>::
firstElement () const
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  return ((Node*) ivTable.ivNodes) [ivTable.ivFirst].ivElement;
}

template <class Element, class ElementOps>
inline Element const& IGDilutedSequence <Element, ElementOps>::
lastElement () const
{
#ifndef INO_CHECKS
  ICHECK (!isEmpty (), IEmptyException, ICollectionIsEmptyText)
#endif
  return ((Node*) ivTable.ivNodes) [ivTable.ivLast].ivElement;
}

template <class Element, class ElementOps>
inline Element const& IGDilutedSequence <Element, ElementOps>::
elementAtPosition (IPosition position) const
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
#endif
  IDilutedSequenceImpl::Cursor cursor;
  IDilutedSequenceImpl::setToPosition (position, cursor);
  return ((Node*) ivTable.ivNodes) [cursor.ivIndex].ivElement;
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
setToLast (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  return IDilutedSequenceImpl::setToLast (((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
setToPrevious (ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IDilutedSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return IDilutedSequenceImpl::setToPrevious (((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
setToPosition (IPosition position, ICursor& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements(),
          IPositionInvalidException, IInvalidPositionText)
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  IDilutedSequenceImpl::setToPosition
    (position, ((Cursor&)cursor).ivImpl);
}

#ifndef __BIN_COMPAT_BASE_43__
template <class Element, class ElementOps>
inline IPosition IGDilutedSequence <Element, ElementOps>::
position (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
  return IDilutedSequenceImpl::position
    (((Cursor&)cursor).ivImpl);
}
#endif

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
isFirst (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IDilutedSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Cursor const&)cursor).ivImpl.ivIndex == ivTable.ivFirst;
}

template <class Element, class ElementOps>
inline IBoolean IGDilutedSequence <Element, ElementOps>::
isLast (ICursor const& cursor) const
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IDilutedSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  return ((Cursor const&)cursor).ivImpl.ivIndex == ivTable.ivLast;
}

template <class Element, class ElementOps>
inline long IGDilutedSequence <Element, ElementOps>::
compare (IGDilutedSequence < Element, ElementOps > const& sequence,
         long (*comparisonFunction)
           (Element const&, Element const&)) const {
  return IDilutedSequenceImpl::compare (sequence, comparisonFunction);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
addAsFirst (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  IDilutedSequenceImpl::addAsFirst (&element, ((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
addAsFirst (Element const& element)
{ Cursor cursor (*this);
  addAsFirst (element, cursor);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
addAsNext (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IDilutedSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  IDilutedSequenceImpl::addAsNext (&element, ((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
addAsPrevious (Element const& element, ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
  ICHECK (cursor.isValid (),
          ICursorInvalidException, IInvalidCursorText)
#endif
#ifdef IALL_CHECKS
  ICHECK (IDilutedSequenceImpl::checkIndex
          (((Cursor const&)cursor).ivImpl.ivIndex),
          ICursorInvalidException, ICursorNotContainedText)
#endif
  IDilutedSequenceImpl::addAsPrevious
    (&element, ((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
addAtPosition (IPosition position, Element const& element)
{ Cursor cursor (*this);
  addAtPosition (position, element, cursor);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
addAtPosition (IPosition position,
               Element const& element,
               ICursor& cursor)
{
#ifndef INO_CHECKS
  ICHECK (1 <= position && position <= numberOfElements() + 1,
          IPositionInvalidException, IInvalidPositionText)
  ICHECK (((Cursor const&)cursor).isFor (*this),
          ICursorInvalidException, ICursorNotForThisText)
#endif
  IDilutedSequenceImpl::addAtPosition
    (position, &element, ((Cursor&)cursor).ivImpl);
}

template <class Element, class ElementOps>
inline void IGDilutedSequence <Element, ElementOps>::
sort (long (*comparisonFunction) (Element const&, Element const&))
{ IDilutedSequenceImpl::sort (comparisonFunction);
}

// ************************** IDilutedSequence *************************

template <class Element>
inline IDilutedSequence < Element >::
IDilutedSequence (INumber n)
: IGDilutedSequence < Element, IStdOps < Element > > (n)
{
}
