#ifndef _ITIME_
#define _ITIME_
/*******************************************************************************
* FILE NAME: itime.hpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ITime                                                                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <istring.hpp>

struct _CTIME;

#pragma pack(4)

class ostream;

class ITime : public IBase {
public:
/*------------------------ Constructors --------------------------------------*/
  ITime       ( );

  ITime       ( long seconds );

  ITime       ( unsigned hours,
                unsigned minutes,
                unsigned seconds = 0 );

  ITime       ( const ITime &aTime );

  ITime       ( const _CTIME &cTime );

/*-------------------------- Current Time ------------------------------------*/
static ITime
  now         ( );

/*-------------------------- Diagnostics -------------------------------------*/
IString
  asString    ( const char *fmt = "%X" ) const;

friend ostream
 &operator << ( ostream     &aStream,
                const ITime &aTime );


/*-------------------------- Time Queries ------------------------------------*/
long
  asSeconds   ( ) const;

_CTIME
  asCTIME     ( ) const;

unsigned
  hours       ( ) const,
  minutes     ( ) const,
  seconds     ( ) const;

/*-------------------------- Comparisons -------------------------------------*/
Boolean
  operator == ( const ITime &aTime ) const,
  operator != ( const ITime &aTime ) const,
  operator <  ( const ITime &aTime ) const,
  operator <= ( const ITime &aTime ) const,
  operator >  ( const ITime &aTime ) const,
  operator >= ( const ITime &aTime ) const;

/*------------------------- Manipulation -------------------------------------*/
ITime
  operator +  ( const ITime &aTime ) const,
  operator -  ( const ITime &aTime ) const;

ITime
 &operator += ( const ITime &aTime ),
 &operator -= ( const ITime &aTime );


protected:
/*------------------------- Implementation -----------------------------------*/
ITime
 &initialize  ( long seconds );

private:
/*------------------------- PRIVATE ------------------------------------------*/
long
  ticks;

}; // ITime

#pragma pack()

#ifndef I_NO_INLINES
  #include <itime.inl>
#endif

#endif /* _ITIME_ */
