#ifndef _ITITLE_
  #define _ITITLE_
/*******************************************************************************
* FILE NAME: ititle.hpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*    Declaration of the class:                                                 *
*      ITitle                                                                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <itextctl.hpp>
#include <ireslib.hpp>

class IString;
class IColor;
class ITitleData;

#pragma pack(4)

class ITitle : public ITextControl {
typedef  ITextControl
  Inherited;
public:
/*----------------------------- Constructors ---------------------------------*/
  ITitle ( IWindow*           owner,
           const char*        objectName = 0,
           const char*        viewName   = 0,
           unsigned long      viewNumber = 0);

  ITitle ( IWindow*           owner,
           const IResourceId& objectName,
           const IResourceId& viewName   = 0,
           unsigned long      viewNumber = 0);

virtual
  ~ITitle ( );


/*--------------------------- Title Components -------------------------------*/
virtual IString
  objectText ( ) const,
  viewText   ( ) const;

virtual ITitle
 &setObjectText ( const char*       objectName ),
 &setObjectText ( const IResourceId& objectNameResId ),
 &setViewText   ( const char*       viewName ),
 &setViewText   ( const IResourceId& viewNameResId ),
 &setViewNumber ( unsigned long     viewNumber ),
 &setTitleText  ( const char*       objectName,
                  const char*       viewName          = 0,
                  unsigned long     viewNum           = 0 ),
 &setTitleText  ( const IResourceId& objectNameResId,
                  const IResourceId& viewNameResId    = 0,
                  unsigned long     viewNum           = 0 );

unsigned long
  viewNumber() const;

/*--------------------------------- Colors -----------------------------------*/
virtual IColor
  borderColor                 () const,
  activeColor                 () const,
  inactiveColor               () const,
  activeTextForegroundColor   () const,
  activeTextBackgroundColor   () const,
  inactiveTextForegroundColor () const,
  inactiveTextBackgroundColor () const;

virtual ITitle
  &setActiveTextForegroundColor     (const IColor& color),
  &setActiveTextBackgroundColor     (const IColor& color),
  &setInactiveTextForegroundColor   (const IColor& color),
  &setInactiveTextBackgroundColor   (const IColor& color),
  &resetActiveTextForegroundColor   (),
  &resetActiveTextBackgroundColor   (),
  &resetInactiveTextForegroundColor (),
  &resetInactiveTextBackgroundColor ();

/*----------------------- Notification Event Descriptions --------------------*/
// Attribute Change Notifications
static INotificationId const
  activeTextForegroundColorId,
  activeTextBackgroundColorId,
  inactiveTextForegroundColorId,
  inactiveTextBackgroundColorId,
  objectTextId,
  viewTextId,
  viewNumberId;

/*-------------------------- Observer Notification ---------------------------*/
virtual ITitle
  &enableNotification (Boolean enable = true);

protected:

private:
/*------------------------------ Hidden Members ------------------------------*/
  ITitle       (const ITitle& title );
ITitle
 &operator=    (const ITitle& title );

/*--------------------------------- Private ----------------------------------*/
IWindow
 *pwinClOwner;

ITitleData
 *fTitleData;

public:
#if (IC_OBSOLETE <= IC_OBSOLETE_1)
/*----------------------------- Obsolete Members -----------------------------*/
enum ColorArea {
  activeFill,
  inactiveFill,
  activeTextForeground,
  inactiveTextForeground,
  activeTextBackground,
  inactiveTextBackground,
  border };
ITitle
  &setColor ( ColorArea     value,
              const IColor& color );
IColor
  color ( ColorArea value ) const;
virtual ITitle
 &setViewNum  ( unsigned long viewNum );
unsigned long
  viewNum() const;
#endif
};

#pragma pack()

#endif  /* ifndef _ITITLE_ */
