/********************************************************************************/
/* File:  itk_seq.hpp                                                           */
/* Class: ITypeCodeSequence                                                     */
/*                                                                              */
/* Type code for sequence                                                       */
/********************************************************************************/

/******************************************************************************/
/*                                                                            */
/* IBM(R) VisualAge(TM) C++ for OS/2(R), Version 3                            */
/* (C) Copyright IBM Corp. 1991, 1995.                                        */
/*  - Licensed Material - Program-Property of IBM - All Rights Reserved.      */
/* US Government Users Restricted Rights - Use, duplication or disclosure     */
/* restricted by GSA ADP Schedule Contract with IBM Corp.                     */
/*                                                                            */
/******************************************************************************/

#ifndef _ITK_SEQ_HPP_
#define _ITK_SEQ_HPP_

#include "itypecod.hpp"
#include <istring.hpp>
#include "dax.hpp"


class EXPORT ITypeCodeSequence : public ITypeCode
{

public:

/*------------------------- Constructors ---------------------------------------
| There is 1 way to construct instances of this class.                         |
| parms:                                                                       |
|  pSeqElementTypeI - type of element in sequence                              |
|  lMaxNumElements  -  max number of sequence elements or 0                    |
|                                                                              |
|    Example of Construction:                                                  |
| ITypeCodeString *pStr = new ITypeCodeString(100);                            |
| ITypeCodeSequence  *pSeq  = new ITypeCodeSequence (pStr,100);                |
------------------------------------------------------------------------------*/
  ITypeCodeSequence (const ITypeCode *pSeqElementTypeI, long lMaxNumElements = 0);
  virtual ~ITypeCodeSequence ( );

  // compare 2 typecodes
  Boolean
    operator == (const ITypeCode &a ) const;

  // returns the type of what typecode describes
  TCKind
    kind() const;

  ITypeCodeSequence*
    duplicate() const;

  // returns the number of parameters that make up the receiving typecode
  long
    paramCount() const;

  // returns the indexed parameter from the receiving typecode. Index starts at 0.
  IAny
    parameter (long index) const;

private:
  ITypeCodeSequence ( const ITypeCodeSequence &typeCodeSeq );

  ITypeCode *pSeqElementType;
  long lMaxNumberOfElements;

};


#ifndef _ITK_SEQ_INL_
#include "itk_seq.inl"
#endif

#endif
