/********************************************************************************/
/* File:  itk_unio.hpp                                                          */
/* Class: ITypeCodeUnion                                                        */
/*                                                                              */
/* Type code for union                                                          */
/********************************************************************************/

/******************************************************************************/
/*                                                                            */
/* IBM(R) VisualAge(TM) C++ for OS/2(R), Version 3                            */
/* (C) Copyright IBM Corp. 1991, 1995.                                        */
/*  - Licensed Material - Program-Property of IBM - All Rights Reserved.      */
/* US Government Users Restricted Rights - Use, duplication or disclosure     */
/* restricted by GSA ADP Schedule Contract with IBM Corp.                     */
/*                                                                            */
/******************************************************************************/

#ifndef _ITK_UNIO_HPP_
#define _ITK_UNIO_HPP_

#include "itypecod.hpp"
#include <istring.hpp>
#include "dax.hpp"

class PICListElementList;

class EXPORT ITypeCodeUnion : public ITypeCode
{

public:

/*------------------------- Constructors ---------------------------------------
| There is 1 way to construct instances of this class.                         |
| parms:                                                                       |
|  strUnionName - union name                                                   |
|  lNumElements  - number of elements in ...                                   |
|  ...           - IString strMemberName, ITypeCode *typeCodeMember,           |
|                 these 2 fields are repeated as needed)                       |
|                                                                              |
|    Example of Construction:                                                  |
| ITypeCodeString *pStr = new ITypeCodeString(100);                            |
| ITypeCodeUnion *pUnion = new ITypeCodeUnion("Animal", 2,                     |
|                                              ,"Name",pStr);                  |
------------------------------------------------------------------------------*/
  ITypeCodeUnion (const IString &strUnionNameI, long lNumElementsI, ... );
  virtual ~ITypeCodeUnion ( );

  // compare 2 typecodes
  Boolean
    operator == (const ITypeCode &a ) const;

  // returns the type of what typecode describes
  TCKind
    kind() const;

  ITypeCodeUnion*
    duplicate() const;

  long
    paramCount() const;

  IAny
    parameter ( long index ) const;

private:
  ITypeCodeUnion ( const ITypeCodeUnion &typeCodeUnion );

  IString strName;
  long lNumElements;
  PICListElementList *p_elements;           // member List

};


#ifndef _ITK_UNIO_INL_
#include "itk_unio.inl"
#endif

#endif
