#ifndef _IUSERENV_HPP_
#define _IUSERENV_HPP_

/*******************************************************************************
* File: iuserenv.hpp                                                           *
* Class: IUserEnvironment                                                      *
*                                                                              *
* The IUserEnvironment allows a client to establish the global context for     *
* datastore access.                                                            *
*                                                                              *
* Associated with the IUserEnvironment is any data that must be global.  This  *
* includes options (see below), the defined connections (see the               *
* IDatastoreConnection class), and information on which connection is current. *
*******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* IBM(R) VisualAge(TM) C++ for OS/2(R), Version 3                            */
/* (C) Copyright IBM Corp. 1991, 1995.                                        */
/*  - Licensed Material - Program-Property of IBM - All Rights Reserved.      */
/* US Government Users Restricted Rights - Use, duplication or disclosure     */
/* restricted by GSA ADP Schedule Contract with IBM Corp.                     */
/*                                                                            */
/******************************************************************************/

#include "idaxbs.h"
#include "dax.hpp"
#include <istdntfy.hpp>
#include "iany.hpp"
#include "idscon.hpp"
#include "idsexc.hpp"

class IString;
class AnyOptionSet;
class DSCElementSet;
class DatastoreMgrCli_Proxy;
class DatastoreMgrSQL_Proxy;

#pragma pack(4)

class EXPORT IUserEnvironment : public IStandardNotifier {

public:

  IUserEnvironment ();
  virtual
    ~IUserEnvironment ();

  void
    getOption (long option, IAny& value)
     throw(IOptionInvalid);
  void
    setOption (long option, const IAny& value)
     throw (IValueTypeInvalid,
            IValueInvalid,
            IOptionInvalid);

   static INotificationId
    setOptionId;

private:

  AnyOptionSet     *pIOptKeySet;            // option dictionary
  DSCElementSet    *pIDSCSet;               // DSC dictionary

    /*------------------------------------------------------------------------------
    |   addDatastoreConnection - Used by an IDatastoreConnection to add itself     |
    |                            to the IUserEnvironment.                          |
    |                                                                              |
    |   removeDatastoreConnection - Used by an IDatastoreConnection to remove      |
    |                               itself from the IUserEnvironment.              |
    |                                                                              |
    |   getDatastoreConnections   - Used to get by the DSM to get datastore        |
    |                               connections.                                   |
    ------------------------------------------------------------------------------*/

  friend IDatastoreConnection::IDatastoreConnection(IUserEnvironment&);
  friend IDatastoreConnection::IDatastoreConnection();
  friend IDatastoreConnection::~IDatastoreConnection();
  friend DatastoreMgrCli_Proxy;
  friend DatastoreMgrSQL_Proxy;
  friend void IDatastoreConnection::setUe(IUserEnvironment * userEnv);

  Boolean addDatastoreConnection    (IDatastoreConnection *dsc);
  Boolean removeDatastoreConnection (IDatastoreConnection *dsc);

  void              *getDatastoreConnections();
  void              *getUserEnvironmentHandle() {return pUserEnvironmentHandle;}
  void              *pUserEnvironmentHandle;

}; // IUSERENV

#pragma pack()

#endif /* _IUSERENV_HPP_ */
